////////////////////////////////////////////////////////////////////////
/// \version 
////////////////////////////////////////////////////////////////////////
#include "CheckDAQChannelMap.h"
#include "DAQChannelMap/DAQChannelMap.h"
#include <vector>
#include <cmath>
#include "TH1.h"
#include "TH2.h"
#include "TH3.h"
#include "TSystem.h"
#include "Config/Config.h"
#include "Config/Param.h"
#include "EventDataModel/EventHandle.h"
#include "CheckDAQChannelMap.h"
#include "JobControl/Exception.h"
#include "JobControl/ModuleFactory.h"
#include "RawData/DAQHeader.h"
#include "RawData/RawDigit.h"
//#include "NOvAFMWK/Utilities/EDMUtils.h"
#include "Geometry/geo.h"
#include "Geometry/PlaneGeo.h"
//#include "CMap/CMap.h"
#include "Header/Header.h"

#include "XMLInterface/xmli.h"
#include "Config/cfg.h"
#include "JobControl/jobc.h"
#include "EventDataModel/edm.h"
#include "Geometry/Geometry.h"
//#include "NOvAFMWK/Utilities/RandomHandler.h"

#include "DAQDataFormats/RawEvent.h"

//#include "DAQ2RawDigit/FlatDAQData/FlatDAQData.h"
#include "DAQDataFormats/BitFields.h"

#include "IoModules/RWFactory.h"
#include <cstdlib>
#include <iostream>
#include <JobControl/Module.h>
#include "Utilities/EDMUtils.h"

static int nevents = 0;

using namespace rawp;
using namespace daqdataformats;
using namespace daqchannelmap;

MODULE_DECL(rawp::CheckDAQChannelMap);


//......................................................................

CheckDAQChannelMap::CheckDAQChannelMap(const char* version) :
  jobc::Module("rawp::CheckDAQChannelMap")
{

  this->SetCfgVersion(version);
  this->SetWatch("CheckDAQChannelMapping", "default");


}

//......................................................................

void CheckDAQChannelMap::Update(const cfg::Config& c) {
  // if 0, then it means all
  // Count from 1

    int diblock;
    int dcm;
    int dcmport;
    int pixel;

   //c("detector").Get();
   c("diblock").Get(diblock);
   c("dcm").Get(dcm);
   c("dcmport").Get(dcmport);
   c("pixel").Get(pixel);


   //_detector = NDOS_ID;

   _diblock = diblock;
   _dcm = dcm;
   _dcmport = dcmport;
   _pixel = pixel;





}

//......................................................................

CheckDAQChannelMap::~CheckDAQChannelMap()
{
}

//......................................................................

jobc::Result CheckDAQChannelMap::Reco(edm::EventHandle& evt)
{
evt.Header().SetEvent(nevents);



  std::cout << "From CheckDAQChannelMap::Reco. run    : " << evt.Header().Run()
 	    << " subrun : " << evt.Header().Subrun() 
 	    << " event  : " << evt.Header().Event() << std::endl;



  geo::Geometry& geo = util::EDMUtils::GetGeometry(evt);
  std::string detector_gdml = geo.GDMLFile();


  if     (  detector_gdml.find("neardet")!= std::string::npos)
      _detector = NEARDET_ID;
  else if(  detector_gdml.find("ndos")!= std::string::npos)
      _detector = NDOS_ID;
  else if(  detector_gdml.find("fardet")!= std::string::npos)
      _detector = FARDET_ID;
  else {
      std::cout<<"\nUnknown detector!!!!!!!!!!!!!";
      return jobc::kFailed;
  }

  std::cout<<"\nNeVents ="<<nevents<<"\n";

  ///setup the Header
  //hdr::Header header;

  hdr::Header header;
  header.SetVersion      ("MonteCarlo", "DAQsim");
  header.SetRandomSeed   ("DAQsim"    , 0);
  header.SetConfiguration("DAQsim"    , "muons");
  //header.SetConfiguration("Geometry"  , "ndos.gdml");
  //lsheader.SetConfiguration("DAQsim"    , nova_jobc::XMLConfiguration());

  
  //Create vector of Digit Lists
  std::vector<rawdata::RawDigit> rawdigitlist(0);

  //ev.print();
  
  rawdata::RawDigit digit;


  DAQChannelMap* chanMap = DAQChannelMap::getInstance();
  //cmap::CMap &cmap = util::EDMUtils::GetCMap(evt);

  this->getLimits(_diblock, _dcm, _dcmport, _pixel);

  int rawdigitnumber = 0;
  for(uint32_t idiblock = _diblockLimit[0]; idiblock < _diblockLimit[1]; ++idiblock)
  //for(uint32_t idiblock = 3; idiblock < 4; ++idiblock)
  for(uint32_t idcm     = _dcmLimit[0]    ; idcm     < _dcmLimit[1]    ; ++idcm){

      if (_dcmport == 0)this->getDCMPortLimits(idiblock, idcm);

       //std::cout<<"\nDCMport limits "<<_dcmportLimit[0] << " | "<<_dcmportLimit[1];
               
      for(uint32_t idcmport = _dcmportLimit[0]; idcmport < _dcmportLimit[1]; ++idcmport){
      //for(uint32_t idcmport = 33; idcmport < 51; ++idcmport){


          idcmport = this->isForbidden(idiblock,idcm,idcmport);
          //std::cout<<"\nport = "<< idcmport;


            for(uint32_t ipixel   = _pixelLimit[0]  ; ipixel   < _pixelLimit[1]  ; ++ipixel){
            
                dchan daq_channel = chanMap->encodeDChan(_detector,
                                                         idiblock,
                                                         idcm,
                                                         idcmport,
                                                         ipixel);

               std::cout<<"\nport = "<<idiblock<<"   "<<idcm<<"   "<<idcmport;



                lchan offline_channel = chanMap->encodeLChan(daq_channel);


                digit.SetChannel(offline_channel);
                digit.SetTDC(0,1000 + 2000*idiblock + 100*idcm);
                digit.SetTDC(0,1000);
                digit.SetADC(0,1000);
                rawdigitlist.push_back(digit);
                ++rawdigitnumber;

                //std::cout<<"\nplaneHere="<<plane;
                //std::cout<<"    cellHere="<<cell;

      }
     }
  }

  std::cout<<"\n Total of "<<rawdigitnumber<<" RawDigits filled";

      

      

  /*
      for(uint32_t plane = 0; plane < 186; ++plane){

        digit.SetChannel(this->getChannel(plane, 5));
        digit.SetTDC(0,1000+i*10);
        digit.SetADC(0,20+i*5);
        rawdigitnumber++;
        if(fillrawdigit == rawdigitnumber||fillrawdigit==0)rawdigitlist.push_back(digit);

      }//end of loop over planes

  */


  




  ///clear the evt as it is the same handle for every spill


  //if(!fIsHeaderPut){
        evt.Clear();
        std::cout<<"From  CheckDAQChannelMap::Reco. Putting the header\n";
        evt.DAQ().Put(header, ".");
        std::cout<<"From  CheckDAQChannelMap::Reco. End of Pitting the header\n";
  //}
  
  // Write the raw digits into the event
  if(evt.Raw().GetFolder(".") == 0)
    evt.Raw().MakeFolder(".");

  for (unsigned int i=0; i<rawdigitlist.size(); ++i) {
    evt.Raw().Put(rawdigitlist[i],".");
  }


  std::cout<<"End of job\n";


  ++nevents;
  return jobc::kPassed;

}

////////////////////////////////////////////////////////////////////////


uint32_t CheckDAQChannelMap::getChannel(uint32_t plane, uint32_t cell){

  uint32_t chan = 0x0;
  BITFIELDSET(chan,plane , MASK_OFFLINE_PLANE   , SHIFT_OFFLINE_PLANE);
  BITFIELDSET(chan,cell  , MASK_OFFLINE_CELL    , SHIFT_OFFLINE_CELL);

  return chan;
}



void CheckDAQChannelMap::getLimits(uint32_t diblock, uint32_t dcm, uint32_t dcmport, uint32_t pixel){






  if(diblock == 0){
    _diblockLimit[0] = 1;
    _diblockLimit[1] = NUM_DIBLOCK_DET[_detector] + 1;
  }
  else{
    _diblockLimit[0] = diblock;
    _diblockLimit[1] = diblock + 1;
  }
  if(dcm == 0){
    _dcmLimit[0] = 1;
    _dcmLimit[1] = NUM_DCMS_DIBLOCK[_detector] + 1;
  }
  else{
    _dcmLimit[0] = dcm;
    _dcmLimit[1] = dcm + 1;
  }
  if(dcmport == 0){

  }
  else{
    _dcmportLimit[0] = dcmport;
    _dcmportLimit[1] = dcmport + 1;
  }
  if(pixel == 0){
    _pixelLimit[0] = 1;
    _pixelLimit[1] = 33;
  }
  else{
    _pixelLimit[0] = pixel;
    _pixelLimit[1] = pixel + 1;
  }



}

void CheckDAQChannelMap::getDCMPortLimits(uint32_t currentdiblock, uint32_t currentdcm){

    _dcmportLimit[0] = 1;

    if(_detector == NEARDET_ID || _detector == NDOS_ID){
        if(currentdiblock < 3  && currentdcm < 3)
                _dcmportLimit[1] = 64;
        if(currentdiblock < 3  && currentdcm == 3)
                _dcmportLimit[1] = 48;
        if(currentdiblock == 3 && currentdcm == 1)
                _dcmportLimit[1] = 65;
        if(currentdiblock == 3 && currentdcm == 2)
                _dcmportLimit[1] = 63;
        if(currentdiblock == 3 && currentdcm == 3)
                _dcmportLimit[1] = 47;
        if(currentdiblock == 4 && currentdcm == 1)
                _dcmportLimit[1] = 23;
        if(currentdiblock == 4 && currentdcm == 2)
                _dcmportLimit[1] = 24;
        if(currentdiblock == 4 && currentdcm == 3)
                _dcmportLimit[1] = 8;
        return;
    }

    // so it is the far detector alright
    if(_detector == FARDET_ID){
    	if(DetectorDiBlockConfig[_detector][currentdiblock] == AB_BLOCK ||
           DetectorDiBlockConfig[_detector][currentdiblock] == BA_BLOCK ){
            	_dcmportLimit[1] = 64;
            	return;
    	}
    
    int yplanes;
    int xplanes;

    
    // Calculate number of planes (since there is no BB block)
    	if(DetectorDiBlockConfig[_detector][currentdiblock] == AA_BLOCK){
        	yplanes = NUM_Y_PLANES_A_BLOCK * 2;
        	xplanes = NUM_X_PLANES_A_BLOCK * 2;
    	}
    	else{
    		yplanes = NUM_Y_PLANES_A_BLOCK + NUM_Y_PLANES_B_BLOCK;
    		xplanes = NUM_X_PLANES_A_BLOCK + NUM_X_PLANES_B_BLOCK;
    	}
            
    	// y plane
    	if(currentdcm < NUM_X_DCMS_FARDET + START_CHAN_IDX)
        	_dcmportLimit[1] = 32 + xplanes + START_CHAN_IDX;
    	else
        	_dcmportLimit[1] = 32 + yplanes + START_CHAN_IDX;
    }
        
}

uint32_t CheckDAQChannelMap::isForbidden(uint32_t idiblock,uint32_t idcm,uint32_t idcmport){

    if(_detector == NEARDET_ID || _detector == NDOS_ID){
        if(idiblock == 4 && idcm ==  1 && idcmport == 7) return 17;
        if(idiblock == 4               && idcmport == 8) return 17;
        if(idiblock < 3 && idcmport == 48) return 49;
        if(idiblock == 3 && idcm == 2 && idcmport == 47) return 49;
        if(idcm == 3 && idcmport == 17) return 33;
        return idcmport;
    }
    // so it is the far detector alrigh

    else if(_detector == FARDET_ID){
    	int yplanes;
   	int xplanes;
    	unsigned int planes;
    // Calculate number of planes (since there is no BB block)
    	if(DetectorDiBlockConfig[_detector][idiblock] == AA_BLOCK){
        	yplanes = NUM_Y_PLANES_A_BLOCK * 2 ;
        	xplanes = NUM_X_PLANES_A_BLOCK * 2;
    	}
    	else{
        	yplanes = NUM_Y_PLANES_A_BLOCK + NUM_Y_PLANES_B_BLOCK;
        	xplanes = NUM_X_PLANES_A_BLOCK + NUM_X_PLANES_B_BLOCK;
    	}
    

    	if(idcm < NUM_X_DCMS_FARDET + START_CHAN_IDX)planes = xplanes;
    	else planes = yplanes;


    	//std::cout<<"\nplanes = "<<xplanes<<" |  "<<yplanes;

    	if(idcmport == planes + 16 + 1)return 49;

    	
    }
    return idcmport;
}
