////////////////////////////////////////////////////////////////////////
/// \version 
////////////////////////////////////////////////////////////////////////
#include "FillRawDigits.h"
#include <vector>
#include <cmath>
#include "TH1.h"
#include "TH2.h"
#include "TH3.h"
#include "TSystem.h"
#include "Config/Config.h"
#include "Config/Param.h"
#include "EventDataModel/EventHandle.h"
#include "JobControl/Exception.h"
#include "JobControl/ModuleFactory.h"
#include "RawData/DAQHeader.h"
#include "RawData/RawDigit.h"
//#include "NOvAFMWK/Utilities/EDMUtils.h"
#include "Geometry/geo.h"
#include "Geometry/PlaneGeo.h"
//#include "CMap/CMap.h"
#include "Header/Header.h"

#include "XMLInterface/xmli.h"
#include "Config/cfg.h"
#include "JobControl/jobc.h"
#include "EventDataModel/edm.h"
#include "Geometry/Geometry.h"
//#include "NOvAFMWK/Utilities/RandomHandler.h"

#include "DAQDataFormats/RawEvent.h"

//#include "DAQ2RawDigit/FlatDAQData/FlatDAQData.h"
#include "DAQDataFormats/BitFields.h"

#include "IoModules/RWFactory.h"
#include <cstdlib>
#include <iostream>

static int nevents = 0;

using namespace rawp;
using namespace daqdataformats;


MODULE_DECL(rawp::FillRawDigits);


//......................................................................

FillRawDigits::FillRawDigits(const char* version) :
  jobc::Module("rawp::FillRawDigits")
{

  this->SetCfgVersion(version);
 

}

//......................................................................

void FillRawDigits::Update(const cfg::Config& /*c*/) { }

//......................................................................

FillRawDigits::~FillRawDigits()
{
}

//......................................................................

jobc::Result FillRawDigits::Reco(edm::EventHandle& evt)
{
evt.Header().SetEvent(nevents);

  std::cout << "From FillRawDigits::Reco. run    : " << evt.Header().Run()
 	    << " subrun : " << evt.Header().Subrun() 
 	    << " event  : " << evt.Header().Event() << std::endl;


  std::cout<<"\nNeVents ="<<nevents<<"\n";

  ///setup the Header
  //hdr::Header header;

  hdr::Header header;
  header.SetVersion      ("MonteCarlo", "DAQsim");
  header.SetRandomSeed   ("DAQsim"    , 0);
  header.SetConfiguration("DAQsim"    , "muons");
  header.SetConfiguration("Geometry"  , "ndos.gdml");
  //lsheader.SetConfiguration("DAQsim"    , nova_jobc::XMLConfiguration());

  
  //Create vector of Digit Lists
  std::vector<rawdata::RawDigit> rawdigitlist(0);

  //ev.print();
  
  rawdata::RawDigit digit;


  //int fillrawdigit = 621;
  int fillrawdigit = 0;
  int rawdigitnumber = 0;

  for(int i=-1; i < nevents; ++i){

      //first 5 Blocks
      for(uint32_t ihorizontal = 1; ihorizontal < 78; ++ihorizontal)
      for(uint32_t icell = 0; icell < 96; ++icell)

      {
            uint32_t plane = ihorizontal*2 - 1;
            digit.SetChannel(this->getChannel(plane, icell));
            digit.SetTDC(0,1000);
            digit.SetADC(0,1000);
            rawdigitnumber++;
            if(fillrawdigit == rawdigitnumber||fillrawdigit==0)rawdigitlist.push_back(digit);

      }

      //first 5 Blocks
      for(uint32_t ivertical = 1; ivertical < 79; ++ivertical)
      for(uint32_t icell = 0; icell < 64; ++icell)

      {
            uint32_t plane = (ivertical - 1) * 2;
            digit.SetChannel(this->getChannel(plane, icell));
            digit.SetTDC(0,1000);
            digit.SetADC(0,1000);
            rawdigitnumber++;
            if(fillrawdigit == rawdigitnumber||fillrawdigit==0)rawdigitlist.push_back(digit);

      }


      // 6-th block
      for(uint32_t ivertical = 79; ivertical < 95; ++ivertical)
      for(uint32_t icell = 0; icell < 64; ++icell)
      {
            uint32_t plane = 77 + 79 + (ivertical - 79)*2 -1;
            digit.SetChannel(this->getChannel(plane, icell));
            digit.SetTDC(0,1000);
            digit.SetADC(0,1000);
            rawdigitnumber++;
            if(fillrawdigit == rawdigitnumber||fillrawdigit==0)rawdigitlist.push_back(digit);

      }

      for(uint32_t ihorizontal = 78; ihorizontal < 93; ++ihorizontal)
      for(uint32_t icell = 0; icell < 96; ++icell)

      {
            uint32_t plane = 77 + 79 + (ihorizontal - 78)*2;
            digit.SetChannel(this->getChannel(plane, icell));
            digit.SetTDC(0,1000);
            digit.SetADC(0,1000);
            rawdigitnumber++;
            if(fillrawdigit == rawdigitnumber||fillrawdigit==0)rawdigitlist.push_back(digit);

      }

      // 7-th block
      for(uint32_t ivertical = 95; ivertical < 101; ++ivertical)
      for(uint32_t icell = 0; icell < 64; ++icell)
      {
            uint32_t plane = 93 + 95 + (ivertical - 95)*2 -1;
            digit.SetChannel(this->getChannel(plane, icell));
            digit.SetTDC(0,1000);
            digit.SetADC(0,1000);
            rawdigitnumber++;
            if(fillrawdigit == rawdigitnumber||fillrawdigit==0)rawdigitlist.push_back(digit);

      }

      for(uint32_t ihorizontal = 93; ihorizontal < 100; ++ihorizontal)
      for(uint32_t icell = 0; icell < 96; ++icell)

      {
            uint32_t plane = 93 + 94 + (ihorizontal - 93)*2 - 1;
            digit.SetChannel(this->getChannel(plane, icell));
            digit.SetTDC(0,1000);
            digit.SetADC(0,1000);
            rawdigitnumber++;
            if(fillrawdigit == rawdigitnumber||fillrawdigit==0)rawdigitlist.push_back(digit);

      }


      

      

  /*
      for(uint32_t plane = 0; plane < 186; ++plane){

        digit.SetChannel(this->getChannel(plane, 5));
        digit.SetTDC(0,1000+i*10);
        digit.SetADC(0,20+i*5);
        rawdigitnumber++;
        if(fillrawdigit == rawdigitnumber||fillrawdigit==0)rawdigitlist.push_back(digit);

      }//end of loop over planes

  */


  }




  ///clear the evt as it is the same handle for every spill


  //if(!fIsHeaderPut){
        evt.Clear();
        std::cout<<"From  FillRawDigits::Reco. Putting the header\n";
        evt.DAQ().Put(header, ".");
        std::cout<<"From  FillRawDigits::Reco. End of Pitting the header\n";
  //}
  
  // Write the raw digits into the event
  if(evt.Raw().GetFolder(".") == 0)
    evt.Raw().MakeFolder(".");

  for (unsigned int i=0; i<rawdigitlist.size(); ++i) {
    evt.Raw().Put(rawdigitlist[i],".");
  }


  std::cout<<"End of job\n";


  ++nevents;
  return jobc::kPassed;

}

////////////////////////////////////////////////////////////////////////


uint32_t FillRawDigits::getChannel(uint32_t plane, uint32_t cell){

  uint32_t chan = 0x0;
  BITFIELDSET(chan,plane , MASK_OFFLINE_PLANE   , SHIFT_OFFLINE_PLANE);
  BITFIELDSET(chan,cell  , MASK_OFFLINE_CELL    , SHIFT_OFFLINE_CELL);

  return chan;
}

