#include <cppunit/extensions/HelperMacros.h>
#include <DAQChannelMap/ChannelMapException.h>
#include <DAQChannelMap/DAQChannelMap.h>
#include "NovaDAQConventions/DAQConventions.h"

#if DAQCHANNELMAP_VERSION == 2
#define GETDETECTOR getDetector
#define GETDIBLOCK  getDiBlock
#define GETDCM      getDCM
#define GETFEB      getDCMLink
#define GETPIXEL    getPixel
#define GETPLANE    getPlane
#define GETCELL     getCell
#define PUT_ENUM(INPUT) CHAN_IDX_0_1                 \
                        INPUT##_NEARDET,             \
                        INPUT##_FARDET,              \
                        INPUT##_NDOS,                \
                        INPUT##_TESTBEAM_2Block_1x1, \
                        INPUT##_TESTBEAM_3Block_1x1, \
                        INPUT##_TESTBEAM_2Block_2x2, \
                        INPUT##_TESTBEAM_3Block_2x2, \
                        INPUT##_IPND
#define NUMDCMPORTS     NUM_DCM_PORTS
#define NUMPIXELS       NUM_PIXELS
#define NEARDETID       novadaq::cnv::kNEARDET
#define FARDETID        novadaq::cnv::kFARDET

#define GETPLANES(CHMAP, DET) CHMAP->getTotalNumberOfPlanes(DET);

#define GETDCMOFFLINE(LCHAN, DET) getDCMOffline(LCHAN)

#define GETDCMLINKOFFLINE(LCHAN, DET) getDCMLinkOffline(LCHAN)

#define ENCODEDCHAN(LCHAN, DET) encodeDChan(LCHAN)

#else
#define GETDETECTOR getDetector
#define GETDIBLOCK  getDiBlock
#define GETDCM      getDCMLocal
#define GETFEB      getDCMPort
#define GETPIXEL    getPixel
#define GETPLANE    getPlaneGlobal
#define GETCELL     getCellGlobal
#define PUT_ENUM(INPUT) CHAN_IDX_0_1       \
                        DAQChannelMap::INPUT##_IPND,      \
                        DAQChannelMap::INPUT##_NDOS,      \
                        DAQChannelMap::INPUT##_NEARDET,   \
                        DAQChannelMap::INPUT##_FARDET
#define NUMDCMPORTS     DAQChannelMap::NUM_DCM_PORTS
#define NUMPIXELS       DAQChannelMap::NUM_PIXELS
#define NEARDETID       DAQChannelMap::NEARDET_ID
#define FARDETID        DAQChannelMap::FARDET_ID

#define GETPLANES(CHMAP, DET) 256

#define GETDCMOFFLINE(LCHAN, DET) getDCMDiBlockLocalOffline(LCHAN, DET)

#define GETDCMLINKOFFLINE(LCHAN, DET) getLinkDCMLocalOffline(LCHAN, DET)

#define ENCODEDCHAN(LCHAN, DET) encodeDChan(LCHAN, DET)

#endif

using namespace daqchannelmap;

class DAQChannelMapBenchMarkTest
{
public:

  DAQChannelMapBenchMarkTest():vector_count(0){};
  ~DAQChannelMapBenchMarkTest(){std::cout<<"Number of tests  ="<<vector_count<<"\n";};

  void testBasicChannelGetOnline();
  
private:

  int vector_count;

};

void DAQChannelMapBenchMarkTest::testBasicChannelGetOnline() {
  // Retrieve the channel map class (singleton)
  DAQChannelMap* chanMap = DAQChannelMap::getInstance();


  const unsigned int NUM_DIBLOCK_DET[] = { PUT_ENUM(NUM_DIBLOCK) };
  const unsigned int NUM_DCMS_DIBLOCK[]= { PUT_ENUM(NUM_DCMS_DIBLOCK) };

  /*
  // Test channel encoding/decoding
  for(int det             = NEARDETID; det < FARDETID;           ++det){
    for(int diblock       = 1; diblock < NUM_DIBLOCK_DET[det];  ++diblock){
      for(int dcm         = 1; dcm     < NUM_DCMS_DIBLOCK[det]; ++dcm    ){
	for(int port      = 1; port    < NUMDCMPORTS;         ++port   ){
	  for(int pixel   = 1; pixel   < (NUMPIXELS+1);        ++pixel  ){
	    dchan theChan = chanMap->encodeDChan(det,diblock,dcm,port,pixel);
	    lchan lChan   = chanMap->encodeLChan(theChan);
	    dchan dChan2  = chanMap->encodeDChan(lChan);

            vector_count++;
	    //chanMap->GETDETECTOR(theChan);
	    //chanMap->GETDIBLOCK(theChan);
	    //chanMap->GETDCM(theChan);
	    //chanMap->GETFEB(theChan);
	    //chanMap->GETPIXEL(theChan);

            //chanMap->GETCELL(theChan);
            //chanMap->GETPLANE(theChan);


	  } // endfor pixel
	} // endfor port
      } // endfor dcm
    } // endfor diblock
  } // endfor det
   */

  for(int det             = NEARDETID; det < FARDETID;           ++det){

      uint32_t planes = GETPLANES(chanMap, det);

      //std::cout<<"planes="<<planes<<"\n";

      for(uint32_t iplane = 0; iplane < planes; ++iplane){

          //uint32_t cells = chanMap->getNumberOfChannelsInPlane(det, iplane);
          uint32_t cells = 64;

            for(uint32_t icell = 0; icell < cells; ++icell){

                uint32_t lChan  = chanMap->encodeLChan(det, iplane, icell);

                
                //const diblock_t     diblock   = chanMap->getDiBlockOffline(lChan);
                //const uint32_t      dcm_id    = chanMap->GETDCMOFFLINE(lChan, det);
                //const uint32_t    dcm_link  = chanMap->GETDCMLINKOFFLINE(lChan, det);
                //const pixel_t       pixel     = chanMap->getPixelOffline(lChan);

                //const apd_pixel_t   apd_pixel = chanMap->pixel2APDPixel(pixel);

                uint32_t dChan = chanMap->ENCODEDCHAN(lChan, det);
                uint32_t lChan2 = chanMap->encodeLChan(dChan);


                vector_count++;

            }



      }


  }
  //std::cout<<NUM_DCMS_DIBLOCK[NEARDETID]<<"\n";
  //exit(1);

}

int main(){


    DAQChannelMapBenchMarkTest bm_test;


    for(uint32_t i = 0; i < 500; ++i)
        bm_test.testBasicChannelGetOnline();






    return 0;
}