#ifndef RAWMICROBLOCKHEADERV0_H
#define RAWMICROBLOCKHEADERV0_H
#include "DAQDataFormats/RawDAQData.h"
#include "DAQDataFormats/Macros.h"
#include "DAQDataFormats/BitFields.h"
#include "DAQDataFormats/RawDCMChan.h"

/// It is requred to
///   1. include DAQDataFormats/DataFormatFirst.inc
///   2. define CURRENT_CLASS
///   3.    define CURRENT_CLASS_VERSION for version of the class
///      OR define LAST_CLASS_VERSION    for the main structure of the class
///   4. (optional) define CURRENT_FIELDS_SUB_NAMESPACE
///   5. include DAQDataFormats/DataFormatLast.inc
/// before the declaration of the DataFormat version class in the header.
#include "DAQDataFormats/DataFormatFirst.inc"
#define  CURRENT_CLASS                RawMicroBlockHeader
#define  CURRENT_CLASS_VERSION        0
#define  CURRENT_FIELDS_SUB_NAMESPACE rawmicroblockhdr
#include "DAQDataFormats/DataFormatLast.inc"

namespace daqdataformats {
namespace  VERSION_NAMESPACE {
            namespace CURRENT_FIELDS_SUB_NAMESPACE {
        
                static const uint32_t RAWMICROBLOCKHEADER_SIZE          = 1;
                static const uint32_t RAWMICROBLOCKHEADER_SIZE_PLUS_ONE = RAWMICROBLOCKHEADER_SIZE + 1;
            
                enum MicroBlockHeaderMASKS{
                    MASK_Partition                = 0xFC000000, //! Partition                   ( 6 bits)
                    MASK_V0                       = 0x02000000, //! First Version bit           ( 1 bit)
                    MASK_DestBuffID               = 0x01FF0000, //! Dest Buffer ID Mask         ( 9 bits, 512 buffer nodes)
                    MASK_DCMChan                  = 0x0000FFFF,
                };

                enum MicroBlockHeaderWORDS{
                    WORD_Partition                = 0,
                    WORD_V0                       = 0,
                    WORD_DestBuffID               = 0,
                    WORD_DCMChan                  = 0,
                };

                DEFINE_SHIFT_FROM_MASK(Partition);
                DEFINE_SHIFT_FROM_MASK(V0);
                DEFINE_SHIFT_FROM_MASK(DestBuffID);
                DEFINE_SHIFT_FROM_MASK(DCMChan);
            }// end of namespace rawmicroblockhdr
    


  class RawMicroBlockHeader: public PREVIOUS_VERSION_CLASS_NAME {

        public:

            // Constructors
            RawMicroBlockHeader(const version_t);
            RawMicroBlockHeader(const RawMicroBlockHeader&);
            ~RawMicroBlockHeader();

            void     init           ();
            using    PREVIOUS_VERSION_CLASS_NAME::print;
            void     print          (std::ostream& os=std::cout) const;
            uint32_t sizeofdata     () const {return CURRENT_FIELDS_SUB_NAMESPACE::RAWMICROBLOCKHEADER_SIZE;}
            bool     setBufferSource(const void* pointer);

            /// RawMicroBlockHeader is a fixed size format
            using    RawDAQData::readData;
            void*    readData(const void* pointer){return RawDAQData::readDataGeneral(pointer);};

            daqdataformats::RawDCMChan* getDCMChan() {return _DCMChan;}

            bool setAllParameters(const void*);

            uint32_t getDiblockId()   const {return _DCMChan->getDiblockId();}
            uint32_t getDCMId()       const {return _DCMChan->getDCMId();}
            uint32_t getDetId()       const {return _DCMChan->getDetId();}
            uint32_t getGlobalDCMID() const {return _DCMChan->getGlobalDCMID();}

            bool setDiblockId  (const uint32_t a) {return _DCMChan->setDiblockId(a);}
            bool setDCMId      (const uint32_t a) {return _DCMChan->setDCMId(a);}
            bool setDetId      (const uint32_t a) {return _DCMChan->setDetId(a);}
            bool setGlobalDCMID(const uint32_t a) {return _DCMChan->setGlobalDCMID(a);}

            // V1 field is defined in versions greater than 0
            uint32_t getV1(              ) const { GENERATE_FUNCTION_IS_NOT_ALLOWED; return 0;}
            bool     setV1(const uint32_t)       { return GENERATE_FUNCTION_IS_NOT_ALLOWED;}

        protected:

            IMPLEMENT_FIRST_GET_AND_SET(DestBuffID, uint32_t)
            IMPLEMENT_FIRST_GET_AND_SET(Partition , uint32_t)
            IMPLEMENT_FIRST_GET_AND_SET(V0        , uint32_t)

	    INIT_FIRST(getVersion    , const, uint32_t, )
            INIT_FIRST(setVersion    ,      , bool    ,uint32_t)


        protected:
            daqdataformats::RawDCMChan* _DCMChan;  //<! DCM identifier
  };


}} // END NAMESPCE daqdataformats
#endif /* RAWMICROBLOCKHEADERV0_H */
