#ifndef RAWNANOSLICEHEADER_H
#define RAWNANOSLICEHEADER_H
#include "DAQDataFormats/RawNanoSliceHeaderV4.h"

/// It is requred to
///   1. include DAQDataFormats/DataFormatFirst.inc
///   2. define CURRENT_CLASS
///   3.    define CURRENT_CLASS_VERSION for version of the class
///      OR define LAST_CLASS_VERSION    for the main structure of the class
///   4. (optional) define CURRENT_FIELDS_SUB_NAMESPACE
///   5. include DAQDataFormats/DataFormatLast.inc
/// before the declaration of the DataFormat version class in the header.
#include "DAQDataFormats/DataFormatFirst.inc"
#define  CURRENT_CLASS                RawNanoSliceHeader
#define  LAST_CLASS_VERSION           4
#define  CURRENT_FIELDS_SUB_NAMESPACE nanosliceheader
#include "DAQDataFormats/DataFormatLast.inc"


namespace daqdataformats {

    namespace CURRENT_FIELDS_SUB_NAMESPACE {
        static const version_t RAWNANOSLICE_LATEST_VERSION  = LAST_CLASS_VERSION;
        static const version_t RAWNANOSLICE_DEFAULT_VERSION = 0;
    }

    class RawNanoSliceHeader: public PREVIOUS_VERSION_CLASS_NAME{
        public:
            // Constructors
            RawNanoSliceHeader(const version_t version = CURRENT_FIELDS_SUB_NAMESPACE::RAWNANOSLICE_DEFAULT_VERSION);//! The default constructor
	    RawNanoSliceHeader(const daqdataformats::NanoSliceHeader& Header);      //! Constructor with Data initialization
            RawNanoSliceHeader(const RawNanoSliceHeader& copy_in);  //! Deep copy constructor
            RawNanoSliceHeader(const void* buffer, const version_t version = rawdaqdataformat::DAQDATAFORMAT_UNKNOWN_VERSION );
            /// Destructor
            ~RawNanoSliceHeader();

            IMPLEMENT_MAIN_STANDARD

            /*
            IMPLEMENT_GET_AND_SET_MAIN(DataPresent, bool);
            IMPLEMENT_GET_AND_SET_MAIN(LinkStatus , bool);
            IMPLEMENT_GET_AND_SET_MAIN(DebugMode  , bool);
            IMPLEMENT_GET_MAIN        (Version  , uint32_t)
            IMPLEMENT_GET_AND_SET_MAIN(FEBId    , uint32_t)
            IMPLEMENT_GET_AND_SET_MAIN(FEBStatus, uint32_t)
            IMPLEMENT_GET_AND_SET_MAIN(PixelAddr, uint32_t)
            IMPLEMENT_GET_AND_SET_MAIN(Sequence , uint32_t)
            IMPLEMENT_GET_AND_SET_MAIN(Pad3     , uint32_t)
            IMPLEMENT_GET_AND_SET_MAIN(DAQEnabled   , bool)
            IMPLEMENT_GET_AND_SET_MAIN(BufferEmpty  , bool)
            IMPLEMENT_GET_AND_SET_MAIN(BufferFull   , bool)
            IMPLEMENT_GET_AND_SET_MAIN(CommError    , bool)
            IMPLEMENT_GET_AND_SET_MAIN(PacketError  , bool)
            IMPLEMENT_GET_AND_SET_MAIN(OverflowError, bool)
            IMPLEMENT_GET_AND_SET_MAIN(ADCError     , bool)
            */
            

            IMPLEMENT_VOID_RETURNING_FUNCTION_POINTER_1PAR    (initWithData , ,       const NanoSliceHeader&)
            IMPLEMENT_FUNCTION_RETURNING_FUNCTION_POINTER_1PAR(setParameters, , bool, const NanoSliceHeader& )

            IMPLEMENT_FIGUREOUT_VERSION_STATIC(RawNanoSliceHeader)

        protected:
            //IMPLEMENT_SET_MAIN(Version, uint32_t)

            void             initWithDataInternal();
            bool             setFunctionPointers();

        private:
            /// Pointer to NanoSlice Header parameters
            /// used in init when using Constructor with Data initialization only.
            /// After that this variable is irrelevant
            NanoSliceHeader* _nanosliceHeader;

    };
} // END NAMESPCE daqdataformats
#endif /* RAWNANOSLICEHEADER_H */
