#ifndef RAWNANOSLICEHEADERV0_H
#define RAWNANOSLICEHEADERV0_H
#include "DAQDataFormats/RawDAQData.h"
#include "DAQDataFormats/Macros.h"
#include "DAQDataFormats/BitFields.h"

/// It is requred to
///   1. include DAQDataFormats/DataFormatFirst.inc
///   2. define CURRENT_CLASS
///   3.    define CURRENT_CLASS_VERSION for version of the class
///      OR define LAST_CLASS_VERSION    for the main structure of the class
///   4. (optional) define CURRENT_FIELDS_SUB_NAMESPACE
///   5. include DAQDataFormats/DataFormatLast.inc
/// before the declaration of the DataFormat version class in the header.
#include "DAQDataFormats/DataFormatFirst.inc"
#define  CURRENT_CLASS                RawNanoSliceHeader
#define  CURRENT_CLASS_VERSION        0
#define  CURRENT_FIELDS_SUB_NAMESPACE nanosliceheader
#include "DAQDataFormats/DataFormatLast.inc"

namespace daqdataformats {

    //! Data structure for holding the unpacked FEB nanoslice header information
  //! Data structure for holding all the nanoslice data of interest
    typedef struct {
       //uint32_t PAD3;
       uint32_t FrameSequenceNumber;
       //uint32_t PAD5;
       uint32_t PixelAddress;
       uint32_t FEBStatus;
       uint32_t FEBLinkID;
       uint32_t Version;

       bool DebugMode;
       bool DataPresent;
       bool LinkStatus;
     } NanoSliceHeader;


     namespace  VERSION_NAMESPACE {
        namespace CURRENT_FIELDS_SUB_NAMESPACE {

            static const uint32_t  RAWNANOSLICEHEADER_SIZE = 1; ///< NanoSliceHeader size in words
            static const uint32_t  PAD3_DEFAULT_VALUE      = 1; ///< Default value for Pad3

            enum NanoSliceHeaderMASKS {
                MASK_LinkStatus   = 0x00000001, //! Link Status                         (1 bit)
                MASK_DataPresent  = 0x00000002, //! Denis Perevalov Present/Absent flag (1 bit)
                MASK_DebugMode    = 0x00000004, //! Debug Mode                          (1 bit)
                MASK_FEBId        = 0x000001F8, //! FEB ID                              (6 bit)
                MASK_FEBStatus    = 0x0000FE00, //! FEB Status                          (7 bit)
                MASK_PixelAddr    = 0x001F0000, //! Pixel Address                       (5 bit)
                MASK_Version      = 0x03E00000, //! version number                      (5 bit)
                MASK_Sequence     = 0x1C000000, //! Sequence number                     (3 bit)
                MASK_Pad3         = 0xE0000000, //! Pad3                                (3 bit)
                MASK_DAQEnabled   = 0x00000200, //! DAQ Enabled Status Bit              (1 bit)
                MASK_BufferEmpty  = 0x00000400, //! APD Data Buffer Empty               (1 bit)
                MASK_BufferFull   = 0x00000800, //! APD Data Buffer Full                (1 bit)
                MASK_CommError    = 0x00001000, //! FEB Communications Error            (1 bit)
                MASK_PacketError  = 0x00002000, //! FEB Packet Error                    (1 bit)
                MASK_OverflowError= 0x00004000, //! FEB Data Buff Overflow Error        (1 bit)
                MASK_ADCError     = 0x00008000, //! FEB ADC Error                       (1 bit)
	    };// end of NanoSliceHeaderMASKS

            enum NanoSliceHeaderWORDS {
                WORD_LinkStatus   = 0, //! Link Status                         (1 bit)
                WORD_DataPresent  = 0, //! Denis Perevalov Present/Absent flag (1 bit)
                WORD_DebugMode    = 0, //! Debug Mode                          (1 bit)
                WORD_FEBId        = 0, //! FEB ID                              (6 bit)
                WORD_FEBStatus    = 0, //! FEB Status                          (7 bit)
                WORD_PixelAddr    = 0, //! Pixel Address                       (5 bit)
                WORD_Version      = 0, //! version number                      (5 bit)
                WORD_Sequence     = 0, //! Sequence number                     (3 bit)
                WORD_Pad3         = 0, //! Pad3                                (3 bit)
                WORD_DAQEnabled   = 0, //! DAQ Enabled Status Bit              (1 bit)
                WORD_BufferEmpty  = 0, //! APD Data Buffer Empty               (1 bit)
                WORD_BufferFull   = 0, //! APD Data Buffer Full                (1 bit)
                WORD_CommError    = 0, //! FEB Communications Error            (1 bit)
                WORD_PacketError  = 0, //! FEB Packet Error                    (1 bit)
                WORD_OverflowError= 0, //! FEB Data Buff Overflow Error        (1 bit)
                WORD_ADCError     = 0, //! FEB ADC Error                       (1 bit)
	    };// end of NanoSliceHeaderWORD

            DEFINE_SHIFT_FROM_MASK( LinkStatus   );
            DEFINE_SHIFT_FROM_MASK( DataPresent  );
            DEFINE_SHIFT_FROM_MASK( DebugMode    );
            DEFINE_SHIFT_FROM_MASK( FEBId        );
            DEFINE_SHIFT_FROM_MASK( FEBStatus    );
            DEFINE_SHIFT_FROM_MASK( PixelAddr    );
            DEFINE_SHIFT_FROM_MASK( Version      );
            DEFINE_SHIFT_FROM_MASK( Sequence     );
            DEFINE_SHIFT_FROM_MASK( Pad3         );
            DEFINE_SHIFT_FROM_MASK( DAQEnabled   );
            DEFINE_SHIFT_FROM_MASK( BufferEmpty  );
            DEFINE_SHIFT_FROM_MASK( BufferFull   );
            DEFINE_SHIFT_FROM_MASK( CommError    );
            DEFINE_SHIFT_FROM_MASK( PacketError  );
            DEFINE_SHIFT_FROM_MASK( OverflowError);
            DEFINE_SHIFT_FROM_MASK( ADCError     );

         }// end of nanosliceheader namespace

    class RawNanoSliceHeader : public PREVIOUS_VERSION_CLASS_NAME {
    
        public:
            RawNanoSliceHeader (const version_t); ///< The default constructor
            RawNanoSliceHeader (const RawNanoSliceHeader&);
            ~RawNanoSliceHeader(); ///< Destructor

            bool     advanceSequence();
            void     init            ();
            using    PREVIOUS_VERSION_CLASS_NAME::print;
            void     print           (std::ostream& os=std::cout) const;
            uint32_t sizeofdata      () const { return CURRENT_FIELDS_SUB_NAMESPACE::RAWNANOSLICEHEADER_SIZE;}
            using    PREVIOUS_VERSION_CLASS_NAME::readData;
            void*    readData        (const void* pointer){return RawDAQData::readDataGeneral(pointer);};


            /// CAUTION!!!!!
            /// It is  assume that these methods will
            /// be the same for all versions of NanoSlice Header
            IMPLEMENT_GET_AND_SET_BOOL(DataPresent)
            IMPLEMENT_GET_AND_SET_BOOL(LinkStatus)
            IMPLEMENT_GET_AND_SET_BOOL(DebugMode)
            IMPLEMENT_GET_AND_SET_BOOL(DAQEnabled)
            IMPLEMENT_GET_AND_SET_BOOL(BufferEmpty)
            IMPLEMENT_GET_AND_SET_BOOL(BufferFull)
            IMPLEMENT_GET_AND_SET_BOOL(CommError)
            IMPLEMENT_GET_AND_SET_BOOL(PacketError)
            IMPLEMENT_GET_AND_SET_BOOL(OverflowError)
            IMPLEMENT_GET_AND_SET_BOOL(ADCError)
            IMPLEMENT_GET             (Version  , uint32_t)
            IMPLEMENT_GET_AND_SET     (FEBId, uint32_t)
            IMPLEMENT_GET_AND_SET     (FEBStatus, uint32_t)
            IMPLEMENT_GET_AND_SET     (PixelAddr, uint32_t)
            IMPLEMENT_GET_AND_SET     (Sequence, uint32_t)
            IMPLEMENT_GET_AND_SET     (Pad3, uint32_t)
        protected:
            IMPLEMENT_SET             (Version  , uint32_t)
            /*
            IMPLEMENT_FIRST_GET_AND_SET(FEBId    , uint32_t)
            IMPLEMENT_FIRST_GET_AND_SET(FEBStatus, uint32_t)
            IMPLEMENT_FIRST_GET_AND_SET(PixelAddr, uint32_t)
            IMPLEMENT_FIRST_GET_AND_SET(Sequence , uint32_t)
            IMPLEMENT_FIRST_GET_AND_SET(Version  , uint32_t)
            IMPLEMENT_FIRST_GET_AND_SET(Pad3     , uint32_t)
            IMPLEMENT_FIRST_GET_AND_SET_BOOL(DataPresent)
            IMPLEMENT_FIRST_GET_AND_SET_BOOL(LinkStatus)
            IMPLEMENT_FIRST_GET_AND_SET_BOOL(DebugMode)
            IMPLEMENT_FIRST_GET_AND_SET_BOOL(DAQEnabled)
            IMPLEMENT_FIRST_GET_AND_SET_BOOL(BufferEmpty)
            IMPLEMENT_FIRST_GET_AND_SET_BOOL(BufferFull)
            IMPLEMENT_FIRST_GET_AND_SET_BOOL(CommError)
            IMPLEMENT_FIRST_GET_AND_SET_BOOL(PacketError)
            IMPLEMENT_FIRST_GET_AND_SET_BOOL(OverflowError)
            IMPLEMENT_FIRST_GET_AND_SET_BOOL(ADCError)
            */
            
            INIT_FIRST(initWithData , , void, const NanoSliceHeader&);
            INIT_FIRST(setParameters, , bool, const NanoSliceHeader&);

    };// end of RawNanoSliceHeader
}} // END NAMESPCE 


#endif /* RAWNANOSLICEHEADERV0_H */
