#ifndef FEBSIMULATOR_H
#define FEBSIMULATOR_H

#include "DAQDataFormats/RawNanoSlice.h"	/* Use the nanoslice data format */

namespace daqdataformats {

    typedef uint32_t link_id_t;
    
    const float DEFAULT_HIT_PROBABILITY = 0.1;

    class FEBSimulator {


        public:

            FEBSimulator(version_t versionNanoSlice = nanosliceheader::RAWNANOSLICE_LATEST_VERSION);
            FEBSimulator(link_id_t linkID, version_t versionNanoSlice = nanosliceheader::RAWNANOSLICE_LATEST_VERSION);
            FEBSimulator(link_id_t linkID, float probability, version_t versionNanoSlice=nanosliceheader::RAWNANOSLICE_LATEST_VERSION);
            ~FEBSimulator();

            void  advanceSEQ();		/* Advance the Sequence Number */
            void  resetSEQ();		/* Reset the Sequence Number */

            void setLinkID        (link_id_t linkID);
            void setHitProbability(float     probability);

            //! Generator methods produce a NanoSlice
            void generateNanoSlice();
            void generateNanoSlice(uint32_t PixelID);
            void generateNanoSlice(uint32_t PixelID, bool ForceGeneration);
            void generateNanoSlice(uint32_t PixelID, uint32_t TimeStamp, uint32_t PulseHeight, bool ForceGeneration);
            void generateNanoSlice(uint32_t PixelID, uint32_t TimeStamp, const std::vector<uint32_t>& data);

            bool nonEmptyNanoSlice();
  
            RawNanoSlice* getLastNanoSlice();


        private:
            uint32_t      fSEQ;		     /* Frame Sequence Number 3 bits */
            link_id_t     fFEBLinkID;	     /* FEB Link Identifier 5 bits */
            float         fHitProbability;   /*Probability of Generating a hit */

            uint32_t      fVersionNanoSlice; /* FEB Link Identifier 5 bits */

            RawNanoSlice* fLastNanoSlice;    /* Last Generated Nanoslice */

    };// end of FEBSimulator

} /* END NAMESPACE daqdataformats */
#endif /* FEBSIMULATOR_H */
