#ifndef RAWCONFIGURATIONBLOCK_H
#define RAWCONFIGURATIONBLOCK_H
#include "DAQDataFormats/RawConfigurationTail.h"
#include "DAQDataFormats/RawConfigurationBlockV1.h"

// This include is here for the temporary function getDAQSystemID
// It needs to be gone after they are removed
#include "DAQDataFormats/RawConfigurationSystemID.h"

/// It is requred to
///   1. include DAQDataFormats/DataFormatFirst.inc
///   2. define CURRENT_CLASS
///   3.    define CURRENT_CLASS_VERSION for version of the class
///      OR define LAST_CLASS_VERSION    for the main structure of the class
///   4. (optional) define CURRENT_FIELDS_SUB_NAMESPACE
///   5. include DAQDataFormats/DataFormatLast.inc
/// before the declaration of the DataFormat version class in the header.
#include "DAQDataFormats/DataFormatFirst.inc"
#define  CURRENT_CLASS                RawConfigurationBlock
#define  LAST_CLASS_VERSION           1
#define  CURRENT_FIELDS_SUB_NAMESPACE configurationblock
#include "DAQDataFormats/DataFormatLast.inc"

namespace daqdataformats {

    class RawConfigurationBlock: public PREVIOUS_VERSION_CLASS_NAME{
        public:

            /// Constructors
            RawConfigurationBlock (const version_t version = rawconfhdr::RAWCONFIGURATIONBLOCKHEADER_LATEST_VERSION); //! The default constructor
            RawConfigurationBlock (const RawConfigurationBlock& copy_in);  //! Deep copy constructor
            RawConfigurationBlock (const void* dadablockheader_buffer, const version_t version = rawdaqdataformat::DAQDATAFORMAT_UNKNOWN_VERSION); //! Shallow copy constructor
            /// Destructor
            ~RawConfigurationBlock();

            IMPLEMENT_MAIN_STANDARD
            IMPLEMENT_MAIN_setBufferSource

            bool close(const bool produce_crc=false){RETURN_FUNCTION(close)(produce_crc);}
            
            IMPLEMENT_FUNCTION_RETURNING_FUNCTION_POINTER_0PAR (getHeader        ,       , RawConfigurationHeader*)
            IMPLEMENT_FUNCTION_RETURNING_FUNCTION_POINTER_0PAR (getTail          ,       , RawConfigurationTail*)
            IMPLEMENT_FUNCTION_RETURNING_FUNCTION_POINTER_0PAR (getTailConst     , const , RawConfigurationTail*)
            IMPLEMENT_FUNCTION_RETURNING_FUNCTION_POINTER_1PAR (setFloatingSystemIDToBufferSource, const, bool, const void*)
            IMPLEMENT_FUNCTION_RETURNING_FUNCTION_POINTER_1PAR (addSystemID                      ,      , bool, const void*)
            IMPLEMENT_SET_MAIN(FloatingSystemID, uint32_t)

            IMPLEMENT_GET_MAIN(MarkerHi1             , uint32_t)
            IMPLEMENT_GET_MAIN(MarkerHi2             , uint32_t)
            IMPLEMENT_GET_MAIN(MarkerLo1             , uint32_t)
            IMPLEMENT_GET_MAIN(MarkerLo2             , uint32_t)
            IMPLEMENT_GET_MAIN(ConfigurationVersion  , uint32_t)
            IMPLEMENT_GET_MAIN(ConfigurationBlockSize, uint32_t)
            IMPLEMENT_GET_MAIN(SystemNumber          , uint32_t)

            IMPLEMENT_GET_AND_SET_MAIN(DetectorID       , uint32_t)
            IMPLEMENT_GET_AND_SET_MAIN(ResourceManagerID, uint32_t)
            IMPLEMENT_GET_AND_SET_MAIN(RunNumber        , uint32_t)
            IMPLEMENT_GET_AND_SET_MAIN(RunType          , uint32_t)
            IMPLEMENT_GET_AND_SET_MAIN(Partition        , uint32_t)
            IMPLEMENT_GET_AND_SET_MAIN(Subrun           , uint32_t)
            IMPLEMENT_GET_AND_SET_MAIN(SystemVersionIDX , uint32_t)

            IMPLEMENT_FUNCTION_RETURNING_FUNCTION_POINTER_0PAR (getSystemID      ,       , RawConfigurationSystemID*)
            IMPLEMENT_FUNCTION_RETURNING_FUNCTION_POINTER_0PAR (getSystemIDConst , const , RawConfigurationSystemID*)

            RawConfigurationSystemID* getSystemID(const uint32_t isystem)            {RETURN_FUNCTION(getSystemIDWithInput)(isystem);}
            RawConfigurationSystemID* getSystemIDConst(const uint32_t isystem) const {RETURN_FUNCTION(getSystemIDConstWithInput)(isystem);}

    protected:
            IMPLEMENT_FUNCTION_RETURNING_FUNCTION_POINTER_1PAR (getSystemIDWithInput      ,       , RawConfigurationSystemID*, const uint32_t)
            IMPLEMENT_FUNCTION_RETURNING_FUNCTION_POINTER_1PAR (getSystemIDConstWithInput , const , RawConfigurationSystemID*, const uint32_t)

            IMPLEMENT_FIGUREOUT_VERSION_STATIC_WITH_CUSTOM_GET_VERSION(RawConfigurationHeader, getConfigurationHeaderVersion)
            bool setFunctionPointers();

    public:
            //******************************************************************
            /// These two are obsolete functions. PLEASE don't use them.
            /// Use
            /// They are here temporarily just to be compatible with old DAQDataFormats
            /// They will be removed!
            //     1. This actually was SystemID version
            uint32_t getDAQSystemID(const uint32_t id)     { return getSystemID(id)->getVersion();}
            //     2. Configuration Block version used to be ConfigurationHeaderVersion
            uint32_t getConfigurationHeaderVersion() const { return getConfigurationVersion();}
            //******************************************************************

    };
  
} // END NAMESPCE daqdataformats
#endif /* RAWCONFIGURATIONBLOCK_H */
