#ifndef RAWDATABLOCKV0_H
#define RAWDATABLOCKV0_H
#include "DAQDataFormats/RawDataBlockHeader.h"

/// It is requred to
///   1. include DAQDataFormats/DataFormatFirst.inc
///   2. define CURRENT_CLASS
///   3.    define CURRENT_CLASS_VERSION for version of the class
///      OR define LAST_CLASS_VERSION    for the main structure of the class
///   4. (optional) define CURRENT_FIELDS_SUB_NAMESPACE
///   5. include DAQDataFormats/DataFormatLast.inc
/// before the declaration of the DataFormat version class in the header.
#include "DAQDataFormats/DataFormatFirst.inc"
#define  CURRENT_CLASS                RawDataBlock
#define  CURRENT_CLASS_VERSION        0
#define  CURRENT_FIELDS_SUB_NAMESPACE datablock
#include "DAQDataFormats/DataFormatLast.inc"


namespace daqdataformats {
    class RawMicroBlock;
}

namespace daqdataformats {
namespace  VERSION_NAMESPACE {

    class RawDataBlock : public PREVIOUS_VERSION_CLASS_NAME {
        protected:
            /// Constructors
            RawDataBlock(const version_t, const uint32_t tlvl);
            RawDataBlock(const RawDataBlock&);
            /// Destructor
            ~RawDataBlock();
        public:

            void     init            ();
            using    RawDAQData::print;
            void     print           (std::ostream& os=std::cout) const;
            uint32_t sizeofdata      () const { return _Header->getDataSize();}
            bool     setBufferSource(const void* pointer);

            using          RawDAQData::readData;
            void*          readData(const void*);

            // Getters
            daqdataformats::RawDataBlockHeader* getHeader()             const { return _Header; }
            daqdataformats::RawMicroBlock*      getFloatingMicroBlock() const { return _FloatingMicroBlock; }


            /// Add the MicroBlock to _Buffer.
            /// If check_mc_flag = true, the method checks whether the microslice
            /// in the microblock is MC generatedand adds an appropriate flag to the DataBlock Header.
            bool addMicroBlock1(const void* microblock_buffer,
                                const bool  check_mc_flag
                               );
            /// Add the MicroBlock of a known size to _Buffer.
            /// If check_mc_flag = true, the method checks whether the microslice
            /// in the microblock is MC generatedand adds an appropriate flag to the DataBlock Header.
            bool addMicroBlock2(const void*    microblock_buffer,
                                const uint32_t microblock_size,
                                const bool     check_mc_flag
                               );
            /// Add the MicroBlock to _Buffer by passing the references of
            /// the MilliSlice and the MicroSlice. MilliSlice is the one that contains the MicroSlice.
            /// If check_mc_flag = true, the method checks whether the microslice
            /// in the microblock is MC generatedand adds an appropriate flag to the DataBlock Header.
            bool addMicroBlock3(const void*     millislice_buffer,
                                const void*     microslice_buffer,
                                const version_t microblock_version,
                                const bool      check_mc_flag
                               );
            /// Add the MicroBlock to _Buffer by passing the references of
            /// the MilliSlice and the MicroSlice of a known size.
            /// MilliSlice is the one that contains the MicroSlice.
            /// If check_mc_flag = true, the method checks whether the microslice
            /// in the microblock is MC generatedand adds an appropriate flag to the DataBlock Header.
            bool addMicroBlock4(const void*     millislice_buffer,
                                const void*     microslice_buffer,
                                const uint32_t  microslice_size,
                                const version_t microblock_version,
                                const bool      check_mc_flag
                               );

        protected:
            
            uint32_t       getPredeterminedSize() const {return 1;}

            readDataStatus readDataUnknownVersion(const int);
            INIT_FIRST(readDataFromFileDescriptor         ,      , readDataStatus, const int);

            INIT_FIRST(close                              ,      , bool          , const bool)
            INIT_FIRST(setFloatingMicroBlockToBufferSource, const, bool          , const void*)

	    INIT_FIRST(clear      , , bool, )
	    INIT_FIRST(softclear  , , bool, )
	    INIT_FIRST(getVersion, const, uint32_t, )
	    INIT_FIRST(getMarker, const, uint32_t, )
	    INIT_FIRST(getTrigNumLo, const, uint32_t, )
	    INIT_FIRST(getTrigNumHi, const, uint32_t, )
	    INIT_FIRST(getTrigNum, const, uint64_t, )
	    INIT_FIRST(getNumMicroBlocks, const, uint16_t, )
	    INIT_FIRST(getBuffId, const, uint16_t, )
	    INIT_FIRST(getDataSize, const, uint32_t, )
	    INIT_FIRST(getMonteCarloFlag, const, bool, )
	    INIT_FIRST(isCRCCalculationUsed, const, bool, )
	    INIT_FIRST(isConnectionInitialization, const, bool, )
	    INIT_FIRST(isCloseConnection, const, bool, )
	    INIT_FIRST(isMissingData, const, bool, )
            INIT_FIRST(setFloatingMicroBlock, const, bool, const uint32_t)
	    INIT_FIRST(getCRC, const, uint32_t, )
            INIT_FIRST(setTrigNumLo, , bool, const uint32_t)
            INIT_FIRST(setTrigNumHi, , bool, const uint32_t)
            INIT_FIRST(setTrigNum, , bool, const uint64_t)
            INIT_FIRST(setBuffId, , bool, const uint16_t)
            INIT_FIRST(setConnectionInitialization, , bool, const bool)
            INIT_FIRST(setMonteCarloFlag, , bool, const bool)
            INIT_FIRST(setCloseConnection, , bool, const bool)
            INIT_FIRST(setMissingData, , bool, const bool)

        protected:
            INIT_CUSTOM_DELEGATE(addMicroBlock1, bool, const void*, const bool);
            INIT_CUSTOM_DELEGATE(addMicroBlock2, bool, const void*, const uint32_t, const bool);
            INIT_CUSTOM_DELEGATE(addMicroBlock3, bool, const void*, const void*   , const version_t, const bool);
            INIT_CUSTOM_DELEGATE(addMicroBlock4, bool, const void*, const void*   , const uint32_t , const version_t, const bool);

            daqdataformats::RawDataBlockHeader*    _Header;                  //! DataBlock header
            mutable daqdataformats::RawMicroBlock* _FloatingMicroBlock;      //! Floating MicroBlock
            mutable int                            _FloatingMicroBlockNumber;//! Floating MicroBlock Number
            version_t                              _microBlockVersion;       ///< Version of the MicroBlock

            uint32_t               _tlvl; // debug "trace level mask"
            size_t                 _fd_read_bytes;
            
        public:
            void tprintf( unsigned lvl, const char *fmt, ... )
                        __attribute__((format(printf, 3, 4))); // g++ note: attributes are not

               			          //  allowed on a function-definition
    };// end of class definition
  
}} // END NAMESPCE daqdataformats
#endif /* RAWDATABLOCKV0_H */
