#ifndef RAWMICROBLOCKV0_H
#define RAWMICROBLOCKV0_H
#include "DAQDataFormats/RawMicroSlice.h"
#include "DAQDataFormats/RawMicroBlockHeader.h"

/// It is requred to
///   1. include DAQDataFormats/DataFormatFirst.inc
///   2. define CURRENT_CLASS
///   3.    define CURRENT_CLASS_VERSION for version of the class
///      OR define LAST_CLASS_VERSION    for the main structure of the class
///   4. (optional) define CURRENT_FIELDS_SUB_NAMESPACE
///   5. include DAQDataFormats/DataFormatLast.inc
/// before the declaration of the DataFormat version class in the header.
#include "DAQDataFormats/DataFormatFirst.inc"
#define  CURRENT_CLASS                RawMicroBlock
#define  CURRENT_CLASS_VERSION        0
#define  CURRENT_FIELDS_SUB_NAMESPACE rawmicroblock
#include "DAQDataFormats/DataFormatLast.inc"

namespace daqdataformats {
namespace  VERSION_NAMESPACE {
    /// MicroBlock header size + MicroSlice Header
    static const uint32_t MICROBLOCK_HEADER_STUFF = rawmicroblockhdr::RAWMICROBLOCKHEADER_SIZE + RAWMICROSLICEHEADER_SIZE;

  class RawMicroBlock : public PREVIOUS_VERSION_CLASS_NAME {

        public:
            // Constructors
            RawMicroBlock (const version_t); //! The default constructor
            RawMicroBlock (const RawMicroBlock&);
            /// Destructor
            ~RawMicroBlock();

            void     init            () {_Header->init();}
            using    PREVIOUS_VERSION_CLASS_NAME::print;
            void     print            (std::ostream& os=std::cout) const;
            uint32_t sizeofdata      () const;
            bool     setBufferSource (const void* pointer);
            
            daqdataformats::RawMicroBlockHeader* getHeader()     const {return _Header;}            //! MicroBlock Header reference
            daqdataformats::RawMicroSlice*       getMicroSlice() const {return _MicroSlice;}        //! The current floating MicroSlice reference

            bool setMicroBlockHeader(const void* millislice_buffer) {return _Header->setAllParameters(millislice_buffer);}
            bool setMicroSlice      (const void* microslice_buffer);

            using PREVIOUS_VERSION_CLASS_NAME::readData;	// unhide the reset of the readData methods
            void*                              readData(const void*);

        protected:
            uint32_t getPredefinedSize() const {return rawmicroblockhdr::RAWMICROBLOCKHEADER_SIZE;}



        protected:
            daqdataformats::RawMicroBlockHeader* _Header;          //! MicroBlock Header
            daqdataformats::RawMicroSlice*       _MicroSlice;      //! MicroSlice
  };


}} // END NAMESPCE daqdataformats
#endif /* RawMicroBlockV0V0_H */
