#ifndef RAWNANOSLICE_H
#define RAWNANOSLICE_H
#include "DAQDataFormats/RawNanoSliceV4.h"

/// It is requred to
///   1. include DAQDataFormats/DataFormatFirst.inc
///   2. define CURRENT_CLASS
///   3.    define CURRENT_CLASS_VERSION for version of the class
///      OR define LAST_CLASS_VERSION    for the main structure of the class
///   4. (optional) define CURRENT_FIELDS_SUB_NAMESPACE
///   5. include DAQDataFormats/DataFormatLast.inc
/// before the declaration of the DataFormat version class in the header.
#include "DAQDataFormats/DataFormatFirst.inc"
#define  CURRENT_CLASS                RawNanoSlice
#define  LAST_CLASS_VERSION           4
#define  CURRENT_FIELDS_SUB_NAMESPACE rawnano
#include "DAQDataFormats/DataFormatLast.inc"


  //! Class to hold the data from the FEBs in correct formats
  /*!
    The RawNanoSlice class is designed to mimic the format of the actual data
    block that the NOvA Front End6
    Boards (FEBS) produce when they detect a signal
    above the zero suppression threshold.

    The class supports general initialization with or with out data blocks as
    well as copyin style initialization for use with STL containers.

    Pack and unpack methods for the data rely on two types of structured data
    blocks that are provided in association with the class.

    The NanoSliceBlock structure breaks the raw data block out into its related
    high and low half words to give the headers, time stamps and pulseheight information

    The NanoSliceHeader structure further breaks out the header blocks into the
    individual data information parts.
  */

namespace daqdataformats {

  class RawNanoSlice: public PREVIOUS_VERSION_CLASS_NAME {
    
    public:
        // Constructors
        RawNanoSlice (const version_t version = nanosliceheader::RAWNANOSLICE_DEFAULT_VERSION); //! The default constructor
        RawNanoSlice (const NanoSlice&);               //! Constructor with Data initialization
        RawNanoSlice (const RawNanoSlice& copy_in);    //! Deep copy constructor
        RawNanoSlice (const void* nanoslice_buffer, const version_t version = rawdaqdataformat::DAQDATAFORMAT_UNKNOWN_VERSION ); //! Shallow copy constructor
        /// Destructor
        ~RawNanoSlice();

        IMPLEMENT_MAIN_STANDARD
        IMPLEMENT_MAIN_setBufferSource

        /// Get the NanoSlice header pointer
        daqdataformats::RawNanoSliceHeader* getHeader() const  {return _Header;}

        IMPLEMENT_GET_AND_SET_MAIN(PulseHeight, uint32_t);
        IMPLEMENT_GET_AND_SET_MAIN(TimeStamp  , uint32_t);
        IMPLEMENT_GET_MAIN        (Reserved   , uint32_t);
        IMPLEMENT_GET_AND_SET_MAIN(Value1, uint32_t);

        IMPLEMENT_VOID_RETURNING_FUNCTION_POINTER_1PAR    (initWithData , ,       const NanoSlice&)
        IMPLEMENT_FUNCTION_RETURNING_FUNCTION_POINTER_1PAR(setParameters, , bool, const NanoSlice& )
        IMPLEMENT_FUNCTION_RETURNING_FUNCTION_POINTER_0PAR(checkReservedForCafe, const, bool)


        IMPLEMENT_FUNCTION_RETURNING_FUNCTION_POINTER_1PAR(getValue, const, uint32_t, const uint32_t)
        IMPLEMENT_FUNCTION_RETURNING_FUNCTION_POINTER_2PAR(setValue,      , bool    , const uint32_t, const uint32_t)

        IMPLEMENT_FIGUREOUT_VERSION_STATIC(RawNanoSliceHeader)

    private:
        IMPLEMENT_SET_MAIN(Reserved, uint32_t);

        void initWithDataInternal();
        bool setFunctionPointers();

    private:
        /// Pointer to Nano parameters
        /// used in init when using Constructor with Data initialization only.
        /// After that this variable is irrelevant
        NanoSlice*  _nanoSliceParameters;

        
  };// end of RawNanoSlice
  
  
} // END NAMESPCE daqdataformats
#endif /* RAWNANOSLICE_H */
