#ifndef DAQRAWTRIGGERMASK_H
#define DAQRAWTRIGGERMASK_H
#include "DAQDataFormats/RawTriggerMaskV0.h"
#include "DAQDataFormats/TriggerDefines.h"
#include "DAQDataFormats/FunctionBind.h"

/// It is requred to
///   1. include DAQDataFormats/DataFormatFirst.inc
///   2. define CURRENT_CLASS
///   3.    define CURRENT_CLASS_VERSION for version of the class
///      OR define LAST_CLASS_VERSION    for the main structure of the class
///   4. (optional) define CURRENT_FIELDS_SUB_NAMESPACE
///   5. include DAQDataFormats/DataFormatLast.inc
/// before the declaration of the DataFormat version class in the header.
#include "DAQDataFormats/DataFormatFirst.inc"
#define  CURRENT_CLASS                RawTriggerMask
#define  LAST_CLASS_VERSION           0
#define  CURRENT_FIELDS_SUB_NAMESPACE triggermask
#include "DAQDataFormats/DataFormatLast.inc"


namespace daqdataformats {
  
    namespace triggermask {
        static const version_t RAWTRIGGERMASK_LATEST_VERSION = LAST_CLASS_VERSION;
    }

  //! Class to hold the data corresponding to the trigger mask
  /*!
    The RawTriggerMask contains the information on the type of trigger
    that is being issued.

    The class supports general initialization with an internal or external
    data buffer as well as copyin style initialization for use with
    STL containers.
  */
  class RawTriggerMask: public PREVIOUS_VERSION_CLASS_NAME {
    
        public:
      
            RawTriggerMask(const version_t version = triggermask::RAWTRIGGERMASK_LATEST_VERSION);		                            //!< Constructor
            RawTriggerMask(const RawTriggerMask& copy_in);	    //!< Copyin Constructor
            RawTriggerMask(const void* buffer, const version_t version = rawdaqdataformat::DAQDATAFORMAT_UNKNOWN_VERSION);  //!< Constructor with data intialization
            ~RawTriggerMask();                                    //!< Destructor

            IMPLEMENT_MAIN_STANDARD

            IMPLEMENT_GET_MAIN        (Version       , uint32_t)
            IMPLEMENT_GET_AND_SET_MAIN(MonteCarloFlag, uint32_t)
            IMPLEMENT_GET_AND_SET_MAIN(TrigMod       , uint32_t)
            IMPLEMENT_GET_AND_SET_MAIN(TrigMask_Low  , uint32_t)
            IMPLEMENT_GET_AND_SET_MAIN(TrigMask_High , uint32_t)
            IMPLEMENT_GET_AND_SET_MAIN(TrigMask_High2, uint32_t)
            IMPLEMENT_GET_AND_SET_MAIN(Prescale      , uint32_t)
            IMPLEMENT_GET_AND_SET_MAIN(TriggerMask   , uint32_t)


            IMPLEMENT_FIGUREOUT_VERSION_STATIC(RawTriggerMask)

       protected:
            IMPLEMENT_SET_MAIN(Version  , uint32_t)
            bool             setFunctionPointers();
            

  };
  
  
} // END NAMESPACE daqdataformats
#endif /* DAQRAWTRIGGERMASK_H */
