#include <vector>
#include <stdio.h>
#include <stdlib.h>
#include <time.h>
#include <DAQDataFormats/RawMicroSlice.h>

#include "DAQDataFormats/FEBSimulator.h"        // Front Endboard Simulator
#include "DAQDataFormats/TimeStampCounter.h"	// High Resolution Time Stamping

using namespace daqdataformats;

//! A short program to show how to simulate an FEB
int main(){
  
  int seed;
  FEBSimulator* FrontEndBoards[62]; // We make an array of 62 FEB Pointers

  ////////////////////////////////////////////////
  // Determine the speed of CPU on the machine we
  // are running on this is done to calibrate the real
  // clock counter for use when we want to time 
  // things like generation speed

  //  unsigned long long startTime;
  //  unsigned long long stopTime;
  //  unsigned long long* time_array;
  //  int TimeConversion_usec=0;
  // Calculate the conversion from system clock to microseconds */
  //int   TimeConversion_nsec = (int)(CPUSpeed()/100);
  //float conv_nsec = 1E9* 1.0/(CPUSpeed() *1000000);
  //printf("System Speed: %2.8f\tConversion To NanoSeconds: %2.12f\n", CPUSpeed(), conv_nsec);
  // exit(1);

  // See a random number generator off of the system time
  srand(time((time_t*)&seed));
  

  // Now we instantiate the front end boards  
  float hit_probability=0.1;	// 10% probability of a link generating a hit on a given sample
  for(int link_number=0; link_number<62; ++link_number){
    // The constructor can take an identifier (link number)
    FrontEndBoards[link_number] = new FEBSimulator(link_number,hit_probability);
  } // endfor link_number
  // We now have an array of FEBs



  for(int link_number=0; link_number<62; ++link_number){
    for(int pixel_number=0; pixel_number<62; ++pixel_number){
      //      cycle_t sys_timing1.full_time = tsc();
      //      int myTimeStamp = sys_timing1.time[0];
      //      int myPulseHeight = rand() & 0x00000FFF; // Pulse Heights are 12bits

      // There are multiple ways to generate hits.  The simplest is to
      // call the generateNanoSlice() method with no arguement.
      // this method will decide to generate (or not generate) a hit
      // on a random pixel with the current hit probability

      //FrontEndBoards[link_number]->generateNanoSlice();
      // Or we can generate a hit on a given pixel with the hit probability
      FrontEndBoards[link_number]->generateNanoSlice(pixel_number, true);
      // Or we can force a hit to be generated on a given pixel
      // FrontEndBoards[link_number]->generateNanoSlice(pixel_number,true);

      // Or we can force a hit to be generated on a given pixel with definite values
      // FrontEndBoards[link_number]->generateNanoSlice(pixel_number,myTimeStamp,myPulseHeight,true);


      // But if we are using a probability to generate things
      // then we need to check if the last generation produced anything
      // (i.e. a non-empty NanoSlice
      if(FrontEndBoards[link_number]->nonEmptyNanoSlice()){
	// If we did generate something interesting then we print it out
	//FrontEndBoards[link_number]->getLastNanoSlice()->print();
      } // endif
    } // endfor pixel_number
  } // endfor link_number



  RawMicroSlice micro;

  for(int imicro = 0; imicro < 100; ++imicro){
  // Now let's generate some hits
  for(int link_number=0; link_number<62; ++link_number){
    for(int pixel_number=0; pixel_number<62; ++pixel_number){

     micro.addNanoSlice(FrontEndBoards[link_number]->getLastNanoSlice()->getBuffer());

    } // endfor pixel_number
  } // endfor link_number
  }

#ifdef USE_CLOCK_TICK_TIME_EXAMPLE
  // Example of how to clock tick time a given call
  std::vector<unsigned long long> genTimes;
  cycle_t sys_timing1;
  cycle_t sys_timing2;
  sys_timing1.full_time=0;
  sys_timing2.full_time=0;

  for(int j=0; j<10; ++j){
    // sys_timing1.full_time = tsc();
    for(int i=0; i<62; ++i){
      //    FrontEndBoards[i]->GenerateNanoSlice();
      sys_timing1.full_time = tsc();
      FrontEndBoards[i]->generateNanoSlice(j);
      sys_timing2.full_time = tsc();
      if(FrontEndBoards[i]->nonEmptyNanoSlice()){
	FrontEndBoards[i]->printNanoSlice();
      } // endif
      // sys_timing2.full_time = tsc();
      genTimes.push_back((sys_timing2.full_time - sys_timing1.full_time));
    } // endfor i
  } // endfor j

    printf("Time for Generation:");
    int i=0;
    for(std::vector<unsigned long long>::iterator p = genTimes.begin();
      p != genTimes.end(); ++p){
      if(!(i%6)){printf("\n");} i++;
      // printf("%lld\t",(*p * 1.0) / (conv_nsec));
      printf("%2.2f\t",*p * (conv_nsec));
      //printf("%lld\t",*p);
  }
#endif

} // END MAIN
