#include "DAQDataFormats/RawNanoSlice.h"
#include "DAQDataFormats/DataFormatException.h"

/// It is requred to
///   1. include DAQDataFormats/DataFormatFirst.inc
///   2. define CURRENT_CLASS
///   3.    define CURRENT_CLASS_VERSION for version of the class
///      OR define LAST_CLASS_VERSION    for the main structure of the class
///   4. (optional) define CURRENT_FIELDS_SUB_NAMESPACE
///   5. include DAQDataFormats/DataFormatLast.inc
/// before the declaration of the DataFormat version class in the header.
#include "DAQDataFormats/DataFormatFirst.inc"
#define  CURRENT_CLASS                RawNanoSlice
#define  LAST_CLASS_VERSION           4
#define  CURRENT_FIELDS_SUB_NAMESPACE rawnano
#include "DAQDataFormats/DataFormatLast.inc"

namespace daqdataformats{
 
//------------------------Constructors------------------------------------------
/// After this constructor the buffer pointer is set to the internal source
RawNanoSlice::RawNanoSlice(const version_t version):
             PREVIOUS_VERSION_CLASS_NAME(version)
            ,_nanoSliceParameters(NULL)
{
    INITIALIZE_DEBUGGING_VARIABLES;
    /// Perform general routines
    defaultConstructorFixedSize();
}// end of :RawNanoSlice(version_t version)

//------------------------------------------------------------------------------
/// After this constructor the buffer pointer is set to the internal source
RawNanoSlice::RawNanoSlice (const NanoSlice& nanoslice_parameters): 
            PREVIOUS_VERSION_CLASS_NAME(nanoslice_parameters)
           ,_nanoSliceParameters((NanoSlice*)&nanoslice_parameters)
{
    INITIALIZE_DEBUGGING_VARIABLES;
   /// Now, apply the default constructor, passing function pointer to initWithDataInternal
    defaultConstructorFixedSize((const init_t) &RawNanoSlice::initWithDataInternal);
}// end of RawNanoSlice (NanoSlice nanoslice_parameters)


//------------------------------------------------------------------------------
/// After this constructor the buffer pointer is set to the external source
RawNanoSlice::RawNanoSlice(const RawNanoSlice& copy_in): 
             PREVIOUS_VERSION_CLASS_NAME(copy_in)
            ,_nanoSliceParameters(NULL)
{
    INITIALIZE_DEBUGGING_VARIABLES;
    lastOperationsInDeepCopy(copy_in);
    setFunctionPointers();

}// end of RawNanoSlice(const RawNanoSlice& copy_in)

//------------------------------------------------------------------------------
// check how to get the version number
RawNanoSlice::RawNanoSlice(const void* nanoslice_buffer, const version_t version): 
             PREVIOUS_VERSION_CLASS_NAME(version)
            ,_nanoSliceParameters(NULL)
{
    INITIALIZE_DEBUGGING_VARIABLES;
    copy(nanoslice_buffer);
}

//------------------------Destructor--------------------------------------------
RawNanoSlice::~RawNanoSlice() { }

//------------------------------------------------------------------------------
void RawNanoSlice::initWithDataInternal(){  EXECUTE_FUNCTION(initWithData)(*_nanoSliceParameters); }

/// All binding routines
#define STANDARD_ROUTINES                                      \
            BIND_GET_AND_SET(PulseHeight , version0, uint32_t);\
            BIND_GET_AND_SET(TimeStamp   , version0, uint32_t);\
            BIND_GET_AND_SET(Reserved    , version0, uint32_t);\
            BIND_GET_AND_SET(Value1        , version1, uint32_t);\
            BIND_SET        (Parameters  , version0, NanoSlice&);\
            BIND_CUSTOM_FUNCTION(initWithData        , version0,      , void, const NanoSlice&); \
            BIND_CUSTOM_FUNCTION(checkReservedForCafe, version0, const, bool, ); \
            BIND_CUSTOM_FUNCTION(getValue              , version2, const, uint32_t, const uint32_t); \
            BIND_CUSTOM_FUNCTION(setValue              , version2,      , bool    , const uint32_t, const uint32_t); \
            BIND_STADARD_FUNCTION(sizeofdata);                  \
            BIND_STADARD_FUNCTIONS;
            

#include "DAQDataFormats/setFunctionPointers_template.inc"


}// end of namespace


