#include "DAQDataFormats/RawNanoSliceHeader.h"
#include "DAQDataFormats/DataFormatException.h"

/// It is requred to
///   1. include DAQDataFormats/DataFormatFirst.inc
///   2. define CURRENT_CLASS
///   3.    define CURRENT_CLASS_VERSION for version of the class
///      OR define LAST_CLASS_VERSION    for the main structure of the class
///   4. (optional) define CURRENT_FIELDS_SUB_NAMESPACE
///   5. include DAQDataFormats/DataFormatLast.inc
/// before the declaration of the DataFormat version class in the header.
#include "DAQDataFormats/DataFormatFirst.inc"
#define  CURRENT_CLASS                RawNanoSliceHeader
#define  LAST_CLASS_VERSION           4
#define  CURRENT_FIELDS_SUB_NAMESPACE nanosliceheader
#include "DAQDataFormats/DataFormatLast.inc"

namespace daqdataformats {

//------------------------Constructors------------------------------------------
/// After this constructor the buffer pointer is set to the internal source
RawNanoSliceHeader::RawNanoSliceHeader(const version_t version):
     PREVIOUS_VERSION_CLASS_NAME(version)
    ,_nanosliceHeader(NULL)
{
    INITIALIZE_DEBUGGING_VARIABLES;
    /// Apply the default constructor
    defaultConstructorFixedSize();
}

//------------------------------------------------------------------------------
/// After this constructor the buffer pointer is set to the internal source
RawNanoSliceHeader::RawNanoSliceHeader(const NanoSliceHeader& header): 
     PREVIOUS_VERSION_CLASS_NAME(header.Version)
    ,_nanosliceHeader((NanoSliceHeader*)&(header))
{
    INITIALIZE_DEBUGGING_VARIABLES;
    /// Now, apply the default constructor, passing function pointer to initWithData
    defaultConstructorFixedSize((const init_t)&RawNanoSliceHeader::initWithDataInternal);
}

//------------------------------------------------------------------------------
void RawNanoSliceHeader::initWithDataInternal(){ initWithData(*_nanosliceHeader);}

//------------------------------------------------------------------------------
RawNanoSliceHeader::RawNanoSliceHeader(const RawNanoSliceHeader& copy_in): 
    PREVIOUS_VERSION_CLASS_NAME(copy_in)
   ,_nanosliceHeader(NULL)
{
    INITIALIZE_DEBUGGING_VARIABLES;
    lastOperationsInDeepCopy(copy_in);
}

//------------------------------------------------------------------------------
// Constructing the nanoslice header form a pointer to an existing nanoslice header
RawNanoSliceHeader::RawNanoSliceHeader(const void* nanosliceheader_buffer, const version_t version):
    PREVIOUS_VERSION_CLASS_NAME(version)
    ,_nanosliceHeader(NULL)
{
    /// Use default Shallow copy constructor
    copy(nanosliceheader_buffer);
}

//------------------------Destructor--------------------------------------------
RawNanoSliceHeader::~RawNanoSliceHeader() { }

/// All binding routines
#define STANDARD_ROUTINES                                       \
            /*                                                  \
            BIND_GET_AND_SET(DataPresent  , version0, bool);    \
            BIND_GET_AND_SET(LinkStatus   , version0, bool);    \
            BIND_GET_AND_SET(DebugMode    , version0, bool);    \
            BIND_GET_AND_SET(FEBId        , version0, uint32_t);\
            BIND_GET_AND_SET(FEBStatus    , version0, uint32_t);\
            BIND_GET_AND_SET(PixelAddr    , version0, uint32_t);\
            BIND_GET_AND_SET(Sequence     , version0, uint32_t);\
            BIND_GET_AND_SET(Version      , version0, uint32_t);\
            BIND_GET_AND_SET(Pad3         , version0, uint32_t);\
            BIND_GET_AND_SET(DAQEnabled   , version0, bool);    \
            BIND_GET_AND_SET(BufferEmpty  , version0, bool);    \
            BIND_GET_AND_SET(BufferFull   , version0, bool);    \
            BIND_GET_AND_SET(CommError    , version0, bool);    \
            BIND_GET_AND_SET(PacketError  , version0, bool);    \
            BIND_GET_AND_SET(OverflowError, version0, bool);    \
            BIND_GET_AND_SET(ADCError     , version0, bool);    \
            */                                                  \
            BIND_CUSTOM_FUNCTION(initWithData, version0, , void, const NanoSliceHeader&); \
            BIND_SET            (Parameters   , version0, NanoSliceHeader&);\
            BIND_STADARD_FUNCTION(sizeofdata); \
            BIND_STADARD_FUNCTIONS;
            
#include "DAQDataFormats/setFunctionPointers_template.inc"

}// end of namespace
