                                                                     
                                                                     
                                                                     
                                             
/*************************************************
 * Very Simple Checksum example                  *
 *                                               *
 * Performs a CRC32 on a buffer, appends the     *
 * the result back to the buffer, then checks    *
 * the transmission for error                    *
 *                                               *
 * anorman@fnal.gov                              *
 *************************************************/

#include <boost/crc.hpp>
#include <stdio.h>


int main(){

  // We just use a stock polynomial (CRC32)
  boost::crc_32_type result;

  // Define a data buffer, here I'm just making a simple one
  unsigned long long buffer[8] = {
    0xFF00FF00AABBCCDDll,
    0x0010101010101010ll,
    0x4245678987654567ll,
    0xabcdef1234567890ll,
    0x0001234101113121ll,
    0x9876567898756789ll,
    0x0913231231231234ll,
    0x0ll
  };


  



  unsigned int* buff32ptr;
  //char* buff8ptr;
  //char* ptr;
  // char* buff;


  // Perform a Check Sum calculation on the buffer
  // the result is a 32bit checksum word
  result.process_bytes((char*)buffer, 56);
  printf("Checksum: 0x%0x\n",result.checksum());

  // Now we append the checksum to the data block
  // for transmission and then on the other side
  // we want to check to see if we have any errors

  buff32ptr = (unsigned int*)&buffer;  // Grab a 32bit boundary pointer to the buffer
 
  // We append the compliment of the checksum
  buff32ptr[14] = ~(result.checksum());	 // Append the ~checksum word to the end of the buffer

  // Print out the resulting data matrix with the appended checksum word
  printf("Data Matrix\n");
  for(int i=0; i < 14; ++i){
    printf("\t0x%08x",buff32ptr[i]);
    if(!((i+1)%4)){printf("\n");}
  }
  printf("\n\n");

  //
  // Transmit/Read/Write the full data block
  //
  // If we want an error to occur uncomment the following line:
  // buff32ptr[3]=0x00113411;

  // Reset the checksum calculator
  result.reset();

  // Take the checksum of the full block
  result.process_bytes((char*)buffer,60);

  // The compliment of the final checksum on the transmitted block, should be 0x0
  if(~result.checksum() == 0){
    printf("*** Block Check Success *** Final Checksum: 0x%0x\n",~result.checksum());
    return 0;
  }else{
    printf("!!! Block Check Failed  !!! Error Detected in final checksum: 0x%0x\n",~result.checksum());
    return 1;
  }

  return -1;
}
