#include <vector>
#include <stdio.h>
#include <stdlib.h>
#include <iostream>
#include <fcntl.h>
#include <sys/stat.h>
#include "DAQDataFormats/RawMicroSlice.h"
#include "DAQDataFormats/RawEvent.h"



#include <fcntl.h>                        // O_RDONLY
#include <getopt.h>		// getopt
#include "boost/regex.hpp"
#include <sys/time.h>		// gettimeofday, struct timeval
#include <poll.h>		// poll
#include <sys/stat.h>		// fstat, struct stat
#include <sys/socket.h>		// socket
#include <netinet/in.h>		// struct sockaddr_in
#include <netdb.h>		// gethostbyname, struct hostent





#define DAQDATAFORMATSDUMP 1

#define MAXIMUM_BUFFER_SIZE 1000000
//using namespace mcdatadaq;

using namespace daqdataformats;

#define USAGE "\
  Usage: %s [options]\n\
Example: %s --f=1 --i=microslices.raw\n\
Options:\n\
 --f=<format_id>         DAQDataFormats: 1=MicroSlices 2=Events\n\
 --i <filename>          Input file name\n\
 ", basename(argv[0]), basename(argv[0])

#define RAWTRIGGERBLOCK_SIZE 16

struct Options {
    int         format;
    char*       filename;
};


char filaneme_default[10] = "uuu";
Options gOpts = {format:0, filename:filaneme_default};

int
get_int_or_die( char *argv[], char const *optarg, char const * msg )
{   if (!isdigit(optarg[0]))
    {   printf( "%s\n", msg );
	printf( USAGE );  exit( 0 );
    }
    return strtoul(optarg,NULL,0);
}
int
parse_args( int argc, char *argv[] )
{   int	c;
    boost::cmatch mm;

    

    for(int inum = 0; inum< 2; inum++){
        int                  option_index=0;
	static struct option long_options[]=
	    {   //name,          has_arg, flag,val
		{"help",  	  0,       0,  'h' },
		{"f",   	  1,       0,   -1 },
		{"i",          1,       0,   -2 },
		{0, 0, 0, 0}
	    };
	c = getopt_long( argc, argv, "?hvp:", long_options, &option_index );
	switch (c)
	{
	case -1:
            gOpts.format = get_int_or_die(argv,optarg,"invalid id" );
	    break;
	case -2:
            gOpts.filename = optarg;
            break;
	case '?': case 'h':
            {
            printf( USAGE );  exit( 0 );
            break;
            }
	default:
	    printf("?? getopt returned character code 0%o ??\n", c);
	}// end of switch

    }

    return 1;
}   // parse_args

uint64_t
get_us_timeofday()
{   timeval tv;
    gettimeofday( &tv, NULL );
    uint64_t xx=tv.tv_sec;
    xx *= 1000000;
    xx += tv.tv_usec;
    return xx;
}   // get_us_timeofday

void
do_pause(char const *msg)
{   printf("%s - press return to continue...",msg); fflush(stdout);
    char linebuf[1024];
    fgets(linebuf,sizeof(linebuf),stdin);
    printf("\n");
}   // do_pause




void intro();


//! A short program to test RawMicroSlice
int main(int argc, char *argv[]){
    intro();

    
    if(argc <  3){
        std::cout<<"Should be 2 arguments\n";
	printf( USAGE );  exit( 0 );
    }



    parse_args( argc, argv );   // optind defined in <getopt.h>


    if(gOpts.format != 1 && gOpts.format !=2){
        std::cout<<"Wrong DAQDataFormat\n";
	printf( USAGE );  exit( 0 );
    }


    unsigned int  fileformat_version_number[1];
    unsigned int* pfileformat_version_number = (unsigned int*)fileformat_version_number;

    struct stat file_status;
    stat(gOpts.filename, &file_status);

    int file_handler = open(gOpts.filename, O_RDONLY);
    read(file_handler, pfileformat_version_number,4);

    if(gOpts.format == 2){

      for(int ievent=0;ievent<1;++ievent){
        RawEvent event;
        event.readData(pfileformat_version_number);
        uint32_t* currentpointer = (uint32_t*) event.getBuffer();;

        if(event.getHeaderMarkerLeft() != raweventhdr::Marker_value[0])
            read(file_handler, currentpointer, 4);
        currentpointer++;
        // read the size
        read(file_handler, currentpointer, 4);
        currentpointer++;

        // read the rest of the event
        read(file_handler, currentpointer, 4 * (event.sizeofdata() - 2));


        event.print();

      }
        return 1;
    }



    std::cout<<"MockDataDAQ Version = "<<fileformat_version_number[0]<<std::endl;



    int file_size = file_status.st_size;
    int current_byte_count = 4; // version is already read

    // read MicroSlices
    {
        RawMicroSlice microslice;
        uint32_t* currentpointer;

        // Read MicroSlices
        // loop over MicroSlices
        for(;current_byte_count < file_size;){

            // Set the buffer to the beginning of the MicroSlice
            currentpointer = (uint32_t*)microslice.getBuffer();

            // read the first word
            read(file_handler, currentpointer, 4);
            currentpointer++;
            current_byte_count += 4;


            //Continue reading
            read(file_handler, currentpointer, microslice.getByteCount());
            current_byte_count += microslice.getByteCount();

            microslice.print();
        }// end of loop over microslices
    }// end of reading Microslices





    
    close(file_handler);

    return 0;
} // END MAIN


//---------------------------intro----------------------------------------------
// Print an introductory message
void intro(){
    int version = DAQDATAFORMATSDUMP;
    std::cout<<"\n****************************************\n";
    std::cout<<"**  DAQDataFormats Dump version "<<version;
    if(version<10 )std::cout<<" ";if(version<100)std::cout<<" ";
    std::cout<<"   **\n****************************************\n\n";
}
