/* 
 * File:   ConfigurationBlockUnitTest.cpp
 * Author: denis
 * 
 * Created on September 24, 2012, 10:18 AM
 */
//#include "DAQDataFormats/../unittest/RawDAQDataUnitTest.h"
//#include "DAQDataFormats/../unittest/ConfigurationBlockConstructor.h"
#include "RawDAQDataUnitTest.h"
#include "ConfigurationBlockConstructor.h"

namespace daqdataformats {

class ConfigurationBlockUnitTest : public RawDAQDataUnitTest {
    CPPUNIT_TEST_SUITE(ConfigurationBlockUnitTest);
    CPPUNIT_TEST(basicFieldGetAndSetChecks);
    CPPUNIT_TEST(testReadDataIntoSameVersion);
    CPPUNIT_TEST(testReadDataIntoUnknownVersion);
    CPPUNIT_TEST(checkDeepCopyConstructor);
    CPPUNIT_TEST(checkShallowCopyConstructor);
    CPPUNIT_TEST_SUITE_END();
public:
    ConfigurationBlockUnitTest();
    ~ConfigurationBlockUnitTest();

    DEFINE_checkDeepCopyConstructor(RawConfigurationBlock)
    DEFINE_checkShallowCopyConstructor(RawConfigurationBlock)
    DEFINE_testReadDataIntoSameVersion
    DEFINE_testReadDataIntoUnknownVersion

private:
    void basicFieldGetAndSetChecksForGivenVersion(const version_t);

private:
    configurationblock::ConfigurationBlockConstructor* _mainDFConstructor;
};

//------------------------------------------------------------------------------
ConfigurationBlockUnitTest::ConfigurationBlockUnitTest():
     RawDAQDataUnitTest(new configurationblock::ConfigurationBlockConstructor())
    ,_mainDFConstructor((configurationblock::ConfigurationBlockConstructor*)_dataFormatConstructor)
{
}

//------------------------------------------------------------------------------
ConfigurationBlockUnitTest::~ConfigurationBlockUnitTest() { }

//------------------------------------------------------------------------------
void ConfigurationBlockUnitTest::basicFieldGetAndSetChecksForGivenVersion(const version_t version) {

    RawConfigurationBlock* df = (RawConfigurationBlock*)_mainDFConstructor->getEmptyDataFormat(version);

    CPPUNIT_ASSERT(version == (version_t) df->getConfigurationVersion());

    ASSERT_PRINT_IS_NOT_UNKNOWN;


    df = (RawConfigurationBlock*)_mainDFConstructor->getFilledDataFormat(version);


    RawConfigurationSystemID* sysid = (RawConfigurationSystemID*)_mainDFConstructor->_configurationSystemIDConstructor.getFilledDataFormat(0);
    CPPUNIT_ASSERT(compareDataFormats(df, sysid) == FIRST_CONTAINS_SECOND);
    sysid->setVersionName("That's funny");
    CPPUNIT_ASSERT(compareDataFormats(df, sysid) == FIRST_CONTAINS_SECOND);
    sysid->setVersionName("Ho Ho Ho");
    CPPUNIT_ASSERT(compareDataFormats(df, sysid) == FIRST_CONTAINS_SECOND);







    delete df;
}

//CPPUNIT_TEST_SUITE_REGISTRATION(ConfigurationBlockUnitTest);
CPPUNIT_TEST_SUITE_NAMED_REGISTRATION(ConfigurationBlockUnitTest, "DAQDataFormats");
CPPUNIT_REGISTRY_ADD_TO_DEFAULT("DAQDataFormats");

}// end of class
