/* 
 * File:   ConfigurationHeaderUnitTest.cpp
 * Author: denis
 * 
 * Created on September 24, 2012, 10:18 AM
 */
//#include "DAQDataFormats/../unittest/RawDAQDataUnitTest.h"
//#include "DAQDataFormats/../unittest/ConfigurationHeaderConstructor.h"
#include "RawDAQDataUnitTest.h"
#include "ConfigurationHeaderConstructor.h"

namespace daqdataformats {

class ConfigurationHeaderUnitTest : public RawDAQDataUnitTest {
    CPPUNIT_TEST_SUITE(ConfigurationHeaderUnitTest);
    CPPUNIT_TEST(basicFieldGetAndSetChecks);
    CPPUNIT_TEST(testReadDataIntoSameVersion);
    CPPUNIT_TEST(testReadDataIntoUnknownVersion);
    CPPUNIT_TEST(checkDeepCopyConstructor);
    CPPUNIT_TEST(checkShallowCopyConstructor);
    CPPUNIT_TEST_SUITE_END();
public:
    ConfigurationHeaderUnitTest();
    ~ConfigurationHeaderUnitTest();

    DEFINE_checkDeepCopyConstructor(RawConfigurationHeader)
    DEFINE_checkShallowCopyConstructor(RawConfigurationHeader)
    DEFINE_testReadDataIntoSameVersion
    DEFINE_testReadDataIntoUnknownVersion

private:
    void basicFieldGetAndSetChecksForGivenVersion(const version_t);

private:
    rawconfhdr::ConfigurationHeaderConstructor* _mainDFConstructor;
};

//------------------------------------------------------------------------------
ConfigurationHeaderUnitTest::ConfigurationHeaderUnitTest():
     RawDAQDataUnitTest(new rawconfhdr::ConfigurationHeaderConstructor())
    ,_mainDFConstructor((rawconfhdr::ConfigurationHeaderConstructor*)_dataFormatConstructor)
{
}

//------------------------------------------------------------------------------
ConfigurationHeaderUnitTest::~ConfigurationHeaderUnitTest() { }

//------------------------------------------------------------------------------
void ConfigurationHeaderUnitTest::basicFieldGetAndSetChecksForGivenVersion(const version_t version) {

    RawConfigurationHeader* df = (RawConfigurationHeader*)_mainDFConstructor->getEmptyDataFormat(version);

    ASSERT_CHECK_FIELD     (DetectorID);
    ASSERT_CHECK_FIELD     (ConfigurationBlockSize);
    ASSERT_CHECK_FIELD     (ResourceManagerID);
    ASSERT_CHECK_FIELD     (RunNumber);
    ASSERT_CHECK_FIELD     (RunType);
    ASSERT_CHECK_FIELD     (ConfigurationIDX);
    ASSERT_CHECK_FIELD     (Partition);
    ASSERT_CHECK_FIELD     (Subrun);
    ASSERT_CHECK_FIELD     (SystemNumber);
    ASSERT_CHECK_FIELD     (SystemVersionIDX);

    CPPUNIT_ASSERT(version == (version_t) df->getConfigurationHeaderVersion());

    ASSERT_PRINT_IS_NOT_UNKNOWN;
    
    delete df;
}

//CPPUNIT_TEST_SUITE_REGISTRATION(ConfigurationHeaderUnitTest);
CPPUNIT_TEST_SUITE_NAMED_REGISTRATION(ConfigurationHeaderUnitTest, "DAQDataFormats");
CPPUNIT_REGISTRY_ADD_TO_DEFAULT("DAQDataFormats");

}// end of class
