/* 
 * File:   ConfigurationSystemIDConstructor.cpp
 * Author: denis
 * 
 * Created on September 20, 2012, 2:18 PM
 */
#include "DAQDataFormats/../unittest/ConfigurationSystemIDConstructor.h"

/// It is requred to
///   1. include DAQDataFormats/DataFormatFirst.inc
///   2. define CURRENT_CLASS
///   3.    define CURRENT_CLASS_VERSION for version of the class
///      OR define LAST_CLASS_VERSION    for the main structure of the class
///   4. (optional) define CURRENT_FIELDS_SUB_NAMESPACE
///   5. include DAQDataFormats/DataFormatLast.inc
/// before the declaration of the DataFormat version class in the header.
#include "DAQDataFormats/DataFormatFirst.inc"
#define  CURRENT_CLASS                RawConfigurationSystemID
#define  LAST_CLASS_VERSION           0
#define  CURRENT_FIELDS_SUB_NAMESPACE rawconfsysid
#include "DAQDataFormats/DataFormatLast.inc"

namespace daqdataformats {
namespace CURRENT_FIELDS_SUB_NAMESPACE {

//------------------------------------------------------------------------------
ConfigurationSystemIDConstructor::ConfigurationSystemIDConstructor():
    daqdataformats::RawDAQDataConstructor(daqdataformats::CURRENT_FIELDS_SUB_NAMESPACE::RAWCONFIGURATIONSYSTEMID_LATEST_VERSION)
{
}

//------------------------------------------------------------------------------
ConfigurationSystemIDConstructor::~ConfigurationSystemIDConstructor() { }

//------------------------------------------------------------------------------
daqdataformats::RawDAQData* ConfigurationSystemIDConstructor::getFilledDataFormat(const daqdataformats::version_t ver) const {
    RawConfigurationSystemID* out = (RawConfigurationSystemID*)getEmptyDataFormat(ver);
    fillDataFormat(out);
    return out;
}

//------------------------------------------------------------------------------
daqdataformats::RawDAQData* ConfigurationSystemIDConstructor::getEmptyDataFormat(const daqdataformats::version_t ver) const{
    return new daqdataformats::RawConfigurationSystemID(ver);
}

//------------------------------------------------------------------------------
bool ConfigurationSystemIDConstructor::fillDataFormat (daqdataformats::RawDAQData* dataformat) const {
    daqdataformats::RawConfigurationSystemID* cname = (daqdataformats::RawConfigurationSystemID*)dataformat;
    setFieldToTestValue_DAQSystemID(cname);
    setFieldToTestValue_DAQSubSystemID(cname);
    setFieldToTestValue_VersionName(cname);

    return true;
}


//------------------------------------------------------------------------------
bool ConfigurationSystemIDConstructor::checkField_VersionName(daqdataformats::CURRENT_CLASS* dataformat) const {

            if(!setFieldToTestValue_VersionName(dataformat)) {
                    LOG_ERROR("UNITTEST") << std::endl<< __PRETTY_FUNCTION__
                                          <<"\nsetFieldToTestValue_"<<"Name"<<" failed.\n"
                                          <<"Test value = "<<TEST_VALUE_VersionName<<"\n";
                    if(getVersion(dataformat) > rawdaqdataformat::DAQDATAFORMAT_UNKNOWN_VERSION)
                        LOG_ERROR("UNITTEST")<<"version =  "<<getVersion(dataformat)<<"\n";
                    return false;
            }

            if(TEST_VALUE_VersionName.compare(getFieldValue_VersionName(dataformat)) != 0 ){
                    LOG_ERROR("UNITTEST") << std::endl<< __PRETTY_FUNCTION__
                                          <<"\ngetFieldToTestValue_"<<"Name"<<" failed.\n"
                                          <<"Test value = "<<TEST_VALUE_VersionName<<"!"
                                          <<" but get is "
                                          << getFieldValue_VersionName(dataformat)<<"!\n";
                    if(getVersion(dataformat) > rawdaqdataformat::DAQDATAFORMAT_UNKNOWN_VERSION)
                        LOG_ERROR("UNITTEST")<<"version =  "<<getVersion(dataformat)<<"\n";
                    return false;
            }

            std::string suceed_string = "Checking BitField in ";
            suceed_string.append(__PRETTY_FUNCTION__);
            suceed_string.append(" field ");
            suceed_string.append("Name");
            suceed_string.append(" suceeded for version =  ");
            suceed_string.append(boost::lexical_cast<std::string>(getVersion(dataformat)));
            suceed_string.append(" (set to ");
            suceed_string.append(boost::lexical_cast<std::string>(TEST_VALUE_VersionName));
            suceed_string.append(" received ");
            suceed_string.append(boost::lexical_cast<std::string>(getFieldValue_VersionName(dataformat)));
            suceed_string.append(").");

            if(!setFieldToZeroValue_VersionName(dataformat)){
                    LOG_ERROR("UNITTEST")<<"\nsetFieldToZeroValue_"<<"Name"<<" failed.\n"
                             <<"version =  "<<getVersion(dataformat)<<"\n";
                    return false;
            }
            if(getFieldValue_VersionName(dataformat).size() != 0) {
                    LOG_ERROR("UNITTEST")<<"\ngetFieldValue_"<<"Name"<<" failed.\n"
                             <<"Should be zero"
                             <<" but get is "<<getFieldValue_VersionName(dataformat)<<"\n"
                             <<"version =  "<<getVersion(dataformat)<<"\n";
                    return false;
            }
            LOG_DEBUG("UNITTEST")<< suceed_string;
            return true;
}





}}// end of namespaces
