/* 
 * File:   DataBlockConstructor.cpp
 * Author: denis
 * 
 * Created on September 20, 2012, 2:18 PM
 */

#include <DAQDataFormats/RawDataBlock.h>

#include "DAQDataFormats/../unittest/DataBlockConstructor.h"

/// It is requred to
///   1. include DAQDataFormats/DataFormatFirst.inc
///   2. define CURRENT_CLASS
///   3.    define CURRENT_CLASS_VERSION for version of the class
///      OR define LAST_CLASS_VERSION    for the main structure of the class
///   4. (optional) define CURRENT_FIELDS_SUB_NAMESPACE
///   5. include DAQDataFormats/DataFormatLast.inc
/// before the declaration of the DataFormat version class in the header.
#include "DAQDataFormats/DataFormatFirst.inc"
#define  CURRENT_CLASS                RawDataBlock
#define  LAST_CLASS_VERSION           0
#define  CURRENT_FIELDS_SUB_NAMESPACE datablock
#include "DAQDataFormats/DataFormatLast.inc"

//#define DBSIZE daqdataformats::DEFAULT_DATABLOCK_SIZE
#define DBSIZE ((const uint32_t)0)

namespace daqdataformats {

DataBlockConstructor::DataBlockConstructor():
    daqdataformats::RawDAQDataConstructor(daqdataformats::datablockheader::RAWDATABLOCKHEADER_LATEST_VERSION)
{
}
DataBlockConstructor::~DataBlockConstructor() { }

//------------------------------------------------------------------------------
daqdataformats::RawDAQData* DataBlockConstructor::getFilledDataFormat(const daqdataformats::version_t ver) const {
    RawDataBlock* datablock = constructDataBlock(ver);
    fillDataFormat(datablock);
    return datablock;
}

//------------------------------------------------------------------------------
daqdataformats::RawDAQData* DataBlockConstructor::getEmptyDataFormat(const daqdataformats::version_t ver) const {

    RawDataBlock* datablock = constructDataBlock(ver);

    if(ver != rawdaqdataformat::DAQDATAFORMAT_UNKNOWN_VERSION)
        datablock->close();


    return datablock;
}

//------------------------------------------------------------------------------
daqdataformats::RawDataBlock* DataBlockConstructor::constructDataBlock(const daqdataformats::version_t ver) const{
    return new daqdataformats::RawDataBlock(DBSIZE, (const uint32_t)0, ver);
}

//------------------------------------------------------------------------------
bool DataBlockConstructor::fillDataFormat (daqdataformats::RawDAQData* dataformat) const {
    daqdataformats::RawDataBlock* datablock = (daqdataformats::RawDataBlock*)dataformat;
    _DataBlockHeaderConstructor.fillDataFormat(datablock->getHeader());

    /// initialize DB header
    datablock->getHeader()->init();

    daqdataformats::RawMilliSlice* millislice = (daqdataformats::RawMilliSlice*)_MilliSliceConstructor.getFilledDataFormat(-1);

    /// adding microblock 2
    for(uint32_t imicroslice = 0; imicroslice < millislice->getMicrosliceNumber(); ++imicroslice){
        millislice->setFloatingMicroSlice(imicroslice);
        datablock->addMicroBlock(millislice->getBuffer(), millislice->getFloatingMicroSlice()->getBuffer());
    }

    /// adding microblock 1
    RawMicroBlock* microblock1 = (RawMicroBlock*)_MicroBlockConstructor.getFilledDataFormat(microblock::RAWMICROBLOCK_LATEST_VERSION);
    datablock->addMicroBlock(microblock1->getBuffer());


    datablock->close();

    delete millislice;
    delete microblock1;

    return true;
}

//------------------------------------------------------------------------------
// Gets the DataBlock version
version_t DataBlockConstructor::getDataBlockVersion(const RawDataBlock* nano) const{
    return (version_t) nano->getHeader()->getVersion();
}

}// end of namespaces
