/* 
 * File:   DataBlockHeaderUnitTest.cpp
 * Author: denis
 * 
 * Created on September 24, 2012, 10:18 AM
 */
//#include "DAQDataFormats/../unittest/RawDAQDataUnitTest.h"
//#include "DAQDataFormats/../unittest/DataBlockHeaderConstructor.h"
#include "RawDAQDataUnitTest.h"
#include "DataBlockHeaderConstructor.h"

namespace daqdataformats {

class DataBlockHeaderUnitTest : public RawDAQDataUnitTest {
    CPPUNIT_TEST_SUITE(DataBlockHeaderUnitTest);
    CPPUNIT_TEST(basicFieldGetAndSetChecks);
    CPPUNIT_TEST(testReadDataIntoSameVersion);
    CPPUNIT_TEST(testReadDataIntoUnknownVersion);
    CPPUNIT_TEST(checkDeepCopyConstructor);
    CPPUNIT_TEST(checkShallowCopyConstructor);
    CPPUNIT_TEST_SUITE_END();
public:
    DataBlockHeaderUnitTest();
    ~DataBlockHeaderUnitTest();

    DEFINE_checkDeepCopyConstructor(RawDataBlockHeader)
    DEFINE_checkShallowCopyConstructor(RawDataBlockHeader)
    DEFINE_testReadDataIntoSameVersion
    DEFINE_testReadDataIntoUnknownVersion

private:
    void basicFieldGetAndSetChecksForGivenVersion(const version_t);

private:
    datablockheader::DataBlockHeaderConstructor* _mainDFConstructor;
};

//------------------------------------------------------------------------------
DataBlockHeaderUnitTest::DataBlockHeaderUnitTest():
     RawDAQDataUnitTest(new datablockheader::DataBlockHeaderConstructor())
    ,_mainDFConstructor((datablockheader::DataBlockHeaderConstructor*)_dataFormatConstructor)
{
}

//------------------------------------------------------------------------------
DataBlockHeaderUnitTest::~DataBlockHeaderUnitTest() { }

//------------------------------------------------------------------------------
void DataBlockHeaderUnitTest::basicFieldGetAndSetChecksForGivenVersion(const version_t version) {

    RawDataBlockHeader* df = (RawDataBlockHeader*)_mainDFConstructor->getEmptyDataFormat(version);


    ASSERT_CHECK_FIELD(TrigNumLo);
    ASSERT_CHECK_FIELD(TrigNumHi);
    ASSERT_CHECK_FIELD(NumMicroBlocks);
    ASSERT_CHECK_FIELD(BuffId);
    ASSERT_CHECK_FIELD(DataSize);
    ASSERT_CHECK_FIELD(MonteCarloFlag);
    ASSERT_CHECK_FIELD(CRCCalculationUsed);
    ASSERT_CHECK_FIELD(ConnectionInitialization);
    ASSERT_CHECK_FIELD(CloseConnection);
    ASSERT_CHECK_FIELD(MissingData);

    CPPUNIT_ASSERT(df->checkMarker());

    CPPUNIT_ASSERT(version == (version_t) df->getVersion());

    ASSERT_PRINT_IS_NOT_UNKNOWN;

    delete df;
}

//CPPUNIT_TEST_SUITE_REGISTRATION(DataBlockHeaderUnitTest);
CPPUNIT_TEST_SUITE_NAMED_REGISTRATION(DataBlockHeaderUnitTest, "DAQDataFormats");
CPPUNIT_REGISTRY_ADD_TO_DEFAULT("DAQDataFormats");

}// end of class
