/* 
 * File:   DataBlockUnitTest.cpp
 * Author: denis
 * 
 * Created on September 24, 2012, 10:18 AM
 */
#include <DAQDataFormats/RawDataBlock.h>

//#include "DAQDataFormats/../unittest/RawDAQDataUnitTest.h"
//#include "DAQDataFormats/../unittest/DataBlockConstructor.h"
#include "RawDAQDataUnitTest.h"
#include "DataBlockConstructor.h"

namespace daqdataformats {

class DataBlockUnitTest : public RawDAQDataUnitTest {
    CPPUNIT_TEST_SUITE(DataBlockUnitTest);
    CPPUNIT_TEST(basicFieldGetAndSetChecks);
    CPPUNIT_TEST(testReadDataIntoSameVersion);
    CPPUNIT_TEST(testReadDataIntoUnknownVersion);
    CPPUNIT_TEST(checkDeepCopyConstructor);
    CPPUNIT_TEST(checkShallowCopyConstructor);
    CPPUNIT_TEST_SUITE_END();
public:
    DataBlockUnitTest();
    ~DataBlockUnitTest();

    DEFINE_checkDeepCopyConstructor(RawDataBlock)
    DEFINE_checkShallowCopyConstructor(RawDataBlock)
    DEFINE_testReadDataIntoSameVersion
    DEFINE_testReadDataIntoUnknownVersion
            
private:
    void basicFieldGetAndSetChecksForGivenVersion(const version_t);

private:
    DataBlockConstructor* _mainDFConstructor;
};

//------------------------------------------------------------------------------
DataBlockUnitTest::DataBlockUnitTest():
     RawDAQDataUnitTest(new DataBlockConstructor())
    ,_mainDFConstructor((DataBlockConstructor*)_dataFormatConstructor)
{
}

//------------------------------------------------------------------------------
DataBlockUnitTest::~DataBlockUnitTest() { }

//------------------------------------------------------------------------------
void DataBlockUnitTest::basicFieldGetAndSetChecksForGivenVersion(const version_t version) {

    RawDataBlock* df = (RawDataBlock*)_mainDFConstructor->getEmptyDataFormat(version);

    CPPUNIT_ASSERT(version == (version_t) df->getHeader()->getVersion());

    CPPUNIT_ASSERT(compareDataFormats(df, df->getHeader()) == FIRST_CONTAINS_SECOND);

    ASSERT_PRINT_IS_NOT_UNKNOWN;


    delete df;
}

//CPPUNIT_TEST_SUITE_REGISTRATION(DataBlockUnitTest);
CPPUNIT_TEST_SUITE_NAMED_REGISTRATION(DataBlockUnitTest, "DAQDataFormats");
CPPUNIT_REGISTRY_ADD_TO_DEFAULT("DAQDataFormats");

}// end of class
