/* 
 * File:   EventConstructor.cpp
 * Author: denis
 * 
 * Created on September 20, 2012, 2:18 PM
 */

#include <DAQDataFormats/RawEvent.h>

#include "DAQDataFormats/../unittest/EventConstructor.h"

/// It is requred to
///   1. include DAQDataFormats/DataFormatFirst.inc
///   2. define CURRENT_CLASS
///   3.    define CURRENT_CLASS_VERSION for version of the class
///      OR define LAST_CLASS_VERSION    for the main structure of the class
///   4. (optional) define CURRENT_FIELDS_SUB_NAMESPACE
///   5. include DAQDataFormats/DataFormatLast.inc
/// before the declaration of the DataFormat version class in the header.
#include "DAQDataFormats/DataFormatFirst.inc"
#define  CURRENT_CLASS                RawEvent
#define  LAST_CLASS_VERSION           1
#define  CURRENT_FIELDS_SUB_NAMESPACE rawevent
#include "DAQDataFormats/DataFormatLast.inc"

//#define DBSIZE daqdataformats::DEFAULT_DATABLOCK_SIZE
#define DBSIZE ((const uint32_t)20)

namespace daqdataformats {

EventConstructor::EventConstructor():
    daqdataformats::RawDAQDataConstructor(raweventhdr::RAWEVENTHEADER_LATEST_VERSION)
{
}
EventConstructor::~EventConstructor() { }

//------------------------------------------------------------------------------
daqdataformats::RawDAQData* EventConstructor::getFilledDataFormat(const daqdataformats::version_t ver) const {
    RawEvent* event = NULL;

    try{
        event = constructEvent(ver);
    }
    catch(...){
        LOG_ERROR("UNITTEST")<<"Could not Construct RawEvent of version "<<ver<<"\n";
        throw;
        return NULL;
    }

    try{
        fillDataFormat(event);
    }
    catch(...){
        LOG_ERROR("UNITTEST")<<"Could not Fill RawEvent of version "<<ver<<"\n";
        throw;
        return NULL;
    }

    return event;
}

//------------------------------------------------------------------------------
daqdataformats::RawDAQData* EventConstructor::getEmptyDataFormat(const daqdataformats::version_t ver) const {

    RawEvent* event = constructEvent(ver);

    if(ver != rawdaqdataformat::DAQDATAFORMAT_UNKNOWN_VERSION)
        event->close();


    return event;
}

//------------------------------------------------------------------------------
daqdataformats::RawEvent* EventConstructor::constructEvent(const daqdataformats::version_t ver) const{
    return new daqdataformats::RawEvent(DBSIZE, ver);
}

//------------------------------------------------------------------------------
bool EventConstructor::fillDataFormat (daqdataformats::RawDAQData* dataformat) const {

    daqdataformats::RawEvent* event = (daqdataformats::RawEvent*)dataformat;

    RawTrigger* trigger = NULL;
    const version_t trigger_ver = 0;
    try{
        trigger = (RawTrigger*)_TriggerConstructor.getFilledDataFormat(trigger_ver);
    }
    catch(...){
        LOG_ERROR("UNITTEST")<<"Could not get Trigger of version "<<trigger_ver<<"\n";
        throw;
        return NULL;
    }

    try{
        event->setTrigger(trigger->getBuffer());
    }
    catch(...){
        LOG_ERROR("UNITTEST")<<"Could not set Trigger\n";
        throw;
        return NULL;
    }

    RawDataBlock* datablock = NULL;
    const version_t db_ver = 0;
    try{
        datablock = (RawDataBlock*)_DataBlockConstructor.getFilledDataFormat(db_ver);
    }
    catch(...){
        LOG_ERROR("UNITTEST")<<"Could not get DataBlock of version "<<db_ver<<"\n";
        throw;
        return NULL;
    }

    const uint32_t ndatablocks = 4;
    for(uint32_t idb = 0; idb < ndatablocks; ++idb){
        try{
            event->addDataBlock(datablock->getBuffer());
        }
        catch(...){
            LOG_ERROR("UNITTEST")<<"Could not add DataBlock of version "<<db_ver<<" on "<<idb<<" add.\n";
            throw;
            return NULL;
        }
    }


    try{
        event->close();
    }
    catch(...){
        LOG_ERROR("UNITTEST")<<"Could not close RawEvent of version "<<event->getVersion()<<"\n";
        throw;
        return NULL;
    }


    delete trigger;
    delete datablock;
    return true;
}

//------------------------------------------------------------------------------
// Gets the Event version
version_t EventConstructor::getEventVersion(const RawEvent* nano) const{
    return (version_t) nano->getHeader()->getVersion();
}

}// end of namespaces
