/* 
 * File:   EventHeaderConstructor.cpp
 * Author: denis
 * 
 * Created on September 20, 2012, 2:18 PM
 */
#include "DAQDataFormats/../unittest/EventHeaderConstructor.h"

/// It is requred to
///   1. include DAQDataFormats/DataFormatFirst.inc
///   2. define CURRENT_CLASS
///   3.    define CURRENT_CLASS_VERSION for version of the class
///      OR define LAST_CLASS_VERSION    for the main structure of the class
///   4. (optional) define CURRENT_FIELDS_SUB_NAMESPACE
///   5. include DAQDataFormats/DataFormatLast.inc
/// before the declaration of the DataFormat version class in the header.
#include "DAQDataFormats/DataFormatFirst.inc"
#define  CURRENT_CLASS                RawEventHeader
#define  LAST_CLASS_VERSION           1
#define  CURRENT_FIELDS_SUB_NAMESPACE raweventhdr
#include "DAQDataFormats/DataFormatLast.inc"

namespace daqdataformats {
namespace CURRENT_FIELDS_SUB_NAMESPACE {

//------------------------------------------------------------------------------
EventHeaderConstructor::EventHeaderConstructor():
    daqdataformats::RawDAQDataConstructor(daqdataformats::CURRENT_FIELDS_SUB_NAMESPACE::RAWEVENTHEADER_LATEST_VERSION)
{
}

//------------------------------------------------------------------------------
EventHeaderConstructor::~EventHeaderConstructor() { }

//------------------------------------------------------------------------------
daqdataformats::RawDAQData* EventHeaderConstructor::getFilledDataFormat(const daqdataformats::version_t ver) const {
    RawEventHeader* out = (RawEventHeader*)getEmptyDataFormat(ver);
    fillDataFormat(out);
    return out;
}

//------------------------------------------------------------------------------
daqdataformats::RawDAQData* EventHeaderConstructor::getEmptyDataFormat(const daqdataformats::version_t ver) const{
    return new daqdataformats::RawEventHeader(ver);
}

//------------------------------------------------------------------------------
bool EventHeaderConstructor::fillDataFormat (daqdataformats::RawDAQData* dataformat) const {
    daqdataformats::RawEventHeader* eventheader = (daqdataformats::RawEventHeader*)dataformat;

    setFieldToTestValue_DataLoggerID(eventheader);
    setFieldToTestValue_RunNumber(eventheader);
    setFieldToTestValue_RunType(eventheader);
    setFieldToTestValue_ConfigIDX(eventheader);
    setFieldToTestValue_Partition(eventheader);
    setFieldToTestValue_Subrun(eventheader);
    setFieldToTestValue_EventNumberLo(eventheader);
    setFieldToTestValue_EventNumberHi(eventheader);
    setFieldToTestValue_EventType(eventheader);
    setFieldToTestValue_EventSize(eventheader);
    setFieldToTestValue_EventNumber(eventheader);
    if(eventheader->getVersion() > 0){
        setFieldToTestValue_TriggerFilled(eventheader);
        setFieldToTestValue_TailFilled(eventheader);
    }
    setFieldToTestValue_MonteCarloFlag(eventheader);
    setFieldToTestValue_CRCCalculationUsed(eventheader);
    setFieldToTestValue_EventIncomplete(eventheader);
    setFieldToTestValue_EventNumber(eventheader);
    
    return true;
}

}}// end of namespaces
