/* 
 * File:   EventHeaderUnitTest.cpp
 * Author: denis
 * 
 * Created on September 24, 2012, 10:18 AM
 */
//#include "DAQDataFormats/../unittest/RawDAQDataUnitTest.h"
//#include "DAQDataFormats/../unittest/EventHeaderConstructor.h"
#include "RawDAQDataUnitTest.h"
#include "EventHeaderConstructor.h"

namespace daqdataformats {

class EventHeaderUnitTest : public RawDAQDataUnitTest {
    CPPUNIT_TEST_SUITE(EventHeaderUnitTest);
    CPPUNIT_TEST(basicFieldGetAndSetChecks);
    CPPUNIT_TEST(testReadDataIntoSameVersion);
    CPPUNIT_TEST(testReadDataIntoUnknownVersion);
    CPPUNIT_TEST(checkDeepCopyConstructor);
    CPPUNIT_TEST(checkShallowCopyConstructor);
    CPPUNIT_TEST_SUITE_END();
public:
    EventHeaderUnitTest();
    ~EventHeaderUnitTest();

    DEFINE_checkDeepCopyConstructor(RawEventHeader)
    DEFINE_checkShallowCopyConstructor(RawEventHeader)
    DEFINE_testReadDataIntoSameVersion
    DEFINE_testReadDataIntoUnknownVersion

private:
    void basicFieldGetAndSetChecksForGivenVersion(const version_t);

private:
    raweventhdr::EventHeaderConstructor* _mainDFConstructor;
};

//------------------------------------------------------------------------------
EventHeaderUnitTest::EventHeaderUnitTest():
     RawDAQDataUnitTest(new raweventhdr::EventHeaderConstructor())
    ,_mainDFConstructor((raweventhdr::EventHeaderConstructor*)_dataFormatConstructor)
{
}

//------------------------------------------------------------------------------
EventHeaderUnitTest::~EventHeaderUnitTest() { }

//------------------------------------------------------------------------------
void EventHeaderUnitTest::basicFieldGetAndSetChecksForGivenVersion(const version_t version) {

    RawEventHeader* df = (RawEventHeader*)_mainDFConstructor->getEmptyDataFormat(version);

    ASSERT_CHECK_FIELD(DataLoggerID);
    ASSERT_CHECK_FIELD(RunNumber);
    ASSERT_CHECK_FIELD(RunType);
    ASSERT_CHECK_FIELD(ConfigIDX);
    ASSERT_CHECK_FIELD(Partition);
    ASSERT_CHECK_FIELD(Subrun);
    ASSERT_CHECK_FIELD(EventNumberLo);
    ASSERT_CHECK_FIELD(EventNumberHi);
    ASSERT_CHECK_FIELD(EventType);
    ASSERT_CHECK_FIELD(EventSize);
    ASSERT_CHECK_FIELD(DataBlockNumber);

    if(version > 0){
        ASSERT_CHECK_FIELD(TriggerFilled);
        ASSERT_CHECK_FIELD(TailFilled);
    }


    ASSERT_CHECK_FIELD(MonteCarloFlag);
    ASSERT_CHECK_FIELD(CRCCalculationUsed);
    ASSERT_CHECK_FIELD(EventIncomplete);
    ASSERT_CHECK_FIELD(EventNumber);

    CPPUNIT_ASSERT(version == (version_t) df->getVersion());

    ASSERT_PRINT_IS_NOT_UNKNOWN;

    delete df;
}

//CPPUNIT_TEST_SUITE_REGISTRATION(EventHeaderUnitTest);
CPPUNIT_TEST_SUITE_NAMED_REGISTRATION(EventHeaderUnitTest, "DAQDataFormats");
CPPUNIT_REGISTRY_ADD_TO_DEFAULT("DAQDataFormats");

}// end of class
