/* 
 * File:   EventUnitTest.cpp
 * Author: denis
 * 
 * Created on September 24, 2012, 10:18 AM
 */
#include <DAQDataFormats/RawEvent.h>

//#include "DAQDataFormats/../unittest/RawDAQDataUnitTest.h"
//#include "DAQDataFormats/../unittest/EventConstructor.h"
#include "RawDAQDataUnitTest.h"
#include "EventConstructor.h"

namespace daqdataformats {

class EventUnitTest : public RawDAQDataUnitTest {
    CPPUNIT_TEST_SUITE(EventUnitTest);
    CPPUNIT_TEST(basicFieldGetAndSetChecks);
    CPPUNIT_TEST(testReadDataIntoSameVersion);
    CPPUNIT_TEST(testReadDataIntoUnknownVersion);
    CPPUNIT_TEST(checkDeepCopyConstructor);
    CPPUNIT_TEST(checkShallowCopyConstructor);
    CPPUNIT_TEST_SUITE_END();
public:
    EventUnitTest();
    ~EventUnitTest();

    DEFINE_checkDeepCopyConstructor(RawEvent)
    DEFINE_checkShallowCopyConstructor(RawEvent)
    DEFINE_testReadDataIntoSameVersion
    DEFINE_testReadDataIntoUnknownVersion
            
private:
    void basicFieldGetAndSetChecksForGivenVersion(const version_t);

private:
    EventConstructor* _mainDFConstructor;
};

//------------------------------------------------------------------------------
EventUnitTest::EventUnitTest():
     RawDAQDataUnitTest(new EventConstructor())
    ,_mainDFConstructor((EventConstructor*)_dataFormatConstructor)
{
}

//------------------------------------------------------------------------------
EventUnitTest::~EventUnitTest() { }

//------------------------------------------------------------------------------
void EventUnitTest::basicFieldGetAndSetChecksForGivenVersion(const version_t version) {

    RawEvent* df = (RawEvent*)_mainDFConstructor->getEmptyDataFormat(version);

    CPPUNIT_ASSERT(version == (version_t) df->getHeader()->getVersion());

    CPPUNIT_ASSERT(compareDataFormats(df, df->getHeader()) == FIRST_CONTAINS_SECOND);

    ASSERT_PRINT_IS_NOT_UNKNOWN;

    delete df;

    df = (RawEvent*)_mainDFConstructor->getFilledDataFormat(version);

    
    delete df;

}

//CPPUNIT_TEST_SUITE_REGISTRATION(EventUnitTest);
CPPUNIT_TEST_SUITE_NAMED_REGISTRATION(EventUnitTest, "DAQDataFormats");
CPPUNIT_REGISTRY_ADD_TO_DEFAULT("DAQDataFormats");

}// end of class
