/* 
 * File:   MicroBlockConstructor.cpp
 * Author: denis
 * 
 * Created on September 20, 2012, 2:18 PM
 */

#include <DAQDataFormats/RawMicroBlock.h>
#include <DAQDataFormats/RawMicroBlockHeader.h>

#include "DAQDataFormats/../unittest/MicroBlockConstructor.h"

/// It is requred to
///   1. include DAQDataFormats/DataFormatFirst.inc
///   2. define CURRENT_CLASS
///   3.    define CURRENT_CLASS_VERSION for version of the class
///      OR define LAST_CLASS_VERSION    for the main structure of the class
///   4. (optional) define CURRENT_FIELDS_SUB_NAMESPACE
///   5. include DAQDataFormats/DataFormatLast.inc
/// before the declaration of the DataFormat version class in the header.
#include "DAQDataFormats/DataFormatFirst.inc"
#define  CURRENT_CLASS                RawMicroBlock
#define  LAST_CLASS_VERSION           1
#define  CURRENT_FIELDS_SUB_NAMESPACE microblock
#include "DAQDataFormats/DataFormatLast.inc"

namespace daqdataformats {
namespace CURRENT_FIELDS_SUB_NAMESPACE {

MicroBlockConstructor::MicroBlockConstructor():
    daqdataformats::RawDAQDataConstructor(daqdataformats::microblock::RAWMICROBLOCK_LATEST_VERSION)
{
}
MicroBlockConstructor::~MicroBlockConstructor() { }

//------------------------------------------------------------------------------
daqdataformats::RawDAQData* MicroBlockConstructor::getFilledDataFormat(const daqdataformats::version_t ver) const {
    RawMicroBlock* out = new daqdataformats::RawMicroBlock(ver);
    fillDataFormat(out);
    return out;
}

//------------------------------------------------------------------------------
daqdataformats::RawDAQData* MicroBlockConstructor::getEmptyDataFormat(const daqdataformats::version_t ver) const {
    RawMicroBlock* microblock = new daqdataformats::RawMicroBlock(ver);

    if(ver != rawdaqdataformat::DAQDATAFORMAT_UNKNOWN_VERSION){
        RawMilliSlice* milli = (RawMilliSlice*)_milliSliceConstructor.getEmptyDataFormat(rawdaqdataformat::DAQDATAFORMAT_UNKNOWN_VERSION);
        RawMicroSlice* micro = (RawMicroSlice*)_milliSliceConstructor._microSliceConstructor.getEmptyDataFormat(rawdaqdataformat::DAQDATAFORMAT_UNKNOWN_VERSION);

        microblock->setMicroBlockHeader(milli->getBuffer());
        microblock->setMicroSlice(micro->getBuffer());

        delete milli;
        delete micro;
    }
    
    return microblock;
}

//------------------------------------------------------------------------------
bool MicroBlockConstructor::fillDataFormat (daqdataformats::RawDAQData* dataformat) const {
    daqdataformats::RawMicroBlock* microblock = (daqdataformats::RawMicroBlock*)dataformat;

    RawMilliSlice* milli = (RawMilliSlice*)_milliSliceConstructor.getFilledDataFormat(rawdaqdataformat::DAQDATAFORMAT_UNKNOWN_VERSION);
    RawMicroSlice* micro = (RawMicroSlice*)_milliSliceConstructor._microSliceConstructor.getFilledDataFormat(rawdaqdataformat::DAQDATAFORMAT_UNKNOWN_VERSION);
    
    microblock->setMicroBlockHeader(milli->getBuffer());
    microblock->setMicroSlice(micro->getBuffer());
    
    delete milli;
    delete micro;

    return true;
}

}}// end of namespaces
