/* 
 * File:   MicroBlockUnitTest.cpp
 * Author: denis
 * 
 * Created on September 24, 2012, 10:18 AM
 */
#include <DAQDataFormats/RawMicroBlock.h>

//#include "DAQDataFormats/../unittest/RawDAQDataUnitTest.h"
//#include "DAQDataFormats/../unittest/MicroBlockConstructor.h"
#include "RawDAQDataUnitTest.h"
#include "MicroBlockConstructor.h"

#include "DAQDataFormats/../unittest/NanoSliceConstructor.h"
#include "DAQDataFormats/../unittest/MilliSliceConstructor.h"

namespace daqdataformats {

class MicroBlockUnitTest : public RawDAQDataUnitTest {
    CPPUNIT_TEST_SUITE(MicroBlockUnitTest);
    CPPUNIT_TEST(basicFieldGetAndSetChecks);
    CPPUNIT_TEST(testReadDataIntoSameVersion);
    CPPUNIT_TEST(testReadDataIntoUnknownVersion);
    CPPUNIT_TEST(loopedcheckDeepCopyConstructor);
    CPPUNIT_TEST(loopedcheckShallowCopyConstructor);
    CPPUNIT_TEST_SUITE_END();
public:
    MicroBlockUnitTest();
    ~MicroBlockUnitTest();

    DEFINE_testReadDataIntoSameVersion
    DEFINE_testReadDataIntoUnknownVersion

    DEFINE_checkDeepCopyConstructor(RawMicroBlock)
    DEFINE_checkShallowCopyConstructor(RawMicroBlock)
    void loopedcheckDeepCopyConstructor();
    void loopedcheckShallowCopyConstructor();
            
private:
    void basicFieldGetAndSetChecksForGivenVersion(const version_t);

private:
    microblock::MicroBlockConstructor* _mainDFConstructor;
};

//------------------------------------------------------------------------------
MicroBlockUnitTest::MicroBlockUnitTest():
     RawDAQDataUnitTest(new microblock::MicroBlockConstructor())
    ,_mainDFConstructor((microblock::MicroBlockConstructor*)_dataFormatConstructor)
{
}

//------------------------------------------------------------------------------
MicroBlockUnitTest::~MicroBlockUnitTest() { }

//------------------------------------------------------------------------------
void MicroBlockUnitTest::basicFieldGetAndSetChecksForGivenVersion(const version_t version) {

    
    LOG_DEBUG("UNITTEST")<<"MicroBlockUnitTest::basicFieldGetAndSetChecksForGivenVersion for version "<<version<<std::endl;

    /// Loop over nanoslices
    rawnano::NanoSliceConstructor nano_constructor;

    MicroSliceConstructor* micro_constructor = &(_mainDFConstructor->_milliSliceConstructor._microSliceConstructor);
    for(micro_constructor->_currentNanoSliceVersion = 0; micro_constructor->_currentNanoSliceVersion <= nano_constructor.getMaximumVersion(); ++(micro_constructor->_currentNanoSliceVersion)){

                RawMicroBlock* microblock = new RawMicroBlock(version);

                RawMilliSlice* milli = (RawMilliSlice*)_mainDFConstructor->_milliSliceConstructor.getFilledDataFormat(rawdaqdataformat::DAQDATAFORMAT_UNKNOWN_VERSION);
                RawMicroSlice* micro = (RawMicroSlice*)_mainDFConstructor->_milliSliceConstructor._microSliceConstructor.getFilledDataFormat(rawdaqdataformat::DAQDATAFORMAT_UNKNOWN_VERSION);

                microblock->setMicroBlockHeader(milli->getBuffer());
                microblock->setMicroSlice(micro->getBuffer());

                delete microblock;
    }
    LOG_DEBUG("UNITTEST")<<"End MicroBlockUnitTest::basicFieldGetAndSetChecksForGivenVersion for version "<<version<<std::endl;
}

//------------------------------------------------------------------------------
void MicroBlockUnitTest::loopedcheckDeepCopyConstructor(){

    rawnano::NanoSliceConstructor nano_constructor;

    MicroSliceConstructor* micro_constructor = &(_mainDFConstructor->_milliSliceConstructor._microSliceConstructor);

    for(micro_constructor->_currentNanoSliceVersion = 0; micro_constructor->_currentNanoSliceVersion <= nano_constructor.getMaximumVersion(); ++(micro_constructor->_currentNanoSliceVersion)){
        LOG_DEBUG("UNITTEST")<< "From "<<__PRETTY_FUNCTION__<<". Checking Nano version "<<micro_constructor->_currentNanoSliceVersion<<std::endl;
        checkDeepCopyConstructor();
    }

}

//------------------------------------------------------------------------------
void MicroBlockUnitTest::loopedcheckShallowCopyConstructor(){

    rawnano::NanoSliceConstructor nano_constructor;

    MicroSliceConstructor* micro_constructor = &(_mainDFConstructor->_milliSliceConstructor._microSliceConstructor);

    for(micro_constructor->_currentNanoSliceVersion = 0; micro_constructor->_currentNanoSliceVersion <= nano_constructor.getMaximumVersion(); ++(micro_constructor->_currentNanoSliceVersion)){
        LOG_DEBUG("UNITTEST")<< "From "<<__PRETTY_FUNCTION__<<". Checking Nano version "<<micro_constructor->_currentNanoSliceVersion<<std::endl;
        checkShallowCopyConstructor();
    }

}


//CPPUNIT_TEST_SUITE_REGISTRATION(MicroBlockUnitTest);
CPPUNIT_TEST_SUITE_NAMED_REGISTRATION(MicroBlockUnitTest, "DAQDataFormats");
CPPUNIT_REGISTRY_ADD_TO_DEFAULT("DAQDataFormats");

}// end of class
