/* 
 * File:   MicroSliceConstructor.cpp
 * Author: denis
 * 
 * Created on September 20, 2012, 2:18 PM
 */
#include <DAQDataFormats/RawNanoSlice.h>
#include <DAQDataFormats/RawMicroSlice.h>

#include "DAQDataFormats/../unittest/NanoSliceConstructor.h"
#include "DAQDataFormats/../unittest/MicroSliceConstructor.h"


namespace daqdataformats {

//------------------------------------------------------------------------------
MicroSliceConstructor::MicroSliceConstructor():_currentNanoSliceVersion(0){ }

//------------------------------------------------------------------------------
MicroSliceConstructor::~MicroSliceConstructor() { }

//------------------------------------------------------------------------------
daqdataformats::RawDAQData* MicroSliceConstructor::getFilledDataFormat(const version_t) const {
    RawMicroSlice* out = (RawMicroSlice*)getEmptyDataFormat(rawdaqdataformat::DAQDATAFORMAT_UNKNOWN_VERSION);
    fillDataFormat(out);
    return out;
}

//------------------------------------------------------------------------------
daqdataformats::RawDAQData* MicroSliceConstructor::getEmptyDataFormat(const version_t) const {
    return new daqdataformats::RawMicroSlice();
}

//------------------------------------------------------------------------------
bool MicroSliceConstructor::fillDataFormat (daqdataformats::RawDAQData* dataformat) const {
    daqdataformats::RawMicroSlice* microslice = (daqdataformats::RawMicroSlice*)dataformat;

    microslice->setVersion(0);
    setFieldToTestValue_Time(microslice);
    setFieldToTestValue_MonteCarloFlag(microslice);

    rawnano::NanoSliceConstructor nano_constructor;

    RawNanoSlice* nano1 = (RawNanoSlice*)nano_constructor.getFilledDataFormat(_currentNanoSliceVersion);

    RawNanoSlice* nano2 = (RawNanoSlice*)nano_constructor.getFilledDataFormat(_currentNanoSliceVersion);

    if(_currentNanoSliceVersion == 0){
        nano2->setPulseHeight(nano2->getPulseHeight() + 1);
    }
    if(_currentNanoSliceVersion == 1){
        nano2->setValue1(nano2->getValue1() + 1);
    }

    microslice->addNanoSlice(nano1->getBuffer());
    microslice->addNanoSlice(nano2->getBuffer());

    delete nano1;
    delete nano2;

    return true;
}

//------------------------------------------------------------------------------
/// Gets the NanoSlices version in the MicroSlice
version_t MicroSliceConstructor::getNanoSlicesVersion(const RawMicroSlice* micro) const{

    if(micro->getNumNanoSlices() == 0) return rawdaqdataformat::DAQDATAFORMAT_UNKNOWN_VERSION;

    /// right now it's unknown
    version_t nano_ver = rawdaqdataformat::DAQDATAFORMAT_UNKNOWN_VERSION;

    /// loop over nanoslices
    for(uint32_t inano = 0; inano < micro->getNumNanoSlices(); ++inano){
        micro->setFloatingNanoSlice(inano);

        const version_t nano_ver_current = _nanosliceConstructor.getVersion(micro->getFloatingNanoSlice());

        if(nano_ver_current == rawdaqdataformat::DAQDATAFORMAT_UNKNOWN_VERSION){
            LOG_ERROR("UNITTEST")<<"Failure in "<<__PRETTY_FUNCTION__<<" at "<<__LINE__<<"\n"
                                 <<" Output nanoslice version is rawdaqdataformat::DAQDATAFORMAT_UNKNOWN_VERSION\n";
            return rawdaqdataformat::DAQDATAFORMAT_UNKNOWN_VERSION;
        }

        if(nano_ver == rawdaqdataformat::DAQDATAFORMAT_UNKNOWN_VERSION){
            nano_ver = nano_ver_current;
        }
        else{
            if(nano_ver != nano_ver_current){
                LOG_ERROR("UNITTEST")<<"Failure in "<<__PRETTY_FUNCTION__<<" at "<<__LINE__<<"\n"
                                     <<" versions are inconsistent: at nanoslice "<<inano<<"\n"
                                     <<"ver0="<<nano_ver<<" ver1="<<nano_ver_current<<"\n";
                return rawdaqdataformat::DAQDATAFORMAT_UNKNOWN_VERSION;
            }
        }
    }// end of loop over nanoslices

    return nano_ver;
}

}// end of namespaces
