/* 
 * File:   MicroSliceUnitTest.cpp
 * Author: denis
 * 
 * Created on September 24, 2012, 10:18 AM
 */
//#include "DAQDataFormats/../unittest/RawDAQDataUnitTest.h"
//#include "DAQDataFormats/../unittest/MicroSliceConstructor.h"
#include "RawDAQDataUnitTest.h"
#include "MicroSliceConstructor.h"
#include "DAQDataFormats/../unittest/NanoSliceConstructor.h"

namespace daqdataformats {

class MicroSliceUnitTest : public RawDAQDataUnitTest {
    CPPUNIT_TEST_SUITE(MicroSliceUnitTest);
    CPPUNIT_TEST(basicFieldGetAndSetChecks);
    CPPUNIT_TEST(loopedcheckDeepCopyConstructor);
    CPPUNIT_TEST(loopedcheckShallowCopyConstructor);
    CPPUNIT_TEST(testReadDataIntoSameVersion);
    CPPUNIT_TEST_SUITE_END();
public:
    MicroSliceUnitTest();
    ~MicroSliceUnitTest();

    DEFINE_checkDeepCopyConstructor(RawMicroSlice)
    DEFINE_checkShallowCopyConstructor(RawMicroSlice)
    void loopedcheckDeepCopyConstructor();
    void loopedcheckShallowCopyConstructor();

    DEFINE_testReadDataIntoSameVersion

    void basicFieldGetAndSetChecks();

private:
    MicroSliceConstructor* _mainDFConstructor;
};

//------------------------------------------------------------------------------
MicroSliceUnitTest::MicroSliceUnitTest():
     RawDAQDataUnitTest(new MicroSliceConstructor())
    ,_mainDFConstructor((MicroSliceConstructor*)_dataFormatConstructor)
{
}

//------------------------------------------------------------------------------
MicroSliceUnitTest::~MicroSliceUnitTest() { }

//------------------------------------------------------------------------------
void MicroSliceUnitTest::basicFieldGetAndSetChecks() {

    /// Checking just construction
    {
        RawMicroSlice* df = (RawMicroSlice*)_mainDFConstructor->getEmptyDataFormat(rawdaqdataformat::DAQDATAFORMAT_UNKNOWN_VERSION);
        ASSERT_CHECK_FIELD(MonteCarloFlag);
        ASSERT_CHECK_FIELD(Time);
        delete df;
    }

    /// Now Checking filling MicroSlice with NanoSlices
    rawnano::NanoSliceConstructor nano_constructor;
    for(_mainDFConstructor->_currentNanoSliceVersion = 0; _mainDFConstructor->_currentNanoSliceVersion <= nano_constructor.getMaximumVersion(); ++(_mainDFConstructor->_currentNanoSliceVersion)){
        LOG_DEBUG("UNITTEST")<< "From "<<__PRETTY_FUNCTION__<<". Checking Nano version "<<_mainDFConstructor->_currentNanoSliceVersion<<std::endl;

        daqdataformats::RawMicroSlice* microslice = (daqdataformats::RawMicroSlice*)_mainDFConstructor->getEmptyDataFormat(rawdaqdataformat::DAQDATAFORMAT_UNKNOWN_VERSION);
        microslice->setVersion(0);
        _mainDFConstructor->setFieldToTestValue_Time(microslice);
        _mainDFConstructor->setFieldToTestValue_MonteCarloFlag(microslice);
        LOG_DEBUG("UNITTEST")<< "From "<<__PRETTY_FUNCTION__<<". Checking Nano version "<<_mainDFConstructor->_currentNanoSliceVersion<<std::endl;

        RawNanoSlice* nano1 = (RawNanoSlice*)nano_constructor.getFilledDataFormat(_mainDFConstructor->_currentNanoSliceVersion);

        RawNanoSlice* nano2 = (RawNanoSlice*)nano_constructor.getFilledDataFormat(_mainDFConstructor->_currentNanoSliceVersion);

        if(_mainDFConstructor->_currentNanoSliceVersion == 0){
            nano2->setPulseHeight(nano2->getPulseHeight() + 1);
        }
        if(_mainDFConstructor->_currentNanoSliceVersion == 1){
            nano2->setValue1(nano2->getValue1() + 1);
        }

        microslice->addNanoSlice(nano1->getBuffer());
        microslice->addNanoSlice(nano2->getBuffer());

        CPPUNIT_ASSERT(compareDataFormats(microslice, microslice->getHeader()      , FIRST_CONTAINS_SECOND));
        CPPUNIT_ASSERT(compareDataFormats(microslice, microslice->getTimingMarker(), FIRST_CONTAINS_SECOND));
        CPPUNIT_ASSERT(compareDataFormats(microslice, nano1                        , FIRST_CONTAINS_SECOND));
        CPPUNIT_ASSERT(compareDataFormats(microslice, nano2                        , FIRST_CONTAINS_SECOND));

        CPPUNIT_ASSERT(microslice->getNumNanoSlices() == 2 );

        CPPUNIT_ASSERT(_mainDFConstructor->checkField_MonteCarloFlag(microslice) );
        CPPUNIT_ASSERT(_mainDFConstructor->checkField_Time(microslice) );

        CPPUNIT_ASSERT(_mainDFConstructor->getNanoSlicesVersion(microslice) == _mainDFConstructor->_currentNanoSliceVersion);
                
        delete nano1;
        delete nano2;
        delete microslice;
    }
}

//------------------------------------------------------------------------------
void MicroSliceUnitTest::loopedcheckDeepCopyConstructor(){

    rawnano::NanoSliceConstructor nano_constructor;

    for(_mainDFConstructor->_currentNanoSliceVersion = 0; _mainDFConstructor->_currentNanoSliceVersion <= nano_constructor.getMaximumVersion(); ++(_mainDFConstructor->_currentNanoSliceVersion)){
        LOG_DEBUG("UNITTEST")<< "From "<<__PRETTY_FUNCTION__<<". Checking Nano version "<<_mainDFConstructor->_currentNanoSliceVersion<<std::endl;
        checkDeepCopyConstructor();
    }

}

//------------------------------------------------------------------------------
void MicroSliceUnitTest::loopedcheckShallowCopyConstructor(){

    rawnano::NanoSliceConstructor nano_constructor;

    for(_mainDFConstructor->_currentNanoSliceVersion = 0; _mainDFConstructor->_currentNanoSliceVersion <= nano_constructor.getMaximumVersion(); ++(_mainDFConstructor->_currentNanoSliceVersion)){
        LOG_DEBUG("UNITTEST")<< "From "<<__PRETTY_FUNCTION__<<". Checking Nano version "<<_mainDFConstructor->_currentNanoSliceVersion<<std::endl;
        checkShallowCopyConstructor();
    }

}

//CPPUNIT_TEST_SUITE_REGISTRATION(MicroSliceUnitTest);
CPPUNIT_TEST_SUITE_NAMED_REGISTRATION(MicroSliceUnitTest, "DAQDataFormats");
CPPUNIT_REGISTRY_ADD_TO_DEFAULT("DAQDataFormats");

}// end of class
