/* 
 * File:   MilliSliceConstructor.cpp
 * Author: denis
 * 
 * Created on September 20, 2012, 2:18 PM
 */

#include <DAQDataFormats/RawMilliSlice.h>
#include "DAQDataFormats/../unittest/MilliSliceConstructor.h"

#include "DAQDataFormats/DataFormatFirst.inc"
#define  CURRENT_CLASS                RawMilliSlice

namespace daqdataformats {

MilliSliceConstructor::MilliSliceConstructor()
{
}
MilliSliceConstructor::~MilliSliceConstructor() { }

//------------------------------------------------------------------------------
daqdataformats::RawDAQData* MilliSliceConstructor::getFilledDataFormat(const daqdataformats::version_t ver) const {
    RawMilliSlice* out = new daqdataformats::RawMilliSlice();
    fillDataFormat(out);
    return out;
}

//------------------------------------------------------------------------------
daqdataformats::RawDAQData* MilliSliceConstructor::getEmptyDataFormat(const daqdataformats::version_t ver) const {
    RawMilliSlice* out = new daqdataformats::RawMilliSlice();
    out->close(true);
    return out;
}

//------------------------------------------------------------------------------
bool MilliSliceConstructor::fillDataFormat (daqdataformats::RawDAQData* dataformat) const {
    daqdataformats::RawMilliSlice* millislice = (daqdataformats::RawMilliSlice*)dataformat;

    millislice->setDiblockId(5);
    millislice->setPartition(6);
    millislice->setTimeStartLow(7);
    millislice->setTimeStartHigh(8);
    millislice->setTimeLength(9);
    millislice->setMilliSliceSEQ(10);
    millislice->setMicroSliceTimeLength_usec(11);
    millislice->setMonteCarloFlag(true);
    millislice->setFrameSEQ(12);

    RawMicroSlice* micro1 = (RawMicroSlice*)_microSliceConstructor.getFilledDataFormat(rawdaqdataformat::DAQDATAFORMAT_UNKNOWN_VERSION);
    RawMicroSlice* micro2 = (RawMicroSlice*)_microSliceConstructor.getFilledDataFormat(rawdaqdataformat::DAQDATAFORMAT_UNKNOWN_VERSION);

    micro2->setTime(micro2->getTime() + 500);

    millislice->addMicroSlice(micro1->getBuffer());
    millislice->addMicroSlice(micro2->getBuffer());

    millislice->close(true);


    delete micro1;
    delete micro2;

    return true;
}

//------------------------------------------------------------------------------
// Gets the NanoSlices version in the MilliSlice
version_t MilliSliceConstructor::getNanoSlicesVersion(const RawMilliSlice* milli) const{

    if(milli->getMicrosliceNumber() == 0) return rawdaqdataformat::DAQDATAFORMAT_UNKNOWN_VERSION;

    /// right now it's unknown
    version_t nano_ver = rawdaqdataformat::DAQDATAFORMAT_UNKNOWN_VERSION;

    /// loop over nanoslices
    for(uint32_t imicro = 0; imicro < milli->getMicrosliceNumber(); ++imicro){
        milli->setFloatingMicroSlice(imicro);

        /// If number of Nanoslices in Microslice is zero, then skip it
        if(milli->getFloatingMicroSlice()->getNumNanoSlices() == 0) continue;

        const version_t nano_ver_current = _microSliceConstructor.getNanoSlicesVersion(milli->getFloatingMicroSlice());

        if(nano_ver_current == rawdaqdataformat::DAQDATAFORMAT_UNKNOWN_VERSION){
            LOG_ERROR("UNITTEST")<<"Failure in "<<__PRETTY_FUNCTION__<<" at "<<__LINE__<<"\n"
                                 <<" Output nanoslice version is rawdaqdataformat::DAQDATAFORMAT_UNKNOWN_VERSION\n";
            return rawdaqdataformat::DAQDATAFORMAT_UNKNOWN_VERSION;
        }

        if(nano_ver == rawdaqdataformat::DAQDATAFORMAT_UNKNOWN_VERSION){
            nano_ver = nano_ver_current;
        }
        else{
            if(nano_ver != nano_ver_current){
                LOG_ERROR("UNITTEST")<<"Failure in "<<__PRETTY_FUNCTION__<<" at "<<__LINE__<<"\n"
                                     <<" versions are inconsistent: at microslice "<<imicro<<"\n"
                                     <<"ver0="<<nano_ver<<" ver1="<<nano_ver_current<<"\n";
                return rawdaqdataformat::DAQDATAFORMAT_UNKNOWN_VERSION;
            }
        }
    }// end of loop over nanoslices

    return nano_ver;
}

}// end of namespaces
