/* 
 * File:   MilliSliceUnitTest.cpp
 * Author: denis
 * 
 * Created on September 24, 2012, 10:18 AM
 */
//#include "DAQDataFormats/../unittest/RawDAQDataUnitTest.h"
//#include "DAQDataFormats/../unittest/MilliSliceConstructor.h"
#include "RawDAQDataUnitTest.h"
#include "MilliSliceConstructor.h"
#include "DAQDataFormats/../unittest/NanoSliceConstructor.h"
#include "DAQDataFormats/../unittest/MicroSliceConstructor.h"

namespace daqdataformats {

class MilliSliceUnitTest : public RawDAQDataUnitTest {
    CPPUNIT_TEST_SUITE(MilliSliceUnitTest);
    CPPUNIT_TEST(basicFieldGetAndSetChecks);
    CPPUNIT_TEST(loopedcheckDeepCopyConstructor);
    CPPUNIT_TEST(loopedcheckShallowCopyConstructor);
    CPPUNIT_TEST(testReadDataIntoSameVersion);
    CPPUNIT_TEST_SUITE_END();
public:
    MilliSliceUnitTest();
    ~MilliSliceUnitTest();

    DEFINE_checkDeepCopyConstructor(RawMilliSlice)
    DEFINE_checkShallowCopyConstructor(RawMilliSlice)
    void loopedcheckDeepCopyConstructor();
    void loopedcheckShallowCopyConstructor();

    DEFINE_testReadDataIntoSameVersion

    void basicFieldGetAndSetChecks();

private:
    /// Check MilliSlice CRC
    void testMilliSliceCRC(RawMilliSlice*);

private:
    MilliSliceConstructor* _mainDFConstructor;
};

//------------------------------------------------------------------------------
MilliSliceUnitTest::MilliSliceUnitTest():
     RawDAQDataUnitTest(new MilliSliceConstructor())
    ,_mainDFConstructor((MilliSliceConstructor*)_dataFormatConstructor)
{
}

//------------------------------------------------------------------------------
MilliSliceUnitTest::~MilliSliceUnitTest() { }

//------------------------------------------------------------------------------
void MilliSliceUnitTest::basicFieldGetAndSetChecks() {


    /// Loop over nanoslices
    rawnano::NanoSliceConstructor nano_constructor;

    MicroSliceConstructor* micro_constructor = &(_mainDFConstructor->_microSliceConstructor);
    for(micro_constructor->_currentNanoSliceVersion = 0; micro_constructor->_currentNanoSliceVersion <= nano_constructor.getMaximumVersion(); ++(micro_constructor->_currentNanoSliceVersion)){

        /// Test filling of the MilliSlice
        {
            daqdataformats::RawMilliSlice* millislice = new RawMilliSlice();

            millislice->setDiblockId(5);
            millislice->setPartition(6);
            millislice->setTimeStartLow(7);
            millislice->setTimeStartHigh(8);
            millislice->setTimeLength(9);
            millislice->setMilliSliceSEQ(10);
            millislice->setMicroSliceTimeLength_usec(11);
            millislice->setMonteCarloFlag(true);
            millislice->setFrameSEQ(12);

            RawMicroSlice* micro1 = (RawMicroSlice*)_mainDFConstructor->_microSliceConstructor.getFilledDataFormat(rawdaqdataformat::DAQDATAFORMAT_UNKNOWN_VERSION);
            RawMicroSlice* micro2 = (RawMicroSlice*)_mainDFConstructor->_microSliceConstructor.getFilledDataFormat(rawdaqdataformat::DAQDATAFORMAT_UNKNOWN_VERSION);
            micro2->setTime(micro2->getTime() + 500);
            millislice->addMicroSlice(micro1->getBuffer());
            millislice->addMicroSlice(micro2->getBuffer());
            millislice->close(true);

            CPPUNIT_ASSERT(compareDataFormats(millislice, micro1) == FIRST_CONTAINS_SECOND);
            CPPUNIT_ASSERT(compareDataFormats(millislice, micro2) == FIRST_CONTAINS_SECOND);

            CPPUNIT_ASSERT(millislice->getMicrosliceNumber() == 2 );


            CPPUNIT_ASSERT(millislice->sizeofdata() - micro1->sizeofdata() - micro2->sizeofdata() < 30);

            CPPUNIT_ASSERT(!isPrintDataFormatUnknownVersion(millislice));

            /// Testing CRC if the MilliSlice
            testMilliSliceCRC(millislice);


            CPPUNIT_ASSERT(_mainDFConstructor->getNanoSlicesVersion(millislice) == micro_constructor->_currentNanoSliceVersion);


            delete micro1;
            delete micro2;
            delete millislice;
        }// end of test filling MilliSlice

        
        RawMilliSlice* df0 = (RawMilliSlice*)_mainDFConstructor->getEmptyDataFormat(rawdaqdataformat::DAQDATAFORMAT_UNKNOWN_VERSION);
        testMilliSliceCRC(df0);
        CPPUNIT_ASSERT(df0->sizeofdata() < 100);

        RawMilliSlice* df1 = (RawMilliSlice*)_mainDFConstructor->getFilledDataFormat(rawdaqdataformat::DAQDATAFORMAT_UNKNOWN_VERSION);
        testMilliSliceCRC(df1);
        CPPUNIT_ASSERT(df1->sizeofdata() < 100);

        CPPUNIT_ASSERT(compareDataFormats(df0, df1) == DATAFORMATS_ARE_DIFFERENT);

        CPPUNIT_ASSERT(!isPrintDataFormatUnknownVersion(df0));
        CPPUNIT_ASSERT(!isPrintDataFormatUnknownVersion(df1));
    
        delete df0;
        delete df1;
    }// end of loop over nanoslices
   
}

//------------------------------------------------------------------------------
// Checking MilliSlice CRC
void MilliSliceUnitTest::testMilliSliceCRC(RawMilliSlice* millislice){
        CPPUNIT_ASSERT(millislice->isCRCCalculationUsed());
        CPPUNIT_ASSERT(millislice->getCRC() > 0);
        CPPUNIT_ASSERT(millislice->calculateCheckSum(0) == 0);
}


//------------------------------------------------------------------------------
void MilliSliceUnitTest::loopedcheckDeepCopyConstructor(){

    rawnano::NanoSliceConstructor nano_constructor;

    MicroSliceConstructor* micro_constructor = &(_mainDFConstructor->_microSliceConstructor);

    for(micro_constructor->_currentNanoSliceVersion = 0; micro_constructor->_currentNanoSliceVersion <= nano_constructor.getMaximumVersion(); ++(micro_constructor->_currentNanoSliceVersion)){
        LOG_DEBUG("UNITTEST")<< "From "<<__PRETTY_FUNCTION__<<". Checking Nano version "<<micro_constructor->_currentNanoSliceVersion<<std::endl;
        checkDeepCopyConstructor();
    }

}

//------------------------------------------------------------------------------
void MilliSliceUnitTest::loopedcheckShallowCopyConstructor(){

    rawnano::NanoSliceConstructor nano_constructor;

    MicroSliceConstructor* micro_constructor = &(_mainDFConstructor->_microSliceConstructor);

    for(micro_constructor->_currentNanoSliceVersion = 0; micro_constructor->_currentNanoSliceVersion <= nano_constructor.getMaximumVersion(); ++(micro_constructor->_currentNanoSliceVersion)){
        LOG_DEBUG("UNITTEST")<< "From "<<__PRETTY_FUNCTION__<<". Checking Nano version "<<micro_constructor->_currentNanoSliceVersion<<std::endl;
        checkShallowCopyConstructor();
    }

}

//CPPUNIT_TEST_SUITE_REGISTRATION(MilliSliceUnitTest);
CPPUNIT_TEST_SUITE_NAMED_REGISTRATION(MilliSliceUnitTest, "DAQDataFormats");
CPPUNIT_REGISTRY_ADD_TO_DEFAULT("DAQDataFormats");

}// end of class
