/* 
 * File:   NanoSliceHeaderConstructor.h
 * Author: denis
 *
 * Created on September 20, 2012, 2:18 PM
 */

#ifndef NANOSLICEHEADERCONSTRUCTOR_H
#define	NANOSLICEHEADERCONSTRUCTOR_H
//#include "DAQDataFormats/../unittest/RawDAQDataConstructor.h"
#include "RawDAQDataConstructor.h"
#include "DAQDataFormats/RawNanoSliceHeader.h"

/// It is requred to
///   1. include DAQDataFormats/DataFormatFirst.inc
///   2. define CURRENT_CLASS
///   3.    define CURRENT_CLASS_VERSION for version of the class
///      OR define LAST_CLASS_VERSION    for the main structure of the class
///   4. (optional) define CURRENT_FIELDS_SUB_NAMESPACE
///   5. include DAQDataFormats/DataFormatLast.inc
/// before the declaration of the DataFormat version class in the header.
#include "DAQDataFormats/DataFormatFirst.inc"
#define  CURRENT_CLASS                RawNanoSliceHeader
#define  LAST_CLASS_VERSION           2
#define  CURRENT_FIELDS_SUB_NAMESPACE nanosliceheader
#include "DAQDataFormats/DataFormatLast.inc"

namespace daqdataformats {
namespace CURRENT_FIELDS_SUB_NAMESPACE {

            enum NanoSliceHeaderTestValues {
                TEST_VALUE_LinkStatus   = 1, //! Link Status                         (1 bit)
                TEST_VALUE_DataPresent  = 1, //! Denis Perevalov Present/Absent flag (1 bit)
                TEST_VALUE_DebugMode    = 1, //! Debug Mode                          (1 bit)
                TEST_VALUE_FEBId        = 2, //! FEB ID                              (6 bit)
                TEST_VALUE_FEBStatus    = 3, //! FEB Status                          (7 bit)
                TEST_VALUE_PixelAddr    = 4, //! Pixel Address                       (5 bit)
                TEST_VALUE_Sequence     = 6, //! Sequence number                     (3 bit)
                TEST_VALUE_Pad3         = 4, //! Pad3                                (3 bit)
                TEST_VALUE_DAQEnabled   = 1, //! DAQ Enabled Status Bit              (1 bit)
                TEST_VALUE_BufferEmpty  = 1, //! APD Data Buffer Empty               (1 bit)
                TEST_VALUE_BufferFull   = 1, //! APD Data Buffer Full                (1 bit)
                TEST_VALUE_CommError    = 1, //! FEB Communications Error            (1 bit)
                TEST_VALUE_PacketError  = 1, //! FEB Packet Error                    (1 bit)
                TEST_VALUE_OverflowError= 1, //! FEB Data Buff Overflow Error        (1 bit)
                TEST_VALUE_ADCError     = 1, //! FEB ADC Error                       (1 bit)
	    };// end of NanoSliceHeaderMASKS

class NanoSliceHeaderConstructor: public daqdataformats::RawDAQDataConstructor {
public:
    NanoSliceHeaderConstructor();
    ~NanoSliceHeaderConstructor();
    DEFINE_TESTVALUE_BOOL(LinkStatus);
    DEFINE_TESTVALUE_BOOL(DataPresent);
    DEFINE_TESTVALUE_BOOL(DebugMode);
    DEFINE_TESTVALUE     (FEBId);
    DEFINE_TESTVALUE     (FEBStatus);
    DEFINE_TESTVALUE     (PixelAddr);
    DEFINE_TESTVALUE     (Sequence);
    DEFINE_TESTVALUE     (Pad3);
    DEFINE_TESTVALUE_BOOL(DAQEnabled);
    DEFINE_TESTVALUE_BOOL(BufferEmpty);
    DEFINE_TESTVALUE_BOOL(BufferFull);
    DEFINE_TESTVALUE_BOOL(CommError);
    DEFINE_TESTVALUE_BOOL(PacketError);
    DEFINE_TESTVALUE_BOOL(OverflowError);
    DEFINE_TESTVALUE_BOOL(ADCError);
    daqdataformats::RawDAQData* getFilledDataFormat(const daqdataformats::version_t)   const;
    daqdataformats::RawDAQData* getEmptyDataFormat (const daqdataformats::version_t)   const;
    bool                        fillDataFormat     (daqdataformats::RawDAQData*)       const;

    version_t getVersion(const daqdataformats::RawDAQData* inver) const {
        const RawNanoSliceHeader* nano_ver = (const RawNanoSliceHeader*)inver;
        return nano_ver->getVersion();
    }
    
private:

};

}}// end of namespaces


#endif	/* NANOSLICEHEADERCONSTRUCTOR_H */

