/* 
 * File:   NanoSliceHeaderUnitTest.cpp
 * Author: denis
 * 
 * Created on September 24, 2012, 10:18 AM
 */
//#include "DAQDataFormats/../unittest/RawDAQDataUnitTest.h"
//#include "DAQDataFormats/../unittest/NanoSliceHeaderConstructor.h"
#include "RawDAQDataUnitTest.h"
#include "NanoSliceHeaderConstructor.h"

namespace daqdataformats {

class NanoSliceHeaderUnitTest : public RawDAQDataUnitTest {
    CPPUNIT_TEST_SUITE(NanoSliceHeaderUnitTest);
    CPPUNIT_TEST(basicFieldGetAndSetChecks);
    CPPUNIT_TEST(testReadDataIntoSameVersion);
    CPPUNIT_TEST(testReadDataIntoUnknownVersion);
    CPPUNIT_TEST(checkDeepCopyConstructor);
    CPPUNIT_TEST(checkShallowCopyConstructor);
    CPPUNIT_TEST_SUITE_END();
public:
    NanoSliceHeaderUnitTest();
    ~NanoSliceHeaderUnitTest();

    DEFINE_checkDeepCopyConstructor(RawNanoSliceHeader)
    DEFINE_checkShallowCopyConstructor(RawNanoSliceHeader)
    DEFINE_testReadDataIntoSameVersion
    DEFINE_testReadDataIntoUnknownVersion

private:
    void basicFieldGetAndSetChecksForGivenVersion(const version_t);

private:
    nanosliceheader::NanoSliceHeaderConstructor* _mainDFConstructor;
};

//------------------------------------------------------------------------------
NanoSliceHeaderUnitTest::NanoSliceHeaderUnitTest():
     RawDAQDataUnitTest(new nanosliceheader::NanoSliceHeaderConstructor())
    ,_mainDFConstructor((nanosliceheader::NanoSliceHeaderConstructor*)_dataFormatConstructor)
{
}

//------------------------------------------------------------------------------
NanoSliceHeaderUnitTest::~NanoSliceHeaderUnitTest() { }

//------------------------------------------------------------------------------
void NanoSliceHeaderUnitTest::basicFieldGetAndSetChecksForGivenVersion(const version_t version) {

    RawNanoSliceHeader* df = (RawNanoSliceHeader*)_mainDFConstructor->getEmptyDataFormat(version);
    ASSERT_CHECK_FIELD(LinkStatus);
    ASSERT_CHECK_FIELD(DataPresent);
    ASSERT_CHECK_FIELD(DebugMode);
    ASSERT_CHECK_FIELD(FEBId);
    ASSERT_CHECK_FIELD(FEBStatus);
    ASSERT_CHECK_FIELD(PixelAddr);
    ASSERT_CHECK_FIELD(Sequence);
    ASSERT_CHECK_FIELD(Pad3);
    ASSERT_CHECK_FIELD(DAQEnabled);
    ASSERT_CHECK_FIELD(BufferEmpty);
    ASSERT_CHECK_FIELD(BufferFull);
    ASSERT_CHECK_FIELD(CommError);
    ASSERT_CHECK_FIELD(PacketError);
    ASSERT_CHECK_FIELD(OverflowError);
    ASSERT_CHECK_FIELD(ADCError);
    CPPUNIT_ASSERT(version == (version_t) df->getVersion());

    ASSERT_PRINT_IS_NOT_UNKNOWN;

    delete df;
}

//CPPUNIT_TEST_SUITE_REGISTRATION(NanoSliceHeaderUnitTest);
CPPUNIT_TEST_SUITE_NAMED_REGISTRATION(NanoSliceHeaderUnitTest, "DAQDataFormats");
CPPUNIT_REGISTRY_ADD_TO_DEFAULT("DAQDataFormats");

}// end of class
