/* 
 * File:   NanoSliceUnitTest.cpp
 * Author: denis
 * 
 * Created on September 24, 2012, 10:18 AM
 */
#include <DAQDataFormats/RawNanoSlice.h>

//#include "DAQDataFormats/../unittest/RawDAQDataUnitTest.h"
//#include "DAQDataFormats/../unittest/NanoSliceConstructor.h"
#include "RawDAQDataUnitTest.h"
#include "NanoSliceConstructor.h"

namespace daqdataformats {

class NanoSliceUnitTest : public RawDAQDataUnitTest {
    CPPUNIT_TEST_SUITE(NanoSliceUnitTest);
    CPPUNIT_TEST(basicFieldGetAndSetChecks);
    CPPUNIT_TEST(testReadDataIntoSameVersion);
    CPPUNIT_TEST(testReadDataIntoUnknownVersion);
    CPPUNIT_TEST(checkDeepCopyConstructor);
    CPPUNIT_TEST(checkShallowCopyConstructor);
    CPPUNIT_TEST_SUITE_END();
public:
    NanoSliceUnitTest();
    ~NanoSliceUnitTest();

    DEFINE_checkDeepCopyConstructor(RawNanoSlice)
    DEFINE_checkShallowCopyConstructor(RawNanoSlice)
    DEFINE_testReadDataIntoSameVersion
    DEFINE_testReadDataIntoUnknownVersion
            
private:
    void basicFieldGetAndSetChecksForGivenVersion(const version_t);

private:
    rawnano::NanoSliceConstructor* _mainDFConstructor;
};

//------------------------------------------------------------------------------
NanoSliceUnitTest::NanoSliceUnitTest():
     RawDAQDataUnitTest(new rawnano::NanoSliceConstructor())
    ,_mainDFConstructor((rawnano::NanoSliceConstructor*)_dataFormatConstructor)
{
}

//------------------------------------------------------------------------------
NanoSliceUnitTest::~NanoSliceUnitTest() { }

//------------------------------------------------------------------------------
void NanoSliceUnitTest::basicFieldGetAndSetChecksForGivenVersion(const version_t version) {

    RawNanoSlice* df = (RawNanoSlice*)_mainDFConstructor->getEmptyDataFormat(version);

    ASSERT_CHECK_FIELD(TimeStamp);
    ASSERT_CHECK_FIELD(PulseHeight);

    if(version == 1){
        ASSERT_CHECK_FIELD(Value1);
        //ASSERT_GETANDSET_IS_NOT_ALLOWED(Value2);
    }

    if(version == 2){
        //ASSERT_GETANDSET_IS_NOT_ALLOWED(Value1);
        //ASSERT_CHECK_FIELD(Value2);
    }
    
    if(version == 0){
        //ASSERT_GETANDSET_IS_NOT_ALLOWED(Value1);
        //ASSERT_GETANDSET_IS_NOT_ALLOWED(Value2);
    }
    
    CPPUNIT_ASSERT(version == (version_t) df->getHeader()->getVersion());

    CPPUNIT_ASSERT(compareDataFormats(df, df->getHeader()) == FIRST_CONTAINS_SECOND);

    ASSERT_PRINT_IS_NOT_UNKNOWN;


    delete df;
}

//CPPUNIT_TEST_SUITE_REGISTRATION(NanoSliceUnitTest);
CPPUNIT_TEST_SUITE_NAMED_REGISTRATION(NanoSliceUnitTest, "DAQDataFormats");
CPPUNIT_REGISTRY_ADD_TO_DEFAULT("DAQDataFormats");

}// end of class
