/* 
 * File:   RawDAQDataUnitTest.cpp
 * Author: denis
 * 
 * Created on September 20, 2012, 4:43 PM
 */

//#include "DAQDataFormats/../unittest/RawDAQDataUnitTest.h"
//#include "DAQDataFormats/../unittest/RawDAQDataConstructor.h"
#include "RawDAQDataUnitTest.h"
#include "RawDAQDataConstructor.h"

#include <string>

#include <boost/algorithm/string.hpp>

namespace daqdataformats {

//------------------------------------------------------------------------------
RawDAQDataUnitTest::RawDAQDataUnitTest(RawDAQDataConstructor* df):
     _dataFormatConstructor(df)
{
}

//------------------------------------------------------------------------------
RawDAQDataUnitTest::~RawDAQDataUnitTest() {
    if(_dataFormatConstructor)delete _dataFormatConstructor;
}

//------------------------------------------------------------------------------
// Check readData for all versions of the dataformat
bool RawDAQDataUnitTest::checkReadDataIntoSameVersion() const {
    VERSION_LOOP_iver(
        LOG_DEBUG("UNITTEST")<<"checkReadDataIntoSameVersion for version "<<iver<<std::endl;
        if(!checkReadDataIntoSameVersion(iver)) return false;
    )
    return true;
}
//------------------------------------------------------------------------------
// Check readData for all versions of the dataformat
bool RawDAQDataUnitTest::checkReadDataIntoUnknownVersion() const{

    VERSION_LOOP_iver(
            LOG_DEBUG("UNITTEST")<<"checkReadDataIntoUnknownVersion for version "<<iver<<std::endl;
            if(!checkReadDataIntoUnknownVersion(iver))
                            return false;
    )

    return true;
}

//------------------------------------------------------------------------------
// Check readData of a given version
bool RawDAQDataUnitTest::checkReadDataIntoSameVersion(const version_t ver)const{
    return checkReadData(ver, ver);
}

//------------------------------------------------------------------------------
// Check readData of a given version
bool RawDAQDataUnitTest::checkReadDataIntoUnknownVersion(const version_t ver) const {
    return checkReadData(ver, rawdaqdataformat::DAQDATAFORMAT_UNKNOWN_VERSION);
}

//------------------------------------------------------------------------------
bool RawDAQDataUnitTest::checkReadData(const version_t initialformat_version,
                                       const version_t readingformat_version) const {

    LOG_DEBUG("UNITTEST")<<__PRETTY_FUNCTION__<<"\n"
                         <<"Versions :"
                         <<" initialformat_version = "<<initialformat_version
                         <<". readingformat_version = "<<readingformat_version<<"\n";

    /// Check Versions. they have to be valid:
    /// 1. (initialformat_version == readingformat_version) AND (initialformat_version != UnknownVersion)
    /// 2. (initialformat_version != UnknownVersion) AND (readingformat_version == UnknownVersion)
    {
        bool are_versions_valid = false;
        if( (initialformat_version == readingformat_version                          ) && (initialformat_version != rawdaqdataformat::DAQDATAFORMAT_UNKNOWN_VERSION)) are_versions_valid = true;
        if( (initialformat_version != rawdaqdataformat::DAQDATAFORMAT_UNKNOWN_VERSION) && (readingformat_version == rawdaqdataformat::DAQDATAFORMAT_UNKNOWN_VERSION)) are_versions_valid = true;

        /// If versions are not valid, exit now
        if(! are_versions_valid){
            LOG_ERROR("UNITTEST")<<"Versions are not valid:"
                                 <<" initialformat_version = "<<initialformat_version
                                 <<". readingformat_version = "<<readingformat_version;
            return false;
        }
    }// end of checking versions

    /// Construct DataFormats
    RawDAQData* format_nonzero_raw = NULL;
    try {
     format_nonzero_raw = _dataFormatConstructor->getFilledDataFormat(initialformat_version);
    }
    catch(...){
        LOG_ERROR("UNITTEST")<<"Could not get a FILLED non-zero Data format of version "<<initialformat_version<<"\n";
        throw;
        return false;
    }
    boost::shared_ptr<RawDAQData> format_nonzero(format_nonzero_raw);


    RawDAQData* format_zero_raw = NULL;
    try{
        format_zero_raw = _dataFormatConstructor->getEmptyDataFormat(readingformat_version);
    }
    catch(...){
        LOG_ERROR("UNITTEST")<<"Could not get an EMPTY zero Data format of version "<<readingformat_version<<"\n";
        throw;
        return false;
    }
    boost::shared_ptr<RawDAQData> format_zero(format_zero_raw);

    //LOG_DEBUG("UNITTEST")<<"Version"<<initialformat_version<<"\n";
    //LOG_DEBUG("UNITTEST")<<*(format_nonzero.get());

    {
    /// Check Printout version for format_nonzero
    if(   initialformat_version != rawdaqdataformat::DAQDATAFORMAT_UNKNOWN_VERSION
       && isPrintDataFormatUnknownVersion(format_nonzero.get())){
            LOG_ERROR("UNITTEST")<<"Failure in "<<__PRETTY_FUNCTION__<<" at "<<__LINE__<<"\n"
                                 <<"Nonzero format should have print with defined version, but it returns:\n"
                                 <<*(format_nonzero.get());
            return false;
    }

    /// Check Printout version for format_zero
    if(   readingformat_version != rawdaqdataformat::DAQDATAFORMAT_UNKNOWN_VERSION
       && isPrintDataFormatUnknownVersion(format_zero.get())){
            LOG_ERROR("UNITTEST")<<"Failure in "<<__PRETTY_FUNCTION__<<" at "<<__LINE__<<"\n"
                                 <<"Zero format should have print with defined version, but it returns:\n"
                                 <<*(format_zero.get());
            return false;
    }

    /// Check Printout version for format_nonzero
    if(   initialformat_version == rawdaqdataformat::DAQDATAFORMAT_UNKNOWN_VERSION
       && !isPrintDataFormatUnknownVersion(format_nonzero.get())){
            LOG_ERROR("UNITTEST")<<"Failure in "<<__PRETTY_FUNCTION__<<" at "<<__LINE__<<"\n"
                                 <<"Nonzero format should have print UNKNOWN format, but it returns:\n"
                                 <<*(format_nonzero.get());
            return false;
    }

    /// Check Printout version for format_zero
    if(   initialformat_version == rawdaqdataformat::DAQDATAFORMAT_UNKNOWN_VERSION
       && !isPrintDataFormatUnknownVersion(format_zero.get())){
            LOG_ERROR("UNITTEST")<<"Failure in "<<__PRETTY_FUNCTION__<<" at "<<__LINE__<<"\n"
                                 <<"Zero format should have print UNKNOWN format, but it returns:\n"
                                 <<*(format_zero.get());
            return false;
    }
    }// end of checking 
    /// Check if DataFormats are the same
    /// At this point they should be different
    try{
        if(compareDataFormats(format_nonzero.get(), format_zero.get()) != DATAFORMATS_ARE_DIFFERENT){
            LOG_ERROR("UNITTEST")<<"Failure in "<<__PRETTY_FUNCTION__<<" at "<<__LINE__<<". "
                                 <<"Formats should be Different.\n"
                                 <<"Nonzero format:\n"
                                 <<*(format_nonzero.get())
                                 <<"Copied format:\n"
                                 <<*(format_zero.get());
            return false;
        }// end of checking if dataformats are different
    }
    catch(...){
        LOG_ERROR("UNITTEST")<<"From "<<__PRETTY_FUNCTION__<<std::endl
                             <<"Could not compare versions "
                             <<initialformat_version<<" and "<<readingformat_version<<std::endl;
    }

    /// Temporary buffer where we are going to put content of the format_nonzero dataformat
    std::vector<uint32_t> buffer_tmp;

    /// Need to resize buffer to the maxumum size
    resizeBufferToSizeOfDataFormats(buffer_tmp, format_nonzero.get(), format_nonzero.get());

    /// Now, this is the pointer to the first element of the Temporary buffer
    const uint32_t* pbuffer = (uint32_t*)(&buffer_tmp[0]);

    /// Write the first dataformat (format_nonzero) into a buffer
    try{
        format_nonzero->writeData(pbuffer);
    }
    catch(...){
        LOG_ERROR("UNITTEST")<<"From "<<__PRETTY_FUNCTION__<<std::endl
                             <<"Could not write "<<initialformat_version<<std::endl;
    }

    /// Read the buffer to the second (empty) dataformat
    try{
        format_zero->readData(pbuffer);
    }
    catch(...){
        LOG_ERROR("UNITTEST")<<"From "<<__PRETTY_FUNCTION__
                             <<". Could not read "<<initialformat_version<<std::endl;
    }

    /// Check if the two dataformats are now the same
    /// At this point they should be the same
    try{
        if(compareDataFormats(format_nonzero.get(), format_zero.get()) != DATAFORMATS_ARE_THE_SAME){
            LOG_ERROR("UNITTEST")<<"Failure in "<<__PRETTY_FUNCTION__<<" at "<<__LINE__<<". "
                                 <<"Formats should be the same. But they are "
                                 <<boost::lexical_cast<std::string>(compareDataFormats(format_nonzero.get(), format_zero.get()))
                                 <<"\nNonzero format:\n"
                                 <<*(format_nonzero.get())
                                 <<"Copied format:\n"
                                 <<*(format_zero.get())
                                 <<"\n Difference:\n"
                                 <<getDifferenceBetweenStrings(format_nonzero.get(), format_zero.get());
                            return false;                                                           
        }// end of checking if the dataformats are the same
    }
    catch(...){
        LOG_ERROR("UNITTEST")<<"From "<<__PRETTY_FUNCTION__<<std::endl
                             <<"Could not compare versions after reading"
                             <<initialformat_version<<" and "<<readingformat_version<<"\n";
    }

    /// All tests succeded
    return true;
}

//------------------------------------------------------------------------------
DATAFORMAT_COMPARE_RESULTS RawDAQDataUnitTest::compareDataFormats(const RawDAQData* format1, const RawDAQData* format2) const
{
    /// Print the first input dataformat into a string
    std::string string1;
    format1->print(string1);

    /// Print the first input dataformat into a string
    std::string string2;
    format2->print(string2);

    /// Compare the two strings and see if they are the same
    if(string1.compare(string2) == 0) return DATAFORMATS_ARE_THE_SAME;

    /// See if first string contains second
    if(string1.find(string2) != std::string::npos) return FIRST_CONTAINS_SECOND;

    /// See if second string contains first
    if(string2.find(string1) != std::string::npos) return SECOND_CONTAINS_FIRST;

    return DATAFORMATS_ARE_DIFFERENT;    
}

//------------------------------------------------------------------------------
bool RawDAQDataUnitTest::compareDataFormats(const RawDAQData*                format1
                                           ,const RawDAQData*                format2
                                           ,const DATAFORMAT_COMPARE_RESULTS what_it_should_be
                                           ) const
{

    /// Print the first input dataformat into a string
    std::string string1;
    format1->print(string1);

    /// Print the first input dataformat into a string
    std::string string2;
    format2->print(string2);


    DATAFORMAT_COMPARE_RESULTS result = DATAFORMATS_ARE_DIFFERENT;

    /// Compare the two strings and see if they are the same
    if     (string1.compare(string2) == 0)              result = DATAFORMATS_ARE_THE_SAME;
    else if(string1.find(string2) != std::string::npos) result = FIRST_CONTAINS_SECOND;
    else if(string2.find(string1) != std::string::npos) result = SECOND_CONTAINS_FIRST;


    if(result != what_it_should_be){
            LOG_ERROR("UNITTEST")<<"Failure in "<<__PRETTY_FUNCTION__<<" at "<<__LINE__<<". "
                                 <<"Formats should be "<<daqdataformats::getNameForResults(what_it_should_be)
                                 <<". But they are "   <<daqdataformats::getNameForResults(result)
                                 <<"\nFirst format:"<<string1
                                 <<"\nSecond format:"<<string2
                                 <<"\n Difference:\n"
                                 <<getDifferenceBetweenStrings(string1, string2);
                            return false;
        }// end of checking if the dataformats are the same

    return(result == what_it_should_be);
}




//------------------------------------------------------------------------------
bool RawDAQDataUnitTest::isPrintDataFormatUnknownVersion(const RawDAQData* df) const{

    /// Print the first input dataformat into a string
    std::string string1;
    df->print(string1);

    /// See if first string contains second
    if(string1.find("DAQDataFormat is of unknown version") != std::string::npos) return true;

    return false;
}

//------------------------------------------------------------------------------
bool RawDAQDataUnitTest::resizeBufferToSizeOfDataFormats(std::vector<uint32_t>& buf, const RawDAQData* f1) const{

    if(buf.size() < f1->sizeofdata())
        buf.resize(f1->sizeofdata());

    return true;
}

bool RawDAQDataUnitTest::resizeBufferToSizeOfDataFormats(std::vector<uint32_t>& buf, const RawDAQData* f1, const RawDAQData* f2) const{
    resizeBufferToSizeOfDataFormats(buf, f1);
    return resizeBufferToSizeOfDataFormats(buf, f2);
}

//------------------------------------------------------------------------------
std::string RawDAQDataUnitTest::getDifferenceBetweenStrings(const RawDAQData* df1, const RawDAQData* df2) const {
    std::string str1, str2;
    df1->print(str1);
    df2->print(str2);
    return getDifferenceBetweenStrings(str1, str2);
}

//------------------------------------------------------------------------------
std::string RawDAQDataUnitTest::getDifferenceBetweenStrings(const std::string str1, const std::string str2) const{

    std::istringstream string_stream_str1(str1);
    std::istringstream string_stream_str2(str2);

    std::vector<std::string> lines_str1;
    std::vector<std::string> lines_str2;

    std::string line_tmp;
    while(std::getline(string_stream_str1, line_tmp)){
        lines_str1.push_back(line_tmp);
    }
    while(std::getline(string_stream_str2, line_tmp)){
        lines_str2.push_back(line_tmp);
    }


    const uint32_t min_lines = std::min(lines_str1.size(), lines_str2.size());

    std::string diff;

    for(uint32_t iline = 0; iline < min_lines; ++iline){
        if(lines_str1[iline].compare(lines_str1[iline]) != 0){
            diff += "line " + boost::lexical_cast<std::string>(iline) + ": ";
            diff += lines_str1[iline] +"VS" + lines_str2[iline] + "\n";
        }
    }

    diff += "ENDOFDIFF";

    return diff;
}

//------------------------------------------------------------------------------
std::string getNameForResults(const DATAFORMAT_COMPARE_RESULTS res)
{
    if     (res == DATAFORMATS_ARE_DIFFERENT) return "\"Data Formats are Different\"";
    else if(res == DATAFORMATS_ARE_THE_SAME)  return "\"Data Formats are Same\"";
    else if(res == FIRST_CONTAINS_SECOND)     return "\"First Format Contains Second\"";
    else if(res == SECOND_CONTAINS_FIRST)     return "\"Second Format Contains First\"";
    return "\"What the heck\"";
}


}// end of namespace
