#include <cppunit/extensions/HelperMacros.h>
#include <DAQDataFormats/RawMilliSlice.h>

using namespace daqdataformats;

class RawMilliSliceTests : public CppUnit::TestFixture {

    CPPUNIT_TEST_SUITE(RawMilliSliceTests);
    CPPUNIT_TEST(testCalculateCheckSum);
    CPPUNIT_TEST(testCalculateCheckSumCorrupt);
    CPPUNIT_TEST_SUITE_END();

public:
    void setUp();
    void tearDown();

    void testCalculateCheckSum();
    void testCalculateCheckSumCorrupt();


private:

};

void RawMilliSliceTests::setUp() {
    // flush stdout to get progress markers to display in realtime
    std::cout.flush();
}

void RawMilliSliceTests::tearDown() {
    // flush stdout to get progress markers to display in realtime
    std::cout.flush();
}

void RawMilliSliceTests::testCalculateCheckSum() {

    RawMilliSlice rawMilliSlice;
    rawMilliSlice.setDetId(1);
    rawMilliSlice.setDCMId(2);
    rawMilliSlice.setDiblockId(3);
    rawMilliSlice.setPartition(4);
    rawMilliSlice.setDestBuffID(5);
    rawMilliSlice.close(true);

    CPPUNIT_ASSERT(rawMilliSlice.calculateCheckSum(0) == 0);

}

void RawMilliSliceTests::testCalculateCheckSumCorrupt() {

    RawMilliSlice rawMilliSlice;
    rawMilliSlice.setDetId(1);
    rawMilliSlice.setDCMId(2);
    rawMilliSlice.setDiblockId(3);
    rawMilliSlice.setPartition(4);
    rawMilliSlice.setDestBuffID(5);
    rawMilliSlice.close();

    uint32_t* buffer_data = (uint32_t*)(rawMilliSlice.getBuffer());
    buffer_data[3] = 0xdeadbeef;

    CPPUNIT_ASSERT(rawMilliSlice.calculateCheckSum(0) != 0);

}

CPPUNIT_TEST_SUITE_NAMED_REGISTRATION(RawMilliSliceTests, "DAQDataFormats");
CPPUNIT_REGISTRY_ADD_TO_DEFAULT("DAQDataFormats");
