/* 
 * File:   RunHeaderUnitTest.cpp
 * Author: denis
 * 
 * Created on September 24, 2012, 10:18 AM
 */
//#include "DAQDataFormats/../unittest/RawDAQDataUnitTest.h"
//#include "DAQDataFormats/../unittest/RunHeaderConstructor.h"
#include "RawDAQDataUnitTest.h"
#include "RunHeaderConstructor.h"

namespace daqdataformats {

class RunHeaderUnitTest : public RawDAQDataUnitTest {
    CPPUNIT_TEST_SUITE(RunHeaderUnitTest);
    CPPUNIT_TEST(basicFieldGetAndSetChecks);
    CPPUNIT_TEST(testReadDataIntoSameVersion);
    CPPUNIT_TEST(testReadDataIntoUnknownVersion);
    CPPUNIT_TEST(checkDeepCopyConstructor);
    CPPUNIT_TEST(checkShallowCopyConstructor);
    CPPUNIT_TEST_SUITE_END();
public:
    RunHeaderUnitTest();
    ~RunHeaderUnitTest();

    DEFINE_checkDeepCopyConstructor(RawRunHeader)
    DEFINE_checkShallowCopyConstructor(RawRunHeader)
    DEFINE_testReadDataIntoSameVersion
    DEFINE_testReadDataIntoUnknownVersion

private:
    void basicFieldGetAndSetChecksForGivenVersion(const version_t);

private:
    rawrunhdr::RunHeaderConstructor* _mainDFConstructor;
};

//------------------------------------------------------------------------------
RunHeaderUnitTest::RunHeaderUnitTest():
     RawDAQDataUnitTest(new rawrunhdr::RunHeaderConstructor())
    ,_mainDFConstructor((rawrunhdr::RunHeaderConstructor*)_dataFormatConstructor)
{
}

//------------------------------------------------------------------------------
RunHeaderUnitTest::~RunHeaderUnitTest() { }

//------------------------------------------------------------------------------
void RunHeaderUnitTest::basicFieldGetAndSetChecksForGivenVersion(const version_t version) {

    RawRunHeader* df = (RawRunHeader*)_mainDFConstructor->getEmptyDataFormat(version);

    ASSERT_CHECK_FIELD(UniversalEventStart);
    ASSERT_CHECK_FIELD(UniversalEventEnd);
    ASSERT_CHECK_FIELD(CRC);
    ASSERT_CHECK_FIELD(TriggerPrescaleListIDX);
    ASSERT_CHECK_FIELD(TriggerListIDX);
    ASSERT_CHECK_FIELD(ValidTriggerTypesHigh2);
    ASSERT_CHECK_FIELD(ValidTriggerTypesHigh);
    ASSERT_CHECK_FIELD(ValidTriggerTypesLow);
    ASSERT_CHECK_FIELD(VTMod);
    ASSERT_CHECK_FIELD(TriggerMC);
    ASSERT_CHECK_FIELD(TriggerCtrlID);
    ASSERT_CHECK_FIELD(TriggerVersion);
    ASSERT_CHECK_FIELD(RunSize);
    ASSERT_CHECK_FIELD(UniversalEventEndHi);
    ASSERT_CHECK_FIELD(UniversalEventEndLo);
    ASSERT_CHECK_FIELD(UniversalEventStartHi);
    ASSERT_CHECK_FIELD(UniversalEventStartLo);
    ASSERT_CHECK_FIELD(TotalEvents);
    ASSERT_CHECK_FIELD(RunEndTime);
    ASSERT_CHECK_FIELD(SubRunEndTime);
    ASSERT_CHECK_FIELD(SubRunStartTime);
    ASSERT_CHECK_FIELD(RunStartTime);
    ASSERT_CHECK_FIELD(DataLoggerID);
    ASSERT_CHECK_FIELD(DataLoggerVersion);
    ASSERT_CHECK_FIELD(RunControlID);
    ASSERT_CHECK_FIELD(RunControlVersion);
    ASSERT_CHECK_FIELD(ConfigurationBlockSet);
    ASSERT_CHECK_FIELD(Subrun);
    ASSERT_CHECK_FIELD(Partition);
    ASSERT_CHECK_FIELD(ConfigIDX);
    ASSERT_CHECK_FIELD(RunType);
    ASSERT_CHECK_FIELD(RunNumber);

    CPPUNIT_ASSERT(version == (version_t) df->getVersion());

    ASSERT_PRINT_IS_NOT_UNKNOWN;

    delete df;
}

//CPPUNIT_TEST_SUITE_REGISTRATION(RunHeaderUnitTest);
CPPUNIT_TEST_SUITE_NAMED_REGISTRATION(RunHeaderUnitTest, "DAQDataFormats");
CPPUNIT_REGISTRY_ADD_TO_DEFAULT("DAQDataFormats");

}// end of class
