/* 
 * File:   TriggerConstructor.cpp
 * Author: denis
 * 
 * Created on September 20, 2012, 2:18 PM
 */

#include <DAQDataFormats/RawTrigger.h>

#include "DAQDataFormats/../unittest/TriggerConstructor.h"

/// It is requred to
///   1. include DAQDataFormats/DataFormatFirst.inc
///   2. define CURRENT_CLASS
///   3.    define CURRENT_CLASS_VERSION for version of the class
///      OR define LAST_CLASS_VERSION    for the main structure of the class
///   4. (optional) define CURRENT_FIELDS_SUB_NAMESPACE
///   5. include DAQDataFormats/DataFormatLast.inc
/// before the declaration of the DataFormat version class in the header.
#include "DAQDataFormats/DataFormatFirst.inc"
#define  CURRENT_CLASS                RawTrigger
#define  LAST_CLASS_VERSION           2
#define  CURRENT_FIELDS_SUB_NAMESPACE rawtrigger
#include "DAQDataFormats/DataFormatLast.inc"

namespace daqdataformats {

TriggerConstructor::TriggerConstructor():
    daqdataformats::RawDAQDataConstructor(daqdataformats::triggerheader::RAWTRIGGERHEADER_LATEST_VERSION)
{
}
TriggerConstructor::~TriggerConstructor() { }

//------------------------------------------------------------------------------
daqdataformats::RawDAQData* TriggerConstructor::getFilledDataFormat(const daqdataformats::version_t ver) const {
    RawTrigger* out = (RawTrigger*)getEmptyDataFormat(ver);
    fillDataFormat(out);
    return out;
}

//------------------------------------------------------------------------------
daqdataformats::RawDAQData* TriggerConstructor::getEmptyDataFormat(const daqdataformats::version_t ver) const {
    return new daqdataformats::RawTrigger(ver);
}

//------------------------------------------------------------------------------
bool TriggerConstructor::fillDataFormat (daqdataformats::RawDAQData* dataformat) const {
    daqdataformats::RawTrigger* trigger = (daqdataformats::RawTrigger*)dataformat;

    _TriggerHeaderConstructor.fillDataFormat(trigger->getTriggerHeader());
    _TriggerMaskConstructor.fillDataFormat(trigger->getTriggerMask());
    _TriggerRangeConstructor.fillDataFormat(trigger->getTriggerRange());
    _TriggerTimeConstructor.fillDataFormat(trigger->getTriggerTime());
    _TriggerTimingMarkerConstructor.fillDataFormat(trigger->getTriggerTimingMarker());
    return true;
}

//------------------------------------------------------------------------------
// Gets the Trigger version
version_t TriggerConstructor::getTriggerVersion(const RawTrigger* nano) const{
    return (version_t) nano->getTriggerHeader()->getVersion();
}

}// end of namespaces
