/* 
 * File:   TriggerHeaderUnitTest.cpp
 * Author: denis
 * 
 * Created on September 24, 2012, 10:18 AM
 */
//#include "DAQDataFormats/../unittest/RawDAQDataUnitTest.h"
//#include "DAQDataFormats/../unittest/TriggerHeaderConstructor.h"
#include "RawDAQDataUnitTest.h"
#include "TriggerHeaderConstructor.h"

namespace daqdataformats {

class TriggerHeaderUnitTest : public RawDAQDataUnitTest {
    CPPUNIT_TEST_SUITE(TriggerHeaderUnitTest);
    CPPUNIT_TEST(basicFieldGetAndSetChecks);
    CPPUNIT_TEST(testReadDataIntoSameVersion);
    CPPUNIT_TEST(testReadDataIntoUnknownVersion);
    CPPUNIT_TEST(checkDeepCopyConstructor);
    CPPUNIT_TEST(checkShallowCopyConstructor);
    CPPUNIT_TEST_SUITE_END();
public:
    TriggerHeaderUnitTest();
    ~TriggerHeaderUnitTest();

    DEFINE_checkDeepCopyConstructor(RawTriggerHeader)
    DEFINE_checkShallowCopyConstructor(RawTriggerHeader)
    DEFINE_testReadDataIntoSameVersion
    DEFINE_testReadDataIntoUnknownVersion

private:
    void basicFieldGetAndSetChecksForGivenVersion(const version_t);

private:
    TriggerHeaderConstructor* _mainDFConstructor;
};

//------------------------------------------------------------------------------
TriggerHeaderUnitTest::TriggerHeaderUnitTest():
     RawDAQDataUnitTest(new TriggerHeaderConstructor())
    ,_mainDFConstructor((TriggerHeaderConstructor*)_dataFormatConstructor)
{
}

//------------------------------------------------------------------------------
TriggerHeaderUnitTest::~TriggerHeaderUnitTest() { }

//------------------------------------------------------------------------------
void TriggerHeaderUnitTest::basicFieldGetAndSetChecksForGivenVersion(const version_t version) {

    RawTriggerHeader* df = (RawTriggerHeader*)_mainDFConstructor->getEmptyDataFormat(version);
    ASSERT_CHECK_FIELD(HWFlag);
    ASSERT_CHECK_FIELD(Source);
    ASSERT_CHECK_FIELD(SourceSubID);
    ASSERT_CHECK_FIELD(SourceID);
    ASSERT_CHECK_FIELD(TriggerNumberLow);
    ASSERT_CHECK_FIELD(TriggerNumberHigh);
    ASSERT_CHECK_FIELD(MasterTriggerNumberLow);
    ASSERT_CHECK_FIELD(MasterTriggerNumberHigh);
    CPPUNIT_ASSERT(version == (version_t) df->getVersion());

    ASSERT_PRINT_IS_NOT_UNKNOWN;

    delete df;
}

//CPPUNIT_TEST_SUITE_REGISTRATION(TriggerHeaderUnitTest);
CPPUNIT_TEST_SUITE_NAMED_REGISTRATION(TriggerHeaderUnitTest, "DAQDataFormats");
CPPUNIT_REGISTRY_ADD_TO_DEFAULT("DAQDataFormats");

}// end of class
