/* 
 * File:   TriggerTimeUnitTest.cpp
 * Author: denis
 * 
 * Created on September 24, 2012, 10:18 AM
 */
//#include "DAQDataFormats/../unittest/RawDAQDataUnitTest.h"
//#include "DAQDataFormats/../unittest/TriggerTimeConstructor.h"
#include "RawDAQDataUnitTest.h"
#include "TriggerTimeConstructor.h"

namespace daqdataformats {

class TriggerTimeUnitTest : public RawDAQDataUnitTest {
    CPPUNIT_TEST_SUITE(TriggerTimeUnitTest);
    CPPUNIT_TEST(basicFieldGetAndSetChecks);
    CPPUNIT_TEST(testReadDataIntoSameVersion);
    CPPUNIT_TEST(testReadDataIntoUnknownVersion);
    CPPUNIT_TEST(checkDeepCopyConstructor);
    CPPUNIT_TEST(checkShallowCopyConstructor);
    CPPUNIT_TEST_SUITE_END();
public:
    TriggerTimeUnitTest();
    ~TriggerTimeUnitTest();

    DEFINE_checkDeepCopyConstructor(RawTriggerTime)
    DEFINE_checkShallowCopyConstructor(RawTriggerTime)
    DEFINE_testReadDataIntoSameVersion
    DEFINE_testReadDataIntoUnknownVersion

private:
    void basicFieldGetAndSetChecksForGivenVersion(const version_t);

private:
    TriggerTimeConstructor* _mainDFConstructor;
};

//------------------------------------------------------------------------------
TriggerTimeUnitTest::TriggerTimeUnitTest():
     RawDAQDataUnitTest(new TriggerTimeConstructor())
    ,_mainDFConstructor((TriggerTimeConstructor*)_dataFormatConstructor)
{
}

//------------------------------------------------------------------------------
TriggerTimeUnitTest::~TriggerTimeUnitTest() { }

//------------------------------------------------------------------------------
void TriggerTimeUnitTest::basicFieldGetAndSetChecksForGivenVersion(const version_t version) {

    RawTriggerTime* df = (RawTriggerTime*)_mainDFConstructor->getEmptyDataFormat(version);

    ASSERT_CHECK_FIELD     (TriggerTimeStampLow);
    ASSERT_CHECK_FIELD     (TriggerTimeStampHigh);
    ASSERT_CHECK_FIELD     (TriggerTimeStamp_sec);
    ASSERT_CHECK_FIELD     (TriggerTimeStamp_usec);
    ASSERT_CHECK_FIELD     (ClockSource);
    ASSERT_CHECK_FIELD     (TriggerTimeStamp);
    ASSERT_CHECK_FIELD     (TriggerTimeStampTvUsec);

    CPPUNIT_ASSERT(version == (version_t) df->getVersion());

    ASSERT_PRINT_IS_NOT_UNKNOWN;

    delete df;
}

//CPPUNIT_TEST_SUITE_REGISTRATION(TriggerTimeUnitTest);
CPPUNIT_TEST_SUITE_NAMED_REGISTRATION(TriggerTimeUnitTest, "DAQDataFormats");
CPPUNIT_REGISTRY_ADD_TO_DEFAULT("DAQDataFormats");

}// end of class
