/* 
 * File:   TriggerUnitTest.cpp
 * Author: denis
 * 
 * Created on September 24, 2012, 10:18 AM
 */
#include <DAQDataFormats/RawTrigger.h>

//#include "DAQDataFormats/../unittest/RawDAQDataUnitTest.h"
//#include "DAQDataFormats/../unittest/TriggerConstructor.h"
#include "RawDAQDataUnitTest.h"
#include "TriggerConstructor.h"

namespace daqdataformats {

class TriggerUnitTest : public RawDAQDataUnitTest {
    CPPUNIT_TEST_SUITE(TriggerUnitTest);
    CPPUNIT_TEST(basicFieldGetAndSetChecks);
    CPPUNIT_TEST(testReadDataIntoSameVersion);
    CPPUNIT_TEST(testReadDataIntoUnknownVersion);
    CPPUNIT_TEST(checkDeepCopyConstructor);
    CPPUNIT_TEST(checkShallowCopyConstructor);
    CPPUNIT_TEST_SUITE_END();
public:
    TriggerUnitTest();
    ~TriggerUnitTest();

    DEFINE_checkDeepCopyConstructor(RawTrigger)
    DEFINE_checkShallowCopyConstructor(RawTrigger)
    DEFINE_testReadDataIntoSameVersion
    DEFINE_testReadDataIntoUnknownVersion
            
private:
    void basicFieldGetAndSetChecksForGivenVersion(const version_t);

private:
    TriggerConstructor* _mainDFConstructor;
};

//------------------------------------------------------------------------------
TriggerUnitTest::TriggerUnitTest():
     RawDAQDataUnitTest(new TriggerConstructor())
    ,_mainDFConstructor((TriggerConstructor*)_dataFormatConstructor)
{
}

//------------------------------------------------------------------------------
TriggerUnitTest::~TriggerUnitTest() { }

//------------------------------------------------------------------------------
void TriggerUnitTest::basicFieldGetAndSetChecksForGivenVersion(const version_t version) {

    RawTrigger* df = (RawTrigger*)_mainDFConstructor->getEmptyDataFormat(version);

    CPPUNIT_ASSERT(version == (version_t) df->getTriggerHeader()->getVersion());

    CPPUNIT_ASSERT(compareDataFormats(df, df->getTriggerHeader()) == FIRST_CONTAINS_SECOND);
    CPPUNIT_ASSERT(compareDataFormats(df, df->getTriggerRange()) == FIRST_CONTAINS_SECOND);
    CPPUNIT_ASSERT(compareDataFormats(df, df->getTriggerMask()) == FIRST_CONTAINS_SECOND);
    CPPUNIT_ASSERT(compareDataFormats(df, df->getTriggerTime()) == FIRST_CONTAINS_SECOND);
    CPPUNIT_ASSERT(compareDataFormats(df, df->getTriggerTimingMarker()) == FIRST_CONTAINS_SECOND);

    ASSERT_PRINT_IS_NOT_UNKNOWN;


    delete df;
}

//CPPUNIT_TEST_SUITE_REGISTRATION(TriggerUnitTest);
CPPUNIT_TEST_SUITE_NAMED_REGISTRATION(TriggerUnitTest, "DAQDataFormats");
CPPUNIT_REGISTRY_ADD_TO_DEFAULT("DAQDataFormats");

}// end of class
