#include <vector>
#include <stdio.h>
#include <stdlib.h>
#include <iostream>
#include <fcntl.h>
#include <sys/stat.h>
#include "DAQDataFormats/RawRun.h"
#include "DAQDataFormats/RawEvent.h"
#include "DAQQualityCheck/QualityCheck.h"

extern "C"{
#include <getopt.h>
}

#include <fcntl.h>                        // O_RDONLY
#include <getopt.h>		// getopt
#include "boost/regex.hpp"
#include <poll.h>		// poll
#include <sys/stat.h>		// fstat, struct stat
#include <netinet/in.h>		// struct sockaddr_in
#include <netdb.h>		// gethostbyname, struct hostent

#define RUNQUALITYCHECK 1
#define MAXIMUM_BUFFER_SIZE 1000000

using namespace daqqc;

#define SIMUSAGE "\
  Usage: %s [options]\n\
Example: %s --f=1 --i=run.raw\n\
Options:\n\
 --f=<format_id>         DAQDataFormats: 1=Run 2=Events\n\
 --i <filename>          Input file name\n\
 ", basename(argv[0]), basename(argv[0])

struct Options {
    int         format;
    char*       filename;
};

Options gOpts = {format:0, filename:"uuu" };

int
get_int_or_die( char *argv[], char const *optarg, char const * msg )
{   if (!isdigit(optarg[0]))
    {   printf( "%s\n", msg );
	printf( SIMUSAGE );  exit( 0 );
    }
    return strtoul(optarg,NULL,0);
}
int
parse_args( int argc, char *argv[] )
{   int	c;
    boost::cmatch mm;

    for(int inum = 0; inum< 2; inum++){
        int                  option_index=0;
	static struct option long_options[]=
	    {   //name,          has_arg, flag,val
		{"help",  	  0,       0,  'h' },
		{"f",   	  1,       0,   -1 },
		{"i",          1,       0,   -2 },
		{0, 0, 0, 0}
	    };
	c = getopt_long( argc, argv, "?hvp:", long_options, &option_index );
	switch (c)
	{
	case -1:
            gOpts.format = get_int_or_die(argv,optarg,"invalid id" );
	    break;
	case -2:
            gOpts.filename = optarg;
            break;
	case '?': case 'h':
            {
            printf( SIMUSAGE );  exit( 0 );
            break;
            }
	default:
	    printf("?? getopt returned character code 0%o ??\n", c);
	}// end of switch

    }

    return 1;
}   // parse_args


void intro();


//! A short program to test QualityCheck
int main(int argc, char *argv[]){
    intro();
    
    if(argc <  3){
        std::cout<<"Should be 2 arguments\n";
	printf( SIMUSAGE );  exit( 0 );
    }

    parse_args( argc, argv );   // optind defined in <getopt.h>

    if(gOpts.format != 1 && gOpts.format !=2){
        std::cout<<"Wrong DAQDataFormat\n";
	printf( SIMUSAGE );  exit( 0 );
    }

    QualityCheck qc;

    unsigned int  fileformat_version_number[1];
    unsigned int* pfileformat_version_number = (unsigned int*)fileformat_version_number;

    struct stat file_status;
    stat(gOpts.filename, &file_status);

    int infile = open(gOpts.filename, O_RDONLY);
    if (!infile)
      printf("File not open!");
    read(infile, pfileformat_version_number,4);

    if(gOpts.format == 2){

      daqdataformats::RawEvent event;

      event.readData(pfileformat_version_number);
      uint32_t* currentpointer = (uint32_t*) event.getBuffer();;
      
      if(event.getHeaderMarkerLeft() != daqdataformats::raweventhdr::Marker_value[0])
	read(infile, currentpointer, 4);
      currentpointer++;
      // read the size
      read(infile, currentpointer, 4);
      currentpointer++;
      
      // read the rest of the event
      read(infile, currentpointer, 4 * (event.sizeofdata() - 2));

      event.print();
      qc.checkEvent(event);
      return 1;
    }

    daqdataformats::RawRunHeader run_header;
    run_header.readData(pfileformat_version_number);
    uint32_t* currentpointer = (uint32_t*) run_header.getBuffer();
    
    if(run_header.getMarkerHi() != daqdataformats::rawrunhdr::Marker_value[0])
      read(infile, currentpointer, 4);
    currentpointer++;
    // read the size
    read(infile, currentpointer, 4*15);
    currentpointer += 15;
    
    // read the rest of the event
    read(infile, currentpointer, 4 * (run_header.getRunSize() - 16));
    
    run_header.print();
    
#ifndef RAWRUN_IS_NOT_IMPLEMENTED    
    qc.checkRun(run);
#endif
    close(infile);
    return 1;
} // END MAIN


//---------------------------intro----------------------------------------------
// Print an introductory message
void intro() {
    int version = RUNQUALITYCHECK;
    std::cout<<"\n****************************************\n";
    std::cout<<"**  Run Quality Check version   "<<version;
    if(version<10 )std::cout<<" ";if(version<100)std::cout<<" ";
    std::cout<<"   **\n****************************************\n\n";
}
