#ifndef _DBUTILS_DAQCONFIG_DBCOLUMNPARAMS_H
#define _DBUTILS_DAQCONFIG_DBCOLUMNPARAMS_H

#include <string>
#include <map>
#include <vector>
#include <stdint.h>  /* uint16_t */

namespace dbutils {
namespace daqconfig {

/**
 * Simple singleton class to provide database table column names and
 * values to be used when selecting subsets of configuration data.
 *
 * @author Kurt Biery
 * @version $Revision: 1.6 $ $Date: 2015/09/09 21:35:38 $
 */

class DBColumnParams {

 public:

    static const std::string DCMID_COLUMN_NAME;  // deprecated
    static const std::string DCMNAME_COLUMN_NAME;
    static const std::string FEBID_COLUMN_NAME;
    static const std::string PIXELID_COLUMN_NAME;
    static const std::string HOSTNAME_COLUMN_NAME;
    static const std::string TDUNAME_COLUMN_NAME;
    static const std::string TDUDELAYTYPE_COLUMN_NAME;

    static const uint16_t LOW_FEB_ID;
    static const uint16_t HIGH_FEB_ID;

    static const uint16_t LOW_PIXEL_ID;
    static const uint16_t HIGH_PIXEL_ID;

    static DBColumnParams& getInstance();

    bool columnKeyToGetDefaultValue(const std::string& columnName,
                                    std::string& columnValue);

 private:

    explicit DBColumnParams();

    // Disallow any copying
    DBColumnParams(DBColumnParams const&); // not implemented
    void operator=(DBColumnParams const&);  // not implemented

    std::map<std::string, std::string> _columnKeysForDefaults;
};

};  // end of namespace daqconfig
};  // end of namespace dbutils

#endif
