#ifndef _DBUTILS_DAQCONFIG_CONFIGDATASELECTOR_H
#define _DBUTILS_DAQCONFIG_CONFIGDATASELECTOR_H

#include <string>
#include <vector>
#include <stdint.h>  /* uint32_t */

namespace dbutils {
namespace daqconfig {

/**
 * Simple container class to hold the identifiers that specify a particular
 * named global configuration.
 *
 * @author Kurt Biery
 * @version $Revision: 1.3 $ $Date: 2011/08/03 19:32:32 $
 */

class ConfigDataSelector {

 public:

    // *****************
    // *** Modifiers ***
    // *****************

    void addSelectionColumn(std::string columnName);
    void addSelectionColumn(std::string columnName,
                            std::string lowSelectionValue,
                            std::string highSelectionValue);

    // *****************
    // *** Accessors ***
    // *****************

    uint32_t getSelectionCount() const {return _selectionColumnList.size();}
    std::string getSelectionColumn(uint32_t selectionIndex) const;
    bool hasSelectionValues(uint32_t selectionIndex) const;
    std::string getSelectionLowValue(uint32_t selectionIndex) const;
    std::string getSelectionHighValue(uint32_t selectionIndex) const;
    std::vector<std::string> getSelectionColumns() const {
        return _selectionColumnList;
    }

 private:

    std::vector<std::string> _selectionColumnList;
    std::vector<bool> _selectionValueFlagList;
    std::vector<std::string> _selectionLowValueList;
    std::vector<std::string> _selectionHighValueList;
};

};  // end of namespace daqconfig
};  // end of namespace dbutils

#endif
