#ifndef _DBUTILS_DAQCONFIG_PIXELENABLEMASKS_H
#define _DBUTILS_DAQCONFIG_PIXELENABLEMASKS_H

#include <DatabaseUtils/DAQConfig/ConfigDataTree.h>
#include <DatabaseUtils/DAQConfig/SubsystemNameSpec.h>
#include <boost/shared_ptr.hpp>
#include <iostream>
#include <string>
#include <vector>

namespace dbutils {
namespace daqconfig {

/**
 * Worker class to handle fetching and formatting of pixel
 * enable mask data (including the FEB enable masks).
 *
 * @author Kurt Biery
 * @version $Revision: 1.5 $ $Date: 2011/07/29 14:31:17 $
 */

class PixelEnableMasks {

 public:

    // ********************
    // *** Constructors ***
    // ********************

    PixelEnableMasks(const SubsystemNameSpec& nameSpec,
                     std::vector<std::string> dcmList);

    PixelEnableMasks(const int64_t& globalConfigId,
                     std::vector<std::string> dcmList);

    virtual ~PixelEnableMasks() {};

    // *****************
    // *** Accessors ***
    // *****************

    bool hasValidData() const {return _hasValidData;}

    int getEnabledPixelCountForDCM(const std::string& dcmName);

    bool getFEBEnableMask(const std::string& dcmName, int64_t& febMask);
    bool febIsEnabled(const std::string& dcmName, const int& febId);
    int getEnabledPixelCountForFEB(const std::string& dcmName,
                                   const int& febId);

    bool getPixelEnableMask(const std::string& dcmName,
                            const int& febId, int32_t& pixelMask);
    bool pixelIsEnabled(const std::string& dcmName,
                        const int& febId, const int& pixelId);

 protected:

    bool _hasValidData;
    boost::shared_ptr<ConfigDataTree> _febEnableMaskDataSet;
    boost::shared_ptr<ConfigDataTree> _pixelEnableDataSet;

};

};  // end of namespace daqconfig
};  // end of namespace dbutil

#endif
