#ifndef _DBUTILS_GUI_BASICEDITORDISPLAY_H
#define _DBUTILS_GUI_BASICEDITORDISPLAY_H

#include <DatabaseUtils/GUI/CustomTabWidget.h>
#include <DatabaseUtils/GUI/SettingsDisplay.h>
#include <DatabaseUtils/DAQConfig/GlobalNameSpec.h>
#include <NovaDAQUtilities/Status.h>
#include <QtGui/QAction>
#include <QtGui/QMainWindow>
#include <QtGui/QCloseEvent>
#include <boost/shared_ptr.hpp>
#include <vector>

namespace dbutils {
namespace gui {

/**
 * Class for providing the core functionality for graphical
 * configuration editors.
 *
 * @author Kurt Biery
 * @version $Revision: 1.8 $ $Date: 2011/12/13 14:54:56 $
 */

class BasicEditorDisplay : public QMainWindow
{
    Q_OBJECT

 public:

    BasicEditorDisplay(QWidget *parent = 0);
    virtual ~BasicEditorDisplay() {}

    virtual bool hasChanges() const = 0;

 public slots:

    void reloadCurrentDataset();
    void loadNewDataset();
    void saveChanges();
    void saveChangesWithNewName();
    void showHelpDialog();

    virtual void reactToDataChange();
    virtual void reactToDataSave();

 protected:

    virtual bool _doReloadCurrentDataset(const dbutils::daqconfig::IDSpec&
                                         idSpec) = 0;
    virtual bool _doLoadNewDataset(const dbutils::daqconfig::IDSpec&
                                   idSpec) = 0;
    virtual bool _doSaveChanges(const dbutils::daqconfig::IDSpec& oldIdSpec,
                                const dbutils::daqconfig::IDSpec& newIdSpec,
                                const dbutils::daqconfig::GlobalNameSpec&
                                newNameSpec) = 0;
    virtual void closeEvent(QCloseEvent* event);

    bool _confirmSave();
    bool _getUserResponseToGlobalCopyFailure(const novadaq::Status& status);

    QAction* _saveAction;
    QAction* _saveAsAction;
    SettingsDisplay* _settingsDisplay;
    boost::shared_ptr<dbutils::daqconfig::GlobalNameSpec> _globalNameSpecPtr;
    int64_t _currentGlobalConfigId;

    void _addHelpMenu(std::string customText = "");

 private:

    std::string _customHelpText;

};

};  // end of namespace gui
};  // end of namespace dbutils

#endif
