#ifndef _DBUTILS_GUI_CONFIGDATATABLEEDITOR_H
#define _DBUTILS_GUI_CONFIGDATATABLEEDITOR_H

#include <DatabaseUtils/GUI/Changeable.h>
#include <QtGui/QMenu>
#include <QtGui/QMouseEvent>
#include <QtGui/QTableView>
#include <boost/shared_ptr.hpp>

namespace dbutils {
namespace gui {

/**
 * Class for displaying a table of configuration data from a single
 * database table.
 *
 * @author Kurt Biery
 * @version $Revision: 1.4 $ $Date: 2013/11/13 18:36:07 $
 */

class ConfigDataTableEditor : public QTableView, public Changeable
{
    Q_OBJECT

 public:

    ConfigDataTableEditor(const std::string& tableDefinitionFileName,
                          QWidget *parent = 0);

    void mousePressEvent(QMouseEvent* event);

    bool loadData(const std::vector<int64_t>& subsystemConfigIdList);
    bool hasChanges() const;
    bool saveChanges(const int64_t& oldSubsystemConfigId,
                     const int64_t& subsystemConfigId);
    bool addInitialRow();

    std::string getTableName() const;

 public slots:

    void updatePopupMenu(const QModelIndex& currentIndex);
    void reactToDataModelChange();
    void reactToDataSave();

    void addRow();
    void deleteRow();

 signals:

    void aChangeWasMade();
    void dataWasSaved();

 private:

    std::string _tableDefinitionFileName;
    QMenu* _popupMenu;
    QAction* _addRowAction;
};

};  // end of namespace gui
};  // end of namespace dbutils

#endif
