#ifndef DBUTILS_GUI_CONFIGDATATABLEMODEL_H
#define DBUTILS_GUI_CONFIGDATATABLEMODEL_H

#include <DatabaseUtils/GUI/TreeItem.h>
#include <NovaDatabase/Table.h>
#include <QtCore/QAbstractItemModel>
#include <QtCore/QModelIndex>
#include <QtCore/QVariant>
#include <boost/shared_ptr.hpp>

namespace dbutils {
namespace gui {

class ConfigDataTableModel : public QAbstractItemModel
{
    Q_OBJECT

 public:

    // ************************************************************
    // constructor/destructor
    // ************************************************************
    ConfigDataTableModel(const std::string& tableDefinitionFileName,
                         QObject *parent = 0);
    virtual ~ConfigDataTableModel() {}

    // ************************************************************
    // the following methods do *not* have implementations in
    // QAbstractItemModel, so they need to be implemented here
    // (or in subclasses)
    // ************************************************************
    virtual int columnCount(const QModelIndex &parent = QModelIndex()) const;
    virtual QVariant data(const QModelIndex &index, int role) const;
    virtual QModelIndex index(int row, int column,
                              const QModelIndex &parent = QModelIndex()) const;
    virtual QModelIndex parent(const QModelIndex &index) const;
    virtual int rowCount(const QModelIndex &parent = QModelIndex()) const;

    // ************************************************************
    // the following methods *do* have implementations in QAbstractItemModel,
    // but want to customize the implementation
    // ************************************************************
    virtual Qt::ItemFlags flags(const QModelIndex &index) const;
    virtual QVariant headerData(int section, Qt::Orientation orientation,
                                int role = Qt::DisplayRole) const;
    virtual bool insertRows(int position, int rows,
                            const QModelIndex &parent = QModelIndex());
    virtual bool removeRows(int position, int rows,
                            const QModelIndex &parent = QModelIndex());
    virtual bool setData(const QModelIndex &index, const QVariant &value,
                         int role = Qt::EditRole);
    virtual bool setHeaderData(int section, Qt::Orientation orientation,
                               const QVariant &value, int role = Qt::EditRole);

    // ************************************************************
    // the following methods are unique to this class and its children
    // ************************************************************
    virtual bool loadData(const std::vector<int64_t>& subsystemConfigIdList);
    virtual bool hasChanges() const;
    virtual bool saveChanges(const int64_t& subsystemConfigId);

    virtual bool addInitialRow();
    virtual bool addRowAfter(const QModelIndex& index);
    virtual bool deleteRow(const QModelIndex& index);

    virtual std::string getTableName() const;

 signals:

    void aChangeWasMade();
    void dataWasSaved();

 protected:

    // ************************************************************
    // protected methods
    // ************************************************************
    TreeItem* _getItem(const QModelIndex &index) const;
    void _getModifiedItems(TreeItem* baseItem,
                           std::vector<TreeItem*>& list) const;

 private:

    // ************************************************************
    // private data
    // ************************************************************
    boost::shared_ptr<nova::database::Table> _dbTablePtr;
    boost::shared_ptr<TreeItem> _rootItem;
    std::vector<std::string> _dataColumnNames;
    std::vector<std::string> _dataColumnTypes;
};

};  // end of namespace gui
};  // end of namespace dbutils

#endif
