#ifndef _DBUTILS_GUI_CONFIGDATATREEEDITOR_H
#define _DBUTILS_GUI_CONFIGDATATREEEDITOR_H

#include <DatabaseUtils/DAQConfig/ConfigDataTree.h>
#include <DatabaseUtils/GUI/Changeable.h>
#include <QtGui/QMenu>
#include <QtGui/QMouseEvent>
#include <QtGui/QTreeView>
#include <boost/shared_ptr.hpp>

namespace dbutils {
namespace gui {

/**
 * Class for displaying a tree of configuration data from a single
 * database table.
 *
 * @author Kurt Biery
 * @version $Revision: 1.6 $ $Date: 2013/11/13 18:36:07 $
 */

class ConfigDataTreeEditor : public QTreeView, public Changeable
{
    Q_OBJECT

 public:

    ConfigDataTreeEditor(QWidget *parent = 0);

    void mousePressEvent(QMouseEvent* event);

    bool loadData(boost::shared_ptr<daqconfig::ConfigDataTree> dataTree);
    bool hasChanges() const;
    bool saveChanges(const int64_t& oldSubsystemConfigId,
                     const int64_t& subsystemConfigId);
    bool addInitialRow();

 public slots:

    void updatePopupMenu(const QModelIndex& currentIndex);
    void reactToDataModelChange();
    void reactToDataSave();
    void refreshDisplayOfChildren(const QModelIndex&, const QModelIndex&);

    void addRowBefore();
    void addRowAfter();
    void addChildRow();
    void editBitmap();

 signals:

    void aChangeWasMade();
    void dataWasSaved();

 private:

    QMenu* _popupMenu;

    QAction* _addRowBeforeAction;
    QAction* _addRowAfterAction;
    QAction* _addChildRowAction;

    QList<QString> _candidatesForInsertBefore;
    QList<QString> _candidatesForInsertAfter;
    QList<QString> _candidatesForChildInsert;
};

};  // end of namespace gui
};  // end of namespace dbutils

#endif
