/****************************************************************************
 **
 ** 03-Feb-2011, KAB:  This file was copied from treemodel.h in the
 ** Editable Tree Model Example from Nokia (copyright notice below).
 ** See http://doc.qt.nokia.com/4.6/itemviews-editabletreemodel.html.
 ** It has been modified to add local customizations.
 **
 **
 ** Copyright (C) 2010 Nokia Corporation and/or its subsidiary(-ies).
 ** All rights reserved.
 ** Contact: Nokia Corporation (qt-info@nokia.com)
 **
 ** This file is part of the examples of the Qt Toolkit.
 **
 ** $QT_BEGIN_LICENSE:LGPL$
 ** Commercial Usage
 ** Licensees holding valid Qt Commercial licenses may use this file in
 ** accordance with the Qt Commercial License Agreement provided with the
 ** Software or, alternatively, in accordance with the terms contained in
 ** a written agreement between you and Nokia.
 **
 ** GNU Lesser General Public License Usage
 ** Alternatively, this file may be used under the terms of the GNU Lesser
 ** General Public License version 2.1 as published by the Free Software
 ** Foundation and appearing in the file LICENSE.LGPL included in the
 ** packaging of this file.  Please review the following information to
 ** ensure the GNU Lesser General Public License version 2.1 requirements
 ** will be met: http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
 **
 ** In addition, as a special exception, Nokia gives you certain additional
 ** rights.  These rights are described in the Nokia Qt LGPL Exception
 ** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
 **
 ** GNU General Public License Usage
 ** Alternatively, this file may be used under the terms of the GNU
 ** General Public License version 3.0 as published by the Free Software
 ** Foundation and appearing in the file LICENSE.GPL included in the
 ** packaging of this file.  Please review the following information to
 ** ensure the GNU General Public License version 3.0 requirements will be
 ** met: http://www.gnu.org/copyleft/gpl.html.
 **
 ** If you have questions regarding the use of this file, please contact
 ** Nokia at qt-info@nokia.com.
 ** $QT_END_LICENSE$
 **
 ****************************************************************************/

#ifndef DBUTILS_GUI_CONFIGDATATREEMODEL_H
#define DBUTILS_GUI_CONFIGDATATREEMODEL_H

#include <DatabaseUtils/GUI/TreeItem.h>
#include <DatabaseUtils/DAQConfig/ConfigDataTree.h>
#include <QtCore/QAbstractItemModel>
#include <QtCore/QModelIndex>
#include <QtCore/QVariant>
#include <boost/shared_ptr.hpp>

namespace dbutils {
namespace gui {

class ConfigDataTreeModel : public QAbstractItemModel
{
    Q_OBJECT

 public:

    // ************************************************************
    // constructor/destructor
    // ************************************************************
    ConfigDataTreeModel(QObject *parent = 0);
    virtual ~ConfigDataTreeModel() {}

    // ************************************************************
    // the following methods do *not* have implementations in
    // QAbstractItemModel, so they need to be implemented here
    // (or in subclasses)
    // ************************************************************
    virtual int columnCount(const QModelIndex &parent = QModelIndex()) const;
    virtual QVariant data(const QModelIndex &index,
                          int role = Qt::DisplayRole) const;
    virtual QModelIndex index(int row, int column,
                              const QModelIndex &parent = QModelIndex()) const;
    virtual QModelIndex parent(const QModelIndex &index) const;
    virtual int rowCount(const QModelIndex &parent = QModelIndex()) const;

    // ************************************************************
    // the following methods *do* have implementations in QAbstractItemModel,
    // but want to customize the implementation
    // ************************************************************
    virtual Qt::ItemFlags flags(const QModelIndex &index) const;
    virtual QVariant headerData(int section, Qt::Orientation orientation,
                                int role = Qt::DisplayRole) const;
    virtual bool insertRows(int position, int rows,
                            const QModelIndex &parent = QModelIndex());
    virtual bool removeRows(int position, int rows,
                            const QModelIndex &parent = QModelIndex());
    virtual bool setData(const QModelIndex &index, const QVariant &value,
                         int role = Qt::EditRole);
    virtual bool setHeaderData(int section, Qt::Orientation orientation,
                               const QVariant &value, int role = Qt::EditRole);

    // ************************************************************
    // the following methods are unique to this class and its children
    // ************************************************************
    QList<QString> allowsInsertBefore(const QModelIndex &index) const;
    QList<QString> allowsInsertAfter(const QModelIndex &index) const;
    QList<QString> allowsChildInsert(const QModelIndex &index) const;

    virtual bool loadData(boost::shared_ptr<daqconfig::ConfigDataTree> dataTree);
    virtual bool hasChanges() const;
    virtual bool saveChanges(const int64_t& subsystemConfigId);

    bool addChildRow(QModelIndex index, QString selectorColumnText,
                     int position = 0);
    bool addRowBefore(const QModelIndex& index,
                      QString selectorColumnText);
    bool addRowAfter(const QModelIndex& index,
                     QString selectorColumnText);

    bool selectorColumnNameForIndex(const QModelIndex& index,
                                    std::string& selectorColumnName) const;
    bool selectorColumnNameForChild(const QModelIndex& index,
                                    std::string& selectorColumnName) const;
    bool dataColumnNameForIndex(const QModelIndex& index,
                                std::string& dataColumnName) const;
    bool dataColumnTypeForIndex(const QModelIndex& index,
                                std::string& dataColumnType) const;

    bool hasSelectors() const {return _selectorColumnNames.size() > 0;}
    bool hasTreeStructure() const {return this->hasSelectors();}

 signals:

    void aChangeWasMade();
    void dataWasSaved();

 private:

    // ************************************************************
    // private methods
    // ************************************************************
    void _setupModelData();
    int _populateSubTree(TreeItem* parent, int level,
                         std::vector<std::string> selectionList);
    TreeItem *_getItem(const QModelIndex &index) const;
    bool _getSelectorColumnName(const QModelIndex& index,
                                std::string& selectorColumnName,
                                int startingOffset = 0) const;
    int _getParentCount(const QModelIndex& index) const;
    void _getModifiedItems(TreeItem* baseItem,
                           std::vector<TreeItem*>& list) const;

    // ************************************************************
    // private data
    // ************************************************************
    boost::shared_ptr<daqconfig::ConfigDataTree> _dataTree;
    boost::shared_ptr<TreeItem> _rootItem;
    std::vector<std::string> _selectorColumnNames;
    std::vector<std::string> _dataColumnNames;
    std::vector<std::string> _dataColumnTypes;
    std::vector<std::string> _keysForDefaultValues;

    boost::shared_ptr<nova::database::Table> _dbTablePtr;
};

};  // end of namespace gui
};  // end of namespace dbutils

#endif
