#ifndef DBUTILS_GUI_NAMEDCONFIGDATAMODEL_H
#define DBUTILS_GUI_NAMEDCONFIGDATAMODEL_H

#include <DatabaseUtils/GUI/TreeItem.h>
#include <DatabaseUtils/DAQConfig/NamedConfigUtils.h>
#include <QtCore/QAbstractItemModel>
#include <QtCore/QModelIndex>
#include <QtCore/QVariant>
#include <boost/shared_ptr.hpp>

namespace dbutils {
namespace gui {

/**
 * 
 *
 * @author Kurt Biery
 * @version $Revision: 1.1 $ $Date: 2011/12/02 23:09:07 $
 */

class NamedConfigDataModel : public QAbstractItemModel
{
    //Q_OBJECT

 public:

    // ************************************************************
    // constructor/destructor
    // ************************************************************
    NamedConfigDataModel(QObject *parent = 0);
    virtual ~NamedConfigDataModel() {}

    // ************************************************************
    // the following methods do *not* have implementations in
    // QAbstractItemModel, so they need to be implemented here
    // (or in subclasses)
    // ************************************************************
    virtual int columnCount(const QModelIndex &parent = QModelIndex()) const;
    virtual QVariant data(const QModelIndex &index,
                          int role = Qt::DisplayRole) const;
    virtual QModelIndex index(int row, int column,
                              const QModelIndex &parent = QModelIndex()) const;
    virtual QModelIndex parent(const QModelIndex &index) const;
    virtual int rowCount(const QModelIndex &parent = QModelIndex()) const;

    // ************************************************************
    // the following methods *do* have implementations in QAbstractItemModel,
    // but want to customize the implementation
    // ************************************************************
    virtual QVariant headerData(int section, Qt::Orientation orientation,
                                int role = Qt::DisplayRole) const;
    virtual bool setHeaderData(int section, Qt::Orientation orientation,
                               const QVariant &value, int role = Qt::EditRole);

    // ************************************************************
    // the following methods are unique to this class and its children
    // ************************************************************
    virtual void loadData(dbutils::daqconfig::NamedConfigUtils::
                          GlobalNameSpecMap nameSpecMap);

 private:

    // ************************************************************
    // private methods
    // ************************************************************
    TreeItem *_getItem(const QModelIndex &index) const;

    // ************************************************************
    // private data
    // ************************************************************
    boost::shared_ptr<TreeItem> _rootItem;

};

};  // end of namespace gui
};  // end of namespace dbutils

#endif
