#ifndef _DBUTILS_GUI_TABBEDSETOFTREEEDITORS_H
#define _DBUTILS_GUI_TABBEDSETOFTREEEDITORS_H

#include <DatabaseUtils/GUI/Changeable.h>
#include <DatabaseUtils/GUI/Storeable.h>
#include <QtGui/QTabWidget>

namespace dbutils {
namespace gui {

/**
 * Class for displaying the tabs that make up the graphical
 * configuration editor.
 *
 * @author Kurt Biery
 * @version $Revision: 1.6 $ $Date: 2013/11/13 18:36:08 $
 */

class TabbedSetOfTreeEditors : public QTabWidget, public Changeable,
        public Storeable
{
    Q_OBJECT

 public:

    TabbedSetOfTreeEditors(const std::string& subsystemName,
                           const std::string& configType,
                           const std::vector<std::string>& tableDefinitionList,
                           QWidget *parent = 0);

    bool loadData(const dbutils::daqconfig::IDSpec& idSpec);
    bool hasChanges() const;
    bool saveChanges(const dbutils::daqconfig::IDSpec& oldGlobalIdSpec,
                     const dbutils::daqconfig::IDSpec& newGlobalIdSpec,
                     const dbutils::daqconfig::GlobalNameSpec&
                     newGlobalNameSpec);
    bool addInitialRow();

    void isTopLevel(bool setting = true) {_isTopLevel = setting;}

 public slots:

    void reactToEditorDataChange();
    void reactToDataSave();

 signals:

    void aChangeWasMade();
    void dataWasSaved();

 private:

    std::string _subsystemName;
    std::string _configType;
    std::vector<std::string> _tableDefinitionList;
    bool _isTopLevel;
};

};  // end of namespace gui
};  // end of namespace dbutils

#endif
