#include <DatabaseUtils/GUI/ConfigSelectionDialog.h>
#include <DatabaseUtils/DAQConfig/NamedConfigUtils.h>
#include <QtGui/QPushButton>
#include <QtGui/QDialogButtonBox>
#include <QtGui/QHBoxLayout>
#include <QtGui/QLabel>
#include <QtGui/QVBoxLayout>

namespace DAQCFG = dbutils::daqconfig;

namespace dbutils {
namespace gui {

ConfigSelectionDialog::
ConfigSelectionDialog(DAQCFG::IDSpec::MAJOR_SYSTEM_TYPE systemType,
                      const std::string& detectorName,
                      QWidget *parent) : QDialog(parent)
{
    setWindowTitle(tr("Configuration Selector"));

    // *********************************************************
    // first, fetch the set of available named configurations
    // *********************************************************
    DAQCFG::NamedConfigUtils::GlobalNameSpecMap globalNameMap;
    if (! DAQCFG::NamedConfigUtils::
        getAvailableGlobalConfigs(systemType, detectorName, globalNameMap)) {

        std::string msgText = "Unable to fetch the available named ";
        msgText.append("global configurations from the database for ");
        msgText.append("the ");
        msgText.append((systemType == DAQCFG::IDSpec::DAQ ?
                       "DAQ" : "SlowControls"));
        msgText.append(" system at the ");
        msgText.append(detectorName);
        msgText.append(" detector");
        QLabel* label = new QLabel(msgText.c_str());

        QDialogButtonBox* buttonBox =
            new QDialogButtonBox(QDialogButtonBox::Cancel);
        buttonBox->setCenterButtons(true);
        connect(buttonBox, SIGNAL(rejected()), this, SLOT(reject()));

        QVBoxLayout *mainLayout = new QVBoxLayout;
        mainLayout->addWidget(label);
        mainLayout->addWidget(buttonBox);
        setLayout(mainLayout);

        return;
    }

    // ******************************************************
    // next, create the tree to display the configurations
    // ******************************************************
    _configTree = new NamedConfigTree();
    _configTree->loadData(globalNameMap);

    // ********************************
    // create the intro text label
    // ********************************
    QLabel* label = new QLabel("Select one of the named configurations from the list below:\n");

    // ********************************
    // create the accept/reject buttons
    // ********************************
    QDialogButtonBox* buttonBox =
        new QDialogButtonBox(QDialogButtonBox::Ok |
                             QDialogButtonBox::Cancel);
    buttonBox->setCenterButtons(true);

    connect(buttonBox, SIGNAL(accepted()), this, SLOT(accept()));
    connect(buttonBox, SIGNAL(rejected()), this, SLOT(reject()));

    // ********************************
    // put it all together
    // ********************************
    QVBoxLayout *mainLayout = new QVBoxLayout;
    mainLayout->addWidget(label);
    mainLayout->addWidget(_configTree);
    mainLayout->addWidget(buttonBox);
    setLayout(mainLayout);
}

std::string ConfigSelectionDialog::getSelectedConfigName() const
{
    return _configTree->getSelectedConfigName();
}

std::string ConfigSelectionDialog::getSelectedDCMOpMode() const
{
    return _configTree->getSelectedDCMOpMode();
}

std::string ConfigSelectionDialog::getSelectedTime() const
{
    return _configTree->getSelectedTime();
}

};  // end of namespace gui
};  // end of namespace dbutils
