#include <DatabaseUtils/GUI/CustomTabBehaviors.h>
#include <DatabaseUtils/GUI/Changeable.h>

namespace dbutils {
namespace gui {

namespace CustomTabBehaviors
{

/**
 * Updates the text colors for the specified tabWidget and tabBar
 * combination to show whether uncommitted changes have been made
 * on each tab.
 *
 * NOTE that the specified tabBar should be the one
 * that is associated with the specified tabWidget.  They only
 * need to be passed to this method separately because the method
 * to fetch the tabBar from a tabWidget is protected, by default.
 */
void updateTabTextColors(QTabWidget* tabWidget, QTabBar* tabBar)
{
    QColor stdColor("black");
    QColor colorForChanges(184, 0, 0);

    for (int idx = 0; idx < tabWidget->count(); ++idx) {
        QWidget* theWidget = tabWidget->widget(idx);
        Changeable* changeableObject = dynamic_cast<Changeable*>(theWidget);
        if (changeableObject->hasChanges()) {
            tabBar->setTabTextColor(idx, colorForChanges);
        }
        else {
            tabBar->setTabTextColor(idx, stdColor);
        }
    }
}

};  // end of namespace CustomTabBehaviors

};  // end of namespace gui
};  // end of namespace dbutils
