#include <DatabaseUtils/GUI/DBParamsGroupBox.h>
#include <NovaDAQUtilities/EnvVarCache.h>
#include <QtGui/QGridLayout>
#include <QtGui/QHBoxLayout>

namespace dbutils {
namespace gui {

DBParamsGroupBox::
DBParamsGroupBox(QWidget *parent) :
    QGroupBox(tr("Current Database Parameters"), parent)
{
    QGridLayout* gridLayout = new QGridLayout;
    NameValueLabelPair* nameValWidget;

    nameValWidget = new NameValueLabelPair("DB Host");
    std::string dbHost =
        novadaq::EnvVarCache::getInstance().getEnvVar("NOVADBHOST");
    nameValWidget->setValue(dbHost);
    gridLayout->addWidget(nameValWidget, 0, 0);


    QHBoxLayout* hboxLayout = new QHBoxLayout;
    hboxLayout->setContentsMargins(0,0,0,0);

    nameValWidget = new NameValueLabelPair("DB Name");
    std::string dbName =
        novadaq::EnvVarCache::getInstance().getEnvVar("NOVADBNAME");
    nameValWidget->setValue(dbName);
    hboxLayout->addWidget(nameValWidget);
    hboxLayout->addSpacing(10);

    nameValWidget = new NameValueLabelPair("DB User");
    std::string dbUser =
        novadaq::EnvVarCache::getInstance().getEnvVar("NOVADBUSER");
    std::string cfgMgrDBUser =
        novadaq::EnvVarCache::getInstance().getEnvVar("CFGMGR_DB_USER");
    std::string simpleUser =
        novadaq::EnvVarCache::getInstance().getEnvVar("USER");
    if (dbUser == "" && cfgMgrDBUser == "") {
        nameValWidget->setValue(simpleUser.c_str());
    }
    else if (dbUser == cfgMgrDBUser) {
        nameValWidget->setValue(dbUser.c_str());
    }
    else {
        std::string tmpString = dbUser;
        tmpString.append("/");
        tmpString.append(cfgMgrDBUser);
        nameValWidget->setValue(tmpString.c_str());
    }
    hboxLayout->addWidget(nameValWidget);
    hboxLayout->addStretch();

    QWidget* tmpWidget = new QWidget();
    tmpWidget->setLayout(hboxLayout);
    gridLayout->addWidget(tmpWidget, 1, 0, Qt::AlignLeft);

    this->setLayout(gridLayout);
}

};  // end of namespace gui
};  // end of namespace dbutils
