#include <DatabaseUtils/GUI/DataTreeCreatorThread.h>
#include <QtCore/QMutexLocker>

using namespace dbutils::daqconfig;

namespace dbutils {
namespace gui {

DataTreeCreatorThread::
DataTreeCreatorThread(const std::string& tableDefinitionFileName,
                      const std::vector<int64_t>& subsystemConfigIdList,
                      QObject *parent) :
    QThread(parent), _tableDefinitionFileName(tableDefinitionFileName),
    _subsystemConfigIdList(subsystemConfigIdList)
{
    this->start();
}

boost::shared_ptr<ConfigDataTree> DataTreeCreatorThread::getTree() {
    QMutexLocker dataTreeLock(&_dataTreeMutex);

    return _dataTree;
}

void DataTreeCreatorThread::run() {
    QMutexLocker dataTreeLock(&_dataTreeMutex);

    //std::cout << "TCT01 " << _tableDefinitionFileName << std::endl;
    _dataTree.reset(new ConfigDataTree(_tableDefinitionFileName,
                                       _subsystemConfigIdList));
    //std::cout << "TCT02 " << _tableDefinitionFileName << std::endl;
}

};  // end of namespace gui
};  // end of namespace dbutils
