#include <DatabaseUtils/GUI/NamedConfigDataModel.h>

using namespace dbutils::daqconfig;

namespace dbutils {
namespace gui {

NamedConfigDataModel::
NamedConfigDataModel(QObject *parent) : QAbstractItemModel(parent)
{
}

void NamedConfigDataModel::
loadData(NamedConfigUtils::GlobalNameSpecMap nameSpecMap)
{
    QVector<QVariant> rootData;
    rootData << "Configuration Name";
    rootData << "DCM Operating Mode";
    rootData << "Time";

    if (_rootItem.get() != 0) {
        QModelIndex dummyIndex;
        this->removeRows(0, _rootItem->childCount(), dummyIndex);
    }
    _rootItem.reset(new TreeItem(rootData));

    NamedConfigUtils::GlobalNameSpecMap::const_iterator iter;
    NamedConfigUtils::GlobalNameSpecMap::const_iterator iterEnd =
        nameSpecMap.end();
    for (iter = nameSpecMap.begin(); iter != iterEnd; ++iter) {
        int newChildIndex = _rootItem->childCount();
        _rootItem->insertChildren(newChildIndex, 1, _rootItem->columnCount());
        TreeItem* thisItem = _rootItem->child(newChildIndex);

        thisItem->setData(0, QVariant((*iter).first.getConfigName().c_str()));
        thisItem->setData(1, QVariant((*iter).first.getDCMOpMode().c_str()));
        thisItem->setData(2, QVariant(NamedConfigUtils::LATEST_TIME_STRING.c_str()));

        std::vector<GlobalNameSpec> detailsList = (*iter).second;
        for (int idx = 0; idx < (int) detailsList.size(); ++idx) {
            newChildIndex = thisItem->childCount();
            thisItem->insertChildren(newChildIndex,1,thisItem->columnCount());
            TreeItem* childItem = thisItem->child(newChildIndex);

            childItem->setData(0, QVariant(detailsList[idx].getConfigName().c_str()));
            childItem->setData(1, QVariant(detailsList[idx].getDCMOpMode().c_str()));
            childItem->setData(2, QVariant(detailsList[idx].getTimestamp().c_str()));
        }
    }
}

int NamedConfigDataModel::columnCount(const QModelIndex & /* parent */) const
{
    return _rootItem->columnCount();
}

QVariant NamedConfigDataModel::data(const QModelIndex &index, int role) const
{
    if (!index.isValid()) {return QVariant();}

    if (role != Qt::DisplayRole && role != Qt::EditRole)
        return QVariant();

    TreeItem *item = _getItem(index);

    return item->data(index.column());
}

QModelIndex NamedConfigDataModel::index(int row, int column,
                                        const QModelIndex &parent) const
{
    if (parent.isValid() && parent.column() != 0) {
        return QModelIndex();
    }

    TreeItem *parentItem = _getItem(parent);

    TreeItem *childItem = parentItem->child(row);
    if (childItem)
        return createIndex(row, column, childItem);
    else
        return QModelIndex();
}

QModelIndex NamedConfigDataModel::parent(const QModelIndex &index) const
{
    if (!index.isValid()) {return QModelIndex();}

    TreeItem *childItem = _getItem(index);
    TreeItem *parentItem = childItem->parent();

    if (parentItem == _rootItem.get())
        return QModelIndex();

    return createIndex(parentItem->childNumber(), 0, parentItem);
}

int NamedConfigDataModel::rowCount(const QModelIndex &parent) const
{
    TreeItem *parentItem = _getItem(parent);

    return parentItem->childCount();
}

QVariant NamedConfigDataModel::
headerData(int section, Qt::Orientation orientation, int role) const
{
    if (orientation == Qt::Horizontal && role == Qt::DisplayRole)
        return _rootItem->data(section);

    return QVariant();
}

bool NamedConfigDataModel::setHeaderData(int section,
                                         Qt::Orientation orientation,
                                         const QVariant &value, int role)
{
    if (role != Qt::EditRole || orientation != Qt::Horizontal)
        return false;

    bool result = _rootItem->setData(section, value);

    if (result)
        emit headerDataChanged(orientation, section, section);

    return result;
}

TreeItem *NamedConfigDataModel::_getItem(const QModelIndex &index) const
{
    if (index.isValid()) {
        TreeItem *item = static_cast<TreeItem*>(index.internalPointer());
        if (item) return item;
    }
    return _rootItem.get();
}

};  // end of namespace gui
};  // end of namespace dbutils
