#include <DatabaseUtils/GUI/NamedConfigTree.h>
#include <DatabaseUtils/GUI/NamedConfigDataModel.h>

using namespace dbutils::daqconfig;

namespace dbutils {
namespace gui {

/**
 * Creates the tree display for editing configuration data from
 * a single database table.
 */
NamedConfigTree::
NamedConfigTree(QWidget *parent) : QTreeView(parent)
{
    this->setAlternatingRowColors(true);
    this->setSelectionBehavior(QAbstractItemView::SelectItems);
    this->setHorizontalScrollMode(QAbstractItemView::ScrollPerPixel);
    this->setAnimated(false);
    this->setAllColumnsShowFocus(true);
    this->setSelectionBehavior(QAbstractItemView::SelectRows);
}

void NamedConfigTree::
loadData(NamedConfigUtils::GlobalNameSpecMap nameSpecMap)
{
    NamedConfigDataModel *model =
        static_cast<NamedConfigDataModel *>(this->model());

    if (model == 0) {
        model = new NamedConfigDataModel(this);
        model->loadData(nameSpecMap);
        this->setModel(model);
    }
    else {
        model->loadData(nameSpecMap);
    }

    this->expandAll();
    for (int column = 0; column < model->columnCount(); ++column) {
        this->resizeColumnToContents(column);
    }
    this->collapseAll();
    //this->expandToDepth(-1);

    QModelIndex firstRow = model->index(0, 0);
    if (firstRow.isValid()) {
        this->setCurrentIndex(firstRow);
    }
}

std::string NamedConfigTree::getSelectedConfigName() const
{
    NamedConfigDataModel *model =
        static_cast<NamedConfigDataModel *>(this->model());
    if (model == 0) {return "";}

    QModelIndex currentIndex = this->selectionModel()->currentIndex();
    if (! currentIndex.isValid()) {return "";}
    QModelIndex columnIndex = currentIndex.sibling(currentIndex.row(),0);
    if (! columnIndex.isValid()) {return "";}

    return model->data(columnIndex).toString().toStdString();
}

std::string NamedConfigTree::getSelectedDCMOpMode() const
{
    NamedConfigDataModel *model =
        static_cast<NamedConfigDataModel *>(this->model());
    if (model == 0) {return "";}

    QModelIndex currentIndex = this->selectionModel()->currentIndex();
    if (! currentIndex.isValid()) {return "";}
    QModelIndex columnIndex = currentIndex.sibling(currentIndex.row(),1);
    if (! columnIndex.isValid()) {return "";}

    return model->data(columnIndex).toString().toStdString();
}

std::string NamedConfigTree::getSelectedTime() const
{
    NamedConfigDataModel *model =
        static_cast<NamedConfigDataModel *>(this->model());
    if (model == 0) {return "";}

    QModelIndex currentIndex = this->selectionModel()->currentIndex();
    if (! currentIndex.isValid()) {return "";}
    QModelIndex columnIndex = currentIndex.sibling(currentIndex.row(),2);
    if (! columnIndex.isValid()) {return "";}

    return model->data(columnIndex).toString().toStdString();
}

};  // end of namespace gui
};  // end of namespace dbutils
