#include <DatabaseUtils/GUI/SettingsDisplay.h>
#include <DatabaseUtils/GUI/DBParamsGroupBox.h>
#include <DatabaseUtils/DAQConfig/NamedConfigUtils.h>
#include <QtGui/QHBoxLayout>

namespace DAQCONFIG = dbutils::daqconfig;

namespace dbutils {
namespace gui {

/**
 * Creates panel that displays the user-selectable parameters.
 */
SettingsDisplay::SettingsDisplay(QWidget *parent) : QWidget(parent)
{
    _namedConfigChoices = new NamedConfigGroupBox();
    DBParamsGroupBox* dbParamsBox = new DBParamsGroupBox(this);

    QHBoxLayout* infoLayout = new QHBoxLayout;
    infoLayout->addWidget(_namedConfigChoices);
    infoLayout->addWidget(dbParamsBox);

    setLayout(infoLayout);
}

/**
 * Destroys the display.
 */
SettingsDisplay::~SettingsDisplay()
{
}

/**
 * Updates the values of the named configuration parameters.
 */
void SettingsDisplay::
updateNamedConfiguration(const DAQCONFIG::GlobalNameSpec& nameSpec)
{
    _namedConfigChoices->setConfigurationName(nameSpec.getConfigName());
    _namedConfigChoices->setDCMOperatingMode(nameSpec.getDCMOpMode());
    _namedConfigChoices->setDetectorName(nameSpec.getDetectorName());
    if (nameSpec.hasTimestamp()) {
        _namedConfigChoices->setTimestamp(nameSpec.getTimestamp());
    }
    else {
        _namedConfigChoices->setTimestamp(DAQCONFIG::NamedConfigUtils::
                                          LATEST_TIME_STRING);
    }
}

};  // end of namespace gui
};  // end of namespace dbutils
