#include <DatabaseUtils/Hardware/Installation.h>
#include <cstdlib>

namespace dbutils {
  namespace hw {

    //**********************************************************************
    
    nova::database::Table* Util::PositionTable(int detid)
    {
      nova::database::Table* t=0;
      if (detid == novadaq::cnv::kNDOS) {
	t = new nova::database::Table();
	t->CreateFromDB("public","hardware_position","ifdbprod.fnal.gov",
			"nova_hardware","5432","nova_reader");
	t->SetIgnoreEnvVar(true);
	std::string detName = novadaq::cnv::DetInfo::GetName(detid);
	t->SetValidityRange("detector",detName);
      }
      
      return t;
    }
    
    bool FEB::Installed(int detid, std::string dcmName, int dcmPort,
			std::string& id, std::string ts)
    {
      nova::database::Table* t = Util::PositionTable(detid);
      
      if (detid == novadaq::cnv::kNDOS) {
	t->SetValidityRange("hw_type","FEB");
	t->SetValidityRange("dcm_name",dcmName);
	t->SetValidityRange("dcm_port",dcmPort);
	std::string vld = "install_date < '" + ts + "' and (remove_date is NULL or remove_date > '" + ts + "')";
	t->SetValiditySQL(vld);
	t->LoadFromDB();
	if (t->NRow() != 1) return false;
	nova::database::Row* r = t->GetRow(0);
	bool ok = r->Col("hw_id")->Get(id);
	delete t;
	return ok;
      }
      
      return false;
    }
    
    bool FEB::GetAllInstalled(int detid, 
			      std::vector<std::string>& dcmName, 
			      std::vector<int>& dcmPort,
			      std::vector<std::string>& febid, std::string ts)
    {
      nova::database::Table* t = Util::PositionTable(detid);
      
      if (detid == novadaq::cnv::kNDOS) {
	t->SetValidityRange("hw_type","FEB");
	std::string vld = "install_date < '" + ts + "' and (remove_date is NULL or remove_date > '" + ts + "')";
	t->SetValiditySQL(vld);
	t->LoadFromDB();
	if (t->NRow() == 0) return false;
	nova::database::Row* r;
	
	std::string dName;
	std::string id;
	int dPort;	  
	bool ok=true;
	for (int i=0; i<t->NRow(); ++i) {
	  r = t->GetRow(i);
	  ok &= r->Col("hw_id")->Get(id);
	  ok &= r->Col("dcm_name")->Get(dName);
	  ok &= r->Col("dcm_port")->Get(dPort);
	  if (ok) {
	    dcmName.push_back(dName);
	    dcmPort.push_back(dPort);
	    febid.push_back(id);
	  }
	}
	delete t;
	return ok;
      }
      
      return false;
    }
    
    bool APD::Installed(int detid, std::string dcmName, int dcmPort,
			std::string& id, std::string ts)
    {
      nova::database::Table* t = Util::PositionTable(detid);
      
      if (detid == novadaq::cnv::kNDOS) {
	t->SetValidityRange("hw_type","APD");
	t->SetValidityRange("dcm_name",dcmName);
	t->SetValidityRange("dcm_port",dcmPort);
	std::string vld = "install_date < '" + ts + "' and (remove_date is NULL or remove_date > '" + ts + "')";
	t->SetValiditySQL(vld);
	t->LoadFromDB();
	if (t->NRow() != 1) return false;
	nova::database::Row* r = t->GetRow(0);
	bool ok = r->Col("hw_id")->Get(id);
	delete t;
	return ok;
      }
      
      return false;
    }

    bool APD::GetAllInstalled(int detid, 
			      std::vector<std::string>& dcmName, 
			      std::vector<int>& dcmPort,
			      std::vector<std::string>& febid, std::string ts)
    {
      nova::database::Table* t = Util::PositionTable(detid);
      
      if (detid == novadaq::cnv::kNDOS) {
	t->SetValidityRange("hw_type","APD");
	std::string vld = "install_date < '" + ts + "' and (remove_date is NULL or remove_date > '" + ts + "')";
	t->SetValiditySQL(vld);
	t->LoadFromDB();
	if (t->NRow() == 0) return false;
	nova::database::Row* r;
	
	std::string dName;
	std::string id;
	int dPort;	  
	bool ok=true;
	for (int i=0; i<t->NRow(); ++i) {
	  r = t->GetRow(i);
	  ok &= r->Col("hw_id")->Get(id);
	  ok &= r->Col("dcm_name")->Get(dName);
	  ok &= r->Col("dcm_port")->Get(dPort);
	  if (ok) {
	    dcmName.push_back(dName);
	    dcmPort.push_back(dPort);
	    febid.push_back(id);
	  }
	}
	delete t;
	return ok;
      }
      
      return false;
    }
    
  }
}
