#include <DatabaseUtils/DAQConfig/NamedConfigUtils.h>
#include <NovaDAQUtilities/EnvVarCache.h>

using namespace dbutils::daqconfig;

/**
 * Command-line utility for printing out the most recent global configuration.
 */

int main(int argc, char **argv)
{
    // parse command line arguments and print usage hints, if needed.
    bool printUsage = false;
    int c;
    while ((c = getopt (argc, argv, "h")) != -1) {
        switch (c)
        {
        case 'h':
            printUsage = true;
            break;
        default:
            printUsage = true;
            break;
        }
    }

    std::string detectorName;
    int partitionNumber = 0;
    if (! printUsage) {
        if ((argc - optind) < 2) {
            printUsage = true;
        }
        else {
            {
                std::string tmpString(argv[optind]);
                detectorName = tmpString;
                ++optind;
            }
            {
                std::string tmpString(argv[optind]);
                try {
                    partitionNumber = boost::lexical_cast<int>(tmpString);
                }
                catch (...) {
                    printUsage = true;
                }
                ++optind;
            }
        }
    }

    // print usage hints, if needed.
    if (printUsage) {
        std::cout << std::endl;
        std::cout << "Usage: " << argv[0] << " <detectorName> "
                  << "<partitionNumber>" << std::endl;
        std::cout << "Where: " << std::endl;
        std::cout << " * detectorName specifies the detector name "
                  << "(e.g. NDOS)" << std::endl;
        std::cout << " * partitionNumber specifies the partition number "
                  << std::endl;
        std::cout << "Examples:" << std::endl;
        std::cout << "   " << argv[0] << " NDOS 0"
                  << std::endl;
        return -1;
    }

    // tell the user which database he/she is using
    std::string dbName =
        novadaq::EnvVarCache::getInstance().getEnvVar("NOVADBNAME");
    std::cout << "*** You are currently using the " << dbName
              << " database. ***" << std::endl;

    // fetch the configuration
    NamedConfigUtils::GlobalNameSpecPtr globalNamePtr;
    globalNamePtr = NamedConfigUtils::
        getNewestInstanceOfLatestUsedDAQGlobalConfig(detectorName,
                                                     partitionNumber);
    if (globalNamePtr.get() == 0) {
        std::cerr << "*** ERROR: Unable to fetch the global "
                  << "configuration for detector \"" << detectorName
                  << "\" and partition \"" << partitionNumber
                  << "\"."<< std::endl;
        exit(2);
    }

    // print out the results
    std::cout << globalNamePtr->toString() << std::endl;

    return 0;
}
