#include <DatabaseUtils/DAQConfig/NamedConfigUtils.h>
#include <NovaDAQUtilities/EnvVarCache.h>

using namespace dbutils::daqconfig;

/**
 * Command-line utility for printing out recent global configurations.
 */

int main(int argc, char **argv)
{
    // parse command line arguments and print usage hints, if needed.
    bool printUsage = false;
    int c;
    while ((c = getopt (argc, argv, "h")) != -1) {
        switch (c)
        {
        case 'h':
            printUsage = true;
            break;
        default:
            printUsage = true;
            break;
        }
    }

    std::string daqOrSlowControls;
    std::string detectorName;
    int resultOffset = 0;
    int resultLimit = 10;
    if (! printUsage) {
        if ((argc - optind) < 4) {
            printUsage = true;
        }
        else {
            {
                std::string tmpString(argv[optind]);
                if (tmpString == "DAQ" || tmpString == "SlowControls") {
                    daqOrSlowControls = tmpString;
                }
                else {
                    printUsage = true;
                }
                ++optind;
            }
            {
                std::string tmpString(argv[optind]);
                detectorName = tmpString;
                ++optind;
            }
            {
                std::string tmpString(argv[optind]);
                try {
                    resultOffset = boost::lexical_cast<int>(tmpString);
                }
                catch (...) {
                    printUsage = true;
                }
                ++optind;
            }
            {
                std::string tmpString(argv[optind]);
                try {
                    resultLimit = boost::lexical_cast<int>(tmpString);
                }
                catch (...) {
                    printUsage = true;
                }
                ++optind;
            }
        }
    }

    // print usage hints, if needed.
    if (printUsage) {
        std::cout << std::endl;
        std::cout << "Usage: " << argv[0] << " <\"DAQ\"|\"SlowControls\"> "
                  << "<detectorName> <resultOffset> <resultLimit>"
                  << std::endl;
        std::cout << "Where: " << std::endl;
        std::cout << " * \"DAQ\"|\"SlowControls\" selects which set "
                  << "of database tables to use" << std::endl;
        std::cout << " * detectorName specifies the detector name "
                  << "(e.g. NDOS)" << std::endl;
        std::cout << " * resultOffset specifies the offset for the result set "
                  << "(number of rows to skip)" << std::endl;
        std::cout << " * resultOffset specifies the limit for the result set "
                  << "(number of rows to be returned)" << std::endl;
        std::cout << "Examples:" << std::endl;
        std::cout << "   " << argv[0] << " DAQ NDOS 0 10"
                  << std::endl;
        return -1;
    }

    // tell the user which database he/she is using
    std::string dbName =
        novadaq::EnvVarCache::getInstance().getEnvVar("NOVADBNAME");
    std::cout << "*** You are currently using the " << dbName
              << " database. ***" << std::endl;

    // fetch the configurations
    NamedConfigUtils::GlobalNameSpecList globalNameList;
    if (! NamedConfigUtils::
        getRecentGlobalConfigs((daqOrSlowControls == "DAQ" ?
                                IDSpec::DAQ : IDSpec::SLOWCONTROLS),
                               detectorName, resultOffset, resultLimit,
                               globalNameList)) {
        std::cerr << "*** ERROR: Unable to fetch the named global "
                  << "configurations for detector \"" << detectorName
                  << "\"."<< std::endl;
        exit(2);
    }

    // print out the results
    if (globalNameList.size() > 0) {
        for (int idx = 0; idx < (int) globalNameList.size(); ++idx) {
            std::cout << globalNameList[idx].toString() << std::endl;
        }
    }
    else {
        std::cout << std::endl << "No global configurations were found "
                  << "for the detector, etc. that you specified!"
                  << std::endl;
        std::cout << "Please double-check that the values for the input "
                  << "parameters are correct..."
                  << std::endl;
    }

    return 0;
}
