#include "DatabaseUtils/RunHistory/RunHistory.h"
#include <vector>

using namespace dbutils;

int main(int argc, char **argv)
{
  if (argc < 4) {
    std::cout << "Usage " << argv[0] << " <runNumber> <detectorName> "
              << "<dcmName>" << std::endl;
    return 1;
  }

  int run = atoi(argv[1]);
  std::string detector = argv[2];
  std::string dcmName = argv[3];

  RunHistory rh(run, detector);

  int64_t _64BitOne = 1;
  int64_t febMask;
  if (rh.FEBEnableMask(dcmName, febMask)) {
    for (int febId = 0; febId < 64; ++febId) {
      if ((febMask & (_64BitOne << febId)) == 0) {
        std::cout << "FEB " << febId << " is disabled." << std::endl;
      }
      else {
        std::cout << "FEB " << febId << " is enabled." << std::endl;
          int32_t pixelMask;
          std::vector<int16_t> thresholdList;
          if (rh.PixelEnableMask(dcmName, febId, pixelMask)) {
            if (rh.PixelThresholdsForFEB(dcmName, febId, thresholdList)) {
              for (int pixelId = 0; pixelId < 32; ++pixelId) {
                if ((pixelMask & (1 << pixelId)) == 0) {
                  std::cout << "  Pixel " << pixelId << " is disabled."
                            << std::endl;
                }
                else {
                  std::cout << "  Pixel " << pixelId << " threshold = "
                            << thresholdList[pixelId] << std::endl;
                }
              }
            }
            else {
              std::cout << "Unable to determine pixel thresholds for DCM "
                        << dcmName << ", FEB " << febId << std::endl;
            }
          }
          else {
            std::cout << "Unable to determine the pixel enable mask for DCM "
                      << dcmName << ", FEB " << febId << std::endl;
          }
      }
    }
  }
  else {
    std::cout << "Unable to determine the FEB enable mask for DCM "
              << dcmName << std::endl;
  }
}
