#include "DatabaseUtils/DAQConfig/NamedConfigUtils.h"
#include <NovaDAQUtilities/EnvVarCache.h>
#include "NovaDatabase/Table.h"
#include <boost/algorithm/string.hpp>

using namespace nova::database;
using namespace dbutils::daqconfig;

int main(int argc, char **argv)
{
    std::string fname =
        "NovaDatabase/tables/DAQConfig/NamedGlobalConfigurations.xml";
    boost::shared_ptr<Table> dbt;
    dbt.reset(new Table(fname.c_str()));
    std::string dbUser =
        novadaq::EnvVarCache::getInstance().getEnvVar("CFGMGR_DB_USER");
    if (dbUser.length() > 0) {
        dbt->SetUser(dbUser);
    }
    dbt->GetConnection();

    dbt->Clear();
    dbt->SetValidityRange(NamedConfigUtils::NAME_COLUMN_NAME,
                          "InitialGlobalConfig");
    dbt->SetValidityRange(NamedConfigUtils::DCMOPMODE_COLUMN_NAME,
                          "DCS");
    dbt->SetValidityRange(NamedConfigUtils::DETECTORNAME_COLUMN_NAME,
                          "NDOS");
    dbt->AddOrderColumn(NamedConfigUtils::ID_COLUMN_NAME);
    dbt->SetOrderDesc();
    dbt->SetVerbosity(1);
    if (! dbt->LoadFromDB() || ! dbt->NRow() == 1) {
        std::cout << "No rows found." << std::endl;
        return -1;
    }
    Row* rowPtr = dbt->GetRow(0);
    Column* colPtr;

    std::vector<std::string> columnNames = dbt->GetColNames();
    for (uint32_t idx = 0; idx < columnNames.size(); ++idx) {

        std::string columnName = boost::to_lower_copy(columnNames[idx]);

        std::string value;
        colPtr = rowPtr->Col(columnName);
        if (colPtr->IsNull()) {value = "NULL";}
        else if (! colPtr->Get(value)) {return false;}
        std::cout << "\t" << columnName << "\t " << value << std::endl;
    }
    dbt->Clear();
    dbt->CloseConnection();

    dbt->GetConnection();
    //std::string cmd = "BEGIN TRANSACTION ISOLATION LEVEL SERIALIZABLE";
    std::string cmd = "BEGIN";
    PGresult* resultSetPtr = 0;
    if (! dbt->ExecuteSQL(cmd, resultSetPtr)) {
        std::cout << "ERROR1" << std::endl;
    }
    PQclear(resultSetPtr);

    std::cout << "Before sleep1" << std::endl;
    sleep(3);
    std::cout << "After sleep1" << std::endl;

    cmd = "LOCK TABLE named_global_configurations in access exclusive mode";
    if (! dbt->ExecuteSQL(cmd, resultSetPtr)) {
        std::cout << "ERROR2" << std::endl;
    }
    PQclear(resultSetPtr);

    std::cout << "Before sleep2" << std::endl;
    sleep(3);
    std::cout << "After sleep2" << std::endl;

    cmd = "SELECT * from named_global_configurations WHERE name='InitialGlobalConfig' and dcmopmode='DCS' and detectorname='NDOS' and isChanging is true ORDER BY id DESC ";
    if (dbt->ExecuteSQL(cmd, resultSetPtr)) {
        if (PQresultStatus(resultSetPtr) == PGRES_TUPLES_OK) {
            std::cout << "Number of rows with changing == true is "
                      << PQntuples(resultSetPtr) << std::endl;
        }
    }
    PQclear(resultSetPtr);

    std::cout << "Before sleep3" << std::endl;
    sleep(5);
    std::cout << "After sleep3" << std::endl;

    cmd = "SELECT * from named_global_configurations WHERE name='InitialGlobalConfig' and dcmopmode='DCS' and detectorname='NDOS' and isChanging is true ORDER BY id DESC ";
    if (dbt->ExecuteSQL(cmd, resultSetPtr)) {
        if (PQresultStatus(resultSetPtr) == PGRES_TUPLES_OK) {
            std::cout << "Number of rows with changing == true is "
                      << PQntuples(resultSetPtr) << std::endl;
        }
    }
    PQclear(resultSetPtr);

    std::cout << "Before sleep4" << std::endl;
    sleep(5);
    std::cout << "After sleep4" << std::endl;

    cmd = "SELECT * from named_global_configurations WHERE name='InitialGlobalConfig' and dcmopmode='DCS' and detectorname='NDOS' ORDER BY id DESC LIMIT 1";
    if (dbt->ExecuteSQL(cmd, resultSetPtr)) {
        if (PQresultStatus(resultSetPtr) == PGRES_TUPLES_OK) {
            for (int iRow = 0; iRow < PQntuples(resultSetPtr); ++iRow) {
                for (int iCol = 0; iCol < PQnfields(resultSetPtr); ++iCol) {
                    std::cout << "\t" << PQfname(resultSetPtr, iCol)
                              << "\t " << PQgetvalue(resultSetPtr, iRow, iCol)
                              << std::endl;
                }
            }
        }
    }
    PQclear(resultSetPtr);

    std::cout << "Before sleep5" << std::endl;
    sleep(5);
    std::cout << "After sleep5" << std::endl;

    cmd = "END";
    resultSetPtr = 0;
    if (! dbt->ExecuteSQL(cmd, resultSetPtr)) {
        std::cout << "ERROR5" << std::endl;
    }
    PQclear(resultSetPtr);

    std::cout << "Before sleep6" << std::endl;
    sleep(5);
    std::cout << "After sleep6" << std::endl;

    dbt->Clear();
    dbt->CloseConnection();

    return 0;
}
