#include "DatabaseUtils/DAQConfig/GlobalNameSpec.h"
#include "DatabaseUtils/DAQConfig/NamedConfigUtils.h"
#include <vector>

using namespace dbutils::daqconfig;

/**
 * Expert tests for NamedConfigUtils class..
 */

int main(int argc, char **argv)
{
    GlobalNameSpec globalNameSpec1("InitialGlobalConfig", "DCS", "NDOS");
    GlobalNameSpec globalNameSpec2("InitialGlobalConfig", "DCS", "NDOS",
                                   "2011-08-01 12:13:14");
    SubsystemNameSpec subsystemNameSpec1("DCSMode_DCMHwCfgNamedSet", "DCS",
                                         "NDOS", "DCMApplication", "hardware");
    SubsystemNameSpec subsystemNameSpec2("DCSMode_DCMHwCfgNamedSet", "DCS",
                                         "NDOS", "DCMApplication", "hardware",
                                         "2011-09-01 04:05:06");
    int64_t globalConfigId;
    std::vector<int64_t> globalConfigIdList;
    int64_t subsystemConfigId;
    std::vector<int64_t> subsystemConfigIdList;
    std::vector<SubsystemNameSpec> subsystemNameSpecList;
    NamedConfigUtils::SubsystemNameSpecMap subsystemNameSpecMap;
    NamedConfigUtils::NamedConfigDetailsPtr detailsPtr;
    std::string idString;
    std::vector<NamedConfigUtils::NamedConfigDetailsPtr> detailsList;
    bool success;


    // **************************************** //
    std::cout << "*** getGlobalConfigId" << std::endl;
    success = NamedConfigUtils::getGlobalConfigId(globalNameSpec1,
                                                  globalConfigId);
    std::cout << "  latest id = " << globalConfigId
              << ", success = " << success << std::endl;
    success = NamedConfigUtils::getGlobalConfigId(globalNameSpec2,
                                                  globalConfigId);
    std::cout << "  01-Aug id = " << globalConfigId
              << ", success = " << success << std::endl;


    // **************************************** //
    std::cout << "*** getSubsystemConfigId" << std::endl;
    success = NamedConfigUtils::getSubsystemConfigId(subsystemNameSpec1,
                                                     subsystemConfigId);
    std::cout << "  latest id = " << subsystemConfigId
              << ", success = " << success << std::endl;
    success = NamedConfigUtils::getSubsystemConfigId(subsystemNameSpec2,
                                                     subsystemConfigId);
    std::cout << "  01-Sep id = " << subsystemConfigId
              << ", success = " << success << std::endl;


    // **************************************** //
    std::cout << "*** getGlobalConfigIdAncestry (by name)" << std::endl;
    globalConfigIdList.clear();
    success =
        NamedConfigUtils::getGlobalConfigIdAncestry(globalNameSpec1,
                                                    globalConfigIdList);
    std::cout << "  latest idList =";
    for (int idx = 0; idx < (int) globalConfigIdList.size(); ++idx) {
        std::cout << " " << globalConfigIdList[idx];
    }
    std::cout << ", success = " << success << std::endl;

    globalConfigIdList.clear();
    success =
        NamedConfigUtils::getGlobalConfigIdAncestry(globalNameSpec2,
                                                    globalConfigIdList);
    std::cout << "  01-Aug idList =";
    for (int idx = 0; idx < (int) globalConfigIdList.size(); ++idx) {
        std::cout << " " << globalConfigIdList[idx];
    }
    std::cout << ", success = " << success << std::endl;


    // **************************************** //
    std::cout << "*** getSubsystemConfigIdAncestry (by name)" << std::endl;
    subsystemConfigIdList.clear();
    success =
        NamedConfigUtils::getSubsystemConfigIdAncestry(subsystemNameSpec1,
                                                       subsystemConfigIdList);
    std::cout << "  latest idList =";
    for (int idx = 0; idx < (int) subsystemConfigIdList.size(); ++idx) {
        std::cout << " " << subsystemConfigIdList[idx];
    }
    std::cout << ", success = " << success << std::endl;

    subsystemConfigIdList.clear();
    success =
        NamedConfigUtils::getSubsystemConfigIdAncestry(subsystemNameSpec2,
                                                       subsystemConfigIdList);
    std::cout << "  01-Aug idList =";
    for (int idx = 0; idx < (int) subsystemConfigIdList.size(); ++idx) {
        std::cout << " " << subsystemConfigIdList[idx];
    }
    std::cout << ", success = " << success << std::endl;


    // **************************************** //
    std::cout << "*** getGlobalConfigIdAncestry (by ID)" << std::endl;
    globalConfigIdList.clear();
    success = NamedConfigUtils::getGlobalConfigId(globalNameSpec1,
                                                  globalConfigId);
    if (success) {
        IDSpec idSpec(globalConfigId, IDSpec::GLOBAL, IDSpec::DAQ);
        success = NamedConfigUtils::getConfigIdAncestry(idSpec,
                                                        globalConfigIdList);
    }
    std::cout << "  latest idList =";
    for (int idx = 0; idx < (int) globalConfigIdList.size(); ++idx) {
        std::cout << " " << globalConfigIdList[idx];
    }
    std::cout << ", success = " << success << std::endl;

    globalConfigIdList.clear();
    success = NamedConfigUtils::getGlobalConfigId(globalNameSpec2,
                                                  globalConfigId);
    if (success) {
        IDSpec idSpec(globalConfigId, IDSpec::GLOBAL, IDSpec::DAQ);
        success = NamedConfigUtils::getConfigIdAncestry(idSpec,
                                                        globalConfigIdList);
    }
    std::cout << "  01-Aug idList =";
    for (int idx = 0; idx < (int) globalConfigIdList.size(); ++idx) {
        std::cout << " " << globalConfigIdList[idx];
    }
    std::cout << ", success = " << success << std::endl;


    // **************************************** //
    std::cout << "*** getSubsystemConfigIdAncestry (by ID)" << std::endl;
    subsystemConfigIdList.clear();
    success = NamedConfigUtils::getSubsystemConfigId(subsystemNameSpec1,
                                                     subsystemConfigId);
    if (success) {
        IDSpec idSpec(subsystemConfigId, IDSpec::SUBSYSTEM, IDSpec::DAQ);
        success = NamedConfigUtils::getConfigIdAncestry(idSpec,
                                                        subsystemConfigIdList);
    }
    std::cout << "  latest idList =";
    for (int idx = 0; idx < (int) subsystemConfigIdList.size(); ++idx) {
        std::cout << " " << subsystemConfigIdList[idx];
    }
    std::cout << ", success = " << success << std::endl;

    subsystemConfigIdList.clear();
    success = NamedConfigUtils::getSubsystemConfigId(subsystemNameSpec2,
                                                     subsystemConfigId);
    if (success) {
        IDSpec idSpec(subsystemConfigId, IDSpec::SUBSYSTEM, IDSpec::DAQ);
        success = NamedConfigUtils::getConfigIdAncestry(idSpec,
                                                        subsystemConfigIdList);
    }
    std::cout << "  01-Sep idList =";
    for (int idx = 0; idx < (int) subsystemConfigIdList.size(); ++idx) {
        std::cout << " " << subsystemConfigIdList[idx];
    }
    std::cout << ", success = " << success << std::endl;


    // **************************************** //
    std::cout << "*** getSubsystemConfigIdFromGlobalName" << std::endl;
    success = NamedConfigUtils::
        getSubsystemConfigIdFromGlobalName(globalNameSpec1,
                                           "DCMApplication", "hardware",
                                           subsystemConfigId);
    std::cout << "  latest id = " << subsystemConfigId
              << ", success = " << success << std::endl;
    success = NamedConfigUtils::
        getSubsystemConfigIdFromGlobalName(globalNameSpec2,
                                           "DCMApplication", "hardware",
                                           subsystemConfigId);
    std::cout << "  01-Aug id = " << subsystemConfigId
              << ", success = " << success << std::endl;


    // **************************************** //
    std::cout << "*** getSubsystemConfigIdAncestryFromGlobalId" << std::endl;
    subsystemConfigIdList.clear();
    success = NamedConfigUtils::getGlobalConfigId(globalNameSpec1,
                                                  globalConfigId);
    if (success) {
        IDSpec idSpec(globalConfigId, IDSpec::GLOBAL, IDSpec::DAQ);
        success = NamedConfigUtils::
            getSubsystemConfigIdAncestryFromGlobalId(idSpec,
                                                     "DCMApplication",
                                                     "hardware",
                                                     subsystemConfigIdList);
    }
    std::cout << "  latest idList =";
    for (int idx = 0; idx < (int) subsystemConfigIdList.size(); ++idx) {
        std::cout << " " << subsystemConfigIdList[idx];
    }
    std::cout << ", success = " << success << std::endl;

    subsystemConfigIdList.clear();
    success = NamedConfigUtils::getGlobalConfigId(globalNameSpec2,
                                                  globalConfigId);
    if (success) {
        IDSpec idSpec(globalConfigId, IDSpec::GLOBAL, IDSpec::DAQ);
        success = NamedConfigUtils::
            getSubsystemConfigIdAncestryFromGlobalId(idSpec,
                                                     "DCMApplication",
                                                     "hardware",
                                                     subsystemConfigIdList);
    }
    std::cout << "  01-Aug idList =";
    for (int idx = 0; idx < (int) subsystemConfigIdList.size(); ++idx) {
        std::cout << " " << subsystemConfigIdList[idx];
    }
    std::cout << ", success = " << success << std::endl;


    // **************************************** //
    std::cout << "*** getSubsystemConfigIdsFromGlobalId" << std::endl;
    subsystemConfigIdList.clear();
    subsystemNameSpecList.clear();
    success = NamedConfigUtils::getGlobalConfigId(globalNameSpec1,
                                                  globalConfigId);
    if (success) {
        IDSpec idSpec(globalConfigId, IDSpec::GLOBAL, IDSpec::DAQ);
        success = NamedConfigUtils::
            getSubsystemConfigIdsFromGlobalId(idSpec,
                                              subsystemConfigIdList,
                                              subsystemNameSpecList);
    }
    std::cout << "  latest idList =";
    for (int idx = 0; idx < (int) subsystemConfigIdList.size(); ++idx) {
        std::cout << " " << subsystemConfigIdList[idx];
        std::cout << " (" << subsystemNameSpecList[idx].getSubsystemName()
                  << ", " << subsystemNameSpecList[idx].getConfigType()
                  << ")";
    }
    std::cout << ", success = " << success << std::endl;

    subsystemConfigIdList.clear();
    subsystemNameSpecList.clear();
    success = NamedConfigUtils::getGlobalConfigId(globalNameSpec2,
                                                  globalConfigId);
    if (success) {
        IDSpec idSpec(globalConfigId, IDSpec::GLOBAL, IDSpec::DAQ);
        success = NamedConfigUtils::
            getSubsystemConfigIdsFromGlobalId(idSpec,
                                              subsystemConfigIdList,
                                              subsystemNameSpecList);
    }
    std::cout << "  01-Aug idList =";
    for (int idx = 0; idx < (int) subsystemConfigIdList.size(); ++idx) {
        std::cout << " " << subsystemConfigIdList[idx];
        std::cout << " (" << subsystemNameSpecList[idx].getSubsystemName()
                  << ", " << subsystemNameSpecList[idx].getConfigType()
                  << ")";
    }
    std::cout << ", success = " << success << std::endl;


    // **************************************** //
    std::cout << "*** getNewestGlobalConfigFromSubsystemName" << std::endl;
    detailsPtr = NamedConfigUtils::
        getNewestGlobalConfigFromSubsystemName(subsystemNameSpec1);
    success = NamedConfigUtils::getFieldValue(detailsPtr, "id",
                                              idString);
    std::cout << "  latest id = " << idString
              << ", success = " << success << std::endl;
    detailsPtr = NamedConfigUtils::
        getNewestGlobalConfigFromSubsystemName(subsystemNameSpec2);
    success = NamedConfigUtils::getFieldValue(detailsPtr, "id",
                                              idString);
    std::cout << "  01-Sep id = " << idString
              << ", success = " << success << std::endl;


    // **************************************** //
    std::cout << "*** getNewestGlobalConfigFromSubsystemId" << std::endl;
    success = NamedConfigUtils::getSubsystemConfigId(subsystemNameSpec1,
                                                     subsystemConfigId);
    if (success) {
        IDSpec idSpec(subsystemConfigId, IDSpec::SUBSYSTEM, IDSpec::DAQ);
        detailsPtr = NamedConfigUtils::
            getNewestGlobalConfigFromSubsystemId(idSpec);
        success = NamedConfigUtils::getFieldValue(detailsPtr, "id",
                                                  idString);
    }
    std::cout << "  latest id = " << idString
              << ", success = " << success << std::endl;
    success = NamedConfigUtils::getSubsystemConfigId(subsystemNameSpec2,
                                                     subsystemConfigId);
    if (success) {
        IDSpec idSpec(subsystemConfigId, IDSpec::SUBSYSTEM, IDSpec::DAQ);
        detailsPtr = NamedConfigUtils::
            getNewestGlobalConfigFromSubsystemId(idSpec);
        success = NamedConfigUtils::getFieldValue(detailsPtr, "id",
                                                  idString);
    }
    std::cout << "  01-Sep id = " << idString
              << ", success = " << success << std::endl;


    // **************************************** //
    std::cout << "*** getAvailableSubsystemConfigs" << std::endl;
    subsystemNameSpecList.clear();
    success = NamedConfigUtils::
        getAvailableSubsystemConfigs(IDSpec::DAQ,
                                     "NDOS", "DCMApplication", "hardware",
                                     subsystemNameSpecMap);
    std::cout << " ";
    NamedConfigUtils::SubsystemNameSpecMap::const_iterator iter;
    NamedConfigUtils::SubsystemNameSpecMap::const_iterator iterEnd =
        subsystemNameSpecMap.end();
    for (iter = subsystemNameSpecMap.begin(); iter != iterEnd; ++iter) {
        std::cout << " [" << (*iter).first.getConfigName()
                  << ", " << (*iter).first.getDCMOpMode()
                  << "]";
    }
    std::cout << ", success = " << success << std::endl;


    // **************************************** //
    std::cout << "*** getGlobalConfigDetailAncestry (by name)" << std::endl;
    detailsList.clear();
    detailsList =
        NamedConfigUtils::getGlobalConfigDetailAncestry(globalNameSpec1);
    std::cout << "  latest idList =";
    for (int idx = 0; idx < (int) detailsList.size(); ++idx) {
        success = NamedConfigUtils::getFieldValue(detailsList[idx], "id",
                                                  idString);
        std::cout << " " << idString << " (success = " << success << ")";
    }
    std::cout << std::endl;
    detailsList.clear();
    detailsList =
        NamedConfigUtils::getGlobalConfigDetailAncestry(globalNameSpec2);
    std::cout << "  01-Sep idList =";
    for (int idx = 0; idx < (int) detailsList.size(); ++idx) {
        success = NamedConfigUtils::getFieldValue(detailsList[idx], "id",
                                                  idString);
        std::cout << " " << idString << " (success = " << success << ")";
    }
    std::cout << std::endl;


    // **************************************** //
    std::cout << "*** getGlobalConfigDetailAncestry (by ID)" << std::endl;
    detailsList.clear();
    success = NamedConfigUtils::getGlobalConfigId(globalNameSpec1,
                                                  globalConfigId);
    std::cout << "  latest idList =";
    if (success) {
        IDSpec idSpec(globalConfigId, IDSpec::GLOBAL, IDSpec::DAQ);
        detailsList = NamedConfigUtils::getConfigDetailAncestry(idSpec);
        for (int idx = 0; idx < (int) detailsList.size(); ++idx) {
            success = NamedConfigUtils::getFieldValue(detailsList[idx], "id",
                                                      idString);
            std::cout << " " << idString << " (success = " << success << ")";
        }
    }
    std::cout << std::endl;
    detailsList.clear();
    success = NamedConfigUtils::getGlobalConfigId(globalNameSpec2,
                                                  globalConfigId);
    std::cout << "  01-Sep idList =";
    if (success) {
        IDSpec idSpec(globalConfigId, IDSpec::GLOBAL, IDSpec::DAQ);
        detailsList = NamedConfigUtils::getConfigDetailAncestry(idSpec);
        for (int idx = 0; idx < (int) detailsList.size(); ++idx) {
            success = NamedConfigUtils::getFieldValue(detailsList[idx], "id",
                                                      idString);
            std::cout << " " << idString << " (success = " << success << ")";
        }
    }
    std::cout << std::endl;


    // **************************************** //
    std::cout << "*** getSubsystemConfigDetailAncestry" << std::endl;
    detailsList.clear();
    success = NamedConfigUtils::getSubsystemConfigId(subsystemNameSpec1,
                                                     subsystemConfigId);
    std::cout << "  latest idList =";
    if (success) {
        IDSpec idSpec(subsystemConfigId, IDSpec::SUBSYSTEM, IDSpec::DAQ);
        detailsList = NamedConfigUtils::getConfigDetailAncestry(idSpec);
        for (int idx = 0; idx < (int) detailsList.size(); ++idx) {
            success = NamedConfigUtils::getFieldValue(detailsList[idx], "id",
                                                      idString);
            std::cout << " " << idString << " (success = " << success << ")";
        }
    }
    std::cout << std::endl;
    detailsList.clear();
    success = NamedConfigUtils::getSubsystemConfigId(subsystemNameSpec2,
                                                     subsystemConfigId);
    std::cout << "  01-Aug idList =";
    if (success) {
        IDSpec idSpec(subsystemConfigId, IDSpec::SUBSYSTEM, IDSpec::DAQ);
        detailsList = NamedConfigUtils::getConfigDetailAncestry(idSpec);
        for (int idx = 0; idx < (int) detailsList.size(); ++idx) {
            success = NamedConfigUtils::getFieldValue(detailsList[idx], "id",
                                                      idString);
            std::cout << " " << idString << " (success = " << success << ")";
        }
    }
    std::cout << std::endl;

    return 0;
}
