#include <cppunit/extensions/HelperMacros.h>
#include <NovaDAQUtilities/CppUnitTestDriver.h>
#include <DatabaseUtils/SimpleParameterSet.h>

/**
 * Unit tests for the SimpleParameterSet class.
 *
 * @author Kurt Biery
 */

using namespace dbutils;

class SimpleParamSetUnitTest : public CppUnit::TestFixture
{
    CPPUNIT_TEST_SUITE(SimpleParamSetUnitTest);
    CPPUNIT_TEST(testTheBasics);
    CPPUNIT_TEST_SUITE_END();

public:
    void setUp();
    void tearDown();

    void testTheBasics();
};

void
SimpleParamSetUnitTest::setUp()
{
    // flush stdout to get progress markers to display in a timely way
    std::cout.flush();
}

void
SimpleParamSetUnitTest::tearDown()
{
    // flush stdout to get progress markers to display in a timely way
    std::cout.flush();
}

void
SimpleParamSetUnitTest::testTheBasics()
{
    SimpleParameterSet paramSet;

    paramSet.put("AAA", "123");
    paramSet.put("BBB", "pdq");
    paramSet.put("CCC", "12.5");
    paramSet.put("DDD", "Hello, World");
    paramSet.put("TRUE", "t");
    paramSet.put("FALSE", "f");

    int64_t bigNumber = 9123450006789;
    std::string bigNumberString = boost::lexical_cast<std::string>(bigNumber);
    paramSet.put("bigNumber", bigNumberString);

    std::string valueString;
    CPPUNIT_ASSERT(paramSet.get("AAA", valueString));
    CPPUNIT_ASSERT(valueString == "123");

    CPPUNIT_ASSERT(paramSet.get("BBB", valueString));
    CPPUNIT_ASSERT(valueString == "pdq");

    CPPUNIT_ASSERT(paramSet.get("CCC", valueString));
    CPPUNIT_ASSERT(valueString == "12.5");

    CPPUNIT_ASSERT(paramSet.get("DDD", valueString));
    CPPUNIT_ASSERT(valueString == "Hello, World");

    CPPUNIT_ASSERT(paramSet.get("TRUE", valueString));
    CPPUNIT_ASSERT(valueString == "t");

    CPPUNIT_ASSERT(paramSet.get("FALSE", valueString));
    CPPUNIT_ASSERT(valueString == "f");

    CPPUNIT_ASSERT(! paramSet.get("ZZZ", valueString));

    int intValue;
    CPPUNIT_ASSERT(paramSet.get("AAA", intValue));
    CPPUNIT_ASSERT(intValue == 123);

    double doubleValue;
    CPPUNIT_ASSERT(paramSet.get("CCC", doubleValue));
    CPPUNIT_ASSERT(doubleValue == 12.5);

    bool boolValue;
    CPPUNIT_ASSERT(paramSet.get("TRUE", boolValue));
    CPPUNIT_ASSERT(boolValue);

    CPPUNIT_ASSERT(paramSet.get("FALSE", boolValue));
    CPPUNIT_ASSERT(! boolValue);

    CPPUNIT_ASSERT(paramSet.get("bigNumber", valueString));
    CPPUNIT_ASSERT(valueString == bigNumberString);

    int64_t testValue1;
    CPPUNIT_ASSERT(paramSet.get("bigNumber", testValue1));
    CPPUNIT_ASSERT(testValue1 == bigNumber);

    int32_t testValue2;
    CPPUNIT_ASSERT(! paramSet.get("bigNumber", testValue2));

    paramSet.clear();
    CPPUNIT_ASSERT(! paramSet.get("AAA", valueString));
    CPPUNIT_ASSERT(! paramSet.get("BBB", valueString));
    CPPUNIT_ASSERT(! paramSet.get("CCC", valueString));
    CPPUNIT_ASSERT(! paramSet.get("DDD", valueString));
}

CPPUNIT_TEST_SUITE_REGISTRATION(SimpleParamSetUnitTest);
