import psycopg2
import csv
import datetime
import time
import argparse
import os

# try connecting to the database
# use the Fermilab (FL) database for testing information and the Ash River (AR)
# for the most up-to-date installation information

#read in the password for the db
pswdloc  = os.environ['NOVADBPWDFILE']
pswdfile = open(pswdloc, "r")
pswd     = pswdfile.read()

SQL = "dbname=nova_hardware host=ifdbprod.fnal.gov user=nova_reader password={} port=5432".format(pswd)

try:
    connFL = psycopg2.connect(SQL)
except:
    print "Unable to connect to the database"
    exit(0)

# set up the argument parser, --safe tells the program to make the safe voltage files
parser      = argparse.ArgumentParser()
parser.add_argument("--safe", help="set voltages 30V below nominal", action="store_true")
parser.add_argument("--dcm",  help="specify which dcm to set the HV for, e.g. dcm-2-01", default="dcm-2")
args        = parser.parse_args()
useSafeMode = args.safe
dcm         = args.dcm

curFL = connFL.cursor()
    
SQL = "set search_path to ashriverprod_factory;"
curFL.execute(SQL)

# get the relevant information from the APD table
# create a separate dictionary for each temperature range, > 0 and < 0
# if the current APD serial number exists in the dictionary, check the
# batch number and take the highest and overwrite the current entry if necessary
SQL = "select * from public.apd_stats"
curFL.execute(SQL)
rows  = curFL.fetchall()
dummy = 'dummyapd1'
apdStatsLowT  = dict(dummy = (-999, -999, 999.) )
apdStatsHighT = dict(dummy = (-999,  999, 999.) )
for row in rows:
    # grab the last 7 characters of the apd SN only
    # first determine how many characters are in the string
    # then only keep the last 7
    apd_stats_id   = row[0] 
    batch_no       = row[1] 
    apd_id         = row[2] 
    target_degc    = row[3] 
    vb             = row[4] 
    isrc           = row[5] 
    id_sum         = row[6] 
    ip_sum         = row[7] 
    vr             = row[8] 
    m_min          = row[9] 
    m_max          = row[10]
    m_stdev        = row[11]
    qe_mean        = row[12]
    qe_min         = row[13]
    qe_max         = row[14]
    qe_stdev       = row[15]
    refd_a_mean    = row[16]
    refd_a_stdev   = row[17]
    thmstr_mean    = row[18]
    thmstr_stdev   = row[19]
    tec_degc_mean  = row[20]
    tec_degc_stdev = row[21]
    stats = (target_degc, apd_stats_id, batch_no, vb, isrc, id_sum, ip_sum, vr, m_min, m_max, m_stdev, qe_mean, qe_min, qe_max, qe_stdev, refd_a_mean, refd_a_stdev, thmstr_mean, thmstr_stdev, tec_degc_mean, tec_degc_stdev)

    # decide which temperature we are looking at 
    # and also use the most recent test
    if target_degc > 0:
        if apd_id not in apdStatsHighT: 
            apdStatsHighT[apd_id] = stats
        else :
            prevStats = apdStatsHighT.get(apd_id)
            prevBatch = prevStats[0]
            if batch_no > prevBatch: 
                apdStatsHighT[apd_id] = stats
    else:
        if apd_id not in apdStatsLowT: 
            apdStatsLowT[apd_id] = stats
        else :
            prevStats = apdStatsLowT.get(apd_id)
            prevBatch = prevStats[0]
            if batch_no > prevBatch: 
                apdStatsLowT[apd_id] = stats

bogus = (-999., -999, -999, -999., -999., -999., -999., -999., -999., -999., -999., -999., -999., -999., -999., -999., -999., -999., -999., -999., -999.)

resultsFile = open("APDInfo.csv", "w")
resultsFile.write("APD SN, target_degc, apd_stats_id, batch_no, vb, isrc, id_sum, ip_sum, vr, m_min, m_max, m_stdev, qe_mean, qe_min, qe_max, qe_stdev, refd_a_mean, refd_a_stdev, thmstr_mean, thmstr_stdev, tec_degc_mean, tec_degc_stdev,target_degc, apd_stats_id, batch_no, vb, isrc, id_sum, ip_sum, vr, m_min, m_max, m_stdev, qe_mean, qe_min, qe_max, qe_stdev, refd_a_mean, refd_a_stdev, thmstr_mean, thmstr_stdev, tec_degc_mean, tec_degc_stdev\n")

for apd_id in apdStatsHighT.iterkeys():
    warm = apdStatsHighT.get(apd_id)
    if apd_id in apdStatsLowT :
        cold = apdStatsLowT.get(apd_id)
        warmcold = '{},{},{}\n'.format(apd_id, warm, cold)
        resultsFile.write(warmcold)
    else :
        warmcold = '{},{},{}\n'.format(apd_id, warm, bogus)
        resultsFile.write(warmcold)

for apd_id in apdStatsLowT.iterkeys():
    cold = apdStatsLowT.get(apd_id)
    if apd_id not in apdStatsHighT :
        warmcold = '{},{},{}\n'.format(apd_id, bogus, cold)
        resultsFile.write(warmcold)

resultsFile.close()
