#/bin/bash
#

############################################################################################
### This script is supposed to run through all the steps                        
### necessary to set the high voltages on new APDs.  
###                           
### It relies on a collection of scripts created by B. Rebel, P. Vahle, L. Mualem, T. Xin  
###
### Email M. Colo (mcolo@email.wm.edu) for questions about this script.         
###########################################################################################

if [ -z $NOVADBPWDFILE ]
then
	echo "NOVADBPWDFILE is not set."
	echo "APDHVSetting.py cannot run."
	exit 1
fi

HVPATH=$HVPATH:$PWD
DATE=$(date +%Y%m%d)
PREVHITEMPFILE=$(ls -t ../recipes/hv_High*.csv | head -n 1)
if [ -n $PREVHITEMPFILE ]
then
if [ $PREVHITEMPFILE == ../recipes/hv_HighT_$DATE.csv ]
then
	PREVHITEMPFILE=$(ls -t ../recipes/hv_High*.csv | head -n 2 | tail -n 1)
fi
fi

PREVLOTEMPFILE=$(ls -t ../recipes/hv_Low*.csv | head -n 1)
if [ -n $PREVHITEMPFILE ]
then
if [ $PREVLOTEMPFILE == ../recipes/hv_LowT_$DATE.csv ]
then
	PREVLOTEMPFILE=$(ls -t ../recipes/hv_Low*.csv | head -n 2 | tail -n 1)
fi
fi

echo "$(tput setaf 3)$(tput bold)***********************************$(tput sgr 0)"
echo "$(tput setaf 3)$(tput bold)***** Running APDHVSetting.py *****$(tput sgr 0)"
echo "$(tput setaf 3)$(tput bold)***********************************$(tput sgr 0)"
echo " "
echo "(this scripts generates the HV recipes and outputs them into hv_HighT.csv and hv_LowT.csv)"

python ./APDHVSetting.py "$@"

case $1 in
    '-h'|'--help')
	exit
				
      	;;
    *)
        ;;
esac

echo "$(tput setaf 2)$(tput bold)**************************************$(tput sgr 0)"
echo "$(tput setaf 2)$(tput bold)***** Generating Checklist Plots *****$(tput sgr 0)"
echo "$(tput setaf 2)$(tput bold)**************************************$(tput sgr 0)"

echo "Generating Cold Temperature plots"
root -q -l 'plotapdhv.C++("hv_LowT.csv")'
echo "Generating Warm Temperature plots"
root -q -l 'plotapdhv.C++("hv_HighT.csv")'
if [ -n $PREVHITEMPFILE ]
then
echo "Generating Cold Temperature difference plots"
ARGUMENT=diffapdhv.C++\(\"hv_LowT.csv\",\"$PREVLOTEMPFILE\"\)
root -q -l $ARGUMENT
fi
if [ -n $PREVLOTEMPFILE ]
then
echo "Generating Warm Temperature difference plots"
ARGUMENT=diffapdhv.C++\(\"hv_HighT.csv\",\"$PREVHITEMPFILE\"\)
root -q -l $ARGUMENT
fi

read -r -p "$(tput setaf 2)$(tput bold)Did the checklist plots look fine? Y/n $(tput sgr 0)" response
case $response in
    [yY][eE][sS]|[yY])
				
		;;
    *)
        echo "Aborted"
        exit
        ;;
esac

echo "$(tput setaf 4)$(tput bold)************************$(tput sgr 0)"
echo "$(tput setaf 4)$(tput bold)***** Moving Files *****$(tput sgr 0)"
echo "$(tput setaf 4)$(tput bold)************************$(tput sgr 0)"
echo " "
echo "(saving the .csv files in ../recipes/ with older generated files...)"

cp 'hv_HighT.csv' '../recipes/hv_HighT_'$DATE'.csv'
cp 'hv_LowT.csv' '../recipes/hv_HighT_'$DATE'.csv'
mv 'plotapdhv.log' '../logs/plotapdhv_'$DATE'.log'
mv 'diffapdhv.log' '../logs/diffapdhv_'$DATE'.log'
        
read -r -p "$(tput setaf 4)$(tput bold)Copy the recipes to novadcs-far-master-02? Y/n $(tput sgr 0)" response

case $response in
    [yY][eE][sS]|[yY])
				
		;;
    *)
        echo "Nothing to do."
        exit
        ;;
esac
#Copy the .csv files to novadcs-far-master-02

scp hv_HighT.csv novadcs@novadcs-far-master-02.fnal.gov:/nova/config/FarDet/DCS/HighVoltageAndTemp/HighT/
scp hv_LowT.csv novadcs@novadcs-far-master-02.fnal.gov:/nova/config/FarDet/DCS/HighVoltageAndTemp/LowT/
scp '../recipes/hv_HighT_'$DATE'.csv' novadcs@novadcs-far-master-02.fnal.gov:/nova/config/FarDet/DCS/HighVoltageAndTemp/HighT/
scp '../recipes/hv_LowT_'$DATE'.csv' novadcs@novadcs-far-master-02.fnal.gov:/nova/config/FarDet/DCS/HighVoltageAndTemp/LowT/

echo " "
echo " "
echo "$(tput setaf 5)$(tput bold)**************************************$(tput sgr 0)"
echo "$(tput setaf 5)$(tput bold)***** Starting DCS Config Editor *****$(tput sgr 0)"
echo "$(tput setaf 5)$(tput bold)**************************************$(tput sgr 0)"
echo " "
echo "At this point the DCS Config Editor will open."
echo "Please go to Action -> Load HV and Temperatures and select the folder containing the .csv files."
echo "You will have to repeat this process twice, once for warm settings (the folder is called HighT)"
echo "And once for cold settings (the folder is called LowT)"
echo "When you are done, close the DCS Config Editor and this script will continue"
echo " "
echo "If the script hangs after you have closed the GUI, X11 is preventing ssh from disconnecting."
echo "In that case press Ctrl+C to continue."

ssh -K -Y -l novadcs novadcs-far-master-02.fnal.gov 'source /home/novadcs/DAQOperationsTools/novadaq_setup.sh ; startDCSConfigEditor.sh'

echo "$(tput setaf 5)$(tput bold)Congratulations! The new recipes have been created and are ready to be loaded.$(tput sgr 0)"
echo "At this point you can have the control room load the new recipes by running \"Configure COld APDs\""
echo "in the APD Temperature Monitor next time a new run is started."
