#ifndef DISPATCHERCMD_H
#define DISPATCHERCMD_H

#include <sys/types.h>
#include <stdint.h>
#include <vector>

// namespace EventDispatcher {

// #define USE_MAGIC_NO

class DispatcherCMD {

 public:


  // Magic numbers used as sanity checks
  enum MagicNumbers{
    DSPCMD_BYTECOUNTMAGIC1             = 0xBBADBEEF,
    DSPCMD_BYTECOUNTMAGIC2             = 0xBEEFBBAD,
    DSPCMD_BYTECOUNTMAGIC_B3           = 0xBB,
    DSPCMD_BYTECOUNTMAGIC_B2           = 0xAD,
    DSPCMD_BYTECOUNTMAGIC_B1           = 0xBE,
    DSPCMD_BYTECOUNTMAGIC_B0           = 0xFF,

    DSPCMD_BYTECOUNTMAGIC_B7           = 0xBE,
    DSPCMD_BYTECOUNTMAGIC_B6           = 0xEF,
    DSPCMD_BYTECOUNTMAGIC_B5           = 0xBB,
    DSPCMD_BYTECOUNTMAGIC_B4           = 0xAD,

  };

  enum DispatcherCommands{
    DSPCMD_NOT_SET                    = 0,
    DSPCMD_ACK                        = 6,
    DSPCMD_IDENT                      = 'I',
    DSPCMD_DATA_VERSION               = 'V',
    DSPCMD_SET_PRESCALE               = 'P',
    DSPCMD_SET_EVENTSTREAM_SINGLE     = 'S',
    DSPCMD_SET_EVENTSTREAM_MAX_EVENTS = 'M',
    DSPCMD_SET_EVENTSTREAM_CONTINOUS  = 'C',
    DSPCMD_SET_TRIGGER_MASK_1         = 100,
    DSPCMD_SET_TRIGGER_MASK_2         = 101,
    DSPCMD_SET_TRIGGER_MASK_3         = 102,
    DSPCMD_START_DATAFLOW             = 23,
    DSPCMD_STOP_DATAFLOW              = 24,
    DSPCMD_NEXT_EVENT                 = 25,
    DSPCMD_END                        = 'E',
    DSPCMD_END_EVENT_STREAM           = 0xFFFFFFFF
  };

  enum DispatcherCommandDefaults{
    DEFAULT_SINGLE_EVENT_MODE = -1,
    DEFAULT_MAX_EVENTS        = 1000,
    DEFAULT_TRIGGER_MASK      = 0xFFFFFFFF,
    DEFAULT_PRESCALE_MASK     = 0xFFFFFFFF,
    DEFAULT_PRESCALE_VALUE    = 1
  };

  struct DspCmd {
    DispatcherCommands cmd;
    std::vector<int32_t> data;
  };


  DispatcherCMD(                                                        void* output_buff, bool ack=false);
  DispatcherCMD(DispatcherCommands cmd,                                 void* output_buff, bool ack=false);
  DispatcherCMD(DispatcherCommands cmd, int32_t value,                  void* output_buff, bool ack=false);
  DispatcherCMD(DispatcherCommands cmd, int32_t value1, int32_t value2, void* output_buff, bool ack=false);
  DispatcherCMD(DispatcherCommands cmd, std::vector<int32_t> values,    void* output_buff, bool ack=false);

  // Constructor for deserialization of a received command
  DispatcherCMD(uint8_t* input_buffer);
  
  ~DispatcherCMD();
  
  void addCommand(DispatcherCommands cmd, int32_t value);
  void addCommand(DispatcherCommands cmd, std::vector<int32_t> values);

  void reset();
  ssize_t write();
  //bool sendList();

  static uint8_t* InsertByteCountMagic(uint8_t* insertion_pointer); //! Function to insert magic numbers into the buffer

  DispatcherCommands   getCommand(){return aCommand.cmd;};
  std::vector<int32_t> getCommandData(){return aCommand.data;};

  void* getExternalBuffer(){return external_buff;}


  static bool cmdNeedsAck( DispatcherCommands cmd){

    switch( cmd ){
      // Commands that need Acknowledgement
    case DSPCMD_IDENT :                     
    case DSPCMD_SET_EVENTSTREAM_SINGLE :    
      return true;
      break;
      // Commands that do NOT need Acknowledgement
    case DSPCMD_DATA_VERSION :              
    case DSPCMD_SET_PRESCALE :              
    case DSPCMD_SET_EVENTSTREAM_MAX_EVENTS :
    case DSPCMD_SET_EVENTSTREAM_CONTINOUS : 
    case DSPCMD_SET_TRIGGER_MASK_1 :        
    case DSPCMD_SET_TRIGGER_MASK_2 :        
    case DSPCMD_SET_TRIGGER_MASK_3 :        
    case DSPCMD_START_DATAFLOW :            
    case DSPCMD_STOP_DATAFLOW :             
    case DSPCMD_NEXT_EVENT :                 
    case DSPCMD_END :                       
    case DSPCMD_END_EVENT_STREAM :          
    case DSPCMD_NOT_SET :                    
    case DSPCMD_ACK :
      return false;
      break;
    default:
      return false;
    }
  }

  static bool cmdNeedsAck( DispatcherCMD& cmd){ return cmdNeedsAck(cmd.getCommand());  }

 protected:


 private:

  void Init(void* output_buff);
  void setCommandDefault(DispatcherCommands cmd);
  void serializeCmd();
  bool deserializeCmd(void* input_buffer);

  DspCmd aCommand;                //! A command
  bool Acknowledge_flag;

  void*   external_buff;          //! Pointer to the external output buffer
  uint8_t internal_buff[256];     //! 256byte internal buffer for serialization

  // Base address of the buffer we write to
  union {
    void*     buff_base;
    uint8_t*  buff8_base;
    uint16_t* buff16_base;
    uint32_t* buff32_base;
  };

  union {
    uint32_t magicNumberWord[2];
    uint8_t  magicNumberBytes[8];
  };
  
  // Current address of insertion point in the buffer we write to
  union {
    void*     buff;
    uint8_t*  buff8;
    uint16_t* buff16;
    uint32_t* buff32;
  };

};

//} // endnamespace

#endif /* DISPATCHERCMD_H */
