#include "EventDispatcher_CommandSet/DispatcherCmd.h"

#include <sys/types.h>
#include <stdint.h>
#include <unistd.h>
#include <string.h>

// These macros allow for the correct setting and extracting of bit fields
// from the given size word.
//#if __BYTE_ORDER == __LITTLE_ENDIAN
//#define SWAB32(word) (word)
//#define SWAB64(word) (word)
//#elif __BYTE_ORDER == __BIG_ENDIAN
#define SWAB32(word) ( (((word) & 0xFF000000) >> 24) | (((word) & 0x00FF0000) >> 8) | (((word) & 0x0000FF00) << 8 | (((word) & 0x000000FF) << 24)) )
#define SWAB64(word) ( SWAB32( (uint32_t)((word)>>32) ) | (uint64_t)SWAB32( (uint32_t)(word) )<<32 )
//#endif


DispatcherCMD::DispatcherCMD(uint8_t* input_buffer){
  Acknowledge_flag = false;
  external_buff = NULL;

  deserializeCmd(input_buffer);
  
  buff_base = internal_buff;
  buff = buff_base;
  
}

DispatcherCMD::DispatcherCMD(void* output_buff, bool ack){
  // Initialize
  Init(output_buff);
  Acknowledge_flag = ack;
}

DispatcherCMD::DispatcherCMD(DispatcherCommands cmd, void* output_buff, bool ack){
  // Initialize
  Init(output_buff);
  Acknowledge_flag = ack;
  // Setup the command with default values
  setCommandDefault(cmd);
  // Serialize the command to the buffer
  serializeCmd();
}

DispatcherCMD::DispatcherCMD(DispatcherCommands cmd, int32_t value, void* output_buff, bool ack){
  // Initialize
  Init(output_buff);
  Acknowledge_flag = ack;

  // Setup the command
  aCommand.cmd = cmd;
  aCommand.data.push_back(value);

  // Serialize the command to the buffer
  serializeCmd();
}

DispatcherCMD::DispatcherCMD(DispatcherCommands cmd, int32_t value1, int32_t value2, void* output_buff, bool ack){
  // Initialize
  Init(output_buff);

  Acknowledge_flag = ack;
  
  // Setup the command
  aCommand.cmd = cmd;
  aCommand.data.push_back(value1);
  aCommand.data.push_back(value2);

  // Serialize the command to the buffer
  serializeCmd();
}

DispatcherCMD::DispatcherCMD(DispatcherCommands cmd, std::vector<int32_t> values, void* output_buff, bool ack){
  // Initialize
  Init(output_buff);
  
  Acknowledge_flag = ack;

  // Setup the command
  aCommand.cmd = cmd;
  aCommand.data = values;

  // Serialize the command to the buffer
  serializeCmd();
}


DispatcherCMD::~DispatcherCMD(){
}

void DispatcherCMD::addCommand(DispatcherCommands cmd, int32_t value){
  aCommand.cmd = cmd;
  aCommand.data.clear();
  aCommand.data.push_back(value);

  serializeCmd();
  return;
}

void DispatcherCMD::addCommand(DispatcherCommands cmd, std::vector<int32_t> values){
  aCommand.cmd = cmd;
  aCommand.data.clear();
  aCommand.data = values;

  serializeCmd();
  return;
}

void DispatcherCMD::reset(){
  // Zero the internal buffer
  memset(internal_buff, 0, 256);

  // Reset the insertion point of the 
  // the internal buffer
  buff = buff_base;
  *buff8 =0;
  buff8++;

  aCommand.cmd = DSPCMD_NOT_SET;
  aCommand.data.clear();
}

ssize_t DispatcherCMD::write(){
  ssize_t cmdsize;

  // Calculate the length of the command
  cmdsize = buff8 - buff8_base + 1;
  
  // Copy the serialized command out to the external buffer
  memcpy(external_buff, internal_buff, cmdsize);

  return cmdsize;
}

void DispatcherCMD::Init(void* output_buffer){
  Acknowledge_flag = false;
  
  external_buff = output_buffer;

  // Zero the internal buffer
  memset(internal_buff, 0, 256);

  // We initially serialize to the internal
  // buffer
  buff_base = internal_buff;

  // Set the buffer insertion point 
  // to the start of the buffer
  buff = buff_base;

  // Write a leading zero byte count
  *buff8 = 0;

  // Advance one byte so we are
  // ready for command insertion
  buff8++;
  
  aCommand.cmd = DSPCMD_NOT_SET;
  aCommand.data.clear();
}

uint8_t* DispatcherCMD::InsertByteCountMagic(uint8_t* insert_buffer){

  // Copy the insertion pointer so we don't modify the original
  uint8_t* local_buffp = insert_buffer;  

  // Insert (write) the 64Bit magic number "0xBBADBEEF 0xBEEFBBAD"
  *local_buffp = (uint8_t)DSPCMD_BYTECOUNTMAGIC_B0;  ++local_buffp;
  *local_buffp = (uint8_t)DSPCMD_BYTECOUNTMAGIC_B1;  ++local_buffp;
  *local_buffp = (uint8_t)DSPCMD_BYTECOUNTMAGIC_B2;  ++local_buffp;
  *local_buffp = (uint8_t)DSPCMD_BYTECOUNTMAGIC_B3;  ++local_buffp;
  *local_buffp = (uint8_t)DSPCMD_BYTECOUNTMAGIC_B4;  ++local_buffp;
  *local_buffp = (uint8_t)DSPCMD_BYTECOUNTMAGIC_B5;  ++local_buffp;
  *local_buffp = (uint8_t)DSPCMD_BYTECOUNTMAGIC_B6;  ++local_buffp;
  *local_buffp = (uint8_t)DSPCMD_BYTECOUNTMAGIC_B7;  ++local_buffp;
  return local_buffp;
}

bool DispatcherCMD::deserializeCmd(void* input_buffer){
  bool valid   = true;
  uint8_t size = 0;

  // First take off the byte count
  size = ((uint8_t*)input_buffer)[0];

#ifdef USE_MAGIC_NO
  // Then take off the magic numbers
  uint32_t* magic = (uint32_t*) (&(((uint8_t*)input_buffer)[1]));
  
  if( !(DSPCMD_BYTECOUNTMAGIC1 == magic[0] )){
    valid = false;
  }

  if( !(DSPCMD_BYTECOUNTMAGIC2 == magic[1] )){
    valid = false;
  }

  // Check the magic numbers
  // if the check fails then set valid = false
#endif

  // Copy the buffer
  if(valid){
    memcpy(internal_buff, input_buffer, size);
  }else{
    return false;  // Invalid buffer
  }

  // Figure out where the end of the buffer is
  uint8_t* endbuff = internal_buff + size;

  // At this point we have a local copy of the data
  // we set a pointer to the start of it
  buff8 = internal_buff;
  size  = buff8[0];
  ++buff8;

#ifdef USE_MAGIC_NO
  magicNumberWord[0] = buff32[0];
  magicNumberWord[1] = buff32[1];
  ++buff32;  // Advance over the first magic word
  ++buff32;  // Advance over the second magic word
#endif
  // Then take the command character
  aCommand.cmd = static_cast<DispatcherCommands>(buff8[0]);
  ++buff8;
  
  // Unpack the data words into a the data vector untill we
  // get to the end of the buffer
  while( endbuff > buff8){
    aCommand.data.push_back(SWAB32(buff32[0]));
    ++buff32;
  }

  // The last byte in the buffer should be the end mark, check that it is
  if(DSPCMD_END != endbuff[0]){
    valid = false;
  }
  
  return valid;
}

void DispatcherCMD::serializeCmd(){
  // Zero the leading byte count
  *buff8_base = 0;

#ifdef USE_MAGIC_NO
  buff8 = InsertByteCountMagic(buff8);
#endif

  // If we need to embed an acknowledgement flag
  // put that in next
  if(Acknowledge_flag){
    *buff8 = (uint8_t)DSPCMD_ACK;
    buff8++;
  }

  // Write the command into the buffer
  *buff8 = (uint8_t)aCommand.cmd;
  buff8++;
  
  // Write the data values into the buffer
  for(uint i=0; i<aCommand.data.size();++i ){
    *buff32 = SWAB32(aCommand.data[i]);
    buff32++;
  }

  // Write the end value
  *buff8 = (uint8_t)DSPCMD_END;
  buff8++;

  // Rewrite the leading byte count
  // first calculate the command size in bytes and check that it is 
  // able to fit into 1 byte (i.e. a value less than 255)
  uint8_t  cmdsize = (buff8 - buff8_base -1);
  uint32_t bsize   = (buff8 - buff8_base -1);
  if(bsize < 255){
    *buff8_base = cmdsize;
  }else{
    // If the size is over 1 byte we put in the overflow value
    *buff8_base = 255;
  }

  // Back up the insertion point one byte 
  // incase we want to insert more commands
  buff8--;
  
  return;
}

void DispatcherCMD::setCommandDefault(DispatcherCommands cmd){
  // Setup the command with default parameters
  aCommand.cmd = cmd;
  
  switch(cmd){
    // This block of commands have no parameter
  case DSPCMD_IDENT:
  case DSPCMD_DATA_VERSION:
  case DSPCMD_START_DATAFLOW:
  case DSPCMD_STOP_DATAFLOW:
  case DSPCMD_NEXT_EVENT:
  case DSPCMD_SET_EVENTSTREAM_SINGLE:
    break;
    // This block of commands take one parameter
  case DSPCMD_SET_EVENTSTREAM_CONTINOUS:
    aCommand.data.push_back(-1);
    break;
  case DSPCMD_SET_EVENTSTREAM_MAX_EVENTS:
    aCommand.data.push_back(DEFAULT_MAX_EVENTS);
    break;
  case DSPCMD_SET_TRIGGER_MASK_1:
  case DSPCMD_SET_TRIGGER_MASK_2:
  case DSPCMD_SET_TRIGGER_MASK_3:
    aCommand.data.push_back(DEFAULT_TRIGGER_MASK);
    // This block of commands take two parameters
  case DSPCMD_SET_PRESCALE:
    aCommand.data.push_back(DEFAULT_PRESCALE_MASK);
    aCommand.data.push_back(DEFAULT_PRESCALE_MASK);
    break;
  default:
    break;
  }//endswitch
  return; 
}



