/**
 * NdmcClientService header file
 * $Id: NdmcClientService.h,v 1.2 2010/07/27 15:11:21 kasahara Exp $
 */
#ifndef __NDMC_CLIENTSERVICE_H_
#define __NDMC_CLIENTSERVICE_H_

#include "NovaDAQMonitorClient/Ndmc.h"
extern "C" {
#include "ganglia.h"
}

NDMC_NAMESPACE_BEGIN

/**
 * NdmcClientService
 *
 * This class implements the NdmcClientService singleton interface to the Ganglia
 * global environment for initiating the Ganglia global context, udp send channels,
 * and gmond configuration.  An NdmcClient would use this service for access
 * this Ganglia data, which should be common for all clients on this node.  
 * 
 * @author Susan Kasahara
 * @author <a href="mailto:schubert@physics.umn.edu">schubert@physics.umn.edu</a> 
 * @date May 5 2010
 */

class NdmcClientService {

 private:

  // Constructor/Destructor private.  Access via static instance() method
  NdmcClientService();
  virtual ~NdmcClientService();

 public:

  static NdmcClientService& instance();
  static std::string getCONFIGFILE() { return _CONFIGFILE; }
  static void setCONFIGFILE(std::string configfile) { _CONFIGFILE = configfile; }

  // State changing methods

  // State testing methods
  bool isEnabled() const { return _enabled; }
  Ganglia_pool_t getGlobalContext() const { return _globalContext; }
  Ganglia_udp_send_channels_t getSendChannels() const { return _sendChannels; }

 private:

  bool _enabled; /**< NdmcClientService has been initialized */
  Ganglia_pool_t _globalContext; /**< Ganglia global context */
  Ganglia_gmond_config_t _gmondConfig; /**< Ganglia gmond configuration */
  Ganglia_udp_send_channels_t _sendChannels; /**< Ganglia send channels */
  
  bool _initGanglia();

  // Disable copy and assignment ctor by making them private and w/no imp
  NdmcClientService(const NdmcClientService&); /**< Disabled */
  NdmcClientService& operator=(const NdmcClientService&); /**< Disabled */

  static std::string _CONFIGFILE;

};

NDMC_NAMESPACE_END

#endif

