/**
 * NdmcMetricABC header file
 * $Id: NdmcMetricABC.h,v 1.5 2011/11/14 01:12:27 kasahara Exp $
 */
#ifndef __NDMC_METRICABC_H_
#define __NDMC_METRICABC_H_  // enums and typedefs

NDMC_NAMESPACE_BEGIN

/**
 * NdmcMetricABC
 *
 * This class is an abstract base class for a custom metric to be sent by a client
 * to the DAQ Monitor Server.
 * 
 * @author Susan Kasahara 
 * @author <a href="mailto:schubert@physics.umn.edu">schubert@physics.umn.edu</a>
 * @date May 06 2010 (creation)
 */
class NdmcMetricABC {

 public:

  // Constructors/Destructors
  NdmcMetricABC() {}
  virtual ~NdmcMetricABC() {}

  // State changing methods
  virtual bool send() = 0;
  virtual void resetValue() = 0;
  virtual void setPartition(int32_t partition) = 0;
  virtual void setApplicationName(std::string appName) = 0;
  virtual void setGroupName(std::string groupName) = 0;

  // Accessor methods
  virtual std::string getName() const = 0;
  virtual int32_t getPartition() const = 0;
  virtual std::string getApplicationName() const = 0;
  virtual std::string getGroupName() const = 0;

 private:

};

NDMC_NAMESPACE_END

#endif

