/**
 * NdmcTrace header file
 * 
 * This file should be included for sending TRACE messages. 
 * Because trace header files associated with other packages may redefine
 * the TRACE_NAME variable, this header file should be the last header
 * file included in a NovaDAQMonitorClient class implementation file.  
 *
 * Normally trace output is only output to the trace circular buffer.
 * To have the trace messages output to stdout via printf:
 *   echo NDMC=0xffffffff,0x0,0x0,0xffffffff > /proc/trace/level
 *   echo mode=0x9 > /proc/trace/control
 * before executing the test binaries and then return to the default:
 *   echo NDMC=0xffffffff,0x0,0x0,0x0 > /proc/trace/level
 *   echo mode=0x1 > /proc/trace/control
 * after job execution.
 * The four /proc/trace/level bit masks allow for 32 message levels for
 * each of 4 different logging backends.  The first logging backend is the
 * trace circular buffer.  The last logging backend is a trace defined
 * printf function.  The last three logging backends can be set to user-defined
 * functions.
 * The 4 backends are activated using the mode bit mask in the 
 * /proc/trace/control file, e.g. mode = 0xf activates all four.

 * $Id: NdmcTrace.h,v 1.1.1.1 2010/04/28 18:13:15 kasahara Exp $
 */

#ifndef __NDMC_TRACE_H_
#define __NDMC_TRACE_H_

#include <stdio.h>
#include "NovaDAQMonitorClient/Ndmc.h"

extern "C" {

#ifdef TRACE_NAME
#  undef TRACE_NAME
#endif
#define TRACE_NAME "NDMC"
#ifdef TRACE_FUNCTIONS
# undef TRACE_FUNCTIONS
#endif
#include <sys/sysinfo.h>
#if defined(TRACE_KERNEL) 
#  include <linux/trace.h>  
#else
#  ifdef TRACE
#    undef TRACE
#  endif
#  define TRACE(lvl,fmt,args...) if ( lvl <= NDMC_NAMESPACE::getTraceLevel() ) { fprintf(stderr,fmt,## args); fprintf(stderr,"\n"); }
#endif /* TRACE_KERNEL */

}

#endif // __NDMC_TRACE_H_ 
