//  This file (NdmcClient_test.cc) was created by Ron Rechenmacher <ron@fnal.gov> on
//  Mar 31, 2011. "TERMS AND CONDITIONS" governing this file are in the README
//  or COPYING file. If you do not have such a file, one can be obtained by
//  contacting Ron or Fermi Lab in Batavia IL, 60510, phone: 630-840-3000.
//  $RCSfile: NdmcClient_test.cc,v $
//  rev="$Revision: 1.2 $$Date: 2011/04/02 17:40:22 $";

#include <stdio.h>		// printf
#include <stdlib.h>		// srand

#include "NovaDAQMonitorClient/NdmcClient.h"
#include "NovaDAQMonitorClient/NdmcMetric.h"

int
main(  int	argc
     , char	*argv[] )
{
    nova::novadaqmonitorclient::NdmcClient	_daqMonitorClient;  // starts a thread
    int						sts;

    // metric 1 - memory buffer occupied by incomplete events
    nova::novadaqmonitorclient::NdmcMetric<double>*	met_buf_incomplete;
    sts = _daqMonitorClient.createMetric<double>(  "buf_incomplete"
					     , "Bytes", nova::novadaqmonitorclient::RATE
					     , met_buf_incomplete );
    if (sts < 0)
    {   printf("Failed to create metric 1\n");
	return (1);
    }

    // metric 2 - memory buffer occupied by complete events
    nova::novadaqmonitorclient::NdmcMetric<double>*	met_buf_complete;
    sts = _daqMonitorClient.createMetric<double>(  "buf_complete"
					     , "Bytes", nova::novadaqmonitorclient::RATE
					     , met_buf_complete );
    if (sts < 0)
    {   printf("Failed to create metric 2\n");
	return (1);
    }


    // metric 3 - total memory buffer size
    nova::novadaqmonitorclient::NdmcMetric<uint32_t>*	met_err_count;
    sts = _daqMonitorClient.createMetric<uint32_t>(  "err_count"
					     , "Count", nova::novadaqmonitorclient::RATE
					     , met_err_count );
    if (sts < 0)
    {   printf("Failed to create metric 3\n");
	return (1);
    }


    unsigned int random_min_complete = 40000;
    unsigned int random_min_incomplete = 10000;
    double random_range = 10000.;

    srand( 1 );

    while (1)
    {
	// First metric is memory buffer occupied by incomplete events
	double buf_incomplete = 
	    random_range*(unsigned int)( (double)rand() / ((double)(RAND_MAX)+(double)(1)) )
	    + random_min_incomplete;
	_daqMonitorClient.setValue( met_buf_incomplete, buf_incomplete );

	// Second metric is memory buffer occupied by complete events
	double buf_complete = 
	    random_range*(unsigned int)( (double)rand() / ((double)(RAND_MAX)+(double)(1)) )
	    + random_min_complete;
	_daqMonitorClient.setValue( met_buf_complete, buf_complete );

	// Third metric is total memory buffer size
	uint32_t err_count = rand() % 5;
	_daqMonitorClient.setValue( met_err_count, err_count );

	sleep(1);
    }
    return (0);
}   // main
