#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <time.h>

#ifdef __cplusplus
extern "C" {
#endif
#include "ganglia.h"
#ifdef __cplusplus
} 
#endif

Ganglia_pool_t global_context;
Ganglia_metric_t gmetric;
Ganglia_gmond_config_t gmond_config;
Ganglia_udp_send_channels_t send_channels;

int main ( int argc, char *argv[] ) {

  int rval;

  /* create the global context */
  global_context = Ganglia_pool_create(NULL);
  if ( !global_context ) {
    fprintf(stderr,"Unable to create global context. Exiting.\n");
    exit(1);
  }

  /* parse the configuration file */
  int fall_back_to_default = 1;  // use default location for configuration /etc/ganglia/gmond.conf
  char gmondconfigstr[] = "/etc/ganglia/gmond.conf";
  gmond_config = Ganglia_gmond_config_create(gmondconfigstr,fall_back_to_default);

  /* build the udp send channels */
  send_channels = Ganglia_udp_send_channels_create(global_context, gmond_config);
  if ( !send_channels ) {
    fprintf(stderr,"Unable to create ganglia send channels. Exiting.\n");
    exit(1);
  }

  /* create the message container */
  gmetric = Ganglia_metric_create(global_context);
  if ( !gmetric ) {
      fprintf(stderr,"Unable to allocate gmetric structure.  Exiting.\n");
      exit(1);
  }  

  unsigned int random_min_complete = 40000;
  unsigned int random_min_incomplete = 10000;
  unsigned int random_range = 10000;
  unsigned int constant_value = 100000;

  while (1) {

    srand(time(NULL));

    // First metric is memory buffer occupied by incomplete events
    unsigned int buf_incomplete = 
      random_range*(unsigned int)( (double)rand() / ((double)(RAND_MAX)+(double)(1)) )
                       + random_min_incomplete;
    char sbuf_incomplete[20];
    sprintf(sbuf_incomplete,"%i",buf_incomplete*1000);

    // Second metric is memory buffer occupied by complete events
    unsigned int buf_complete = 
      random_range*(unsigned int)( (double)rand() / ((double)(RAND_MAX)+(double)(1)) )
                       + random_min_complete;
    char sbuf_complete[20];
    sprintf(sbuf_complete,"%i",buf_complete*1000);

    // Third metric is total memory buffer size
    unsigned int buf_total = constant_value;
    char sbuf_total[20];
    sprintf(sbuf_total,"%i",buf_total*1000);


    // Arguments are: gmetric structure, 
    //                name of the metric (char*), 
    //                value of the metric (char*),
    //     type of the metric (string|int8|uint8|int16|uint16|int32|uint32|float|double) (char*)
    //     unit of the metric (char*) if given, 
    //     slope of metric (zero | positive | negative | both) (slope_t=cstr_to_slope(char*)), 
    //     tmax max time in sec between gmetric calls (int),
    //     dmax lifetime in sec of this metric (int)
    char namestr[] = "buf_incomplete";
    char typestr[] = "uint32";
    char unitstr[] = "Bytes";
    char slopestr[] = "both";
    rval = Ganglia_metric_set(gmetric,namestr,sbuf_incomplete,typestr,unitstr,
                              cstr_to_slope(slopestr),60,0);
    switch(rval)
      {
      case 1:
        fprintf(stderr,"gmetric parameters invalid. exiting.\n");
        exit(1); 
      case 2:
        fprintf(stderr,"one of your parameters has an invalid character '\"'. exiting.\n");
        exit(1);
      case 3:
        fprintf(stderr,"the type parameter is not a valid type. exiting.\n");
        exit(1);
      case 4:
        fprintf(stderr,"the value parameter does not represent a number. exiting.\n");
        exit(1);
      }

    rval = Ganglia_metric_send(gmetric, send_channels);
    if ( rval ) {
        fprintf(stderr,"There was an error sending to %d of the send channels.\n", rval);
    }

    char namestr2[] = "buf_complete";
    char typestr2[] = "uint32";
    char unitstr2[] = "Bytes";
    char slopestr2[] = "both";
    rval = Ganglia_metric_set(gmetric,namestr2,sbuf_complete,typestr2,unitstr2,
                              cstr_to_slope(slopestr2),60,0);
    switch(rval)
      {
      case 1:
        fprintf(stderr,"gmetric parameters invalid. exiting.\n");
        exit(1); 
      case 2:
        fprintf(stderr,"one of your parameters has an invalid character '\"'. exiting.\n");
        exit(1);
      case 3:
        fprintf(stderr,"the type parameter is not a valid type. exiting.\n");
        exit(1);
      case 4:
        fprintf(stderr,"the value parameter does not represent a number. exiting.\n");
        exit(1);
      }

    rval = Ganglia_metric_send(gmetric, send_channels);
    if ( rval ) {
        fprintf(stderr,"There was an error sending to %d of the send channels.\n", rval);
    }

    char namestr3[] = "buf_total";
    char typestr3[] = "uint32";
    char unitstr3[] = "Bytes";
    char slopestr3[] = "both";
    rval = Ganglia_metric_set(gmetric,namestr3,sbuf_total,typestr3,unitstr3,
                              cstr_to_slope(slopestr3),60,0);
    switch(rval)
      {
      case 1:
        fprintf(stderr,"gmetric parameters invalid. exiting.\n");
        exit(1); 
      case 2:
        fprintf(stderr,"one of your parameters has an invalid character '\"'. exiting.\n");
        exit(1);
      case 3:
        fprintf(stderr,"the type parameter is not a valid type. exiting.\n");
        exit(1);
      case 4:
        fprintf(stderr,"the value parameter does not represent a number. exiting.\n");
        exit(1);
      }

    rval = Ganglia_metric_send(gmetric, send_channels);
    if ( rval ) {
        fprintf(stderr,"There was an error sending to %d of the send channels.\n", rval);
    }

    sleep(15);
  }


  /* cleanup */
  Ganglia_metric_destroy(gmetric); /* not really necessary but for symmetry */
  Ganglia_pool_destroy(global_context);

  return 0;
}

