#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <time.h>
#include "NovaDAQMonitorClient/NdmcMetric.h"

using namespace NDMC_NAMESPACE;

int main ( int argc, char *argv[] ) {


  // The arguments are:
  // metric name, 
  // unit of the metric, e.g. "Bytes", "Hz","Tax Dollars Consumed".  This appears in the
  //                     ganglia graph produced in the ganglia display.  
  // slope (GANGLIA_SLOPE_ZERO, POSITIVE, NEGATIVE, BOTH)
  //       If slope GANGLIA_SLOPE_POSITIVE is chosen, the resulting rrd database entry will 
  //       have a Data Source Type (DST) COUNTER format.  All others will result in a 
  //       database entry DST with GAUGE format.
  //       A DST of COUNTER format will save the rate of change
  //       of the value over a step period.  This assumes that the value is always
  //       always increasing.  GAUGE does not save the rate of change, it saves the
  //       the value itself.
  //
  // tmax = the maximum time in seconds between gmetric calls, default='60'.
  // dmax = the lifetime in seconds of this metric, default='0' => forever.
  //        The default setting is not recommended as with this setting the last metric
  //        sent will persist forever in the display. 
  // A rule of thumb is to set tmax to twice as long as the sampling interval, and dmax
  // four times as long as the sampling interval.  


  // Set the sampling interval to 15 seconds.
  int32_t samplingInterval = 15;
  int32_t tmax = 2*samplingInterval;
  int32_t dmax = 2*tmax;

  // An example of managing metrics individually.
  NdmcMetric<double> ndmc_buf_incomplete("buf_incomplete","Bytes",VALUE,tmax,dmax,
                                          GANGLIA_SLOPE_BOTH);
  NdmcMetric<double> ndmc_buf_complete("buf_complete","Bytes",VALUE,
                                          tmax,dmax,GANGLIA_SLOPE_BOTH);
  NdmcMetric<uint32_t> ndmc_err_count("err_count","Count",VALUE,
                                          tmax,dmax,GANGLIA_SLOPE_BOTH);

  unsigned int random_min_complete = 40000;
  unsigned int random_min_incomplete = 10000;
  double random_range = 10000.;

  srand(time(NULL));

  while (1) {

    // First metric is memory buffer occupied by incomplete events
    double buf_incomplete = 
      random_range*( (double)rand() / ((double)(RAND_MAX)+(double)(1)) )
                       + random_min_incomplete;
    ndmc_buf_incomplete.setValue(buf_incomplete);
    ndmc_buf_incomplete.send();

    // Second metric is memory buffer occupied by complete events
    double buf_complete = 
      random_range*( (double)rand() / ((double)(RAND_MAX)+(double)(1)) )
                       + random_min_complete;
    ndmc_buf_complete.setValue(buf_complete);
    ndmc_buf_complete.send();

    int err_count = rand() % 5;
    ndmc_err_count.setValue(err_count);
    ndmc_err_count.send();

    sleep(samplingInterval);
  }

  return 0;
}

