#include "NdmcMetricTest.h"
using namespace NDMC_NAMESPACE;

/**
 * Register the NdmcMetricTest suite in the registry of suites in a "folder"
 * of name "NovaDAQMonitorClient".  Store the folder in the default registry path.
 * $Id: NdmcMetricTest.cpp,v 1.18 2014/02/03 00:39:56 kasahara Exp $
 */
CPPUNIT_TEST_SUITE_NAMED_REGISTRATION( NdmcMetricTest, "NovaDAQMonitorClient" );
CPPUNIT_REGISTRY_ADD_TO_DEFAULT("NovaDAQMonitorClient");

/**
 * A public method to setUp the environment for the test.  This will be invoked before
 * every test in the fixture to get a fresh start.
 */
void NdmcMetricTest::setUp() {

  _configFile = getConfigFullFilePathName("unittest/gmond.conf");
  NdmcClientService::setCONFIGFILE(_configFile);

}
/**
 * A public method to tearDown the environment for the test.  This will be invoked after
 * every test in the fixture to clean up allocated memory.
 */
void NdmcMetricTest::tearDown() {
  std::cout.flush();
}

/**
 * A test method to test that NdmcMetric::instance()
 * works correctly.
 */
void NdmcMetricTest::testCtor() {

  ::mf::LogInfo("unittest") << "NdmcMetricTest::testCtor." << std::endl;

  std::string name="dummy";
  std::string unit="bytes";
  ganglia_slope_t slope = GANGLIA_SLOPE_POSITIVE;

  NdmcMetric<uint32_t> metric(name.c_str(),unit.c_str(),VALUE,15,5,slope);
  CPPUNIT_ASSERT(metric.getName() == name);
  CPPUNIT_ASSERT(metric.getUnit() ==  unit);
  CPPUNIT_ASSERT(metric.getMetricType() == VALUE);
  CPPUNIT_ASSERT(metric.getMaxDeltaTime() == 15);
  CPPUNIT_ASSERT(metric.getLifeTime() == 5);
  CPPUNIT_ASSERT(metric.getSlope() == slope);
  CPPUNIT_ASSERT(metric.getType() == "uint32");
  CPPUNIT_ASSERT(metric.getValue() == 0);
  CPPUNIT_ASSERT(metric.getSumW() == 0);
  CPPUNIT_ASSERT(metric.getNW() == 0);
  CPPUNIT_ASSERT(metric.getSumWX() == 0);
  CPPUNIT_ASSERT(metric.getSumWXSq() == 0);
  CPPUNIT_ASSERT(metric.isValueSet() == false);
  CPPUNIT_ASSERT(metric.getPartition() == -1);
  CPPUNIT_ASSERT(metric.getApplicationName() == "");
  CPPUNIT_ASSERT(metric.getGroupName() == "");
  CPPUNIT_ASSERT(metric.getMetricName() == metric.getName());
  uint32_t lastValueSent = 99;
  CPPUNIT_ASSERT(metric.getLastValueSent(lastValueSent) == false);
  CPPUNIT_ASSERT(lastValueSent == 99);

  int32_t partition = 2;
  std::string appName = "dcmapp";
  std::string groupName = "dcmapp P2";
  NdmcMetric<uint32_t> metric2(name.c_str(),unit.c_str(),VALUE,15,5,slope,
                               partition, appName, groupName);
  CPPUNIT_ASSERT(metric2.getName() == name);
  CPPUNIT_ASSERT(metric2.getUnit() ==  unit);
  CPPUNIT_ASSERT(metric2.getMetricType() == VALUE);
  CPPUNIT_ASSERT(metric2.getMaxDeltaTime() == 15);
  CPPUNIT_ASSERT(metric2.getLifeTime() == 5);
  CPPUNIT_ASSERT(metric2.getSlope() == slope);
  CPPUNIT_ASSERT(metric2.getType() == "uint32");
  CPPUNIT_ASSERT(metric2.getValue() == 0);
  CPPUNIT_ASSERT(metric2.getSumW() == 0);
  CPPUNIT_ASSERT(metric2.getNW() == 0);
  CPPUNIT_ASSERT(metric2.getSumWX() == 0);
  CPPUNIT_ASSERT(metric2.getSumWXSq() == 0);
  CPPUNIT_ASSERT(metric2.isValueSet() == false);
  CPPUNIT_ASSERT(metric2.getPartition() == partition);
  CPPUNIT_ASSERT(metric2.getApplicationName() == appName);
  CPPUNIT_ASSERT(metric2.getGroupName() == groupName);
  CPPUNIT_ASSERT(metric2.getMetricName() == "dummy_Adcmapp_P2");

}

/**
 * A test method to test that NdmcMetric::getType
 * works correctly for uint32_t.
 */
void NdmcMetricTest::testGetTypeUint32_t() {

  ::mf::LogInfo("unittest") << "NdmcMetricTest::testGetTypeUint32_t." << std::endl;

  std::string name="dummy";
  NdmcMetric<uint32_t> metric(name.c_str());
  CPPUNIT_ASSERT(metric.getType() == "uint32");

}

/**
 * A test method to test that NdmcMetric::getType
 * works correctly for int32_t.
 */
void NdmcMetricTest::testGetTypeInt32_t() {

  ::mf::LogInfo("unittest") << "NdmcMetricTest::testGetTypeInt32_t." << std::endl;

  std::string name="dummy";
  NdmcMetric<int32_t> metric(name.c_str());
  CPPUNIT_ASSERT(metric.getType() == "int32");
 
}

/**
 * A test method to test that NdmcMetric::getType
 * works correctly for uint16_t.
 */
void NdmcMetricTest::testGetTypeUint16_t() {

  ::mf::LogInfo("unittest") << "NdmcMetricTest::testGetTypeUint16_t." << std::endl;

  std::string name="dummy";
  NdmcMetric<uint16_t> metric(name.c_str());
  CPPUNIT_ASSERT(metric.getType() == "uint16");
 
}

/**
 * A test method to test that NdmcMetric::getType
 * works correctly for int16_t.
 */
void NdmcMetricTest::testGetTypeInt16_t() {

  ::mf::LogInfo("unittest") << "NdmcMetricTest::testGetTypeInt16_t." << std::endl;

  std::string name="dummy";
  NdmcMetric<int16_t> metric(name.c_str());
  CPPUNIT_ASSERT(metric.getType() == "int16");
 
}

/**
 * A test method to test that NdmcMetric::getType
 * works correctly for uint8_t.
 */
void NdmcMetricTest::testGetTypeUint8_t() {

  ::mf::LogInfo("unittest") << "NdmcMetricTest::testGetTypeUint8_t." << std::endl;

  std::string name="dummy";
  NdmcMetric<uint8_t> metric(name.c_str());
  CPPUNIT_ASSERT(metric.getType() == "uint8");
 
}

/**
 * A test method to test that NdmcMetric::getType
 * works correctly for int8_t.
 */
void NdmcMetricTest::testGetTypeInt8_t() {

  ::mf::LogInfo("unittest") << "NdmcMetricTest::testGetTypeInt8_t." << std::endl;

  std::string name="dummy";
  NdmcMetric<int8_t> metric(name.c_str());
  CPPUNIT_ASSERT(metric.getType() == "int8");
 
}


/**
 * A test method to test that NdmcMetric::getType
 * works correctly for float.
 */
void NdmcMetricTest::testGetTypeFloat() {

  ::mf::LogInfo("unittest") << "NdmcMetricTest::testGetTypeFloat." << std::endl;

  std::string name="dummy";
  NdmcMetric<float> metric(name.c_str());
  CPPUNIT_ASSERT(metric.getType() == "float");

 
}

/**
 * A test method to test that NdmcMetric::getType
 * works correctly for double.
 */
void NdmcMetricTest::testGetTypeDouble() {

  ::mf::LogInfo("unittest") << "NdmcMetricTest::testGetTypeDouble." << std::endl;

  std::string name="dummy";
  NdmcMetric<double> metric(name.c_str());
  CPPUNIT_ASSERT(metric.getType() == "double");
 
}

/**
 * A test method to test that NdmcMetric::getType
 * works correctly for unknown type.
 */
void NdmcMetricTest::testGetTypeUnknown() {

  ::mf::LogInfo("unittest") << "NdmcMetricTest::testGetTypeUnknown." << std::endl;

  std::string name="dummy";
  NdmcMetric<uint64_t> metric(name.c_str()); /**< 64-bit not supported by Ganglia */
  CPPUNIT_ASSERT(metric.getType() == "Unknown");
 
}

/**
 * A test method to test that NdmcMetric::setValue
 * works correctly for a metric of type VALUE.
 */
void NdmcMetricTest::testSetValueMetricTypeVALUE() {

  ::mf::LogInfo("unittest") << "NdmcMetricTest::testSetValueMetricTypeVALUE." << std::endl;

  std::string name="dummy";
  NdmcMetric<uint32_t> metricUInt32(name.c_str(),"unit",VALUE); 

  CPPUNIT_ASSERT(metricUInt32.isValueSet() == false);

  CPPUNIT_ASSERT(metricUInt32.setValue(10));
  CPPUNIT_ASSERT(metricUInt32.getValue() == 10);
  CPPUNIT_ASSERT(metricUInt32.isValueSet() == true);

  NdmcMetric<double> metricDouble(name.c_str(),"unit",VALUE);
  CPPUNIT_ASSERT(metricDouble.setValue(15.));
  CPPUNIT_ASSERT(metricDouble.getValue() == 15.);
  CPPUNIT_ASSERT(metricDouble.isValueSet() == true);
  CPPUNIT_ASSERT(metricDouble.setValue(-15.));
  CPPUNIT_ASSERT(metricDouble.getValue() == -15.);
  CPPUNIT_ASSERT(metricDouble.isValueSet() == true);
    
  NdmcMetric<int32_t> metricInt32(name.c_str(),"unit",VALUE);
  CPPUNIT_ASSERT(metricInt32.setValue(-1));
  CPPUNIT_ASSERT(metricInt32.getValue() == -1);
  CPPUNIT_ASSERT(metricInt32.isValueSet() == true);
  CPPUNIT_ASSERT(metricInt32.setValue(15));
  CPPUNIT_ASSERT(metricInt32.getValue() == 15);
  CPPUNIT_ASSERT(metricInt32.isValueSet() == true);

}

/**
 * A test method to test that NdmcMetric::_calculateSendValue
 * works correctly for a metric of type VALUE.
 */
void NdmcMetricTest::testCalculateSendValueMetricTypeVALUE() {

  ::mf::LogInfo("unittest") << "NdmcMetricTest::testCalculateSendValueMetricTypeVALUE." << std::endl;

  std::string name="dummy";
  NdmcMetric<uint32_t> metricUInt32(name.c_str(),"unit",VALUE); 
  CPPUNIT_ASSERT(metricUInt32.setValue(10));
  uint32_t sendValueUInt32 = 0;
  CPPUNIT_ASSERT(metricUInt32._calculateSendValue(sendValueUInt32) == true);
  CPPUNIT_ASSERT(sendValueUInt32 == 10);


  NdmcMetric<double> metricDouble(name.c_str(),"unit",VALUE);
  CPPUNIT_ASSERT(metricDouble.setValue(15.));
  CPPUNIT_ASSERT(metricDouble.setValue(-15));
  double sendValueDouble = 0;
  CPPUNIT_ASSERT(metricDouble._calculateSendValue(sendValueDouble) == true);
  CPPUNIT_ASSERT(sendValueDouble == -15.);
    
  NdmcMetric<int32_t> metricInt32(name.c_str(),"unit",VALUE);
  CPPUNIT_ASSERT(metricInt32.setValue(-1));
  CPPUNIT_ASSERT(metricInt32.setValue(15));
  int32_t sendValueInt32 = 0;
  CPPUNIT_ASSERT(metricInt32._calculateSendValue(sendValueInt32) == true);
  CPPUNIT_ASSERT(sendValueInt32 == 15);

}

/**
 * A test method to test that NdmcMetric::setValue
 * works correctly for a metric of type COUNTER.
 */
void NdmcMetricTest::testSetValueMetricTypeCOUNTER() {

  ::mf::LogInfo("unittest") << "NdmcMetricTest::testSetValueMetricTypeCOUNTER." << std::endl;

  std::string name="dummy";
  NdmcMetric<uint32_t> metricUInt32(name.c_str(),"unit",COUNTER); 
  CPPUNIT_ASSERT(metricUInt32.isValueSet() == false);

  CPPUNIT_ASSERT(metricUInt32.setValue(10));
  CPPUNIT_ASSERT(metricUInt32.getValue() == 10);
  CPPUNIT_ASSERT(metricUInt32.isValueSet() == true);
  CPPUNIT_ASSERT(metricUInt32.setValue(10));
  CPPUNIT_ASSERT(metricUInt32.getValue() == 20);
  CPPUNIT_ASSERT(metricUInt32.isValueSet() == true);


  NdmcMetric<double> metricDouble(name.c_str(),"unit",COUNTER);
  CPPUNIT_ASSERT(metricDouble.isValueSet() == false);
  CPPUNIT_ASSERT(metricDouble.setValue(15.));
  CPPUNIT_ASSERT(metricDouble.getValue() == 15.);
  CPPUNIT_ASSERT(metricDouble.isValueSet() == true);
  CPPUNIT_ASSERT(metricDouble.setValue(-15.));
  CPPUNIT_ASSERT(metricDouble.getValue() == 0.);
  CPPUNIT_ASSERT(metricDouble.isValueSet() == true);

    
  NdmcMetric<int32_t> metricInt32(name.c_str(),"unit",COUNTER);
  CPPUNIT_ASSERT(metricInt32.isValueSet() == false);
  CPPUNIT_ASSERT(metricInt32.setValue(-1));
  CPPUNIT_ASSERT(metricInt32.getValue() == -1);
  CPPUNIT_ASSERT(metricInt32.isValueSet() == true);
  CPPUNIT_ASSERT(metricInt32.setValue(15));
  CPPUNIT_ASSERT(metricInt32.getValue() == 14);
  CPPUNIT_ASSERT(metricInt32.isValueSet() == true);

}

/**
 * A test method to test that NdmcMetric::_calculateSendValue
 * works correctly for a metric of type COUNTER.
 */
void NdmcMetricTest::testCalculateSendValueMetricTypeCOUNTER() {

  ::mf::LogInfo("unittest") << "NdmcMetricTest::testCalculateSendValueMetricTypeCOUNTER." 
                            << std::endl;

  std::string name="dummy";
  NdmcMetric<uint32_t> metricUInt32(name.c_str(),"unit",COUNTER); 
  CPPUNIT_ASSERT(metricUInt32.setValue(10));
  uint32_t sendValueUInt32 = 0;
  CPPUNIT_ASSERT(metricUInt32._calculateSendValue(sendValueUInt32) == true);
  CPPUNIT_ASSERT(sendValueUInt32 == 10);


  NdmcMetric<double> metricDouble(name.c_str(),"unit",COUNTER);
  CPPUNIT_ASSERT(metricDouble.setValue(15.));
  CPPUNIT_ASSERT(metricDouble.setValue(-15));
  double sendValueDouble = 0;
  CPPUNIT_ASSERT(metricDouble._calculateSendValue(sendValueDouble) == true);
  CPPUNIT_ASSERT(sendValueDouble == 0.);
    
  NdmcMetric<int32_t> metricInt32(name.c_str(),"unit",COUNTER);
  CPPUNIT_ASSERT(metricInt32.setValue(-1));
  CPPUNIT_ASSERT(metricInt32.setValue(15));
  int32_t sendValueInt32 = 0;
  CPPUNIT_ASSERT(metricInt32._calculateSendValue(sendValueInt32) == true);
  CPPUNIT_ASSERT(sendValueInt32 == 14);

}

/**
 * A test method to test that NdmcMetric::setValue
 * works correctly for a metric of type RATE.
 */
void NdmcMetricTest::testSetValueMetricTypeRATE() {

  ::mf::LogInfo("unittest") << "NdmcMetricTest::testSetValueMetricTypeRATE." << std::endl;

  std::string name="dummy";
  NdmcMetric<uint32_t> metricUInt32(name.c_str(),"unit",RATE); 
  CPPUNIT_ASSERT(metricUInt32.isValueSet() == false);

  CPPUNIT_ASSERT(metricUInt32.setValue(10));
  CPPUNIT_ASSERT(metricUInt32.getValue() == 10);
  CPPUNIT_ASSERT(metricUInt32.isValueSet() == true);
  CPPUNIT_ASSERT(metricUInt32.setValue(10));
  CPPUNIT_ASSERT(metricUInt32.getValue() == 20);
  CPPUNIT_ASSERT(metricUInt32.isValueSet() == true);


  NdmcMetric<double> metricDouble(name.c_str(),"unit",RATE);
  CPPUNIT_ASSERT(metricDouble.isValueSet() == false);
  CPPUNIT_ASSERT(metricDouble.setValue(15.));
  CPPUNIT_ASSERT(metricDouble.getValue() == 15.);
  CPPUNIT_ASSERT(metricDouble.isValueSet() == true);
  CPPUNIT_ASSERT(metricDouble.setValue(-15.));
  CPPUNIT_ASSERT(metricDouble.getValue() == 0.);
  CPPUNIT_ASSERT(metricDouble.isValueSet() == true);
    
  NdmcMetric<int32_t> metricInt32(name.c_str(),"unit",RATE);
  CPPUNIT_ASSERT(metricInt32.isValueSet() == false);
  CPPUNIT_ASSERT(metricInt32.setValue(-1));
  CPPUNIT_ASSERT(metricInt32.getValue() == -1);
  CPPUNIT_ASSERT(metricInt32.isValueSet() == true);
  CPPUNIT_ASSERT(metricInt32.setValue(15));
  CPPUNIT_ASSERT(metricInt32.getValue() == 14);
  CPPUNIT_ASSERT(metricInt32.isValueSet() == true);

}

/**
 * A test method to test that NdmcMetric::_calculateSendValue
 * works correctly for a metric of type RATE.
 */
void NdmcMetricTest::testCalculateSendValueMetricTypeRATE() {

  ::mf::LogInfo("unittest") << "NdmcMetricTest::testCalculateSendValueMetricTypeRATE." 
                            << std::endl;

  std::string name="dummy";
  NdmcMetric<uint32_t> metricUInt32(name.c_str(),"unit",RATE); 
  CPPUNIT_ASSERT(metricUInt32.setValue(10));
  uint32_t sendValueUInt32 = 0;
  CPPUNIT_ASSERT(metricUInt32._calculateSendValue(sendValueUInt32,9.) == true);
  CPPUNIT_ASSERT(sendValueUInt32 == 1);
  CPPUNIT_ASSERT(metricUInt32._calculateSendValue(sendValueUInt32,15.) == true);
  CPPUNIT_ASSERT(sendValueUInt32 == 0);


  NdmcMetric<double> metricDouble(name.c_str(),"unit",RATE);
  CPPUNIT_ASSERT(metricDouble.setValue(15.));
  CPPUNIT_ASSERT(metricDouble.setValue(-15));
  double sendValueDouble = 0;
  CPPUNIT_ASSERT(metricDouble._calculateSendValue(sendValueDouble,10.) == true);
  CPPUNIT_ASSERT(sendValueDouble == 0.);
  CPPUNIT_ASSERT(metricDouble.setValue(5.));
  CPPUNIT_ASSERT(metricDouble._calculateSendValue(sendValueDouble,10.) == true);
  CPPUNIT_ASSERT(sendValueDouble == 0.5);
  CPPUNIT_ASSERT(metricDouble._calculateSendValue(sendValueDouble,0.) == false);
    
  NdmcMetric<int32_t> metricInt32(name.c_str(),"unit",RATE);
  CPPUNIT_ASSERT(metricInt32.setValue(-1));
  CPPUNIT_ASSERT(metricInt32.setValue(15));
  int32_t sendValueInt32 = 0;
  CPPUNIT_ASSERT(metricInt32._calculateSendValue(sendValueInt32,10) == true);
  CPPUNIT_ASSERT(sendValueInt32 == 1);

}

/**
 * A test method to test that NdmcMetric::setValue
 * works correctly for a metric of type AVERAGE.
 */
void NdmcMetricTest::testSetValueMetricTypeAVERAGE() {

  ::mf::LogInfo("unittest") << "NdmcMetricTest::testSetValueMetricTypeAVERAGE." << std::endl;

  std::string name="dummy";
  NdmcMetric<uint32_t> metricUInt32(name.c_str(),"unit",AVERAGE); 
  CPPUNIT_ASSERT(metricUInt32.isValueSet() == false);

  CPPUNIT_ASSERT(metricUInt32.setValue(10));
  CPPUNIT_ASSERT(metricUInt32.getSumWX() == 10);
  CPPUNIT_ASSERT(metricUInt32.isValueSet() == true);
  CPPUNIT_ASSERT(metricUInt32.setValue(10,2.));
  CPPUNIT_ASSERT(metricUInt32.getSumWX() == 30);
  CPPUNIT_ASSERT(metricUInt32.isValueSet() == true);


  NdmcMetric<double> metricDouble(name.c_str(),"unit",AVERAGE);

  CPPUNIT_ASSERT(metricDouble.isValueSet() == false);
  CPPUNIT_ASSERT(metricDouble.setValue(15.));
  CPPUNIT_ASSERT(metricDouble.getSumWX() == 15.);
  CPPUNIT_ASSERT(metricDouble.isValueSet() == true);
  CPPUNIT_ASSERT(metricDouble.setValue(-15.,2.));
  CPPUNIT_ASSERT(metricDouble.getSumWX() == -15.);
  CPPUNIT_ASSERT(metricDouble.isValueSet() == true);
    
  NdmcMetric<int32_t> metricInt32(name.c_str(),"unit",AVERAGE);

  CPPUNIT_ASSERT(metricInt32.isValueSet() == false);
  CPPUNIT_ASSERT(metricInt32.setValue(-1));
  CPPUNIT_ASSERT(metricInt32.getSumWX() == -1);
  CPPUNIT_ASSERT(metricInt32.isValueSet() == true);
  CPPUNIT_ASSERT(metricInt32.setValue(15,2));
  CPPUNIT_ASSERT(metricInt32.getSumWX() == 29);
  CPPUNIT_ASSERT(metricInt32.isValueSet() == true);

}

/**
 * A test method to test that NdmcMetric::_calculateSendValue
 * works correctly for a metric of type AVERAGE.
 */
void NdmcMetricTest::testCalculateSendValueMetricTypeAVERAGE() {

  ::mf::LogInfo("unittest") << "NdmcMetricTest::testCalculateSendValueMetricTypeAVERAGE." 
                            << std::endl;

  std::string name="dummy";
  NdmcMetric<uint32_t> metricUInt32(name.c_str(),"unit",AVERAGE); 
  CPPUNIT_ASSERT(metricUInt32.setValue(10));
  CPPUNIT_ASSERT(metricUInt32.setValue(10,2.));
  uint32_t sendValueUInt32 = 0;
  CPPUNIT_ASSERT(metricUInt32._calculateSendValue(sendValueUInt32) == true);
  CPPUNIT_ASSERT(sendValueUInt32 == 10);


  NdmcMetric<double> metricDouble(name.c_str(),"unit",AVERAGE);
  CPPUNIT_ASSERT(metricDouble.setValue(15.));
  CPPUNIT_ASSERT(metricDouble.setValue(-15.,2.));
  double sendValueDouble = 0;
  CPPUNIT_ASSERT(metricDouble._calculateSendValue(sendValueDouble) == true);
  CPPUNIT_ASSERT(sendValueDouble == -5.);
    
  NdmcMetric<int32_t> metricInt32(name.c_str(),"unit",AVERAGE);
  CPPUNIT_ASSERT(metricInt32.setValue(-1));
  CPPUNIT_ASSERT(metricInt32.setValue(15,2));
  int32_t sendValueInt32 = 0;
  CPPUNIT_ASSERT(metricInt32._calculateSendValue(sendValueInt32) == true);
  CPPUNIT_ASSERT(sendValueInt32 == 9);

}

/**
 * A test method to test that NdmcMetric::setValue
 * works correctly for a metric of type MINIMUM.
 */
void NdmcMetricTest::testSetValueMetricTypeMINIMUM() {

  ::mf::LogInfo("unittest") << "NdmcMetricTest::testSetValueMetricTypeMINIMUM." << std::endl;

  std::string name="dummy";
  NdmcMetric<uint32_t> metricUInt32(name.c_str(),"unit",MINIMUM); 
  CPPUNIT_ASSERT(metricUInt32.isValueSet() == false);

  CPPUNIT_ASSERT(metricUInt32.setValue(10));
  CPPUNIT_ASSERT(metricUInt32.getValue() == 10);
  CPPUNIT_ASSERT(metricUInt32.isValueSet() == true);
  CPPUNIT_ASSERT(metricUInt32.setValue(10));
  CPPUNIT_ASSERT(metricUInt32.getValue() == 10);
  CPPUNIT_ASSERT(metricUInt32.isValueSet() == true);


  NdmcMetric<double> metricDouble(name.c_str(),"unit",MINIMUM);

  CPPUNIT_ASSERT(metricDouble.isValueSet() == false);
  CPPUNIT_ASSERT(metricDouble.setValue(15.));
  CPPUNIT_ASSERT(metricDouble.getValue() == 15.);
  CPPUNIT_ASSERT(metricDouble.isValueSet() == true);
  CPPUNIT_ASSERT(metricDouble.setValue(-15.));
  CPPUNIT_ASSERT(metricDouble.getValue() == -15.);
  CPPUNIT_ASSERT(metricDouble.isValueSet() == true);
    
  NdmcMetric<int32_t> metricInt32(name.c_str(),"unit",MINIMUM);

  CPPUNIT_ASSERT(metricInt32.isValueSet() == false);
  CPPUNIT_ASSERT(metricInt32.setValue(-1));
  CPPUNIT_ASSERT(metricInt32.getValue() == -1);
  CPPUNIT_ASSERT(metricInt32.isValueSet() == true);
  CPPUNIT_ASSERT(metricInt32.setValue(15));
  CPPUNIT_ASSERT(metricInt32.getValue() == -1);
  CPPUNIT_ASSERT(metricInt32.isValueSet() == true);

}

/**
 * A test method to test that NdmcMetric::_calculateSendValue
 * works correctly for a metric of type MINIMUM.
 */
void NdmcMetricTest::testCalculateSendValueMetricTypeMINIMUM() {

  ::mf::LogInfo("unittest") << "NdmcMetricTest::testCalculateSendValueMetricTypeMINIMUM." 
                            << std::endl;

  std::string name="dummy";
  NdmcMetric<uint32_t> metricUInt32(name.c_str(),"unit",MINIMUM); 
  CPPUNIT_ASSERT(metricUInt32.setValue(10));
  CPPUNIT_ASSERT(metricUInt32.setValue(10));
  uint32_t sendValueUInt32 = 0;
  CPPUNIT_ASSERT(metricUInt32._calculateSendValue(sendValueUInt32) == true);
  CPPUNIT_ASSERT(sendValueUInt32 == 10);


  NdmcMetric<double> metricDouble(name.c_str(),"unit",MINIMUM);
  CPPUNIT_ASSERT(metricDouble.setValue(15.));
  CPPUNIT_ASSERT(metricDouble.setValue(-15));
  double sendValueDouble = 0;
  CPPUNIT_ASSERT(metricDouble._calculateSendValue(sendValueDouble) == true);
  CPPUNIT_ASSERT(sendValueDouble == -15.);
    
  NdmcMetric<int32_t> metricInt32(name.c_str(),"unit",MINIMUM);
  CPPUNIT_ASSERT(metricInt32.setValue(-1));
  CPPUNIT_ASSERT(metricInt32.setValue(15));
  int32_t sendValueInt32 = 0;
  CPPUNIT_ASSERT(metricInt32._calculateSendValue(sendValueInt32) == true);
  CPPUNIT_ASSERT(sendValueInt32 == -1);

}

/**
 * A test method to test that NdmcMetric::setValue
 * works correctly for a metric of type MAXIMUM.
 */
void NdmcMetricTest::testSetValueMetricTypeMAXIMUM() {

  ::mf::LogInfo("unittest") << "NdmcMetricTest::testSetValueMetricTypeMAXIMUM." << std::endl;

  std::string name="dummy";
  NdmcMetric<uint32_t> metricUInt32(name.c_str(),"unit",MAXIMUM); 
  CPPUNIT_ASSERT(metricUInt32.isValueSet() == false);

  CPPUNIT_ASSERT(metricUInt32.setValue(10));
  CPPUNIT_ASSERT(metricUInt32.getValue() == 10);
  CPPUNIT_ASSERT(metricUInt32.isValueSet() == true);
  CPPUNIT_ASSERT(metricUInt32.setValue(10));
  CPPUNIT_ASSERT(metricUInt32.getValue() == 10);
  CPPUNIT_ASSERT(metricUInt32.isValueSet() == true);


  NdmcMetric<double> metricDouble(name.c_str(),"unit",MAXIMUM);

  CPPUNIT_ASSERT(metricDouble.isValueSet() == false);
  CPPUNIT_ASSERT(metricDouble.setValue(15.));
  CPPUNIT_ASSERT(metricDouble.getValue() == 15.);
  CPPUNIT_ASSERT(metricDouble.isValueSet() == true);
  CPPUNIT_ASSERT(metricDouble.setValue(-15.));
  CPPUNIT_ASSERT(metricDouble.getValue() == 15.);
  CPPUNIT_ASSERT(metricDouble.isValueSet() == true);
    
  NdmcMetric<int32_t> metricInt32(name.c_str(),"unit",MAXIMUM);

  CPPUNIT_ASSERT(metricInt32.isValueSet() == false);
  CPPUNIT_ASSERT(metricInt32.setValue(-1));
  CPPUNIT_ASSERT(metricInt32.getValue() == -1);
  CPPUNIT_ASSERT(metricInt32.isValueSet() == true);
  CPPUNIT_ASSERT(metricInt32.setValue(15));
  CPPUNIT_ASSERT(metricInt32.getValue() == 15);
  CPPUNIT_ASSERT(metricInt32.isValueSet() == true);

}

/**
 * A test method to test that NdmcMetric::_calculateSendValue
 * works correctly for a metric of type MAXIMUM.
 */
void NdmcMetricTest::testCalculateSendValueMetricTypeMAXIMUM() {

  ::mf::LogInfo("unittest") << "NdmcMetricTest::testCalculateSendValueMetricTypeMAXIMUM." 
                            << std::endl;

  std::string name="dummy";
  NdmcMetric<uint32_t> metricUInt32(name.c_str(),"unit",MAXIMUM); 
  CPPUNIT_ASSERT(metricUInt32.setValue(10));
  CPPUNIT_ASSERT(metricUInt32.setValue(10));
  uint32_t sendValueUInt32 = 0;
  CPPUNIT_ASSERT(metricUInt32._calculateSendValue(sendValueUInt32) == true);
  CPPUNIT_ASSERT(sendValueUInt32 == 10);


  NdmcMetric<double> metricDouble(name.c_str(),"unit",MAXIMUM);
  CPPUNIT_ASSERT(metricDouble.setValue(15.));
  CPPUNIT_ASSERT(metricDouble.setValue(-15));
  double sendValueDouble = 0;
  CPPUNIT_ASSERT(metricDouble._calculateSendValue(sendValueDouble) == true);
  CPPUNIT_ASSERT(sendValueDouble == 15.);
    
  NdmcMetric<int32_t> metricInt32(name.c_str(),"unit",MAXIMUM);
  CPPUNIT_ASSERT(metricInt32.setValue(-1));
  CPPUNIT_ASSERT(metricInt32.setValue(15));
  int32_t sendValueInt32 = 0;
  CPPUNIT_ASSERT(metricInt32._calculateSendValue(sendValueInt32) == true);
  CPPUNIT_ASSERT(sendValueInt32 == 15);

}

/**
 * A test method to test that NdmcMetric::setValue
 * works correctly for a metric of type STDEV.
 */
void NdmcMetricTest::testSetValueMetricTypeSTDEV() {

  ::mf::LogInfo("unittest") << "NdmcMetricTest::testSetValueMetricTypeSTDEV." << std::endl;

  std::string name="dummy";
  NdmcMetric<uint32_t> metricUInt32(name.c_str(),"unit",STDEV); 
  CPPUNIT_ASSERT(metricUInt32.isValueSet() == false);

  CPPUNIT_ASSERT(metricUInt32.setValue(10));
  CPPUNIT_ASSERT(metricUInt32.getSumWX() == 10);
  CPPUNIT_ASSERT(metricUInt32.isValueSet() == true);
  CPPUNIT_ASSERT(metricUInt32.setValue(10,2.));
  CPPUNIT_ASSERT(metricUInt32.getSumWX() == 30);
  CPPUNIT_ASSERT(metricUInt32.isValueSet() == true);


  NdmcMetric<double> metricDouble(name.c_str(),"unit",STDEV);

  CPPUNIT_ASSERT(metricDouble.isValueSet() == false);
  CPPUNIT_ASSERT(metricDouble.setValue(15.));
  CPPUNIT_ASSERT(metricDouble.getSumWX() == 15.);
  CPPUNIT_ASSERT(metricDouble.isValueSet() == true);
  CPPUNIT_ASSERT(metricDouble.setValue(-15.,2.));
  CPPUNIT_ASSERT(metricDouble.getSumWX() == -15.);
  CPPUNIT_ASSERT(metricDouble.isValueSet() == true);
    
  NdmcMetric<int32_t> metricInt32(name.c_str(),"unit",STDEV);

  CPPUNIT_ASSERT(metricInt32.isValueSet() == false);
  CPPUNIT_ASSERT(metricInt32.setValue(-1));
  CPPUNIT_ASSERT(metricInt32.getSumWX() == -1);
  CPPUNIT_ASSERT(metricInt32.isValueSet() == true);
  CPPUNIT_ASSERT(metricInt32.setValue(15,2));
  CPPUNIT_ASSERT(metricInt32.getSumWX() == 29);
  CPPUNIT_ASSERT(metricInt32.isValueSet() == true);

}

/**
 * A test method to test that NdmcMetric::_calculateSendValue
 * works correctly for a metric of type STDEV.
 */
void NdmcMetricTest::testCalculateSendValueMetricTypeSTDEV() {

  ::mf::LogInfo("unittest") << "NdmcMetricTest::testCalculateSendValueMetricTypeSTDEV." 
                            << std::endl;

  std::string name="dummy";
  NdmcMetric<uint32_t> metricUInt32(name.c_str(),"unit",STDEV); 
  CPPUNIT_ASSERT(metricUInt32.setValue(10));
  CPPUNIT_ASSERT(metricUInt32.setValue(10,2.));
  uint32_t sendValueUInt32 = 0;
  CPPUNIT_ASSERT(metricUInt32._calculateSendValue(sendValueUInt32) == true);
  CPPUNIT_ASSERT(sendValueUInt32 == 0);


  NdmcMetric<double> metricDouble(name.c_str(),"unit",STDEV);
  CPPUNIT_ASSERT(metricDouble.setValue(15.));
  CPPUNIT_ASSERT(metricDouble.setValue(-15.));
  double sendValueDouble = 0;
  CPPUNIT_ASSERT(metricDouble._calculateSendValue(sendValueDouble) == true);
  CPPUNIT_ASSERT(int(sendValueDouble*1000) == 21213);
    
  NdmcMetric<int32_t> metricInt32(name.c_str(),"unit",STDEV);
  CPPUNIT_ASSERT(metricInt32.setValue(-1));
  CPPUNIT_ASSERT(metricInt32.setValue(15));
  CPPUNIT_ASSERT(metricInt32.setValue(15));
  int32_t sendValueInt32 = 0;
  CPPUNIT_ASSERT(metricInt32._calculateSendValue(sendValueInt32) == true);
  CPPUNIT_ASSERT(sendValueInt32 == 9);

}

/**
 * A test method to test that NdmcMetric::getLastValueSent method works correctly.
 */
void NdmcMetricTest::testGetLastValueSent() {

  ::mf::LogInfo("unittest") << "NdmcMetricTest::testGetLastValueSent." << std::endl;
  NdmcMetric<uint32_t> metric("dummy","unit",RATE);
  uint32_t lastValueSent=0;
  CPPUNIT_ASSERT(metric.getLastValueSent(lastValueSent) == false);
  CPPUNIT_ASSERT(metric.setValue(10));
  uint32_t sendValue = 0;
  CPPUNIT_ASSERT(metric._calculateSendValue(sendValue,9.) == true);
  CPPUNIT_ASSERT(sendValue == 1);
  metric._lastValueSent = sendValue;
  metric._valueHasBeenSent = true;
  CPPUNIT_ASSERT(metric.getLastValueSent(lastValueSent) == true);
  CPPUNIT_ASSERT(lastValueSent == sendValue);

  CPPUNIT_ASSERT(metric._calculateSendValue(sendValue,15.) == true);
  CPPUNIT_ASSERT(sendValue== 0);
  metric._lastValueSent = sendValue;
  CPPUNIT_ASSERT(metric.getLastValueSent(lastValueSent) == true);
  CPPUNIT_ASSERT(lastValueSent == sendValue);  

}

/**
 * A test method to test that NdmcMetric::setPartition works correctly.
 */
void NdmcMetricTest::testSetPartition() {

  ::mf::LogInfo("unittest") << "NdmcMetricTest::testSetPartition." 
                            << std::endl;

  std::string name="dummy";
  NdmcMetric<uint32_t> metric(name.c_str(),"unit",STDEV); 

  int32_t partition = 0;
  metric.setPartition(partition);
  CPPUNIT_ASSERT(metric.getPartition() == partition);
  CPPUNIT_ASSERT(metric.getApplicationName() == "");
  CPPUNIT_ASSERT(metric.getGroupName() == "");
  CPPUNIT_ASSERT(metric.getMetricName() == "dummy_P0");

}

/**
 * A test method to test that NdmcMetric::setApplicationName works correctly.
 */
void NdmcMetricTest::testSetApplicationName() {

  ::mf::LogInfo("unittest") << "NdmcMetricTest::testSetApplicationName." 
                            << std::endl;

  std::string name="dummy";
  NdmcMetric<uint32_t> metric(name.c_str(),"unit",STDEV); 

  std::string appName = "MyApp";
  metric.setApplicationName(appName);
  CPPUNIT_ASSERT(metric.getApplicationName() == "myapp"); // converts to lowercase
  CPPUNIT_ASSERT(metric.getMetricName() == "dummy_Amyapp");
  CPPUNIT_ASSERT(metric.getPartition() == -1);
  CPPUNIT_ASSERT(metric.getGroupName() == "");

  int32_t partition = 0;
  metric.setPartition(partition);
  CPPUNIT_ASSERT(metric.getApplicationName() == "myapp");
  CPPUNIT_ASSERT(metric.getPartition() == partition);
  CPPUNIT_ASSERT(metric.getGroupName() == "");
  CPPUNIT_ASSERT(metric.getMetricName() == "dummy_Amyapp_P0");

}

/**
 * A test method to test that NdmcMetric::setGroupName works correctly.
 */
void NdmcMetricTest::testSetGroupName() {

  ::mf::LogInfo("unittest") << "NdmcMetricTest::testSetGroupName." 
                            << std::endl;

  std::string name="dummy";
  NdmcMetric<uint32_t> metric(name.c_str(),"unit",STDEV); 

  std::string groupName = "MyGroup";
  metric.setGroupName(groupName);
  CPPUNIT_ASSERT(metric.getApplicationName() == "");
  CPPUNIT_ASSERT(metric.getMetricName() == "dummy");
  CPPUNIT_ASSERT(metric.getPartition() == -1);
  CPPUNIT_ASSERT(metric.getGroupName() == groupName);

}

/**
 * A test method to test that NdmcMetric::setTitle works correctly.
 */
void NdmcMetricTest::testSetTitle() {

  ::mf::LogInfo("unittest") << "NdmcMetricTest::testSetTitle." 
                            << std::endl;

  std::string name="dummy";
  NdmcMetric<uint32_t> metric(name.c_str(),"unit",STDEV); 
  CPPUNIT_ASSERT(metric.getTitle() == "");

  std::string title = "MyTitle";
  metric.setTitle(title);
  CPPUNIT_ASSERT(metric.getTitle() == title);

}

/**
 * A test method to test that NdmcMetric::setDescription works correctly.
 */
void NdmcMetricTest::testSetDescription() {

  ::mf::LogInfo("unittest") << "NdmcMetricTest::testSetDescription." 
                            << std::endl;

  std::string name="dummy";
  NdmcMetric<uint32_t> metric(name.c_str(),"unit",STDEV); 
  CPPUNIT_ASSERT(metric.getDescription() == "");

  std::string description = "MyDescription";
  metric.setDescription(description);
  CPPUNIT_ASSERT(metric.getDescription() == description);

}



