#include <NovaDAQUtilities/SHMConvention.h>
#include <boost/lexical_cast.hpp>
#include <boost/algorithm/string.hpp>
#include <iostream>
#include <string>
#include <vector>

/**
 * Command-line utility for testing how LocationUtils::findFile() works.
 */

int main(int argc, char **argv) {

    bool printUsage = false;
    int frameworkNumber = 0;
    uint32_t partitionNumber = 0;
    

    int c;
    while ((c = getopt (argc, argv, "z:f:")) != -1) {
        switch (c)
        {
    case 'f':
            {
                std::string tmpString(optarg);
                try {
                   frameworkNumber = boost::lexical_cast<int>(tmpString);
                }
                catch (...) {
                    printUsage = true;
                }
            }
            break;
    case 'z':
            {
                std::string tmpString(optarg);
                try {
                   partitionNumber = boost::lexical_cast<uint32_t>(tmpString);
                }
                catch (...) {
                    printUsage = true;
                }
            }
            break;
        case '?':
            if (optopt == 'z') {
                std::cerr << "Option -z requires an argument." << std::endl;
            }
            else if (optopt == 'f') {
                std::cerr << "Option -f requires an argument." << std::endl;
            }
            else if (isprint (optopt)) {
                std::cerr << "Unknown option '-" << optopt << "'."
                          << std::endl;
            }
            else {
                std::cerr << "Unknown option character '" << optopt << "'."
                          << std::endl;
            }
            printUsage = true;
            break;
        default:
            printUsage = true;
        }
    }


  if (printUsage) {
        std::cerr << "Usage: " << argv[0] << "[-z <partitionNumber>:0] "
                  << " [ -f <frameworkNumber>:0] "
                  << std::endl;
        return -1;
  }

  std::cout<<"Partition "<<partitionNumber<<", Framework="<<frameworkNumber
         <<" key=0x"<<std::hex
         <<novadaq::SHMConvention::getSHMid(partitionNumber, 
                  (novadaq::OFFLINE_FRAMEWORK)  frameworkNumber) <<std::dec<<std::endl;

  return 0;
}
