#include <cppunit/extensions/HelperMacros.h>
#include <NovaDAQUtilities/HexUtils.h>

using namespace novadaq;

class HexUtilTests : public CppUnit::TestFixture
{
    CPPUNIT_TEST_SUITE(HexUtilTests);
    CPPUNIT_TEST(testFormat);
    CPPUNIT_TEST(testValidity);
    CPPUNIT_TEST(testParse);
    CPPUNIT_TEST_SUITE_END();

public:
    void setUp();
    void tearDown();

    void testFormat();
    void testValidity();
    void testParse();

private:

};

void
HexUtilTests::setUp()
{
    // flush stdout to get progress markers to display in a timely way
    std::cout.flush();
}

void
HexUtilTests::tearDown()
{
    // flush stdout to get progress markers to display in a timely way
    std::cout.flush();
}

void
HexUtilTests::testFormat()
{
    int value;
    //int32_t value32;
    int64_t value64;
    std::string hexString;
    std::string decimalString;

    value = 1;
    hexString = HexUtils::format(value);
    CPPUNIT_ASSERT(hexString == "0x00000001");

    value = -1;
    hexString = HexUtils::format(value);
    CPPUNIT_ASSERT(hexString == "0xFFFFFFFF");

    value = 0x1234;
    hexString = HexUtils::format(value, 5);
    CPPUNIT_ASSERT(hexString == "0x01234");

    value64 = -1;
    hexString = HexUtils::format(value64);
    CPPUNIT_ASSERT(hexString == "0xFFFFFFFFFFFFFFFF");

    decimalString = "100";
    hexString = HexUtils::format(decimalString, 2);
    CPPUNIT_ASSERT(hexString == "0x64");

    decimalString = "100";
    hexString = HexUtils::format(decimalString);
    CPPUNIT_ASSERT(hexString == "0x00000064");

    decimalString = "-1";
    hexString = HexUtils::format(decimalString);
    CPPUNIT_ASSERT(hexString == "0xFFFFFFFF");

    decimalString = "-1";
    hexString = HexUtils::format(decimalString, 16);
    CPPUNIT_ASSERT(hexString == "0xFFFFFFFFFFFFFFFF");

    decimalString = "abc";
    try {
        hexString = HexUtils::format(decimalString);
        CPPUNIT_ASSERT(0);
    }
    catch (...) {}

    decimalString = "99999";
    hexString = HexUtils::format(decimalString, 2);
    CPPUNIT_ASSERT(hexString.length() > 4);

    decimalString = "9999999999";
    try {
        hexString = HexUtils::format(decimalString, 2);
        CPPUNIT_ASSERT(0);
    }
    catch (...) {}
}

void
HexUtilTests::testValidity()
{
    CPPUNIT_ASSERT(! HexUtils::isValidHex("0"));
    CPPUNIT_ASSERT(! HexUtils::isValidHex("1"));

    CPPUNIT_ASSERT(HexUtils::isValidHex("  0x01  "));
    CPPUNIT_ASSERT(HexUtils::isValidHex("0x80000000"));
    CPPUNIT_ASSERT(HexUtils::isValidHex("0xffffffff"));
    CPPUNIT_ASSERT(HexUtils::isValidHex("0x123456789abcd"));

    CPPUNIT_ASSERT(! HexUtils::isValidHex("0xabcdefghij"));
    CPPUNIT_ASSERT(! HexUtils::isValidHex("0x0t"));
}

void
HexUtilTests::testParse()
{
    int32_t value32;
    int64_t value64;
    std::string hexString;

    hexString = "0x0";
    CPPUNIT_ASSERT(HexUtils::parse(hexString, value32));
    CPPUNIT_ASSERT(value32 == 0);

    hexString = "0x80000000";
    CPPUNIT_ASSERT(HexUtils::parse(hexString, value32));
    CPPUNIT_ASSERT(value32 == -2147483648);

    hexString = "0xffffffff";
    CPPUNIT_ASSERT(HexUtils::parse(hexString, value32));
    CPPUNIT_ASSERT(value32 == -1);

    hexString = "0x123456789abc";
    CPPUNIT_ASSERT(! HexUtils::parse(hexString, value32));

    hexString = "0x123456789abc";
    CPPUNIT_ASSERT(HexUtils::parse(hexString, value64));

    hexString = "0x0t";
    CPPUNIT_ASSERT(! HexUtils::parse(hexString, value32));

    hexString = " abc ";
    CPPUNIT_ASSERT(! HexUtils::parse(hexString, value32));
}

CPPUNIT_TEST_SUITE_NAMED_REGISTRATION(HexUtilTests, "NovaDAQUtilities");
CPPUNIT_REGISTRY_ADD_TO_DEFAULT("NovaDAQUtilities");
