package novadaq;

import java.util.Iterator;
import java.util.Vector;

/**
 * A singleton class to manage flusing of caches.
 *
 * @author Qiming Lu
 * @version $Revision: 1.1 $ $Date: 2009/12/28 20:56:09 $
 */

public class CacheManager {

    private static Vector<Cache> _registry = new Vector<Cache>();
    private static CacheManager _singletonInstance = null;

    /**
     * Constructor.
     */
    private CacheManager() {};

    /**
     * Returns the single instance of the CacheManagerImpl.
     *
     * @return CacheManagerImpl instance.
     */
    public static synchronized CacheManager getInstance() {
        if (_singletonInstance == null)
            _singletonInstance = new CacheManager();
        return _singletonInstance;
    }

    /**
     * Register the specified cache with the CacheManagerImpl. 
     * Registered caches will be flused when the CacheManagerImpl
     * receives a flush request.
     *
     * @param cache a ref to the cache that should be registered.
     */
    public synchronized void registerCache(Cache cache) {
        _registry.add(cache);
    }

    /**
     * Removes the specified cache from the CacheManager.
     * 
     * @param cache a ref to the cache that should be removed.
     */
    public synchronized void removeCache(Cache cache) {
        _registry.remove(cache);
    }

    /**
     * Flushes the caches that have been registered.
     */
    public synchronized void flushCaches() {
        Iterator iter = _registry.iterator();
        while(iter.hasNext())
            ((Cache)iter.next()).flush();
    }
}
