package novadaq;

/**
 * The base class for cache expiration policies. For convenience,
 * this class provides an implementation of a "never expire" policy,
 * so instances of this class canbe used in cases in which no
 * automatic expiration is desired.
 *
 * @author Qiming Lu
 * @version $Revision: 1.1 $ $Date: 2009/12/28 20:56:09 $
 */

public class CachePolicy {

    /**
     * Constructor.
     */
    public CachePolicy() {
    }

    /**
     * Tells the policy that a cache update was done. This is used to
     * reset the state of the policy to "unexpired" after an update.
     */
    public void cacheUpdated() {
        _markAsUpdated();
    }

    /**
     * Tests if the associated cache has expired based on the criteria
     * in the policy.
     *
     * @return true if the cache has expired, false if not.
     */
    public boolean isExpired() {
        return _checkIfExpired();
    }

    /**
     * Tells the policy that a cache update was done. This is used to
     * reset the state of the policy to "unexpired" after an update.
     */
    protected void _markAsUpdated() {
        // don't need to keep track of when the cache was updated since
        // we never expire in this simple implementation
    }

    /**
     * Tests if the associated cache has expired based on the criteria
     * in the policy.
     *
     * @return true if the cache has expired, false if not.
     */
    protected boolean _checkIfExpired() {
        return false;
    }
}
