package novadaq;

/**
 * This class provides utility serialization methods for
 * use with Castor Java/XML binding.
 *
 * @author Kurt Biery
 * @version $Revision: 1.1 $ $Date: 2009/11/24 22:15:46 $
 */
public abstract class CastorSerializableBase implements XMLSerializable {
    /**
     * Name of the Castor map file to use, if any.
     */
    protected String _castorMapName = null;

    /**
     * Returns a String containing the serialized contents of the object.
     *
     * @return a string containing the serialized object
     * @throws novadaq.DataException if an error occurs serializing the
     *         object data
     */
    public String serialize() throws novadaq.DataException {
        try {
            return CastorXMLHelper.marshal(this, _castorMapName);
        }
        catch (novadaq.Exception excpt) {
            String excptMsg = "Error serializing a message with Castor.";
            throw new novadaq.DataException(excptMsg, excpt, this.toString());
        }
    }

    /**
     * Populates the internal data of the object based on serialized 
     * data in the input String.
     *
     * @param serializedData a string containing serialized object data
     * @throws novadaq.DataException if an error occurs deserializing the
     *         string representation of the object into its internal data
     */
    public void deserialize(String serializedData) throws novadaq.DataException {
        try {
            CastorXMLHelper.unmarshal(this, serializedData, _castorMapName);
        }
        catch (novadaq.Exception excpt) {
            String excptMsg = "Error deserializing a message with Castor.";
            throw new novadaq.DataException(excptMsg, excpt, serializedData);
        }
    }
}
