package novadaq;

/**
 * Utilities for working with the locations of resources.
 *
 * @author Kurt Biery
 * @version $Revision: 1.1 $ $Date: 2009/11/24 22:15:46 $
 */

public class LocationUtils {

    /**
     * Translates the specified URL to a generally accepted form.
     *
     * At the moment, the only translation that is supported is to
     * convert a URL of the form srt://relativePath/filename to
     * ${SRT_PRIVATE_RELEASE}/relativePath/filename or
     * ${SRT_PUBLIC_RELEASE}/relativePath/filename depending
     * on whether a private (test) release has been set up.
     * All other URLs are simply returned with no modification.
     *
     * @param sourceURL The URL string to translate.
     *
     * @return The modified URL string.
     */
    public static String translateURL(String sourceURL) {

        // make a copy of the URL without any leading spaces (just in case)
        String trimmedURL = sourceURL.trim();

        // if the URL uses our SRT notation, do the needed translation
        if (trimmedURL.startsWith("srt://")) {
            String rootPath = System.getenv("SRT_PRIVATE_CONTEXT");
            if (rootPath == null || rootPath.equals(".")) {
                rootPath = System.getenv("SRT_PUBLIC_CONTEXT");
            }
            if (rootPath != null) {
                return (rootPath + trimmedURL.substring(5));
            }
        }

        // if we failed to do the translation, or none was needed, simply
        // return the input URL
        return sourceURL;
    }
}
