package novadaq;

/**
 * A cache expiration policy based on a time interval.
 *
 * @author Qiming Lu
 * @version $Revision: 1.2 $ $Date: 2010/05/04 19:17:06 $
 */

public class TimedCachePolicy extends CachePolicy {

    private long _timeoutInMillis = 0;
    private long _lastUpdateTime = 0;

    /**
     * Constructor.
     *
     * @param timeoutInMinutes the time, in minutes, after which the
     *                         policy will report that a cache has expired.
     */
    public TimedCachePolicy(int timeoutInMinutes) {
        _timeoutInMillis = timeoutInMinutes * 60 * 1000;
    }

    /**
     * Tells the policy that a cache update was done. This is used to
     * reset the state of the policy to "unexpired" after an update.
     */
    protected void _markAsUpdated() {
        _lastUpdateTime = System.currentTimeMillis();
    }

    /**
     * Tests if the associated cache has expired based on the criteria
     * in the policy.
     *
     * @return true if the cache has expired, false if not.
     */
    protected boolean _checkIfExpired() {
        long now = System.currentTimeMillis();
        long cacheAge = now - _lastUpdateTime;
        return (cacheAge >= _timeoutInMillis);
    }
}
