/**
 * @package novadaq
 */
package novadaq;

import java.io.StringReader;
import java.io.StringWriter;
import javax.xml.transform.OutputKeys;
import javax.xml.transform.Transformer;
import javax.xml.transform.TransformerFactory;
import javax.xml.transform.stream.StreamResult;
import javax.xml.transform.stream.StreamSource;

/**
 * Utility class to help with XML formatting.
 *
 * @author Kurt Biery
 * @version $Id: XMLFormatter.java,v 1.1 2009/11/24 21:31:06 biery Exp $
 */

public class XMLFormatter {

    /**
     * Formats the specified string (containing XML) into a more
     * human-readable form (newlines between tags and child tags indented).
     * 
     * @param xmlString the string containing the XML to be formatted
     * @param indent the number of spaces to indent child tags
     * @return the formatted XML string
     * @throws javax.xml.transform.TransformerException if there is a problem
     *         formatting the XML string
     */
    public static String prettyFormat(String xmlString, int indent)
        throws javax.xml.transform.TransformerException
    {
        StreamSource xmlInput = new StreamSource(new StringReader(xmlString));
        StreamResult xmlOutput = new StreamResult(new StringWriter());
        Transformer transformer = TransformerFactory.newInstance().newTransformer(); 
        transformer.setOutputProperty(OutputKeys.INDENT, "yes");
        transformer.setOutputProperty("{http://xml.apache.org/xslt}indent-amount",
                                      Integer.toString(indent));
        transformer.transform(xmlInput, xmlOutput);
        return xmlOutput.getWriter().toString();
    }

}
