package novadaq.demo;

/**
 * Utility class for launching GUI applications.
 *
 * @author Kurt Biery
 * @version $Revision: 1.1 $ $Date: 2009/11/23 18:52:03 $
 */

import java.awt.BorderLayout;
import java.awt.Component;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JMenuBar;
import javax.swing.UIManager;
import java.awt.event.*;

public class ApplicationLauncher implements Runnable {

    /**
     * The application that we need to launch.
     */
    private Launchable _targetApplication;

    /**
     * The name of the application (used for the window title).
     */
    private String _applicationName;

    /**
     * Constructs a launcher instance based on the specified
     * list of user-specified parameters.
     */
    public ApplicationLauncher(Launchable targetApp, String appName) {
        _targetApplication = targetApp;
        _applicationName = appName;

        // submit a request to the GUI thread to finish the launch
        // in a thread-safe way
        javax.swing.SwingUtilities.invokeLater(this);
    }

    /**
     * Launches the application.  This method is part of the Runnable
     * interface and is called at the appropriate time by the Swing
     * GUI thread.
     */
    public void run() {

        // set the GUI look and feel
        String lookAndFeel = "com.sun.java.swing.plaf.motif.MotifLookAndFeel";
        try {
            UIManager.setLookAndFeel(lookAndFeel);
        }
        catch (Exception excpt) {
            System.err.println("Couldn't set specified look and feel ("
                               + lookAndFeel
                               + "), for some reason.");
            System.err.println("Using the default look and feel.");
            excpt.printStackTrace();
        }

        // set nice window decorations.
        JFrame.setDefaultLookAndFeelDecorated(true);

        // create and display the splash screen, if needed
        // if an exception is encountered, display it
        Component splashContents = null;
        JFrame splashFrame = null;
        try {
            splashContents = _targetApplication.getSplashPanel();
        }
        catch (Exception excpt) {
            splashContents = new JLabel(excpt.toString());
        }
        if (splashContents != null) {
            splashFrame = new JFrame(_applicationName + " Splash Screen");
            splashFrame.getContentPane().add(splashContents,
                                             BorderLayout.CENTER);
            splashFrame.pack();
            splashFrame.setVisible(true);
            //splashFrame.show();
        }

        // create and set up the main window.
        JFrame mainFrame = new JFrame(_applicationName);
        mainFrame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);

        // catch any exceptions in application init and GUI creation
        Component mainContents = null;
        try {
            // initialize the application
            _targetApplication.initialize();

            // fetch the display
            mainContents = _targetApplication.getMainDisplay(mainFrame);
        }
        catch (Exception excpt) {
            mainContents = new JLabel(excpt.toString());
        }

        // add the main display to its window
        mainFrame.getContentPane().add(mainContents, BorderLayout.CENTER);

        // add the menu bar to the window, if needed
        JMenuBar menuBar = _targetApplication.getMenuBar();
        if (menuBar != null) {
            mainFrame.setJMenuBar(menuBar);
        }

        // hide the splash screen, if needed
        if (splashFrame != null) {
            //splashFrame.setVisible(false);
            splashFrame.dispose();
            splashFrame = null;
        }

        // testing...
        //WindowStateListener wsListener = new WindowStateListener() {
        //        public void windowStateChanged(WindowEvent winEvt) {
        //            System.out.println("STATE CHANGE: " + winEvt);
        //        }
        //    };
        //mainFrame.addWindowStateListener(wsListener);

        // display the window.
        mainFrame.pack();
        mainFrame.setVisible(true);
        //mainFrame.show();
    }
}
