import static org.junit.Assert.*;
import org.junit.*;

/**
 * JUnit Individual Test
 * The test methods are executed in the following sequence:
 *
 * oneTimeSetUp()
 *
 * setUp()
 * someTestMethod()
 * tearDown()
 *
 * setUp()
 * someTestMethod()
 * tearDown()
 *
 * ...
 *
 * setUp()
 * someTestMethod()
 * tearDown()
 * 
 * oneTimeTearDown()
 *
 * Note: the execution order of testMothod() is not guranteed. Therefore,
 * it is not encouraged to design testMethods() in such a way that one test
 * relies on the results of another test.
 *
 */

public class SimpleJUnitPackageTest1
{
    @BeforeClass
    public static void oneTimeSetUp() {
        // one-time initialization code
    }

    @AfterClass
    public static void oneTimeTearDown() {
        // one-time cleanup code
    }

    @Before
    public void setUp() {
        // initialization code before each test method
    }

    @After
    public void tearDown() {
        // cleanup code after each test method
    }

    @Test
    public void testMethod1() {
        // True test
        assertTrue(1==1);
    }

    @Test
    public void testMethod2() {
        // Equal test
        assertEquals("Assert for equals.", 
                     1 /*expected*/, 
                     1 /*test value*/);
    }

    @Test
    public void testMethod3() {
        // Equal test with delta
        assertEquals("Assert for equals with delta, good for float number test.", 
                     1.0 /*expected*/, 
                     0.6 /*test value*/, 
                     0.5 /*delta*/);
    }

    // This test will be ignored
    @Ignore("Not ready to run")
    @Test
    public void testMethod4() {
        assertTrue("Assert for true.", 1==1);
    }

    // Test if an expected exception is thrown
    @Test(expected = ArithmeticException.class /*expected exception*/)
    public void testMethod5() {
        int i = 1/0;
    }
}
