#!/bin/bash 

usage () {
     echo "archive_links.sh [-r] [-f] filename"
     echo "            default : save links to archive <filename>"
     echo "                 -r : restore links from archive <filename>"
     echo "                 -f : force replacement of archive (no -r) or "
     echo "                      existing links (-r)"
}


set_defaults() 
{
   restore=0
   force_overwrite=0
}
   
get_args() {

   set_defaults

   archive_file=`echo "${@: -1}"`
   shift `expr $OPTIND - 1`

# Parse command line options.
    while getopts hrf OPT; do
        case "$OPT" in
            h)
                usage
                exit 0
                ;;
            r)
                restore=1
                ;;
            f)
                force_overwrite=1
                ;;
            \?)
                # getopts issues an error message
                usage
                exit 1
                ;;
        esac

    done
    # Remove the switches we parsed above.

    shift `expr $OPTIND - 1`

    if [ $# -ne 1 ]; then
	echo "Illegal arguments"
	usage
	exit 1
    fi


}

check_archive() {

	if [ ! -e $archive_file ]; then
		echo "Error: archive file $archive_file does not exist"
		exit 2
	fi

}

check_safe_links() {
	
        # read in all links
	allok=1
	while read line; do 
		linkname=`echo $line |awk '{print $1}'`
		target=`echo $line |awk '{print $2}'`
		ls $linkname >& /dev/null
		status=$?
		if [ $status -eq 0 ]; then
			allok=0
			echo "link $linkname exists:"
			ls -l $linkname
		fi
        done < $archive_file

	if [ $allok -eq 0 ]; then
		echo "Links exist.  Exiting without changes"
		exit  3
	fi

}

restore_links() {
	
        # read in all links
	while read line; do 
		linkname=`echo $line |awk '{print $1}'` 
		target=`echo $line |awk '{print $2}'`
		ls $linkname >& /dev/null
		status=$?
		if [ $status -eq 0 ]; then
			rm -f $linkname
		fi
		ln -s $target $linkname
        done < $archive_file


}
check_safe_archive() {
	
	if [ -e $archive_file ]; then
		echo "Archive file exists. Exiting."
		exit 3
	fi

}
archive_links() {
	
	ls -l |grep "^l" |awk -v OUTPUT=$archive_file '{print $9" "$11 > OUTPUT}' 
}

get_args $*

if [ $restore -eq 1 ]; then
	check_archive 
	if [ $force_overwrite -eq 0 ]; then
		check_safe_links 
	fi
	restore_links 
else 
	if [ $force_overwrite -eq 0 ]; then
		check_safe_archive
	fi
	archive_links
fi


exit 0

