#!/bin/bash 

usage () {
     echo "make_ops_tag.sh -t <tag> -d <detector name> [-F] [-T]"
     echo "      Applieds an operations <tag> to your local DAQOperationsTools for <detector name>"
     echo "                -F : force tag move"
     echo "                -t : test mode - just echo"
}

get_args() {

   set_defaults

# Parse command line options.
    while getopts "ht:d:FT" opt; do
    if [ "$opt" = "-" ]; then
        opt=$OPTARG
    fi
    case $opt in
            h)
                usage
                exit 0
                ;;
            t)
                newTag=${OPTARG}
                ;;
            d)
                detectorId=${OPTARG}
                ;;
            F)
                forceTag=1
                ;;
            T)
                testMode="echo "
                ;;
            \?)
                # getopts issues an error message
                usage
                exit 1
                ;;
        esac

    done
    # Remove the switches we parsed above.

    shift `expr $OPTIND - 1`

    if [ $# -ne 0 ]; then
	echo "Illegal arguments"
	echo " "
	usage
	exit 1
    fi

    if [ -z $detectorId ]; then
	echo "missing detector parameter"
	usage 
        exit 1
    fi

    if [ -z $newTag ]; then
	echo "missing tag parameter"
	usage 
        exit 1
    fi

    if [ $detectorId == "FarDet" ]; then
	detectorOk=1
    elif [ $detectorId == "NearDet" ]; then
	detectorOk=1
    elif [ $detectorId == "NDOS" ]; then
	detectorOk=1
    elif [ $detectorId == "NDSBTest" ]; then
	detectorOk=1
    elif [ $detectorId == "NSBAPD" ]; then
	detectorOk=1
    elif [ $detectorId == "FCCDAQ" ]; then
	detectorOk=1
    elif [ $detectorId == "TestStandFCC" ]; then
	detectorOk=1
    fi

    if [ $detectorOk -ne 1 ]; then
	echo "Invalid detector name $detectorId"
	exit 1
    fi

    if [ $forceTag -eq 1 ]; then
	cvs_cmd="$cvs_cmd -F"
    fi

}

set_defaults() {
    forceTag=0
    testMode=""
    detectorOk=0
    cvs_cmd="cvs tag"
}

checkPWD() {
    if [ `basename $PWD` != "DAQOperationsTools" ]; then
	echo "cd to your working DAQOperationsTools and try again"
	usage
	exit 1
    fi

}

tag_makes() {
    local files=`find . -name GNUmakefile`
    for file in $files; do
    	$testMode eval "$cvs_cmd $newTag $file"
    done
}

tag_script() {
    cd script
    $testMode eval "$cvs_cmd $newTag"
    cd ..
}
tag_setup() {
    cd setup
    $testMode eval "$cvs_cmd $newTag *.sh"
    cd $detectorId
    $testMode eval "$cvs_cmd $newTag *.sh"
    cd ../..
}

get_args $*

checkPWD
tag_makes 
tag_script 
tag_setup 

echo "... done ..."

exit 0

