#include <iostream>
#include <iomanip>
#include <vector>
#include "NovaDAQUtilities/LocationUtils.h"
#include "NovaDatabase/Table.h"

using namespace nova::database;
using namespace std;

int main(int argc, char *argv[])
{
  std::string fname;

  if (argc != 1) {
    cout << "Usage: dbReadTest" << endl;
    exit(1);    
  }

  fname = "NovaDatabase/cxx/test/Badchan.xml";
  
  Table* dbt = new Table(fname.c_str());
  
  cout << dbt->Name() << " has " << dbt->NCol()
       << " columns:" << endl;
  
  dbt->PrintColumns();

  if (!dbt->ExistsInDB()) {
    std::cout << dbt->Name() << " not found in the database." << std::endl;
    return 1;
  }
    
  dbt->SetValidityRange("run",11351,11351);
  dbt->SetValidityRange("subrun",0,0);

  dbt->LoadFromDB();

  int colsize = 0;
  for (int i=0; i<dbt->NCol(); ++i) {
    const Column* col = dbt->GetCol(i);
    colsize += sizeof(*col);
  }
  cout << "colsize = " << colsize << endl;

  int rowsize = 0;
  for (int j=0; j<dbt->NRow(); ++j) {
    Row* row = dbt->GetRow(j);
    rowsize += sizeof(*row);
    for (int i=0; i<row->NCol(); ++i) {
      Column& col = row->Col(i);
      rowsize += sizeof(col);
    }
  }
  cout << "rowsize = " << rowsize << endl;

  dbt->WriteToCSV("Badchan.csv");

  dbt->Clear();
  dbt->SetValidityRange("run",11348,11348);
  dbt->LoadFromDB();
  
  delete dbt;
  /*
  dbt->Clear();
  for (int i=0; i<2; ++i) {
    dbt->LoadFromCSV("Badchan.csv");
    dbt->Clear();
    cout << "sleeping 10 s..." << endl;
    sleep(10);
  }
  */
  
  return 0;
}
