#include <iostream>
#include <iomanip>
#include <vector>
#include "NovaDAQUtilities/LocationUtils.h"
#include "NovaDatabase/Table.h"

using namespace nova::database;
using namespace std;

int main(int argc, char *argv[])
{
  std::string fname;

  if (argc == 1) 
    fname = "NovaDatabase/config/Runs.xml";
  else
    if (argc == 2) 
      fname = argv[1];
    else {
      cout << "Usage: dbWriteTest [XML Table File]" << endl;
      exit(1);
    }
  
  Table* dbt = new Table(fname.c_str());
  
  cout << dbt->Name() << " has " << dbt->NCol()
       << " columns:" << endl;
  
  dbt->PrintColumns();
  
  if (argc == 2) {
    int nRow = dbt->NRow();
    
    if (nRow > 0)
      std::cout << " and " << nRow << " rows." << std::endl;
    else
      std::cout << std::endl;
    
    for (int i=0; i<nRow; ++i) {
      std::cout << i << " " << dbt->GetRow(i) << std::endl;
    }
    return 0;
  }

  if (dbt->ExistsInDB())
    std::cout << dbt->Name() << " exists in the database." << std::endl;
  else
    std::cout << dbt->Name() << " not found in the database." << std::endl;

  Row* row;

  row = dbt->NewRow();
  row->Set(string("run"),57);
  row->Set(string("nsubruns"),3);
  row->Set(string("nevents"),5026);
  row->Set(string("type"),string("Test"));
  row->Set(string("config"),string("TestConfig.xml"));
  row->Set(string("detector"),string("NDOS"));
  row->Set(string("tstart"),string("2010-09-01 10:04:34"));
  row->Set(string("shifter"),string("Jon P."));
  row->Set(string("begin_comment"),string("this is a test of the NOvA DBI"));

  cout << *row << endl;

  dbt->AddRow(row);

  if (dbt->WriteToDB()) {
    Row* row2 = dbt->GetRow(0);
    row2->Update(string("tstop"),string("2010-09-01 10:14:34"));
    row2->Update(string("end_comment"),string("test was successful!"));
    dbt->WriteToDB();
  }

  // now test getting (& keeping) a connection to the dB
  dbt->GetConnection();

  row = dbt->NewRow();
  row->Set(string("run"),58);
  row->Set(string("nsubruns"),5);
  row->Set(string("nevents"),10041);
  row->Set(string("type"),string("Test"));
  row->Set(string("config"),string("TestConfig.xml"));
  row->Set(string("detector"),string("NDOS"));
  row->Set(string("tstart"),string("2010-09-02 10:04:34"));
  row->Set(string("shifter"),string("Jon P."));
  row->Set(string("begin_comment"),string("this is another test of the NOvA DBI"));

  cout << *row << endl;

  dbt->AddRow(row);

  dbt->WriteToDB();

  dbt->CloseConnection();

  delete dbt;

  return 0;
}
