#include <iostream>
#include <iomanip>
#include <vector>
#include <cstdlib>
#include <ctime>

#include "NovaDAQUtilities/LocationUtils.h"
#include "NovaDatabase/Table.h"

using namespace nova::database;
using namespace std;

int main(int argc, char *argv[])
{
  std::string fname;

  if (argc != 1) {
    cout << "Usage: dbUnitTest" << endl;
    exit(1);
  }
  
  fname = "NovaDatabase/cxx/unittest/unitTest.xml";
  
  Table* dbt = new Table(fname.c_str());
  
  if (! dbt->GetConnection()) {
    std::cerr << "Could not connect to the database.  Exiting..." << std::endl;
    exit(2);
  }

  if (dbt->ExistsInDB())
    std::cout << dbt->Name() << " exists in the database." << std::endl;
  else
    std::cout << dbt->Name() << " not found in the database." << std::endl;

  dbt->WriteToDB();
  dbt->WriteToCSV(string("myUnitTest.csv"));

  dbt->Clear();

  fname = "NovaDatabase/cxx/unittest/unitTest.csv";
  dbt->LoadFromCSV(fname);

  // search for a specific row...
  string email;
  for (int i=0; i<dbt->NRow(); ++i) {
    cout << *(dbt->GetRow(i)) << endl;
    if (dbt->GetRow(i)->Col("email")->Get(email)) {
      if (email == string("hh@elsewhere.gov")) {
	dbt->GetRow(i)->Update("email","harry.henry@yahoo.com");
	break;
      }
    }
  }
  dbt->WriteToDB();

  dbt->CloseConnection();

  delete dbt;

  fname = "NovaDatabase/cxx/unittest/unitTest2.xml";
  
  Table* dbt2 = new Table(fname.c_str());

  srand( time(NULL) );

  Row* r = dbt2->NewRow();
  int ic=0;
  for (int i=0; i<1000; ++i) {
    if ((i%100)==0) {
      ic = 0;
      cout << i << endl;
      dbt2->WriteToDB();
      sleep(1);
    }
    r->Clear();
    r->Set("runmin",99);
    r->Set("runmax",99);
    r->Set("subrunmin",0);
    r->Set("subrunmax",999);
    r->Set("channel",ic++);
    float v1 = (float)(rand()%100)/100.;
    r->Set("value1",v1);
    if (v1 < 0.1) {
      float v2 = (float)(rand()%100)/100.;
      r->Set("value2",v2);
    }
    dbt2->AddRow(r);    
  }
  
  dbt2->WriteToDB();

  for (int i=0; i<dbt2->NRow(); ++i)
    cout << *(dbt2->GetRow(i)) << endl;

  dbt2->Clear();
  dbt2->AddDistinctColumn(std::string("channel"));
  dbt2->LoadFromDB();
  
  cout << dbt2->Name() << " has " << dbt2->NRow() << " rows." << endl;
  for (int i=0; i<dbt2->NRow(); ++i)
    cout << *(dbt2->GetRow(i)) << endl;

  // temporary addition by Kurt
  cout << "** Kurt test begin ***" << endl;
  Row* kabRow = dbt2->GetRow(1);
  cout << *kabRow << std::endl;
  int64_t kabVal;
  bool retcode;
  Column* kabCol = kabRow->Col("channel");
  retcode = kabCol->Get(kabVal);
  cout << "channel = " << kabVal << ", retcode = " << retcode << endl;
  kabCol = kabRow->Col("id");
  retcode = kabCol->Get(kabVal);
  cout << "id = " << kabVal << ", retcode = " << retcode << endl;
  retcode = dbt2->GetCurrSeqVal("id", kabVal);
  cout << "latest id = " << kabVal << ", retcode = " << retcode << endl;
  cout << "** Kurt test end ***" << endl;

  dbt2->Clear();
  dbt2->LoadFromDB();
  cout << dbt2->Name() << " has " << dbt2->NRow() << " rows." << endl;

  delete dbt2;

  return 0;
}
