#!/bin/sh

if [[ "$NOVADBNAME" == "" ]]; then
  echo "Please set up the database connection parameters first."
  echo "(e.g. 'source \`which setupNovaDBEnvVars.sh\`')"
  exit
fi

echo "Are you sure that you want to modify the $NOVADBNAME database? [y/n]"
read response

if [[ "$response" == "y" ]]; then

  utc_now=`date -u '+%Y%m%d %H:%M:%S'`

  ############################################
  ### Create the DDTM connect params table ###
  ############################################

  # update the named_subsystem_configurations table to allow for DDTManager entries
  psql -h $NOVADBHOST -d $NOVADBNAME -p $NOVADBPORT -U $NOVADBUSER \
    -c "ALTER TABLE ${NOVADAQ_ENVIRONMENT}.named_subsystem_configurations DROP CONSTRAINT named_subsystem_configurations_subsystemname_check"
  psql -h $NOVADBHOST -d $NOVADBNAME -p $NOVADBPORT -U $NOVADBUSER \
    -c "ALTER TABLE ${NOVADAQ_ENVIRONMENT}.named_subsystem_configurations ADD CONSTRAINT named_subsystem_configurations_subsystemname_check CHECK (subsystemname = 'DCMApplication'::text OR subsystemname = 'GlobalTrigger'::text OR subsystemname = 'DataLogger'::text OR subsystemname = 'BufferNodeEVB'::text OR subsystemname = 'SimulationManager'::text OR subsystemname = 'APDHardware'::text OR subsystemname = 'DDTManager'::text)"

  # create the table
  createTableInDB -d ${NOVADAQ_ENVIRONMENT} -f NovaDatabase/tables/DAQConfig/DDTManagerConnectParameters.xml -h $NOVADBHOST -n $NOVADBNAME -p $NOVADBPORT -u $NOVADBUSER

  # look up the most recent global configuration
  cfg_id=`psql -h $NOVADBHOST -d $NOVADBNAME -p $NOVADBPORT -U $NOVADBUSER -t -c "select id from ${NOVADAQ_ENVIRONMENT}.named_global_configurations order by id desc limit 1"`
  #echo "_${cfg_id}_"

  # create the appropriate entry in the subsystem configuration table
  psql -h $NOVADBHOST -d $NOVADBNAME -p $NOVADBPORT -U $NOVADBUSER \
    -c "INSERT INTO ${NOVADAQ_ENVIRONMENT}.named_subsystem_configurations (name, dcmopmode, detectorname, description, subsystemname, configtype, globalcfgid, inserttime, insertuser, isobsolete, ischanging, iscomplete) VALUES ('DDTManager connect config', 'DCS', '${NOVADAQ_ENVIRONMENT}', 'Creating the DDTManager Connect Configuration', 'DDTManager', 'connections', ${cfg_id}, '${utc_now}', '${USER}', false, true, false)"

  # look up the ID of the subsystem configuration that we just created
  cfg_id=`psql -h $NOVADBHOST -d $NOVADBNAME -p $NOVADBPORT -U $NOVADBUSER -t -c "select id from ${NOVADAQ_ENVIRONMENT}.named_subsystem_configurations where subsystemname='DDTManager' and configtype='connections' order by id desc limit 1"`
  #echo "_${cfg_id}_"

  # create a default entry in the new table
  psql -h $NOVADBHOST -d $NOVADBNAME -p $NOVADBPORT -U $NOVADBUSER \
    -c "INSERT INTO ${NOVADAQ_ENVIRONMENT}.ddt_manager_connect_parameters (hostname, nprocesses, configpath, subsysCfgId, inserttime, insertuser) VALUES ('default', 2, '/home/novadaq/ddt/current.fcl', ${cfg_id}, '${utc_now}', '${USER}')"

  # update the subsystem entry to indicate that we're done
  psql -h $NOVADBHOST -d $NOVADBNAME -p $NOVADBPORT -U $NOVADBUSER \
    -c "UPDATE ${NOVADAQ_ENVIRONMENT}.named_subsystem_configurations SET ischanging=false, iscomplete=true, updatetime='${utc_now}', updateuser='${USER}' where id=${cfg_id}"

fi
