#!/bin/sh

if [[ "$NOVADBNAME" == "" ]]; then
  echo "Please set up the database connection parameters first."
  echo "(e.g. 'source \`which setupNovaDBEnvVars.sh\`')"
  exit
fi

echo "Are you sure that you want to modify the $NOVADBNAME database? [y/n]"
read response

if [[ "$response" == "y" ]]; then

  utc_now=`date -u '+%Y%m%d %H:%M:%S'`

  ########################################
  ### Add the nano-slice version field ###
  ########################################

  # add the "NanoSliceVersion" column to the dcm_fpga_parameters table
  psql -h $NOVADBHOST -d $NOVADBNAME -p $NOVADBPORT -U $NOVADBUSER \
    -c "ALTER TABLE ${NOVADAQ_ENVIRONMENT}.dcm_fpga_parameters ADD COLUMN nanoSliceVersion INTEGER"

  # populate the "NanoSliceVersion" column with initial values
  psql -h $NOVADBHOST -d $NOVADBNAME -p $NOVADBPORT -U $NOVADBUSER \
    -c "UPDATE ${NOVADAQ_ENVIRONMENT}.dcm_fpga_parameters SET nanoSliceVersion=0"

  # update the "update" column to show that something was changed
  psql -h $NOVADBHOST -d $NOVADBNAME -p $NOVADBPORT -U $NOVADBUSER \
    -c "UPDATE ${NOVADAQ_ENVIRONMENT}.dcm_fpga_parameters SET updatetime='${utc_now}', updateuser='${USER}'"

  # modify the nanoSliceVersion column so that NULLs are not allowed
  psql -h $NOVADBHOST -d $NOVADBNAME -p $NOVADBPORT -U $NOVADBUSER \
    -c "ALTER TABLE ${NOVADAQ_ENVIRONMENT}.dcm_fpga_parameters ALTER COLUMN nanoSliceVersion SET NOT NULL"

  #########################################
  ### Create the BNEVB run params table ###
  #########################################

  # create the table
  createTableInDB -d ${NOVADAQ_ENVIRONMENT} -f NovaDatabase/tables/DAQConfig/BNEVBRunParameters.xml -h $NOVADBHOST -n $NOVADBNAME -p $NOVADBPORT -u $NOVADBUSER

  # look up the most recent global configuration
  cfg_id=`psql -h $NOVADBHOST -d $NOVADBNAME -p $NOVADBPORT -U $NOVADBUSER -t -c "select id from ${NOVADAQ_ENVIRONMENT}.named_global_configurations order by id desc limit 1"`
  #echo "_${cfg_id}_"

  # create the appropriate entry in the subsystem configuration table
  psql -h $NOVADBHOST -d $NOVADBNAME -p $NOVADBPORT -U $NOVADBUSER \
    -c "INSERT INTO ${NOVADAQ_ENVIRONMENT}.named_subsystem_configurations (name, dcmopmode, detectorname, description, subsystemname, configtype, globalcfgid, inserttime, insertuser, isobsolete, ischanging, iscomplete) VALUES ('BNEVB run config', 'DCS', '${NOVADAQ_ENVIRONMENT}', 'Creating the BNEVB Run Configuration', 'BufferNodeEVB', 'run', ${cfg_id}, '${utc_now}', '${USER}', false, true, false)"

  # look up the ID of the subsystem configuration that we just created
  cfg_id=`psql -h $NOVADBHOST -d $NOVADBNAME -p $NOVADBPORT -U $NOVADBUSER -t -c "select id from ${NOVADAQ_ENVIRONMENT}.named_subsystem_configurations where subsystemname='BufferNodeEVB' and configtype='run' order by id desc limit 1"`
  #echo "_${cfg_id}_"

  # create a default entry in the new table
  psql -h $NOVADBHOST -d $NOVADBNAME -p $NOVADBPORT -U $NOVADBUSER \
    -c "INSERT INTO ${NOVADAQ_ENVIRONMENT}.bnevb_run_parameters (poolSizeSeconds, subsysCfgId, inserttime, insertuser) VALUES (35.0, ${cfg_id}, '${utc_now}', '${USER}')"

  # update the subsystem entry to indicate that we're done
  psql -h $NOVADBHOST -d $NOVADBNAME -p $NOVADBPORT -U $NOVADBUSER \
    -c "UPDATE ${NOVADAQ_ENVIRONMENT}.named_subsystem_configurations SET ischanging=false, iscomplete=true, updatetime='${utc_now}', updateuser='${USER}' where id=${cfg_id}"

fi
