#include "ShmMilliBlock/ShmMilliBlock.h"
#include "DAQDataFormats/RawMicroSlice.h"
#include "DAQDataFormats/RawMilliBlock.h"
#include "DAQDataFormats/RawMilliBlockHeader.h"
#include "DAQDataFormats/RawMilliSlice.h"
#include "DAQDataFormats/RawNanoSlice.h"
#include "DAQDataFormats/RawNanoSliceHeader.h"

#include <boost/typeof/typeof.hpp>

#include <iostream>


// Jaroslav Zalesak: first version of TimeChecker to validate time stamps inside Milli-,
// Micro- and Nano- slices.

int main()
{
  ShmMilliBlock shared_milli_block;
  RawMilliBlock milli_block;

  unsigned n_got(0), n_ungot(0);
  unsigned global_reads(10);

  FILE *pFile;
  char pf[100];
  bool mcout = true;

  int _ivers(6);
  sprintf(pf,"timechecker.outoput.v%02d.txt",_ivers);

  pFile = fopen(pf,"w");

  while (global_reads > 0){
    // 1. Extract a milliblock from the shared memory segment:

    // The first argument is tmo_us, which could mean "time-out micro seconds".
    // A value of tmo_us = -1 is considered a "block"
    int tmo_us = 10000;
    if (shared_milli_block.getMilliBlock(tmo_us, milli_block))  {
      ++n_got;
      unsigned nano_tot(0);
            int n_slices = milli_block.slices.size();
      std::cout << "This block contains " << n_slices << " millislices." 
		<< std::endl;
	

      // 2. Loop over the millislices in the milliblock:
      unsigned milli_got(0);
      for (BOOST_AUTO(slice, milli_block.slices.begin()); 
	   slice != milli_block.slices.end(); ++slice)
      {
	++milli_got;
	
	uint32_t milTimeStart    = slice->getTimeStart();
	uint32_t milDCMId        = slice->getDCMId();
	uint32_t milMicroSliceNo = slice->getMicrosliceNumber();
	uint32_t milTimeLength   = slice->getTimeLength();
	uint32_t milSliceSize    = slice->getSliceSize();
	uint32_t milDiblockId    = slice->getDiblockId();

	if (mcout) std::cout << "Millislice Info:"
			     << "\n\tVersion:   " << slice->getVersion()
			     << "\n\tDCM ID:    " << milDCMId  << "DiBlock:" << milDiblockId
			     << "\n\tTimeStart: " << milTimeStart
			     << "\n\tTimeLength:" << milTimeLength
			     << "\n\tMicroSliceNo: " << milMicroSliceNo
			     << std::endl;

 	// Now for each millislice we loop over the microslices
	unsigned micro_got(0);
	//	for (uint32_t imicro = 0; slice->setFloatingMicroSlice(imicro); ++imicro) {
	for (uint32_t imicro = 0; imicro < milMicroSliceNo; ++imicro) {
	  slice->setFloatingMicroSlice(imicro);
 	  daqdataformats::RawMicroSlice *theSlice = slice->getFloatingMicroSlice();
	  ++micro_got;

	  uint64_t miTime     = theSlice->getTime();
	  uint32_t miHighTime = theSlice->getHighWord();
	  uint32_t miLowTime  = theSlice->getLowWord();
	  uint32_t miNoNano   = theSlice->getNumNanoSlices();

	  if (mcout) std::cout << "MicroSlice Info:"
			       << "\n\tMicroSliceNo: " << imicro
			       << "\n\tNum of Nanos: " << miNoNano
			       << "\n\tFullTimeInfo: " << miTime
			       << "\n\tHighTimeWord: " << miHighTime
			       << "\n\tLowTimeWord:  " << miLowTime
			       << std::endl;

	  unsigned nano_got(0);
	  for (uint32_t inano = 0; inano < miNoNano; ++inano) {
	    theSlice->setFloatingNanoSlice(inano);

	    daqdataformats::RawNanoSlice *theNanoSlice = theSlice->getFloatingNanoSlice();
	    ++nano_got; ++nano_tot;

	    uint32_t zTime     = theNanoSlice->getTimeStamp();
	    uint32_t zPulse    = theNanoSlice->getPulseHeight();
	    uint32_t zReserved = theNanoSlice->getReserved();

	    if (mcout) std::cout << "NanoSlice Info:"
				 << "\n\tNanoSliceNo: " << inano
				 << "\n\tTimeStart: "   << zTime
				 << "\n\tPulseHeight: " << zPulse
				 << "\n\tReserved: "    << zReserved
				 << std::endl;

	    if (mcout) theNanoSlice->print(); // printing all stuff (header, time stamp, buffer)

	    if ( mcout && (nano_tot % 1 == 0) )
	      std::cout << n_ungot << " n_got=" << n_got 
			<< " nano_tot=" << nano_tot << "_got=" << nano_got
			<< " milli=" << milli_got << " micro=" << micro_got
			<< std::endl;
	    
	    daqdataformats::RawNanoSliceHeader *theNanoSliceHeader = theNanoSlice->getHeader();
	    uint32_t zFEBId   = theNanoSliceHeader->getFEBId();
	    

// 	    std::cout << "NanoSlice Header: " 
// 		      << theNanoSliceHeader->getFEBId()
// 		      << std::endl;

	    //printing to a file	    
	    fprintf(pFile,"%u %u %u %u %u %u  %u %u  %u %u  %u %u  %u %lu %u %u  %u %u %u %u",
		    n_got,milli_got,micro_got,inano,nano_tot,nano_got,
		    milDCMId,milDiblockId,
		    milTimeStart,milTimeLength,
		    milMicroSliceNo,milSliceSize,
		    miNoNano,miTime,miHighTime,miLowTime,
		    zTime,zPulse,zReserved,zFEBId);

// 	    hits->emplace_back(&millislice,
// 			       theSlice->getFloatingNanoSlice());

	  } // Loop over nanoslices (imicro, as ranged using setFloatingNanoSlice)
	} // Loop over microslices (inano, as ranged using setFloatingMicroSlice)
      } // Loop over millislices {millislice ranged over mb.slices}
	
    } else {
    ++n_ungot;
    if (n_ungot % 1 == 0)
      std::cout << n_ungot << "   Could not get Milliblock! have good: " 
		<< n_got
		<< std::endl;
    }
    global_reads--;
  }

  fclose(pFile);

  return 0;

}
