/*  This file (shm_rd.c) was created by Ron Rechenmacher <ron@fnal.gov> on
    May 20, 2011. "TERMS AND CONDITIONS" governing this file are in the README
    or COPYING file. If you do not have such a file, one can be obtained by
    contacting Ron or Fermi Lab in Batavia IL, 60510, phone: 630-840-3000.
    $RCSfile: fast_rd.cc,v $
    rev="$Revision: 1.8 $$Date: 2014/01/22 19:29:02 $";
    */

#include <stdio.h>		// printf
#include <getopt.h>             // getopt_long
#include <libgen.h>		// basename
#include <stdlib.h>		// exit
#include <netinet/in.h>		// sockaddr_in
#include <sys/shm.h>		// shmget
#include <errno.h>		// EEXIST
#include <unistd.h>		// sleep
#include <sys/time.h>		// gettimeofday
#include <string.h>		// memcpy
#include <time.h>		// localtime
#include <stdarg.h>		// va_start
#include <fcntl.h>		// creat
#include "../include/ShmRdWr.h"		// shm_rdwr

#define DFLT_SIZE	(1<<24)
#define DFLT_NUM2N	2

#define USAGE	"\
   Usage: %s <shm_key> [\n\
Examples: %s 0x12345678\n\
Options:\n\
 -n or --bufs-num <num2n>   0=1, 1=2, 2=4, 3=8, ...\n\
 -s or --size <size>\n\
", basename(argv[0]), basename(argv[0])

//static uint32_t tcount=0;
static void tprintf( const char *fmt, ... )
    __attribute__((format(printf, 1, 2))); // g++ note: attributes are not
static void			          //  allowed on a function-definition
tprintf( const char *fmt, ... )
{
    //if (tcount > 25) return;
    //++tcount;
    timeval 	tv;
    char            str[800];
    time_t          local;
    va_list         ap;
    va_start( ap, fmt);
    gettimeofday( &tv, NULL );
    local = tv.tv_sec;
    strftime( str, sizeof(str), "%T", localtime(&local) );
    printf( "%s.%06ld ", str, (long)tv.tv_usec );
    vprintf( fmt, ap );
}   // tprintf



int
main(  int	argc
     , char	*argv[] )
{
    uint32_t    buf_siz=DFLT_SIZE;
    int		bufs2n=DFLT_NUM2N;

    int		opt;
    while (1)
    {   int option_index = 0;
        static struct option long_options[] =
        {   /* name,   has_arg, int* flag, val_for_flag */
            {"buffs-num",  1,      0,         'n'},
            {"size",       1,      0,         's'},
            {0,            0,      0,          0}
        };
        opt = getopt_long (argc, argv, "?hn:s:",
                        long_options, &option_index);
        if (opt == -1) break;
        switch (opt)
        {
        case 'n': bufs2n=strtoul(optarg,NULL,0); break;
        case 's': buf_siz=strtoul(optarg,NULL,0); break;
        case '?': case 'h':  printf( USAGE );  exit( 0 ); break;
        default:  printf ("?? getopt returned character code 0%o ??\n", opt);
        }
    }
    if (argc - optind != 1)
    {   printf( "Need shm id\n" );
        printf( USAGE ); exit( 0 );
    }

    key_t key = strtoul( argv[optind], NULL, 0 );
    printf( "key=0x%08x\n", key );


    if ((buf_siz==DFLT_SIZE) && (bufs2n==DFLT_NUM2N))
    {   printf("about to call attach with (default) params:\n");
	printf("  buffersize=%d\n", buf_siz );
	printf("  buffs_num2n=%d\n", bufs2n );
    }


# if 0   // old/bogus
    // make "bufs2n" part of key for more efficient testing
    if (bufs2n<0 || bufs2n>15) { printf("bufs2n==%d not 0<=x<=7\n",bufs2n); exit(1); }
    if (key & 7)             { printf("low bits of key(0x%08x) not 0\n",key); exit(1); }
    key |= bufs2n;
# endif

    ShmRd        shmRd;
    //int sts = shmRd.attach( key, buf_siz, bufs2n );
    int sts = shmRd.attach();
    if (sts != 0) { perror("shm_attach");exit(1);}
    printf( "shm_ptr=%p\n", shmRd.get_ptr() );
    shmRd.print_header(0);

    timeval   write_time;
    uint32_t *buffer=(uint32_t*)malloc(DFLT_SIZE);
    ShmRdWrCnt  missed;
    uint32_t    cushion;
    uint32_t    more_avail;
    uint64_t now=shmRd.get_us_timeofday();
    uint64_t next_report=now+5000000;
    uint32_t totErrAtt=0, totErrSiz=0, totErrSmall=0, totOK=0, totMissed=0, totPosOvr=0;
    uint32_t totEmpty=0,                                                    prvPosOvr=0;
    uint32_t totChkOK=0, totChkErr=0, totSizErr=0;
    uint32_t totCush=0, prvOK=0;
    while (1)
    {
	sts=shmRd.read( buffer, DFLT_SIZE, write_time, missed, cushion, more_avail );
	if (sts==-1)
	{   ++totErrAtt;
	}
	else if (sts==-2)
	{   ++totErrSiz;
	}
	else if (sts==-3)
	{   ++totErrSmall;
	    totMissed+=missed;
	}
	else if (sts==0)
	{   ++totEmpty;
	    totMissed+=missed;
	    //if(missed)tprintf("Empty missed=%u\n", missed );
	}
	else if (sts < 0)
	{   ++totPosOvr;
	    totMissed+=missed;
	    totCush  += cushion;
	    //if(missed)tprintf("PosOvr missed=%u cushion=%u\n", missed, cushion );
	}
	else
	{   ++totOK;
	    totMissed+=missed;
	    totCush  += cushion;
	    //if(missed)tprintf("OK missed=%u\n", missed );
	    uint32_t size_words=buffer[0];
	    uint32_t chksum=0, jj;
	    for (jj=0;jj<(size_words-1); jj++)
	    {   chksum += buffer[jj];
	    }
	    //tprintf("chksum=0x%08x size_words=%u [1]=%u\n",chksum, size_words, buffer[1] );
	    if ((chksum!=buffer[jj]) || (sts!=(size_words*sizeof(uint32_t))))
	    {
		if (chksum!=buffer[jj]) { ++totChkErr; }
		else                    { ++totSizErr; ++totChkOK; }
		tprintf("FATAL: chksum(0x%08x)!=buffer[jj=%d](0x%08x) size_words=%u sts=%d totOK=%d cush=%d\n"
			,chksum, jj, buffer[jj], size_words, sts, totOK, cushion );
		char fname[80];
		sprintf( fname,"error.dat");
		int fd=creat(fname,0666);
		write(fd, buffer, size_words*sizeof(uint32_t));
		close(fd);
		exit(1);
	    }
	    else
	    {   ++totChkOK;
	    }
	}

	if ((now=shmRd.get_us_timeofday()) > next_report)
	{   next_report=now+5000000;
	    uint32_t deltaCushCnt=(totOK-prvOK)+(totPosOvr-prvPosOvr);
	    float    aveCush=0.0;
	    if (deltaCushCnt) aveCush=(float)totCush/deltaCushCnt;
	    tprintf(    "bufs2n=%u ErrAtt=%u ErrSiz=%u ErrSmall=%u PosOvr=%u OK=%u ChkOK=%u ChkErr=%u Missed=%u Empty=%u aveCush=%5.3f\n"
		    , bufs2n, totErrAtt,totErrSiz,totErrSmall,totPosOvr,totOK,totChkOK,totChkErr,totMissed,totEmpty, aveCush );
	    prvOK=totOK;
	    prvPosOvr=totPosOvr;
	    totCush=0;
	}
    }

    return (0);
}   /* main */


/*
 *  With "tab-width: 4" tabs and spaces could get mixed (especially in
 *  environments where c-backspace-function is backward-delete-char-untabify)
 *  which could make things a mess in other "tab-width" environments. (Always
 *  tabifying would work, but require a lot of discipline.)
 *  Use mode: c++ in list (next) to force major-mode = c++-mode.
 *  Local variables:
 *    tab-width: 8
 *    c-tab-always-indent: nil
 *    c-basic-offset: 4
 *    c-file-offsets: ((substatement-open . 0)
                       (statement         . c-lineup-runin-statements)
                       (label             . -1000)
                       (arglist-intro     . c-lineup-arglist-intro-after-paren)
                       (arglist-cont      . c-lineup-arglist-intro-after-paren)
                       (arglist-cont-nonempty . c-lineup-arglist-intro-after-paren)
                       (arglist-close     . c-lineup-arglist-intro-after-paren)
                      )
 *  End:
 */
