/*  This file (shm_wr.c) was created by Ron Rechenmacher <ron@fnal.gov> on
    May 20, 2011. "TERMS AND CONDITIONS" governing this file are in the README
    or COPYING file. If you do not have such a file, one can be obtained by
    contacting Ron or Fermi Lab in Batavia IL, 60510, phone: 630-840-3000.
    $RCSfile: fast_wr.cc,v $
    rev="$Revision: 1.8 $$Date: 2014/01/22 19:29:02 $";
    */

#include <stdio.h>		// printf
#include <getopt.h>             // getopt_long
#include <libgen.h>		// basename
#include <stdlib.h>		// exit, rand
#include <netinet/in.h>		// sockaddr_in
#include <sys/shm.h>		// shmget
#include <errno.h>		// EEXIST
#include <unistd.h>		// sleep
#include <sys/time.h>		// gettimeofday
#include <string.h>		// memcpy
#include <time.h>		// localtime
#include <stdarg.h>		// va_start
#include <fcntl.h>		// creat
#include "../include/ShmRdWr.h"		// shm_rdwr
#include "TCAttrib.h"		// allow stdin=fd=0 getkeypressed

#define DFLT_SIZE	(1<<24)	// 16 MB

#define USAGE	"\
   Usage: %s <shm_key> [\n\
Examples: %s 0x12345678\n\
Options:\n\
 -n or --bufs-num <num2n>   0=1, 1=2, 2=4, 3=8, ...\n\
 -s or --size <size>\n\
 -p or --pause-us <usec>  pause (in microseconds) after each write\n\
", basename(argv[0]), basename(argv[0])

//static uint32_t tcount=0;
static void tprintf( const char *fmt, ... )
    __attribute__((format(printf, 1, 2))); // g++ note: attributes are not
static void			          //  allowed on a function-definition
tprintf( const char *fmt, ... )
{
    //if (tcount > 25) return;
    //++tcount;
    timeval 	tv;
    char            str[800];
    time_t          local;
    va_list         ap;
    va_start( ap, fmt);
    gettimeofday( &tv, NULL );
    local = tv.tv_sec;
    strftime( str, sizeof(str), "%T", localtime(&local) );
    printf( "%s.%06ld ", str, (long)tv.tv_usec );
    vprintf( fmt, ap );
}   // tprintf


int
main(  int	argc
     , char	*argv[] )
{
    uint32_t    buf_siz=DFLT_SIZE;
    uint32_t	opt_pause=5000;

    int		opt;
    while (1)
    {   int option_index = 0;
        static struct option long_options[] =
        {   /* name,   has_arg, int* flag, val_for_flag */
            {"buffs-num",  1,      0,         'n'},
            {"size",       1,      0,         's'},
            {"pause-us",   1,      0,         'p'},
            {0,            0,      0,          0}
        };
        opt = getopt_long (argc, argv, "?hn:s:p:",
                        long_options, &option_index);
        if (opt == -1) break;
        switch (opt)
        {
        case 'n': printf("under construction\n"); break;
        case 's': buf_siz=strtoul(optarg,NULL,0); break;
        case 'p': opt_pause=strtoul(optarg,NULL,0); break;
        case '?': case 'h':  printf( USAGE );  exit( 0 ); break;
        default:  printf ("?? getopt returned character code 0%o ??\n", opt);
        }
    }
    if (argc - optind != 1)
    {   printf( "Need shm id\n" );
        printf( USAGE ); exit( 0 );
    }

    key_t key = strtoul( argv[optind], NULL, 0 );
    printf( "key=0x%08x\n", key );


    ShmWr        shmWr;
    int sts=shmWr.attach();
    if (sts != 0) { perror("shm_attach");exit(1);}
    shm_rdwr_header *ptr=shmWr.get_ptr();
    printf( "shm_ptr=%p buf_siz=%u\n", ptr, buf_siz );
    shmWr.print_header(1);

    // get a bunch(RAND_BUFFS) of random buffers between (CNT_COMP_OFF+1)*4
    // and buf_siz bytes
#   define CNT_COMP_OFF		4
#   define RAND_BUFFS		(1<<4)
#   define RAND_BUFFS_MSK	(RAND_BUFFS-1)
    srand(1);
    uint32_t	*buffers[RAND_BUFFS];
    uint32_t	chksum, size_words;
    printf("RAND_MAX=%d\n", RAND_MAX );
    for (int ii=0; ii<RAND_BUFFS; ++ii)
    {   uint32_t  size_bytes;
	do
	{   size_bytes=(rand()%buf_siz);
	} while (size_bytes < (CNT_COMP_OFF+1)*4);
	size_words=size_bytes/sizeof(uint32_t);
	buffers[ii] = (uint32_t*)malloc(size_words*sizeof(uint32_t));
	uint32_t jj;
	for (jj=1;jj<size_words; jj++)
	{   buffers[ii][jj] = (ii+1);//rand();
	}
	buffers[ii][0] = size_words;
	buffers[ii][CNT_COMP_OFF] = 0;
	chksum=0;
	for (jj=0;jj<(size_words-1); jj++)
	{   chksum += buffers[ii][jj];
	}
	buffers[ii][jj]=chksum;
	printf("buff[%u]=%p size=%u words chksum=0x%08x\n", ii, buffers[ii], size_words, chksum );
	char fname[80];
	sprintf( fname,"buf_%02d.dat", ii);
	//int fd=creat(fname,0666);
	//write(fd, buffers[ii], size_words*sizeof(uint32_t));
	//close(fd);
    }

    uint32_t cnt_complete=ptr->idxcnt_wr_complete;
    uint32_t prv_cnt_complete=cnt_complete;
    uint32_t tot_bytes=0;
    uint64_t now=shmWr.get_us_timeofday();
    uint64_t last_report_time=now;
    uint64_t next_report=now+5000000;
    TCAttrib::keypress(0);
    int header_out=0;
    float    pause_us=(float)opt_pause;
    while (1)
    {	uint32_t *randbuf = buffers[rand()&RAND_BUFFS_MSK];
	size_words = randbuf[0];
	chksum = randbuf[size_words-1] - randbuf[CNT_COMP_OFF];
	randbuf[CNT_COMP_OFF] = ++cnt_complete;
	//tprintf( "randbuf=%p size=%u [1]=%u chksum: orig:0x%08x now:0x%08x\n"
	//	, randbuf, size_words, randbuf[1], randbuf[size_words-1], chksum+randbuf[CNT_COMP_OFF] );
	randbuf[size_words-1] = chksum+randbuf[CNT_COMP_OFF];
	tot_bytes += size_words*sizeof(uint32_t);

	int sts=shmWr.write( randbuf, size_words*sizeof(uint32_t) );
	if (sts<=0) { tprintf( "write error sts=%d", sts ); exit(1); }

	if ((now=shmWr.get_us_timeofday()) > next_report)
	{
	    float second_since_last_report = (float)(now-last_report_time)/1000000;
	    float buffs_per_sec=(float)(cnt_complete-prv_cnt_complete)/second_since_last_report;
	    float bytes_per_sec=(float)tot_bytes/second_since_last_report;
	    if (header_out++%14 == 0)
	    {   header_out=1;
		printf("#                              i/d\n");
		printf("#          time bufs2n sts pause_us cnt_complete buffs/s     MB/s\n");
	    }
	    tprintf( "%3d %8u  %11u %7.2f %8.3f\n"
		    , sts, (uint32_t)pause_us, cnt_complete, buffs_per_sec, bytes_per_sec/(1024*1024) );
	    next_report=now+5000000;
	    last_report_time=now;
	    prv_cnt_complete=cnt_complete;
	    tot_bytes=0;
	}
	char key;
	if (read(0,&key,1) == 1)
	{
	    switch (key)
	    {
	    case 'i':  pause_us*=1.01; break;
	    case 'I':  pause_us*=1.10; break;
	    case 'd':  pause_us*=0.99009900990099009900990099; break;
	    case 'D':  pause_us*=0.90909090909090909090909090; break;
	    }
	    next_report=shmWr.get_us_timeofday();
	}
	usleep( pause_us );
    }



    return (0);
}   /* main */


/*
 *  With "tab-width: 4" tabs and spaces could get mixed (especially in
 *  environments where c-backspace-function is backward-delete-char-untabify)
 *  which could make things a mess in other "tab-width" environments. (Always
 *  tabifying would work, but require a lot of discipline.)
 *  Use mode: c++ in list (next) to force major-mode = c++-mode.
 *  Local variables:
 *    tab-width: 8
 *    c-tab-always-indent: nil
 *    c-basic-offset: 4
 *    c-file-offsets: ((substatement-open . 0)
                       (statement         . c-lineup-runin-statements)
                       (label             . -1000)
                       (arglist-intro     . c-lineup-arglist-intro-after-paren)
                       (arglist-cont      . c-lineup-arglist-intro-after-paren)
                       (arglist-cont-nonempty . c-lineup-arglist-intro-after-paren)
                       (arglist-close     . c-lineup-arglist-intro-after-paren)
                      )
 *  End:
 */
