/*  This file (shm_rd.c) was created by Ron Rechenmacher <ron@fnal.gov> on
    May 20, 2011. "TERMS AND CONDITIONS" governing this file are in the README
    or COPYING file. If you do not have such a file, one can be obtained by
    contacting Ron or Fermi Lab in Batavia IL, 60510, phone: 630-840-3000.
    $RCSfile: shm_rd.cc,v $
    rev="$Revision: 1.4 $$Date: 2011/11/02 19:16:44 $";
    */
#define USAGE	"\
   Usage: %s <shm_key> [\n\
Examples: %s 0x12345678\n\
Options:\n\
 -n or --bufs-num <num>\n\
 -s or --size <size>\n\
", basename(argv[0]), basename(argv[0])


#include <stdio.h>		// printf
#include <getopt.h>             // getopt_long
#include <libgen.h>		// basename
#include <stdlib.h>		// exit
#include <netinet/in.h>		// sockaddr_in
#include <sys/shm.h>		// shmget
#include <errno.h>		// EEXIST
#include <unistd.h>		// sleep
#include <sys/time.h>		// gettimeofday
#include <string.h>		// memcpy
#include <time.h>		// localtime
#include <stdarg.h>		// va_start
#include "../include/ShmRdWr.h"		// shm_rdwr

#define DFLT_SIZE	0x780000
#define DFLT_NUM2N	2


static void tprintf( const char *fmt, ... )
    __attribute__((format(printf, 1, 2))); // g++ note: attributes are not
static void			          //  allowed on a function-definition
tprintf( const char *fmt, ... )
{   timeval 	tv;
    char            str[800];
    time_t          local;
    va_list         ap;
    va_start( ap, fmt);
    gettimeofday( &tv, NULL );
    local = tv.tv_sec;
    strftime( str, sizeof(str), "%T", localtime(&local) );
    printf( "%s.%06ld ", str, (long)tv.tv_usec );
    vprintf( fmt, ap );
}   // tprintf



int
main(  int	argc
     , char	*argv[] )
{
    uint32_t    buf_siz=DFLT_SIZE;
    int		bufs_2n=DFLT_NUM2N;
    int		pause_before_close=0;

    int		opt;
    while (1)
    {   int option_index = 0;
        static struct option long_options[] =
        {   /* name,   has_arg, int* flag, val_for_flag */
            {"buffs-num",  1,      0,         'n'},
            {"size",       1,      0,         's'},
            {"pause",      0,      0,         'p'},
            {0,            0,      0,          0}
        };
        opt = getopt_long (argc, argv, "?hn:s:p",
                        long_options, &option_index);
        if (opt == -1) break;
        switch (opt)
        {
        case 'n': bufs_2n=strtoul(optarg,NULL,0); break;
        case 's': buf_siz=strtoul(optarg,NULL,0); break;
	case 'p': pause_before_close=1; break;
        case '?': case 'h':  printf( USAGE );  exit( 0 ); break;
        default:  printf ("?? getopt returned character code 0%o ??\n", opt);
        }
    }
    if (argc - optind != 1)
    {   printf( "Need shm id\n" );
        printf( USAGE ); exit( 0 );
    }

    key_t key = strtoul( argv[optind], NULL, 0 );
    printf( "key=0x%08x\n", key );


    if ((buf_siz==DFLT_SIZE) && (bufs_2n==DFLT_NUM2N))
    {   printf("about to call attach with (default) params:\n");
	printf("  buffersize=%d\n", buf_siz );
	printf("  buffs_num2n=%d\n", bufs_2n );
    }

    ShmRd        shmRd;
    int sts = shmRd.attach( key, buf_siz, bufs_2n );
    if (sts != 0) { perror("shm_attach");exit(1);}
    printf( "shm_ptr=%p\n", shmRd.get_ptr() );
    shmRd.print_header(0);

    timeval  write_time;
    uint8_t  buf[0x10000];
    ShmRdWrCnt missed;
    uint32_t   cushion;
    uint32_t   more_avail;
    while (sleep(1) == 0)
    {
	int sts2=shmRd.read( buf, sizeof(buf), write_time, missed, cushion, more_avail );
	tprintf( "shm_read=%d missed=%d cushion=%u\n", sts2, (uint32_t)missed, cushion );
	sts2=shmRd.read_wait( buf, sizeof(buf), write_time, missed, cushion, more_avail
				 , 1500000 );
	tprintf( "shm_read_wait=%d missed=%d cushion=%u\n"
		, sts2, (uint32_t)missed, cushion );
    }

    if (pause_before_close)
    {   printf("press enter to continue...\n");
	char buf[80];
	fgets(buf,sizeof(buf),stdin);
    }
    return (0);
}   /* main */


/*
 *  With "tab-width: 4" tabs and spaces could get mixed (especially in
 *  environments where c-backspace-function is backward-delete-char-untabify)
 *  which could make things a mess in other "tab-width" environments. (Always
 *  tabifying would work, but require a lot of discipline.)
 *  Use mode: c++ in list (next) to force major-mode = c++-mode.
 *  Local variables:
 *    tab-width: 8
 *    c-tab-always-indent: nil
 *    c-basic-offset: 4
 *    c-file-offsets: ((substatement-open . 0)
                       (statement         . c-lineup-runin-statements)
                       (label             . -1000)
                       (arglist-intro     . c-lineup-arglist-intro-after-paren)
                       (arglist-cont      . c-lineup-arglist-intro-after-paren)
                       (arglist-cont-nonempty . c-lineup-arglist-intro-after-paren)
                       (arglist-close     . c-lineup-arglist-intro-after-paren)
                      )
 *  End:
 */
