//  This file (test_shmrw.cc) was created by Ron Rechenmacher <ron@fnal.gov> on
//  Oct  2, 2013. "TERMS AND CONDITIONS" governing this file are in the README
//  or COPYING file. If you do not have such a file, one can be obtained by
//  contacting Ron or Fermi Lab in Batavia IL, 60510, phone: 630-840-3000.
//  $RCSfile: test_shmrw.cc,v $
//  rev="$Revision: 1.1 $$Date: 2013/10/03 03:02:54 $";

#include <stdio.h>		// printf
#include <stdlib.h>		// system
#include <iostream>		// cout
#include "../include/ShmRdWr.h"

int
main(  int	argc
     , char	*argv[] )
{
    ShmRdWr	shmrw;
    char cmd[100];
    snprintf( cmd, sizeof(cmd), "ipcrm -S0x%08x -M0x%08x"
	     , shmrw.getShmKey(), shmrw.getShmKey() );
    printf("the system cmd is: %s\n", cmd);
    system(cmd);
    int sts = shmrw.attach();
    if (sts != 0)
    {  fprintf(stderr,"attach failed -- make sure env.var. %s is set."
	       "(remove shm key w/ ipcrm -M key -S key)\n"
	       , shmrw.envvar() );
       return (1);
    }
    snprintf( cmd, sizeof(cmd), "ipcs | grep -i 0x%08x", shmrw.getShmKey() );
    printf("the system cmd is: %s\n", cmd);
    system(cmd);

    printf("sizeof(ShmRdWr_BufMetadata)=%ld sizeof(ShmRdWr_Rdr)=%ld\n"
	   ,sizeof(ShmRdWr_BufMetadata),    sizeof(ShmRdWr_Rdr) );

#   ifdef TEST
    printf("proving bufcnt_add, bufcnt_delta, bufcnt2idx\n");
    printf("largest multiple=0x%08x\n",shmrw._shm_hdr_ptr->bufs_largest_multiple);
    uint16_t bufs_num=shmrw._shm_hdr_ptr->bufs_num;
    uint32_t wr=shmrw._shm_hdr_ptr->bufs_largest_multiple - 4;
    uint32_t rd=0x0;
    for (int ii=0; ii<8; ++ii)
    {   printf("bufs=%2u bufcnt_delta(wr=0x%08x,rd=0x%08x)=%10u widx=%d\n"
	       ,bufs_num,wr,rd,shmrw.bufcnt_delta(wr,rd), shmrw.bufcnt2idx(wr) );
	wr = shmrw.bufcnt_add(wr,1);
    }
    printf("\n");
    wr=shmrw._shm_hdr_ptr->bufs_largest_multiple - 3;
    rd=shmrw._shm_hdr_ptr->bufs_largest_multiple - 3;
    for (int ii=0; ii<8; ++ii)
    {   uint32_t wr_4=shmrw.bufcnt_add(wr,-bufs_num);
	printf("bufs=%2u bufcnt_delta(wr=0x%08x,rd=0x%08x)=%10u widx=%d wr_4=0x%x\n"
	       ,bufs_num,wr,rd,shmrw.bufcnt_delta(wr,rd), shmrw.bufcnt2idx(wr)
	       ,wr_4 );
	wr = shmrw.bufcnt_add(wr,1);
    }
#   endif


    char buf[120];
    uint32_t ov1, ov2;

    buf[0]=0x55;
    sts = shmrw.write( buf, sizeof(buf) );
    buf[0]=0x56;
#   if 1
    sts = shmrw.write( buf, sizeof(buf), (off_t)-1 );
#   else
    sts = shmrw.write( buf, sizeof(buf), -1 );
#   endif
    printf( "write sts=%d\n",sts );

    std::cout << shmrw;

    sts = shmrw.read( buf, sizeof(buf), ov1, 100 );
    printf( "read     sts=%d buf[0]=0x%02x ov1=%u\n",sts,buf[0],ov1 );

    sts = shmrw.read_grp( buf, sizeof(buf), ov1 );
    printf( "read_grp sts=%d buf[0]=0x%02x ov1=%u\n",sts,buf[0],ov1 );
    std::cout << shmrw;

    sts = shmrw.read_grp( buf, sizeof(buf), ov1, 100 );
    printf( "read_grp sts=%d buf[0]=0x%02x ov1=%u\n",sts,buf[0],ov1 );
    std::cout << shmrw;

    sts = shmrw.read_grp( buf, sizeof(buf), ov1, 100 );
    printf( "read_grp sts=%d buf[0]=0x%02x ov1=%u\n",sts,buf[0],ov1 );
    std::cout << shmrw;


    printf( "empty; write 1 more\n" );

    buf[0]=0x57; sts = shmrw.write( buf, sizeof(buf) );

    time_t tmo=-1;
    ssize_t s1,s2;
    ShmRdWr_Info info;
    s1 = shmrw.read_grp( 0,   0,                       info, tmo, true  );
    std::cout << shmrw; printf("semval SHOULD BE 0\n");
    s2 = shmrw.read_grp( buf, sizeof(buf), ov2,  0 );
    printf( "read_grp s1=%ld s2=%ld buf[0]=0x%02x ov1=%u ov2=%u\n"
	   ,s1,s2,buf[0],ov1,ov2 );


    printf( "empty; write 1 more - zero byte this time\n" );

    sts = shmrw.write( buf, 0 );
    std::cout << shmrw;

    s1 = shmrw.read_grp( 0,   0,                       info, tmo, true  );
    std::cout << shmrw;
    if (info.bufInfo.data_bytes)
	s2 = shmrw.read_grp( buf, sizeof(buf), ov2,  0 );
    printf( "read_grp s1=%ld s2=%ld buf[0]=0x%02x ov1=%u ov2=%u\n"
	   ,s1,s2,buf[0],ov1,ov2 );

    printf( "Simulating overwrite...\n" );

    buf[0]=0x58; sts = shmrw.write( buf, sizeof(buf) );
    buf[0]=0x59; sts = shmrw.write( buf, sizeof(buf) );
    buf[0]=0x5a; sts = shmrw.write( buf, sizeof(buf) );
    buf[0]=0x5b; sts = shmrw.write( buf, sizeof(buf) );

    printf( "start read\n" );
    s1 = shmrw.read_grp( 0, 0, info, tmo, ShmRdWr::semKeepAfterDataAvail  );

    printf( "start write\n" );
    shmrw.write( &buf[40], sizeof(buf)/3,true, 40*sizeof(buf[0]) );

    printf( "attempting to finish read (%ld bytes)\n",info.bufInfo.data_bytes );
    s2 = shmrw.read_grp( buf, sizeof(buf), ov2,  0 );
    printf( "read_grp s1=%ld s2=%ld buf[0]=0x%02x ov1=%u ov2=%u\n"
	   ,s1,s2,buf[0],ov1,ov2 );

    printf( "finish write\n" );
    shmrw.write( &buf[80], sizeof(buf)/3,true );
    shmrw.write( &buf[0],  sizeof(buf)/3,(off_t)0 );

    std::cout << shmrw;

    return (0);
}   // main
