#ifndef _RMS_MESSAGELISTENER_H
#define _RMS_MESSAGELISTENER_H

#include <rms/base/RmsMessage.h>

#include <boost/shared_ptr.hpp>

namespace gov {

namespace fnal {

namespace cd {

namespace rms {

/**
 * This Interface defines the interface used by message listeners in RMS.
 * A message listener is an object that is notified asynchronously when
 * messages arrive (similar to a callback in C).
 *
 * @author Kurt Biery
 * @author Steve Foulkes
 * @version $Revision: 1.1.1.1 $ $Date: 2009/10/09 20:03:00 $
 */

class RmsMessageListener {
 public:
    
    /**
     * Destructor (declared virtual to ensure that derived class
     * destructors are always called).
     */
    virtual ~RmsMessageListener() {}

    /**
     * Passes the specified message to the listener.
     *
     * @param theMessage The message that was received by a consumer
     * or producer and is now being passed to the listener.
     */
    virtual void messageReceived(boost::shared_ptr<base::RmsMessage> 
                                 theMessage) = 0;
};

};  // end of namespace rms

};  // end of namespace cd

};  // end of namespace fnal

};  // end of namespace gov

#endif
