#ifndef _RMS_MESSAGESOURCE_H
#define _RMS_MESSAGESOURCE_H

#include <string>
#include <rms/RmsMessageSchema.h>

namespace gov {

namespace fnal {

namespace cd {

namespace rms {

namespace base {

/**
 * Class to represent a messages's origin.
 *
 * @author Kurt Biery
 * @author Steve Foulkes
 * @version $Revision: 1.1.1.1 $ $Date: 2009/10/09 20:03:00 $
 */

class RmsMessageSource : public gov::fnal::cd::rms::xsd::messageSource {
 public:
    /**
     * Default constructor.
     */
    RmsMessageSource() {}

    /**
     * Constructs an instance from an instance of the XSD base class.
     */
    RmsMessageSource(const xsd::messageSource& baseSource) : 
        xsd::messageSource(baseSource) {}

    /**
     * Get the hostname of the application that sent the message.
     *
     * @return The hostname of the application that sent the message.
     */
    std::string getHostName() const {
        return this->hostName();
    }

    /**
     * Get the name of the application that sent the message.
     *
     * @return The name of the application that sent the message.
     */
    std::string getApplicationName() const {
        return this->applicationName();
    }
  
    /**
     * Set the hostname parameter.
     *
     * @param name The new hostname
     */
    void setHostName(std::string name) {
        this->hostName(name);
    }

    /**
     * Set the application name parameter.
     *
     * @param name The new application name
     */
    void setApplicationName(std::string name) {
        this->applicationName(name);
    }
};

};  // end of namespace base

};  // end of namespace rms

};  // end of namespace cd

};  // end of namespace fnal

};  // end of namespace gov

#endif
