#ifndef _RMS_RUNTIMEEXCEPTION_H
#define _RMS_RUNTIMEEXCEPTION_H

#include <execinfo.h>
#include <stdexcept>
#include <string>
#include <sstream>

#define GENERATE_RMS_RUNTIME_EXCEPTION(msg) \
  throw gov::fnal::cd::rms::base::RmsRuntimeException(__FILE__, __LINE__, msg)

#define GENERATE_RMS_NOTCONNECTED_EXCEPTION(msg) \
throw gov::fnal::cd::rms::base::RmsNotConnectedException(__FILE__, __LINE__, msg)

#define GENERATE_RMS_EXITINGPROCESS_EXCEPTION(msg) \
throw gov::fnal::cd::rms::base::RmsExitingProcessException(__FILE__, __LINE__, msg)

#if CETDDS_DEBUG
#define LOG_EXCEPTION(message,what) std::cerr << message << what << std::endl
#else
#define LOG_EXCEPTION(message,what)
#endif

namespace gov {

namespace fnal {

namespace cd {

namespace rms {

namespace base {

/**
 * Class to report runtime exceptions.
 *
 * @author Kurt Biery
 * @author Steve Foulkes
 * @version $Revision: 1.2 $ $Date: 2011/05/12 17:11:14 $
 */

class RmsRuntimeException : public std::exception {

  public:

    /**
     * Constructs a runtime exception instance from the specified
     * file, line number, and message.
     */
    RmsRuntimeException(const std::string &fileName, int line,
                        const std::string &message) {

        // convert the line number to a string
        std::stringstream tmpStream;
        tmpStream << line;
        std::string lineNumber = tmpStream.str();

        // build up the exception text
        _fullText = message + "\n" + "    RmsRuntimeException generated at " +
            fileName + ":" + lineNumber;

        // add some limited stack trace information
        void *array[10];
        size_t size;
        char **strings;
        size_t idx;

        size = backtrace (array, 10);
        strings = backtrace_symbols (array, size);
        tmpStream.seekp(0);  // clear out the stream
        for (idx = 0; idx < size; idx++) {
            tmpStream << "      " << strings[idx] << std::endl;
        }
        free (strings);

        _fullText += "\n    Stack trace:\n" + tmpStream.str();
    }

    /**
     * Destructor.
     */
    virtual ~RmsRuntimeException() throw() { 
        return; 
    }

    /**
     * Returns a description of the exception.
     */
    const char* what() const throw() {
        return _fullText.c_str();
    }

 private:

    /**
     * The full description of the exception.
     */
    std::string _fullText;

};

class RmsNotConnectedException : public  RmsRuntimeException
{
public:
  RmsNotConnectedException(const std::string &fileName, int line,const std::string &message)
  :RmsRuntimeException(fileName,line,message) {};
};

class RmsExitingProcessException : public  RmsRuntimeException
{
public:
  RmsExitingProcessException(const std::string &fileName, int line,const std::string &message)
  :RmsRuntimeException(fileName,line,message) {};
};

};  // end of namespace base

};  // end of namespace rms

};  // end of namespace cd

};  // end of namespace fnal

};  // end of namespace gov

#endif
 
