#ifndef _RMS_UUID_H
#define _RMS_UUID_H

/**
 * @package gov.fnal.cd.rms.util
 *
 * @brief A collection of classes that simplify miscellaneous
 * tasks.
 *
 * The util package contains helper classes for managing threads,
 * parsing time and date strings, generating UUIDs as well as
 * thread safe data structures.
 */

#include <rms/util/Sha1.h>

#include <boost/random/linear_congruential.hpp>
#include <boost/random/uniform_int.hpp>
#include <boost/random/variate_generator.hpp>

#include <string>
#include <sys/time.h>

namespace gov {

namespace fnal {

namespace cd {

namespace rms {

namespace util {

/**
 * This class generates time based type 1 UUIDs, as specified in RFC4122.
 *
 * @author Kurt Biery
 * @author Steve Foulkes
 * @version $Revision: 1.1.1.1 $ $Date: 2009/10/09 20:03:00 $
 */

class UUIDGenerator {
 public:
    UUIDGenerator(std::string nodeName);
    
    const std::string getUUIDHex();
    const char *getUUID();

 private:
    /**
     * The time at which the last UUID was generated. 
     */
    struct timeval _previousTime;    

    /**
     * This is used to differentiate UUIDs that are generated within the 
     * same clock tick 
     */
    int _overrunCounter;

    /**
     * The name of the node that is generating the UUIDs.  This should 
     * be unique.
     */
    std::string _nodeName;

    /**
     * An SHA1 hash of the nodename 
     */
    unsigned char _nodeNameHash[20]; 

    /** 
     * The clock sequence, a 16 bit random number 
     */
    unsigned char _clockSequence[2]; 

};

};  // end of namespace util

};  // end of namespace rms

};  // end of namespace cd

};  // end of namespace fnal

};  // end of namespace gov

#endif
