#include <rms/provider/MessageSplitter.h>

#include <iostream>
#include <cmath>

namespace gov {

namespace fnal {

namespace cd {

namespace rms {

namespace provider {

/**
 * Split a long string message into several message fragments
 * that can be serialized.
 *
 * @param messageText The message to split
 * @param maxFragmentSize The maximum size of the message fragments
 * that will be generated.
 * @param uuid The UUID of the message
 *
 * @return A vector of message fragments
 */
std::vector <MessageFragment *> 
MessageSplitter::splitMessage(std::string messageText, 
                              int maxFragmentSize,
                              const char *uuid) {
    std::vector <MessageFragment *> messageFragments;
    std::string stringFragment;
    MessageFragment *fragment;
    int index;

    maxFragmentSize -= MessageFragment::getMaxHeaderSize();

    if (maxFragmentSize < 1) {
        return messageFragments;
    }

    int bytesRemaining = messageText.length();

    int numFragments = 
        (int) ceil((float)messageText.length() / maxFragmentSize);

    for (index = 0; bytesRemaining > 0; index++) {
        if (bytesRemaining > maxFragmentSize) {
            stringFragment = 
                messageText.substr(index * maxFragmentSize, maxFragmentSize);
        }
        else {
            stringFragment = 
                messageText.substr(index * maxFragmentSize, bytesRemaining);
        }

        fragment = new MessageFragment(uuid, index, numFragments);
        fragment->setPayload(stringFragment);
        messageFragments.push_back(fragment);
        bytesRemaining -= maxFragmentSize;
    }

    return messageFragments;
}
  
};  // end of namespace provider

};  // end of namespace rms

};  // end of namespace cd

};  // end of namespace fnal

};  // end of namespace gov
