#include <rms/base/RmsDestination.h>
#include <rms/base/RmsRuntimeException.h>
#include <iostream>

using namespace gov::fnal::cd::rms::base;

int main( int argc, char *argv[] )
{
    int testCount = 0;

    // create a sample destination
    testCount++;
    std::cout << "\n>>>>> Test " << testCount << " <<<<<" << std::endl;
    std::cout << "Creating a sample destination with "
              << "three properties..." << std::endl;
    RmsDestination destOrig;
    destOrig.setProperty(RmsDestination::TARGET_PROPERTY_NAME, "sampleTarget");
    destOrig.setProperty(RmsDestination::CHANNEL_PROPERTY_NAME,
                         RmsDestination::CONTROL_CHANNEL);
    destOrig.setProperty(RmsDestination::MESSAGE_TYPE_PROPERTY_NAME,
                         RmsDestination::NOTIFICATION_MESSAGE_TYPE);

    // check the behavior of toString();
    std::cout << "\nResult of toString() call on original "
              << "destination:" << std::endl;
    std::cout << "----------------------------------------" << std::endl;
    std::cout << destOrig.toString() << std::endl;
    std::cout << "----------------------------------------" << std::endl;

    // create a copy of the original destination
    testCount++;
    std::cout << "\n>>>>> Test " << testCount << " <<<<<" << std::endl;
    std::cout << "Creating a copy of the original destination..." << std::endl;
    RmsDestination destCopy(destOrig);

    // check the behavior of toString();
    std::cout << "\nResult of toString() call on "
              << "destination copy:" << std::endl;
    std::cout << "----------------------------------------" << std::endl;
    std::cout << destCopy.toString() << std::endl;
    std::cout << "----------------------------------------" << std::endl;

    // check behavior of equals(), matches(), and hashcode()
    testCount++;
    std::cout << "\n>>>>> Test " << testCount << " <<<<<" << std::endl;
    std::cout << "Does the copy equal the original? "
              << (destCopy.equals(destOrig) ? "yes" : "no")
              << " (it should)" << std::endl;
    std::cout << "Does the copy match the original? "
              << (destCopy.matches(destOrig) ? "yes" : "no")
              << " (it should)" << std::endl;

    // modify the copy
    testCount++;
    std::cout << "\n>>>>> Test " << testCount << " <<<<<" << std::endl;
    std::cout << "Modifying the copy..." << std::endl;
    destCopy.setProperty(RmsDestination::CHANNEL_PROPERTY_NAME,
                         RmsDestination::ALERT_CHANNEL);

    // check the behavior of toString();
    std::cout << "\nResult of toString() call on "
              << "modified copy:" << std::endl;
    std::cout << "----------------------------------------" << std::endl;
    std::cout << destCopy.toString() << std::endl;
    std::cout << "----------------------------------------" << std::endl;

    // check behavior of equals(), matches(), and hashcode()
    testCount++;
    std::cout << "\n>>>>> Test " << testCount << " <<<<<" << std::endl;
    std::cout << "Does the copy now equal the original? "
              << (destCopy.equals(destOrig) ? "yes" : "no")
              << " (it shouldn't)" << std::endl;
    std::cout << "Does the copy now match the original? "
              << (destCopy.matches(destOrig) ? "yes" : "no")
              << " (it shouldn't)" << std::endl;

    // further check of the behavior of matches()
    testCount++;
    std::cout << "\n>>>>> Test " << testCount << " <<<<<" << std::endl;
    std::cout << "Creating a destination to test for matches..." << std::endl;
    RmsDestination matchDest;
    matchDest.setProperty(RmsDestination::TARGET_PROPERTY_NAME,"sampleTarget");
    matchDest.setProperty(RmsDestination::CHANNEL_PROPERTY_NAME,
                          RmsDestination::CONTROL_CHANNEL);
    std::cout << "\nResult of toString() call on "
              <<  "matchTest destination:" << std::endl;
    std::cout << "----------------------------------------" << std::endl;
    std::cout << matchDest.toString() << std::endl;
    std::cout << "----------------------------------------" << std::endl;
    std::cout << "Does the matchTest match the original? "
              << (matchDest.matches(destOrig) ? "yes" : "no")
              << " (it should)" << std::endl;
    std::cout << "Does the matchTest match the copy? "
              << (matchDest.matches(destCopy) ? "yes" : "no")
              << " (it shouldn't)" << std::endl;

    testCount++;
    std::cout << "\n>>>>> Test " << testCount << " <<<<<" << std::endl;
    std::cout << "Modifying the matchTest destination..." << std::endl;
    matchDest.setProperty(RmsDestination::SOURCE_PROPERTY_NAME, "RunControl0");
    std::cout << "\nResult of toString() call on "
              << "modified matchTest destination:" << std::endl;
    std::cout << "----------------------------------------" << std::endl;
    std::cout << matchDest.toString() << std::endl;
    std::cout << "----------------------------------------" << std::endl;
    std::cout << "Does the matchTest match the original? "
              << (matchDest.matches(destOrig) ? "yes" : "no")
              << " (it should)" << std::endl;
    std::cout << "Does the matchTest match the copy? "
              << (matchDest.matches(destCopy) ? "yes" : "no")
              << " (it shouldn't)" << std::endl;

    testCount++;
    std::cout << "\n>>>>> Test " << testCount << " <<<<<" << std::endl;
    std::cout << "Modifying the matchTest destination..." << std::endl;
    matchDest.setProperty(RmsDestination::MESSAGE_TYPE_PROPERTY_NAME,
                          RmsDestination::NOTIFICATION_MESSAGE_TYPE);
    std::cout << "\nResult of toString() call on "
              <<  "modified matchTest destination:" << std::endl;
    std::cout << "----------------------------------------" << std::endl;
    std::cout << matchDest.toString() << std::endl;
    std::cout << "----------------------------------------" << std::endl;
    std::cout << "Does the matchTest match the original? "
              << (matchDest.matches(destOrig) ? "yes" : "no")
              << " (it should)" << std::endl;
    std::cout << "Does the matchTest match the copy? "
              << (matchDest.matches(destCopy) ? "yes" : "no")
              << " (it shouldn't)" << std::endl;

    // check that validation is working
    testCount++;
    std::cout << "\n>>>>> Test " << testCount << " <<<<<" << std::endl;
    std::cout << "Creating a destination to test validation..." << std::endl;
    RmsDestination vdnDest;
    vdnDest.setProperty(RmsDestination::PARTITION_PROPERTY_NAME, "0");
    std::cout << "\nResult of toString() call on "
              << "validation destination:" << std::endl;
    std::cout << "----------------------------------------" << std::endl;
    std::cout << vdnDest.toString() << std::endl;
    std::cout << "----------------------------------------" << std::endl;
    std::cout << "Is the destination valid? (it shouldn't be)" << std::endl;
    try {
        vdnDest.validate();
        std::cout << "  ERROR: the destination was incorrectly "
                  << "determined to be valid!" << std::endl;
    }
    catch (RmsRuntimeException& excpt) {
        std::cout << "  The destination was correctly "
                  << "determined to be invalid:" << std::endl;
        std::cout << "    " << excpt.what() << std::endl;
    }

    testCount++;
    std::cout << "\n>>>>> Test " << testCount << " <<<<<" << std::endl;
    std::cout << "Modifying the validation destination..." << std::endl;
    vdnDest.setProperty(RmsDestination::CHANNEL_PROPERTY_NAME,
                        RmsDestination::MONITOR_CHANNEL);
    std::cout << "\nResult of toString() call on "
              << "validation destination:" << std::endl;
    std::cout << "----------------------------------------" << std::endl;
    std::cout << vdnDest.toString() << std::endl;
    std::cout << "----------------------------------------" << std::endl;
    std::cout << "Is the destination valid? (it shouldn't be)" << std::endl;
    try {
        vdnDest.validate();
        std::cout << "  ERROR: the destination was incorrectly "
                  << "determined to be valid!" << std::endl;
    }
    catch (RmsRuntimeException& excpt) {
        std::cout << "  The destination was correctly "
                  << "determined to be invalid:" << std::endl;
        std::cout << "    " << excpt.what() << std::endl;
    }

    testCount++;
    std::cout << "\n>>>>> Test " << testCount << " <<<<<" << std::endl;
    std::cout << "Modifying the validation destination..." << std::endl;
    vdnDest.setProperty(RmsDestination::TARGET_PROPERTY_NAME, "dcm005");
    std::cout << "\nResult of toString() call on "
              << "validation destination:" << std::endl;
    std::cout << "----------------------------------------" << std::endl;
    std::cout << vdnDest.toString() << std::endl;
    std::cout << "----------------------------------------" << std::endl;
    std::cout << "Is the destination valid? (it should be)" << std::endl;
    try {
        vdnDest.validate();
        std::cout << "  The destination was correctly "
                  << "determined to be valid." << std::endl;
    }
    catch (RmsRuntimeException& excpt) {
        std::cout << "  ERROR: The destination was incorrectly "
                  << "determined to be invalid:" << std::endl;
        std::cout << "    "  << excpt.what() << std::endl;
    }

    testCount++;
    std::cout << "\n>>>>> Test " << testCount << " <<<<<" << std::endl;
    std::cout << "Modifying the validation destination..." << std::endl;
    vdnDest.setProperty(RmsDestination::SOURCE_PROPERTY_NAME, "RunControl2");
    std::cout << "\nResult of toString() call on "
              << "validation destination:" << std::endl;
    std::cout << "----------------------------------------" << std::endl;
    std::cout << vdnDest.toString() << std::endl;
    std::cout << "----------------------------------------" << std::endl;
    std::cout << "Is the destination valid? (it shouldn't be)" << std::endl;
    try {
        vdnDest.validate();
        std::cout << "  ERROR: the destination was incorrectly "
                  << "determined to be valid!" << std::endl;
    }
    catch (RmsRuntimeException& excpt) {
        std::cout << "  The destination was correctly "
                  << "determined to be invalid:" << std::endl;
        std::cout << "    " << excpt.what() << std::endl;
    }

    // check how empty inputs are handled
    testCount++;
    std::cout << "\n>>>>> Test " << testCount << " <<<<<" << std::endl;
    std::cout << "Trying EMPTY property name and value..." << std::endl;
    try {
        destCopy.setProperty("", "Test Value");
        std::cout << "WARNING: an empty property name did NOT " <<
            "generate an exception!  This could lead to " <<
            "undefined behavior!" << std::endl;
    }
    catch (RmsRuntimeException& excpt) {
        std::cout << "setProperty() with an empty property name " <<
            "generated an exception, as expected:\n    " <<
            excpt.what() << std::endl;
    }
    try {
        destCopy.setProperty("Test Name", "");
        std::cout << "WARNING: an empty property value did NOT " <<
            "generate an exception!  This could lead to " <<
            "undefined behavior!" << std::endl;
    }
    catch (RmsRuntimeException& excpt) {
        std::cout << "setProperty() with an empty property value " <<
            "generated an exception, as expected:\n    " <<
            excpt.what() << std::endl;
    }

    // test lookups
    std::string name, value;
    testCount++;
    std::cout << "\n>>>>> Test " << testCount << " <<<<<" << std::endl;
    std::cout << "For the original destination:" << std::endl;
    name = RmsDestination::TARGET_PROPERTY_NAME;
    value = destOrig.getProperty(name);
    std::cout << "    The value for the \"" << name << "\""
              << " property is \"" << value << "\"." << std::endl;
    name = RmsDestination::CHANNEL_PROPERTY_NAME;
    value = destOrig.getProperty(name);
    std::cout << "    The value for the \"" << name << "\""
              << " property is \"" << value << "\"." << std::endl;
    name = "nonsense";
    value = destOrig.getProperty(name);
    std::cout << "    The value for the \"" << name << "\""
              << " property is \"" << value << "\" (should be empty)."
              << std::endl;

    // test constructor with 3 arguments
    testCount++;
    std::cout << "\n>>>>> Test " << testCount << " <<<<<" << std::endl;
    std::cout << "Creating a destination with "
              << "the three-argument constructor..." << std::endl;
    RmsDestination dest3Arg("configManager",
                            RmsDestination::CONFIGURATION_CHANNEL,
                            RmsDestination::REQUEST_MESSAGE_TYPE);
    std::cout << "\nResult of toString() call on 3-arg "
              << "destination:" << std::endl;
    std::cout << "----------------------------------------" << std::endl;
    std::cout << dest3Arg.toString() << std::endl;
    std::cout << "----------------------------------------" << std::endl;

    // test constructor with 4 arguments
    testCount++;
    std::cout << "\n>>>>> Test " << testCount << " <<<<<" << std::endl;
    std::cout << "Creating a destination with "
              << "the four-argument constructor..." << std::endl;
    RmsDestination dest4Arg("allElements",
                            RmsDestination::HEARTBEAT_CHANNEL,
                            RmsDestination::REQUEST_MESSAGE_TYPE,
                            15);
    std::cout << "\nResult of toString() call on 4-arg "
              << "destination:" << std::endl;
    std::cout << "----------------------------------------" << std::endl;
    std::cout << dest4Arg.toString() << std::endl;
    std::cout << "----------------------------------------" << std::endl;

    std::cout << "" << std::endl;

    return 0;
}
