#include <rms/provider/RmsConnection.h>
#include <rms/provider/RmsConnectionFactory.h>
#include <rms/base/RmsDestination.h>
#include <rms/base/RmsMessage.h>
#include <rms/base/RmsMessageBody.h>
#include <rms/RmsProducer.h>

#include <boost/shared_ptr.hpp>

using namespace gov::fnal::cd::rms;

void DestructorTest(void) {
    provider::RmsConnectionFactory connectionFactory;
    boost::shared_ptr<provider::RmsConnection> rmsConnection;

    /**
     * Create an RmsConnection object using the connection factory.
     * We want replies routed to the destination with a target equal
     * to "RmsProducerExample" so we set that as the applicationName
     * and -1 as the partition as we don't care about partitions.
     */
    rmsConnection = connectionFactory.getConnection("RmsDestructorTest", -1);

    /**
     * Create a destination to represent where the messages that are
     * generated here will go to.  For the example, we're sending messages
     * to the "RmsConsumerExample" target.  The messages will use the control
     * channel and be request messages.
     */
    base::RmsDestination exampleDest("RmsDestructorDestination",
                                     base::RmsDestination::CONTROL_CHANNEL,
                                     base::RmsDestination::REQUEST_MESSAGE_TYPE);

    /**
     * Create a producer with the connection, destination and reply destination.
     */
    RmsProducer exampleProducer(rmsConnection, exampleDest);

    exampleProducer.sendString("Hello World!");

    exampleProducer.close();
    rmsConnection->close();

    return;
}

int main(int argc, char **argv) {
    int i, numRuns;

    if (argc != 2) {
        std::cout << "Usage: ./DestructorTest [ number of runs ]" << std::endl;
        return 0;
    }

    numRuns = atoi(argv[1]);

    std::cout << "Going to run the test " << numRuns << " times:" << std::endl;

    for (i = 0; i < numRuns; i++) {
        DestructorTest();
        std::cout << ".";

        if (i % 10 == 0 && i != 0) {
            std::cout << std::endl;
        }
    }
    
    std::cout << std::endl;

    return 0;
}
