#include <rms/provider/EpicsMessenger.h>
#include <rms/provider/ProviderListener.h>
#include <rms/util/UUIDGenerator.h>

#include <cadef.h>

#include <boost/shared_ptr.hpp>

#include <iostream>
#include <vector>
#include <string>

#include <unistd.h>

using namespace std;
using namespace gov::fnal::cd::rms;

class EpicsMessengerTest : public provider::ProviderListener {
public:
  int msgsReceived;

  void textReceived(boost::shared_ptr<string> messageText) {
    cout << "Received message " << *messageText << endl;
    msgsReceived++;
  }

  EpicsMessengerTest() {
    msgsReceived = 0;
  }
};

int main() {
  struct ca_client_context *_caCurrentContext;
  provider::EpicsMessenger *messenger;
  util::UUIDGenerator *uuidGenerator;
  EpicsMessengerTest *test;
  int received;

  test = new EpicsMessengerTest();

  ca_context_create(ca_enable_preemptive_callback);
  _caCurrentContext = ca_current_context();
  ca_attach_context(_caCurrentContext);

  uuidGenerator = new util::UUIDGenerator(std::string("test"));

  messenger = new provider::EpicsMessenger(std::string("control/inbox"), _caCurrentContext, 
					   1024, uuidGenerator);

  if(messenger->getName() != std::string("control/inbox")) {
    cout << "getName() failed" << endl;
    
    return -1;
  }

  messenger->addListener(test);

  sleep(10);

  messenger->removeListener(test);

  received = test->msgsReceived;

  if(received > 0) {
    cout << "I have received " << received << " messages" << endl;
  }
  else {
    cout << "No messages received" << endl;
    return -1;
  }
  
  sleep(10);

  if(received != test->msgsReceived) {
    cout << "Still receiving messages, removeListener() is broken" << endl;
    
    return -1;
  }

  messenger->addListener(test);

  sleep(10);

  if(received == test->msgsReceived) {
    cout << "Did not receive any new messages, removeListener() is broken" << endl;
    
    return -1;
  }

  cout << "All tests passed..." << endl;

  return 0;
}

