#include <NovaDAQUtilities/XMLDeserializationRegistry.h>
#include <rms/base/RmsMessage.h>
#include <rms/util/UUIDGenerator.h>
#include <rms/util/TimeUtils.h>
#include <fstream>
#include <sstream>
#include <iostream>

using namespace gov::fnal::cd::rms;

int main( int argc, char *argv[] )
{
  std::string tempString;

  base::RmsMessageSource msgSource;
  msgSource.setHostName("localhost");
  msgSource.setApplicationName("The world's greatest application");
  //std::cout << msgSource.serialize() << std::endl;

  base::RmsMessageBody msgBody;
  msgBody.setClassName("gov.fnal.cd.rms.HelloWorld");
  msgBody.setContent("<That's all folks!>");

  base::RmsDestination dest1;
  dest1.setProperty("target", "dcm005");
  dest1.setProperty("service", "control");
  //std::cout << dest1.getProperty("target") << std::endl;

  base::RmsDestination dest2;
  dest2.setProperty("target", "RunControl2");
  dest2.setProperty("service", "control");

  util::UUIDGenerator generator(std::string("nodeName"));

  base::RmsMessage rmsMsg;
  rmsMsg.setSource(msgSource);
  rmsMsg.setBody(msgBody);
  rmsMsg.setDestination(dest1);
  rmsMsg.setReplyDestination(dest2);
  rmsMsg.setId(generator.getUUIDHex());
  rmsMsg.setCorrelationId(generator.getUUIDHex());
  rmsMsg.setTimeToLive(5000);
  rmsMsg.setSentTime(gov::fnal::cd::rms::util::TimeUtils::getUniversalPtime());
  rmsMsg.setCorrelationTime(gov::fnal::cd::rms::util::TimeUtils::getUniversalPtime());
  rmsMsg.setPriority(5);

  // extra test - see if cloning works
  boost::shared_ptr<novadaq::XMLSerializable> transPtr = rmsMsg.clone();
  base::RmsMessage *msgPtr = (base::RmsMessage*) transPtr.get();
  msgPtr->setId(generator.getUUIDHex());
  msgPtr->setPriority(10);

  std::string xmlString = rmsMsg.serialize();
  std::cout << "Original message:" << std::endl;
  std::cout << xmlString << std::endl;
  std::cout << std::endl;

  std::cout << "Copy of message:" << std::endl;
  std::cout << msgPtr->serialize() << std::endl;
  base::RmsStatus statObj = msgPtr->getMessageStatus();
  std::cout << "  status = " << statObj.getCode() << std::endl;


  // more extra tests - check if TransmittableFactory works
  novadaq::XMLDeserializationRegistry& tFact =
      novadaq::XMLDeserializationRegistry::getInstance();

  boost::shared_ptr<novadaq::XMLSerializable> ptr1(new base::RmsMessage());

  msgPtr = (base::RmsMessage*) ptr1.get();
  std::cout << "ptr1 status = " << msgPtr->getMessageStatus().getCode() << std::endl;

  tFact.registerClass(ptr1->getClassName(), ptr1);
  boost::shared_ptr<novadaq::XMLSerializable> ptr2;
  ptr2 = tFact.createObject("a.b.c");
  std::cout << "Created a.b.c address = " << ptr2.get() << std::endl;
  ptr2 = tFact.createObject("gov::fnal::cd::rms::base::RmsMessage");
  std::cout << "Created RmsMessage address = " << ptr2.get() << std::endl;

  msgPtr = (base::RmsMessage*) ptr2.get();
  std::cout << "initial ptr2 status = "
            << msgPtr->getMessageStatus().getCode() << std::endl;
  if (msgPtr->hasId()) {
      tempString = msgPtr->getId();
      std::cout << "ptr2 id = " << tempString << std::endl;
  }

  if (ptr2.get() != NULL) {
    try {
      msgPtr = (base::RmsMessage*) ptr2.get();
      msgPtr->deserialize(xmlString);
      std::cout << "Original message after deser and reser:" << std::endl;
      std::cout << ptr2->serialize() << std::endl;
      statObj = msgPtr->getMessageStatus();
      std::cout << "  final ptr2 status = " << statObj.getCode() << std::endl;

      tempString = msgPtr->getId();
      std::cout << "ptr2 id = " << tempString << std::endl;
    }
    catch (const ::xsd::cxx::tree::expected_element< char >& e) {
      std::cout << "ERROR - caught expected_element exception, "
                << "missing element = " << e.name() << std::endl;
    }
  }

  boost::shared_ptr<novadaq::XMLSerializable> ptr8(new base::RmsMessage());
  tFact.registerClass(ptr8->getClassName(), ptr8);

  return 0;
}
