#include <rms/provider/MessageSplitter.h>
#include <rms/provider/MessageAssembler.h>
#include <rms/provider/MessageFragment.h>

#include <iostream>
#include <string>
#include <vector>
#include <cstring>  /* strncpy */

using namespace std;
using namespace gov::fnal::cd::rms::provider;

int main() {
  MessageSplitter splitter;
  MessageAssembler *assembler;
  vector <MessageFragment *> fragments;
  string payload = std::string("test test test test test test");
  char *uuid;

  uuid = new char[16];
  strncpy(uuid, "1234567890123456", 16);

  fragments = splitter.splitMessage(payload, 100, uuid);

  if(fragments.size() != 1) {
    cout << "Did not get the correct number of fragments" << endl;
    
    return -1;
  }

  assembler = new MessageAssembler(fragments[0]);

  if(!assembler->hasCompleteMessage()) {
    cout << "failed to assemble first message" << endl;

    return -1;
  }

  if(assembler->getMessage() != payload) {
    cout << "assembled payload does not match" << endl;
    
    return -1;
  }

  fragments = splitter.splitMessage(payload, 44, uuid);

  if(fragments.size() != 2) {
    cout << "Did not get the correct number of fragments" << endl;
    
    return -1;
  }

  delete assembler;

  assembler = new MessageAssembler(fragments[0]);
  assembler->addFragment(fragments[1]);

  if(!assembler->hasCompleteMessage()) {
    cout << "failed to assemble second message" << endl;

    return -1;
  }

  if(assembler->getMessage() != payload) {
    cout << "assembled payload does not match" << endl;
    cout << "received: " << assembler->getMessage() << endl;

    return -1;
  }

  fragments = splitter.splitMessage(payload, 38, uuid);

  if(fragments.size() != 3) {
    cout << "Did not get the correct number of fragments" << endl;
    
    return -1;
  }

  delete assembler;

  assembler = new MessageAssembler(fragments[0]);
  assembler->addFragment(fragments[1]);
  assembler->addFragment(fragments[2]);

  if(!assembler->hasCompleteMessage()) {
    cout << "failed to assemble second message" << endl;

    return -1;
  }

  if(assembler->getMessage() != payload) {
    cout << "assembled payload does not match" << endl;
    
    return -1;
  }
  
  delete[] uuid;
  delete assembler;

  cout << "All tests passed..." << endl;

  return 0;
}
