#!/bin/bash

#arguments
# $1 as string  // process name
function pidof {
ps -u `whoami` -f |  grep -i "$1" | awk -v psname=$1 '{if($8==psname) print $2}'
}

#arguments
# $1 as string        // process name
# $2 as unsigned int  // kill signal; default is 9 (kill -9)
function kill-if-running {
proc_id=$( pidof $1);
kill_signal=$2;

if [[ $kill_signal == "" ]] ; then
  kill_signal=9;
  echo "Using default kill signal number ($kill_signal)."
fi

if [ $proc_id ]; then
  if ! kill $kill_signal $proc_id > /dev/null 2>&1; then
    echo "Could not kill \"$1\" (pid=$proc_id) with the -$kill_signal signal."
  else
    echo  "Process \"$1\" (pid=$proc_id) was killed with the -$kill_signal signal."
  fi
else
  echo "Process \"$1\" was not running."
fi
}

#arguments
# $1 as string        // command line
# $2 as string        // log file name
# $3 as unsigned int  // duration in seconds; default is 10 seconds
# $4 as string        // kill signal number; default is 9 (kill -9)
function run-process-specified-time {
monitored_process=$1;
monitored_process_log=$2;
run_seconds=$3;
kill_signal=$4;

if [[ $run_seconds == "" ]] ; then
  run_seconds=10;
fi

if [[ $monitored_process_log == "" ]]; then
  monitored_process_log="~/tmp-Monitored-Process-$HOSTNAME.log";
  echo "Setting process output log to $monitored_process_log ."
fi

touch $monitored_process_log

if [[ $kill_signal == "" ]] ; then
  kill_signal=9;
  echo "Using default kill signal number ($kill_signal)."
fi

echo "Starting \"$monitored_process\"."

$monitored_process >> $monitored_process_log 2>&1 &
proc_id="$!"

echo "Started \"$monitored_process\" (pid=$proc_id)."

if [ $proc_id ] ; then
  echo "Running \"$monitored_process\" for $run_seconds seconds."

  sleep  $run_seconds;

  if ! kill -$kill_signal $proc_id ; then
    echo "Could not kill \"$monitored_process\" (pid=$proc_id)."
  else
    echo  "Process \"$monitored_process\" (pid=$proc_id) was killed."
  fi
else
  echo "\"$monitored_process\" was not running."
fi
}

#arguments
# $1 as string        // command line
# $2 as string        // log file name
# $3 as unsigned int  // duration in seconds; default is 10 seconds
# $4 as string        // kill signal number; default is 9 (kill -9)
function run-process-untill-pattern {
monitored_process=$1;
monitored_process_log=$2;
match_pattern=$3;
expect_timeout=$4;

if [[ $run_seconds == "" ]] ; then
  run_seconds=10;
fi

if [[ $monitored_process_log == "" ]]; then
  monitored_process_log="~/tmp-Monitored-Process-$HOSTNAME.log";
  echo "Setting process output log to $monitored_process_log ."
fi

touch $monitored_process_log

if [[ expect_timeout == "" ]] ; then
  expect_timeout=30;
  echo "Setting default expect timeout to $expect_timeout seconds."
fi

echo "Timeout= $expect_timeout, exit pattern=$match_pattern"

expect -c "
set timeout $expect_timeout;
puts \"Starting $monitored_process\";
spawn $monitored_process ;
expect \"$match_pattern\";	
sleep .5;
puts \"Aborting $monitored_process\";
send \\003;"
}

#arguments
# $1 as string        // osdds version
function restart-odds
{
  ospl stop;
  unsetup osdds;
  setup osdds $1;
  ospl start;
}

#arguments
# $1 as string        // osdds version
function restart-odds-X86
{
  restart-odds $1;
}

#arguments
# $1 as string        // osdds version
function restart-odds-PPC
{
  restart-odds $1 -q GCC_4_1_2 -f LinuxPPC; 
}



#arguments
#none
function echo-time {
date "+%m/%d/%Y-%H:%M";
}

function test_run-process-specified-time {
echo "Test 1"
run-process-specified-time "sleep 9" "tmpLogFile.log" 5 2;

echo "Test 2"
run-process-specified-time "sleep 2" "tmpLogFile.log" 5 2;

echo "Test 3"
run-process-specified-time "sleep 15" "tmpLogFile.log" 8;

echo "Test 4"
run-process-specified-time "sleep 15" "tmpLogFile.log";

echo "Test 5"
run-process-specified-time "sleep 2";
}
