#!/bin/sh

MESSAGE_COUNT=300000
MAX_NODE_TEST_TIME=60
TIME_BETWEEN_TESTS=80
DCM_LIST=(dcm-3-01-01 dcm-3-01-02 dcm-3-01-03 dcm-3-02-01 dcm-3-02-02 dcm-3-02-03 dcm-3-03-01 dcm-3-03-02 dcm-3-03-03 dcm-3-04-01 dcm-3-04-02)

if [ "$NOVADAQ_PARTITION_NUMBER" == "" ]; then
    echo "Please set up the online software environment first."
    exit
fi

echo "Are you sure that you want to test DDS on the DCMs in Partition ${NOVADAQ_PARTITION_NUMBER}? [y/n]"
read response

if [[ "$response" != "y" ]]; then
    echo "Aborting..."
    exit
fi

echo "Is DDS running on the DCMs in Partition ${NOVADAQ_PARTITION_NUMBER}? [y/n]"
read response

if [[ "$response" != "y" ]]; then
    echo "Aborting..."
    exit
fi

killall RmsSenderExample
killall RmsReceiverExample

# send a set of messages that is designed to go everywhere as a way to verify
# which DDS daemons are running when looking at the Ganglia plots
date
RmsSenderExample -i -R -s 100 -a RunControl -t allElements -n ${MESSAGE_COUNT}
date

# now loop over each of the target nodes
for NODENAME in "${DCM_LIST[@]}"; do
    APPNAME=${NODENAME}

    echo "*** Starting test of ${APPNAME} on ${NODENAME}."
    ssh -xkn root@${NODENAME} "killall RmsSenderExample"
    ssh -xkn root@${NODENAME} "killall RmsReceiverExample"
    sleep 5

    # outgoing RunControl messages
    ssh -xkn root@${NODENAME} "(source /home/novadaq/DAQOperationsTools/novadaq_setup.sh -z ${NOVADAQ_PARTITION_NUMBER}; nohup RmsReceiverExample -i -n ${MESSAGE_COUNT} -a ${APPNAME}) 2>1 1>/tmp/${APPNAME}_test.log &"
    sleep $TIME_BETWEEN_TESTS

    (date; RmsSenderExample -i -a RunControl -t ${APPNAME} -n ${MESSAGE_COUNT}; date) &

    sleep $MAX_NODE_TEST_TIME
    killall RmsSenderExample
    ssh -xkn root@${NODENAME} "killall RmsReceiverExample"
    sleep $TIME_BETWEEN_TESTS

    # incoming MsgFac messages
    RmsReceiverExample -i -R -a MessageFacility_Message -n ${MESSAGE_COUNT} &
    sleep 5

    ssh -xkn root@${NODENAME} "(source /home/novadaq/DAQOperationsTools/novadaq_setup.sh -z ${NOVADAQ_PARTITION_NUMBER}; nohup RmsSenderExample -i -R -s 100 -n ${MESSAGE_COUNT} -t MessageFacility_Message) 2>1 1>/tmp/${APPNAME}_test2.log &"

    sleep $MAX_NODE_TEST_TIME
    killall RmsReceiverExample
    ssh -xkn root@${NODENAME} "killall RmsSenderExample"
done
