#!/bin/sh

MESSAGE_COUNT=300000
MAX_NODE_TEST_TIME=60
TIME_BETWEEN_TESTS=80
let FIRST_NODE_NUMBER=1
let LAST_NODE_NUMBER=14

if [ "$NOVADAQ_PARTITION_NUMBER" == "" ]; then
    echo "Please set up the online software environment first."
    exit
fi

echo "Are you sure that you want to test DDS on the farm nodes in Partition ${NOVADAQ_PARTITION_NUMBER}? [y/n]"
read response

if [[ "$response" != "y" ]]; then
    echo "Aborting..."
    exit
fi

echo "Is DDS running on the farm nodes in Partition ${NOVADAQ_PARTITION_NUMBER}? [y/n]"
read response

if [[ "$response" != "y" ]]; then
    echo "Aborting..."
    exit
fi

killall RmsSenderExample
killall RmsReceiverExample

# send a set of messages that is designed to go everywhere as a way to verify
# which DDS daemons are running when looking at the Ganglia plots
date
RmsSenderExample -i -R -s 100 -a RunControl -t allElements -n ${MESSAGE_COUNT}
date

# now loop over each of the target nodes
let nodeNumber=${FIRST_NODE_NUMBER}
while [ $nodeNumber -le ${LAST_NODE_NUMBER} ]; do
    sleep $TIME_BETWEEN_TESTS

    numberString=""
    if [ $nodeNumber -lt 10 ]; then
        numberString="0"
    fi
    numberString="${numberString}${nodeNumber}"

    NODENAME=novadaq-ctrl-farm-${numberString}
    APPNAME=bnevb0${numberString}

    echo "*** Starting test of ${APPNAME} on ${NODENAME}."
    ssh -xkn ${NODENAME} "killall RmsSenderExample"
    ssh -xkn ${NODENAME} "killall RmsReceiverExample"
    sleep 2

    # outgoing RunControl messages
    ssh -xkn ${NODENAME} "(source /home/novadaq/DAQOperationsTools/novadaq_setup.sh -z ${NOVADAQ_PARTITION_NUMBER}; nohup RmsReceiverExample -i -n ${MESSAGE_COUNT} -a ${APPNAME}) 2>1 1>/tmp/${APPNAME}_test.log &"
    sleep 5

    (date; RmsSenderExample -i -a RunControl -t ${APPNAME} -n ${MESSAGE_COUNT}; date) &

    sleep $MAX_NODE_TEST_TIME
    killall RmsSenderExample
    ssh -xkn ${NODENAME} "killall RmsReceiverExample"
    sleep $TIME_BETWEEN_TESTS

    # incoming MsgFac messages
    RmsReceiverExample -i -R -a MessageFacility_Message -n ${MESSAGE_COUNT} &
    sleep 5

    ssh -xkn ${NODENAME} "(source /home/novadaq/DAQOperationsTools/novadaq_setup.sh -z ${NOVADAQ_PARTITION_NUMBER}; nohup RmsSenderExample -i -R -s 100 -n ${MESSAGE_COUNT} -t MessageFacility_Message) 2>1 1>/tmp/${APPNAME}_test2.log &"

    sleep $MAX_NODE_TEST_TIME
    killall RmsReceiverExample
    ssh -xkn ${NODENAME} "killall RmsSenderExample"

    let nodeNumber=${nodeNumber}+1
done
