package gov.fnal.cd.rms;

/**
 * This class holds monitoring information about any application
 * that may be broadcast periodically to determine the health
 * of the sending application.
 *
 * @author Kurt Biery
 * @author Steve Foulkes
 * @version $Revision: 1.2 $ $Date: 2009/11/24 23:56:47 $
 */
public class Heartbeat extends novadaq.CastorSerializableBase {
    /**
     * The name of the application that is sending the 
     * heartbeats.
     */
    private String _applicationName = null;

    /**
     * The version of the application that is sending
     * the heartbeats.
     */
    private String _applicationVersion = null;

    /**
     * The time that a heartbeat was sent.
     */
    private long _timestamp = 0;

    /**
     * Create an instance of the hearbeat class with a particular
     * application name, version and timestamp.
     *
     * @param applicationName The application that is sending out the
     * heartbeats
     * @param applicationVersion The version of the application that is
     * sending out heartbeats
     * @param timestamp The time this heartbeat was sent out
     */
    public Heartbeat(String applicationName, String applicationVersion,
		     long timestamp) {
	_applicationName = applicationName;
	_applicationVersion = applicationVersion;
	_timestamp = timestamp;
    }

    /**
     * Create an empty heartbeat class, as all the parameters
     * can be set at a later time.
     */
    public Heartbeat() {
    }

    /**
     * Return the application name that is associated with
     * this heartbeat object.
     *
     * @return The application name
     */
    public String getApplicationName() {
	return _applicationName;
    }
    
    /**
    * Return the application version that is associated with
    * this heartbeat object.
    *
    * @return The application version
    */
    public String getApplicationVersion() {
	return _applicationVersion;
    }

    /**
    * Resturn the timestamp that is associated with
    * this heartbeat object.
    *
    * @return The heartbeat timestamp
    */
    public long getTimestamp() {
	return _timestamp;
    }

    /**
     * Set the application name that is stored in this heartbeat
     * object.
     *
     * @param applicationName The new application name
     */
    public void setApplicationName(String applicationName) {
	_applicationName = applicationName;
    }

    /**
     * Set the application version that is stored in this heartbeat
     * object.
     *
     * @param applicationVersion The new application version.
     */
    public void setApplicationVersion(String applicationVersion) {
	_applicationVersion = applicationVersion;
    }

    /**
     * Set the timestamp that is stored in this heartbeat object.
     *
     * @param timestamp The new timestamp.
     */
    public void setTimestamp(long timestamp) {
	_timestamp = timestamp;
    }
}
