package gov.fnal.cd.rms.base;

import gov.fnal.cd.rms.util.RmsException;

/**
 * An exception class for reporting communication problems in RMS.
 *
 * @author Kurt Biery
 * @author Steve Foulkes
 * @version $Revision: 1.1.1.1 $ $Date: 2009/10/09 20:03:00 $
 */
public class RmsCommunicationException extends RmsException {

    /**
     * The destination associated with the exception, if any.
     */
    private RmsDestination _destination = null;

    /**
     * Constructs a new instance with the specified detail message and cause.
     *
     * @param messageText the error message
     * @param cause the exception or error contained within this exception
     */
    public RmsCommunicationException(String messageText, Throwable cause) {
        super(messageText, cause);
    }

    /**
     * Constructs a new instance with the specified detail message, cause,
     * and associated destination
     *
     * @param messageText the error message
     * @param cause the exception or error contained within this exception
     * @param destination the RmsDestination associated with the exception
     */
    public RmsCommunicationException(String messageText, Throwable cause,
                                     RmsDestination destination) {
        super(messageText, cause);
        _destination = destination;
    }

    /**
     * Returns the RmsDestination associated with the exception or null
     * if none was specified.
     *
     * @return the RmsDestination associated with the exception
     */
    public RmsDestination getDestination() {
        return _destination;
    }
}
