package gov.fnal.cd.rms.base;

import gov.fnal.cd.rms.util.RmsException;

/**
 * An exception class for reporting data problems in RMS.
 *
 * @author Kurt Biery
 * @author Steve Foulkes
 * @version $Revision: 1.1.1.1 $ $Date: 2009/10/09 20:03:00 $
 */
public class RmsDataException extends RmsException {

    /**
     * The data associated with the exception, if any.
     */
    private String _messageData = null;

    /**
     * Constructs a new instance with the specified detail message and cause.
     *
     * @param exceptionText the error message
     * @param cause the exception or error contained within this exception
     */
    public RmsDataException(String exceptionText, Throwable cause) {
        super(exceptionText, cause);
    }

    /**
     * Constructs a new instance with the specified detail message, cause,
     * and data string.
     *
     * @param exceptionText the error message
     * @param cause the exception or error contained within this exception
     * @param data the data string associated with the exception
     */
    public RmsDataException(String exceptionText,Throwable cause,String data) {
        super(exceptionText, cause);
        _messageData = data;
    }

    /**
     * Returns the data associated with the exception or null
     * if none was specified.
     *
     * @return the string data associated with the exception
     */
    public String getData() {
        return _messageData;
    }

    /**
     * Sets the data associated with the exception.
     *
     * @param data the string data associated with the exception
     */
    public void setData(String data) {
        _messageData = data;
    }
}
