package gov.fnal.cd.rms.base;

/**
 * Sample message.
 *
 * @author Kurt Biery
 * @author Steve Foulkes
 * @version $Revision: 1.2 $ $Date: 2009/11/24 23:56:47 $
 */
public class SampleMessage extends novadaq.CastorSerializableBase {
    /**
     * Default constructor.
     */
    public SampleMessage() {}

    /**
     * Returns the integer value.
     */
    public int getIntValue() {
        return _intValue;
    }

    /**
     * Returns the string value.
     */
    public String getStringValue() {
        return _stringValue;
    }

    /**
     * Returns the double array.
     */
    public double[] getValueArray() {
        return _valueArray;
    }

    /**
     * Sets the integer value.
     */
    public void setIntValue(int newValue) {
        _intValue = newValue;
    }

    /**
     * Sets the string value.
     */
    public void setStringValue(String newValue) {
        _stringValue = newValue;
    }

    /**
     * Sets the double array.
     */
    public void setValueArray(double[] newArray) {
        _valueArray = newArray;
    }

    /**
     * Main method for unit testing.
     */
    public static void main(String[] args) {
        int testCount = 0;

        // create a sample message
        testCount++;
        System.out.println("\n>>>>> Test " + testCount + " <<<<<");
        System.out.println("Creating a sample message and storing " +
                           "it in an RmsMessage...");
        double[] tmpArray = new double[5];
        for (int idx = 0; idx < 5; idx++) {
            tmpArray[idx] = (double) (idx * idx);
        }
        SampleMessage sampleMessage = new SampleMessage();
        sampleMessage.setIntValue(100);
        sampleMessage.setStringValue("Hello, world!");
        sampleMessage.setValueArray(tmpArray);
        String xmlString = "";
        System.out.println("Result of serializing the RmsMessage...");
        System.out.println("----------------------------------------");
        try {
            RmsMessage rmsMessage = new RmsMessage(sampleMessage);
            xmlString = rmsMessage.serialize();
            System.out.println(xmlString);
        }
        catch (Exception excpt) {
            System.err.println("Exception caught while serializing");
            excpt.printStackTrace(System.err);
        }
        System.out.println("----------------------------------------");

        // test deserialization
        testCount++;
        System.out.println("\n>>>>> Test " + testCount + " <<<<<");
        System.out.println("Deserializing the message created in " +
                           "the first test...");
        System.out.println("Result of deserializing the XML and " +
                           "and re-serializing the message...");
        System.out.println("----------------------------------------");
        try {
            RmsMessage rmsMessage = new RmsMessage();
            rmsMessage.deserialize(xmlString);
            System.out.println(rmsMessage.serialize());
        }
        catch (Exception excpt) {
            System.err.println("Exception caught while deserializing");
            excpt.printStackTrace(System.err);
        }
        System.out.println("----------------------------------------");

        // test re-creating the body object
        testCount++;
        System.out.println("\n>>>>> Test " + testCount + " <<<<<");
        System.out.println("Recreating the SampleMessage stored in " +
                           "the original RMS message...");
        System.out.println("Result of serializing the reconstructed " +
                           "SampleMessage...");
        System.out.println("----------------------------------------");
        try {
            RmsMessage rmsMessage = new RmsMessage();
            rmsMessage.deserialize(xmlString);

            novadaq.XMLSerializable transObj = rmsMessage.getBodyObject();
            if (transObj != null) {
                System.out.println(transObj.serialize());
                if (transObj instanceof gov.fnal.cd.rms.base.SampleMessage) {
                    System.out.println("Successfully restored SampleMessage " +
                                       "from RMS message body");
                }
            }
        }
        catch (Exception excpt) {
            System.err.println("Exception caught while serializing");
            excpt.printStackTrace(System.err);
        }
        System.out.println("----------------------------------------");

        System.out.println("");
    }

    /**
     * integer attribute
     */
    private int _intValue;

    /**
     * string attribute
     */
    private String _stringValue;

    /**
     * array of double attribute
     */
    private double[] _valueArray;

}
