package gov.fnal.cd.rms.provider;

import java.util.concurrent.LinkedBlockingQueue;

/**
 * This class provides an implementation of the ProviderListener interface
 * that copies the messages that are received into a specified queue.
 *
 * @author Kurt Biery
 * @author Steve Foulkes
 * @version $Revision: 1.1.1.1 $ $Date: 2009/10/09 20:03:00 $
 */
public class BufferedProviderListener implements ProviderListener {
    /**
     * The queue that is used to provide the buffering of the messages
     * that are received from the provider.
     */
    private LinkedBlockingQueue <String> _inputQueue;

    /**
     * Create a new BufferedProviderListener with a specific inputQueue.
     *
     * @param inputQueue The queue that message should be inserted into
     * as they are received.
     */
    public BufferedProviderListener(LinkedBlockingQueue <String> inputQueue) {
        _inputQueue = inputQueue;
    }

    /**
     * Add messages to the queue as they are received.
     *
     * @param messageText The serialized message that will be inserted
     * onto the queue.
     */
    public void textReceived(String messageText) {
        _inputQueue.add(messageText);
    }
}
