package gov.fnal.cd.rms.provider;

import DDS.*;
import java.lang.reflect.*;
import java.util.*;

public class DDSReader {

    private String _messageDataType;

    private DDS.WaitSet _waitSet;
    private DDS.DomainParticipantQosHolder _dpQos;
    private DDS.TopicQosHolder             _tQos;
    private DDS.SubscriberQosHolder        _sQos;
    private DDS.DataReaderQosHolder        _drQos;
    private DDS.DomainParticipantFactory _dpf;
    private DDS.DomainParticipant        _dp;
    private DDS.Topic                    _theTopic;
    private DDS.Subscriber               _subscriber;
    private DDS.StatusCondition          _PP_string_sc;

    //private helloworld.HelloWorldMessageTypeSupport _PP_string_dt;
    //private helloworld.HelloWorldMessageDataReader _PP_string_reader;
    private Object _PP_string_dt;
    private Object _PP_string_reader;

    public DDSReader(String messageDataType, String domainURI,
                     String topicName, String channelName) {
   
        int     result;

        _messageDataType = messageDataType;

        /*
         * Create WaitSet
         */
        _waitSet = new DDS.WaitSet ();

        /*
         * Initialize Qos variables
         */ 
        _dpQos = new DDS.DomainParticipantQosHolder();
        _tQos  = new DDS.TopicQosHolder();
        _sQos  = new DDS.SubscriberQosHolder();
        _drQos = new DDS.DataReaderQosHolder();

        /*
         * Create participant
         */
        _dpf = DDS.DomainParticipantFactory.get_instance ();
        _dpf.get_default_participant_qos (_dpQos);
        _dp = _dpf.create_participant (domainURI, _dpQos.value, null, DDS.ANY_STATUS.value);
        if (_dp == null) {
            System.out.println ("CONSUMER: ERROR - Splice Daemon not running");
            return;
        }
        _dpQos = null;

        /*
         * Modify default Topic Qos settings
         */
        _dp.get_default_topic_qos (_tQos);
        _tQos.value.reliability.kind = DDS.ReliabilityQosPolicyKind.RELIABLE_RELIABILITY_QOS;
        //_tQos.value.history.kind = DDS.HistoryQosPolicyKind.KEEP_ALL_HISTORY_QOS;
        _dp.set_default_topic_qos (_tQos.value);

        /*
         * Create subscriber
         */
        _dp.get_default_subscriber_qos (_sQos);
        _sQos.value.partition.name = new String [1];
        _sQos.value.partition.name[0] = channelName;
        _subscriber = _dp.create_subscriber (_sQos.value, null, DDS.ANY_STATUS.value);
        _sQos = null;

        /*
         * Get default DataReader QoS settings
         */
        _subscriber.get_default_datareader_qos (_drQos);
        //_drQos.value.history.kind = DDS.HistoryQosPolicyKind.KEEP_ALL_HISTORY_QOS;

        // add default Topic QoS settings to DataReader QoS settings
        _subscriber.copy_from_topic_qos(_drQos, _tQos.value);

        /*  Create Topic */
        try {
            Class<?> typeClass =
                Class.forName(_messageDataType + "TypeSupport");
            _PP_string_dt = typeClass.newInstance();

            Class[] argTypes =
                new Class[] {Class.forName("DDS.DomainParticipant"),
                             _messageDataType.getClass()};
            java.lang.reflect.Method registerMethod =
                typeClass.getMethod("register_type", argTypes);
            registerMethod.invoke(_PP_string_dt, _dp,
                                  _messageDataType.replaceAll("\\.", "::"));
        }
        catch (Exception excpt) {
            excpt.printStackTrace(System.err);
        }

        _theTopic =
            _dp.create_topic (topicName, _messageDataType.replaceAll("\\.", "::"),
                              _tQos.value, null, DDS.ANY_STATUS.value);


        /* Create datareader */
        try {
            Class<?> typeClass =
                Class.forName(_messageDataType + "DataReaderHelper");
            Class[] argTypes = new Class[] {Class.forName("java.lang.Object")};
            java.lang.reflect.Method narrowMethod =
                typeClass.getMethod("narrow", argTypes);
            _PP_string_reader = narrowMethod.
                invoke(null,
                       _subscriber.create_datareader (_theTopic,
                                                      _drQos.value, null,
                                                      DDS.ANY_STATUS.value));
        }
        catch (Exception excpt) {
            excpt.printStackTrace(System.err);
        }


        /* Add datareader statuscondition to waitset */
        //_PP_string_sc = _PP_string_reader.get_statuscondition ();
        try {
            Class<?> readerClass = _PP_string_reader.getClass();
            Class[] argTypes = new Class[] {};
            java.lang.reflect.Method scMethod =
                readerClass.getMethod("get_statuscondition", argTypes);
            Object[] args = new Object[] {};
            _PP_string_sc = (DDS.StatusCondition)
                scMethod.invoke(_PP_string_reader, args);
        }
        catch (Exception excpt) {
            excpt.printStackTrace(System.err);
        }

        _PP_string_sc.set_enabled_statuses (DDS.DATA_AVAILABLE_STATUS.value);
        result = _waitSet.attach_condition (_PP_string_sc);
        assert(result == RETCODE_OK.value);

    }


    public synchronized Vector<Object> receive() {
        return receive(-1);
    }

    public synchronized Vector<Object> receive(long timeout) {

        Vector<Object> objectVector = new Vector<Object>();
        long startTime = System.currentTimeMillis();

        int     result;
        int     i;
        int     imax;
        int     j;
        int     jmax;

        DDS.ConditionSeqHolder conditionList = new DDS.ConditionSeqHolder();
        DDS.SampleInfoSeqHolder infoList = new DDS.SampleInfoSeqHolder();
    
        /* Create seqholder */
        Object PP_string_dataList = null;
        Class<?> holderClass = null;
        try {
            holderClass = Class.forName(_messageDataType + "SeqHolder");
            PP_string_dataList = holderClass.newInstance();
        }
        catch (Exception excpt) {
            excpt.printStackTrace(System.err);
        }
    
        DDS.Duration_t wait_timeout;
        if (timeout < 0) {
            wait_timeout =
                new DDS.Duration_t (DDS.DURATION_INFINITE_SEC.value,
                                    DDS.DURATION_INFINITE_NSEC.value);
        }
        else {
            int secValue = (int) (timeout / 1000);
            int nsecValue = (int) ((timeout - (secValue * 1000)) * 1000000);
            wait_timeout =
                new DDS.Duration_t (secValue, nsecValue);
        }

        // loop until either we have some data to return or the timeout
        // has expired
        while (objectVector.size() == 0 &&
               (timeout < 0 || 
                (System.currentTimeMillis() - startTime) < timeout)) {

            //System.out.println("Before wait");
            result = _waitSet._wait (conditionList, wait_timeout);
            //System.out.println("After wait, result = " + result);
            if (result == RETCODE_ALREADY_DELETED.value) {
                return null;  // or throw exception!
            }
            if (conditionList.value != null) {
                imax = conditionList.value.length;
                //System.out.println("ConditionList size = " + imax);
                for (i = 0; i < imax; i++) {
                    if (conditionList.value[i] == _PP_string_sc) {
                        //System.out.println ("DDSReader: Message arrived.");

                        try {
                            Class<?> readerClass = _PP_string_reader.getClass();
                            Class[] argTypes =
                                new Class[] {holderClass, infoList.getClass(),
                                             int.class, int.class, int.class, int.class};
                            java.lang.reflect.Method takeMethod =
                                readerClass.getMethod("take", argTypes);
                            Integer retval = (Integer)
                                takeMethod.invoke(_PP_string_reader, PP_string_dataList,
                                                  infoList, DDS.LENGTH_UNLIMITED.value,
                                                  DDS.ANY_SAMPLE_STATE.value,
                                                  DDS.ANY_VIEW_STATE.value,
                                                  DDS.ANY_INSTANCE_STATE.value);
                            result = retval.intValue();
                        }
                        catch (Exception excpt) {
                            excpt.printStackTrace(System.err);
                        }

                        try {
                            java.lang.reflect.Field valueField = holderClass.getField("value");
                            Object theArray = valueField.get(PP_string_dataList);
                            jmax = java.lang.reflect.Array.getLength(theArray);
                            //System.out.println("jmax = " + jmax);

                            if (jmax != 0) {
                                for (int kdx = 0; kdx < jmax; kdx++) {
                                    if (infoList.value[kdx].valid_data) {
                                        // do we really need to make a copy?
                                        Object tmpObject = java.lang.reflect.Array.get(theArray, kdx);

                                        Class<?> msgType = Class.forName(_messageDataType);
                                        Object theObject = msgType.newInstance();
                                        Field[] fields = msgType.getDeclaredFields();
                                        for (int idx = 0; idx < fields.length; idx++) {
                                            fields[idx].setAccessible(true);
                                            fields[idx].set(theObject, fields[idx].get(tmpObject));
                                        }
                                        objectVector.addElement(theObject);
                                    }
                                }
                                Class<?> readerClass = _PP_string_reader.getClass();
                                Class[] argTypes =
                                    new Class[] {holderClass, infoList.getClass()};
                                java.lang.reflect.Method returnLoanMethod =
                                    readerClass.getMethod("return_loan", argTypes);
                                returnLoanMethod.invoke(_PP_string_reader, PP_string_dataList,
                                                        infoList);
                            } else {
                                System.out.println ("CONSUMER: Message triggered, but no data available");
                            }
                        }
                        catch (Exception excpt) {
                            excpt.printStackTrace(System.err);
                        }
                    
                    } else {
                        System.out.println ("CONSUMER: unknown condition triggered: " + conditionList.value[i]);
                    }
                }
            } else {
                System.out.println ("CONSUMER: unknown condition triggered");
            }
        }

        return objectVector;
    }


    public synchronized void close() {
    
        int     result;

        if (_drQos == null) return;

        try {
            Class<?> subscriberClass = _subscriber.getClass();
            //Class[] argTypes = new Class[] {_PP_string_reader.getClass()};
            Class[] argTypes = new Class[] {DDS.DataReader.class};
            java.lang.reflect.Method delMethod =
                subscriberClass.getMethod("delete_datareader", argTypes);
            delMethod.invoke(_subscriber, _PP_string_reader);
        }
        catch (Exception excpt) {
            excpt.printStackTrace(System.err);
        }

        result = _dp.delete_topic (_theTopic);
        result = _dp.delete_subscriber (_subscriber);
        result = _dpf.delete_participant (_dp);
        _waitSet = null;
        _PP_string_dt = null;
        _dpQos = null;
        _tQos = null;
        _drQos = null;
    
        return;
    }

    public void finalize() {
        close();
    }
}
