package gov.fnal.cd.rms.provider;

import DDS.RETCODE_OK;

public class DDSWriter {

    private String _messageDataType;

    private String myDomain = null;  // use OSPL_URI env var
    private DDS.DomainParticipantFactory       _dpf;
    private DDS.DomainParticipant              _dp;
    private DDS.Publisher                      _publisher;

    //private helloworld.HelloWorldMessageDataWriter   _PP_string_writer;
    private Object _PP_string_writer;

    //private helloworld.HelloWorldMessageTypeSupport  _PP_string_dt;
    private Object _PP_string_dt;

    private DDS.Topic                          _theTopic;

    private DDS.DomainParticipantQosHolder _dpQos;
    private DDS.TopicQosHolder             _tQos;
    private DDS.DataWriterQosHolder        _dwQos;


    public DDSWriter(String messageDataType, String domainURI,
                     String topicName, String channelName) {


        DDS.PublisherQosHolder         pQos;

        _messageDataType = messageDataType;

        /*
         * Initialize Qos variables
         */
        _dpQos = new DDS.DomainParticipantQosHolder();
        _tQos  = new DDS.TopicQosHolder();
        pQos  = new DDS.PublisherQosHolder();
        _dwQos = new DDS.DataWriterQosHolder();

        /*
         * Create participant
         */
        _dpf = DDS.DomainParticipantFactory.get_instance ();
	_dpf.get_default_participant_qos (_dpQos);
        _dp = _dpf.create_participant (myDomain, _dpQos.value, null, DDS.ANY_STATUS.value);
        if (_dp == null) {
            System.out.println ("PRODUCER: ERROR - Splice Daemon not running");
            return;
        }
	_dpQos = null;

        /*
         * Modify default Topic Qos settings
         */
        _dp.get_default_topic_qos (_tQos);
        _tQos.value.reliability.kind = DDS.ReliabilityQosPolicyKind.RELIABLE_RELIABILITY_QOS;
        //_tQos.value.history.kind = DDS.HistoryQosPolicyKind.KEEP_ALL_HISTORY_QOS;
        _dp.set_default_topic_qos (_tQos.value);

        /* 
         * Create publisher
         */
        _dp.get_default_publisher_qos (pQos);
	pQos.value.partition.name = new String [1];
	pQos.value.partition.name[0] = channelName;
        _publisher = _dp.create_publisher (pQos.value, null, DDS.ANY_STATUS.value);
        pQos = null;

	/*
	 * Get default DataWriter QoS settings
	 */
	_publisher.get_default_datawriter_qos (_dwQos);

        // add default Topic QoS settings to DataReader QoS settings
        _publisher.copy_from_topic_qos(_dwQos, _tQos.value);

        /*  Create Topic */
        try {
            Class<?> typeClass =
                Class.forName(_messageDataType + "TypeSupport");
            _PP_string_dt = typeClass.newInstance();

            Class[] argTypes =
                new Class[] {Class.forName("DDS.DomainParticipant"),
                             _messageDataType.getClass()};
            java.lang.reflect.Method registerMethod =
                typeClass.getMethod("register_type", argTypes);
            registerMethod.invoke(_PP_string_dt, _dp,
                                  _messageDataType.replaceAll("\\.", "::"));
            //System.out.println("QQQ " + _messageDataType.replaceAll("\\.", "::"));
        }
        catch (Exception excpt) {
            excpt.printStackTrace(System.err);
        }

        _theTopic =
            _dp.create_topic (topicName, _messageDataType.replaceAll("\\.", "::"),
                              _tQos.value, null, DDS.ANY_STATUS.value);

        /* Create datawriter */
        try {
            Class<?> typeClass =
                Class.forName(_messageDataType + "DataWriterHelper");
            Class[] argTypes = new Class[] {Class.forName("java.lang.Object")};
            java.lang.reflect.Method narrowMethod =
                typeClass.getMethod("narrow", argTypes);
            _PP_string_writer = narrowMethod.
                invoke(null,
                       _publisher.create_datawriter (_theTopic,
                                                     _dwQos.value, null,
                                                     DDS.ANY_STATUS.value));
        }
        catch (Exception excpt) {
            excpt.printStackTrace(System.err);
        }

        // extremely temporary - until we find a way to ensure that the
        // first message after creation gets through reliably
        try {Thread.sleep(500);}
        catch (Exception excpt) {}
    }

    public synchronized int send(Object messageObject) {

        int result = -1;

        Class<?> writerClass = _PP_string_writer.getClass();

        try {
            Class[] argTypes =
                new Class[] {Class.forName(_messageDataType), long.class};
            java.lang.reflect.Method writeMethod =
                writerClass.getMethod("write", argTypes);
            Integer retval = (Integer)
                writeMethod.invoke(_PP_string_writer, messageObject,
                                   new Long(DDS.HANDLE_NIL.value));
            result = retval.intValue();
        }
        catch (Exception excpt) {
            excpt.printStackTrace(System.err);
        }

        return result;
    }

    public synchronized void close() {

        int result;

        if (_dwQos == null) return;

        result = _publisher.delete_datawriter ((DDS.DataWriter) _PP_string_writer);
        result = _dp.delete_topic (_theTopic);
        result = _dp.delete_publisher (_publisher);
        result = _dpf.delete_participant (_dp);
        _PP_string_dt = null;
        _dpQos = null;
        _tQos = null;
        _dwQos = null;

        //System.out.println("PRODUCER: exiting");

        return;
    }

    public void finalize() {
        close();
    }

}
