package gov.fnal.cd.rms.provider;

import gov.fnal.cd.rms.base.RmsDestination;
import gov.fnal.cd.rms.base.RmsMessage;
import gov.fnal.cd.rms.base.RmsMessageSource;

/**
 * This utility class provides methods that are used by, or use,
 * parts of the RMS provider layer.
 *
 * @author Kurt Biery
 * @author Steve Foulkes
 * @version $Revision: 1.2 $ $Date: 2009/11/25 14:52:20 $
 */
public class RmsProviderUtil {

    /**
     * Fills in the standard fields of the specified RmsMessage (fields
     * such as message ID and sent time).
     *
     * @param pendingMessage the RmsMessage to be modified
     * @param rmsConnection the RmsConnection through which the message
     *        will be sent (used for fetching connection-based information)
     * @throws novadaq.DataException if a problem occurs creating or filling
     *         the standard field data
     */
    public static void setStandardFields(RmsMessage pendingMessage,
                                         RmsConnection rmsConnection)
        throws novadaq.DataException {

        // check for valid inputs
        if (pendingMessage == null) {return;}
        if (rmsConnection == null) {return;}

        // fill in the easy stuff
        pendingMessage.setSentTime(new java.util.Date());
        try {
            pendingMessage.setId(rmsConnection.getUUID());
        }
        catch (novadaq.DataException rdExcpt) {
            // including the message data in the novadaq.DataException would be
            // nice to have, but it's not necessary at all, so we try to
            // get it, but continue blithely along if we can't
            String messageData = null;
            try {
                messageData = pendingMessage.serialize();
            }
            catch (Throwable thrw) {  // catch anything and everything!
                messageData = pendingMessage.toString();
            }
            rdExcpt.setData(messageData);
            throw rdExcpt;
        }

        // Since we now have reply destinations on all messages, including
        // replies, the message source isn't really necessary.

        // fill in the message source, if needed
        //RmsMessageSource msgSource = pendingMessage.getSource();
        //if (msgSource == null) {
        //    msgSource = new RmsMessageSource();
        //    try {
        //        String hostString =
        //            java.net.InetAddress.getLocalHost().getHostName();
        //        msgSource.setHostName(hostString);
        //    }
        //    catch (java.net.UnknownHostException uhExcpt) {
        //        // including the message data in the novadaq.DataException would be
        //        // nice to have, but it's not necessary at all, so we try to
        //        // get it, but continue blithely along if we can't
        //        String messageData = null;
        //        try {
        //            messageData = pendingMessage.serialize();
        //        }
        //        catch (Throwable thrw) {  // catch anything and everything!
        //            messageData = pendingMessage.toString();
        //        }
        //
        //        String excptMsg = "Error determining the hostname for a " +
        //            "message source.";
        //        throw new novadaq.DataException(excptMsg, uhExcpt, messageData);
        //    }
        //    msgSource.setApplicationName(rmsConnection.getMessageSource().
        //                                 getProperty(RmsDestination.
        //                                             TARGET_PROPERTY_NAME));
        //    pendingMessage.setSource(msgSource);
        //}
    }
}
