import gov.fnal.cd.rms.provider.RmsConnection;
import gov.fnal.cd.rms.provider.RmsConnectionFactory;

import gov.fnal.cd.rms.base.RmsDestination;
import gov.fnal.cd.rms.base.RmsMessage;
import gov.fnal.cd.rms.base.RmsMessageBody;
import gov.fnal.cd.rms.RmsProducer;

/**
 * Send a loopback message to an RMS destination
 * and see if we get a reply back.
 *
 * @author Kurt Biery
 * @author Steve Foulkes
 * @version $Revision: 1.1.1.1 $ $Date: 2009/10/09 20:03:00 $
 */
public class RmsLoopbackTest {
    private RmsConnection _myConnection;
    private RmsProducer _myProducer;
    private RmsDestination _myDestination;

    /**
     * Create provider, connection and producer objects based
     * upon what was passed in from the command line.
     *
     * @param args The command line arguments where args[0] is the 
     * application name, args[1] is the target name, and args[2] is
     * the target service.
     */ 
    private void initializeRms(String[] args) {
	try {
	    _myConnection = RmsConnectionFactory.getConnection(args[0], -1);
	}
	catch (Exception excp) {
	    System.out.println("Had some problems creating the connection");
	    System.out.println(excp);
	
	    return;
	}

	_myDestination = new RmsDestination(args[1], args[2], RmsDestination.LOOPBACK_MESSAGE_TYPE);

	try {
	    _myProducer = new RmsProducer(_myConnection, _myDestination);
	}
	catch (Exception excp) {
	    System.out.println("Exception when trying to create a producer: " + excp);
	    return;
	}

	return;
    }

    /**
     * Send a loopback message to the destination specified
     * on the command line, and time how long it takes to receive
     * the reply.
     *
     * @param args The command line arguments.
     */
    public static void main(String[] args) {
	RmsLoopbackTest testObject = new RmsLoopbackTest();

	if(args.length != 4) {
	    System.out.println("Usage:");
	    System.out.println("RmsLoopbackTest [node name] [target name] [target service] [timeout]");
	    
	    System.exit(0);
	}

	testObject.initializeRms(args);

	RmsMessage myMessage;
	RmsMessageBody myMessageBody;

	myMessageBody = new RmsMessageBody();
	myMessageBody.setContent("PING");
	myMessage = new RmsMessage(myMessageBody);

	long startTime = System.currentTimeMillis();

	try {
	    testObject._myProducer.sendMessage(myMessage);
	}
	catch (Exception excp) {
	    System.out.println("Sending of the message failed: " + excp);
	}

	int timeout = 0;

	try {
	    timeout = Integer.parseInt(args[3], 10);
	}
	catch (Exception excpt) {}

	if(testObject._myProducer.receiveReply(1000 * timeout) == null) {
	    System.out.println("Request timed out.");
	    System.exit(0);
	}

	long totalTime = System.currentTimeMillis() - startTime;

	System.out.println("Round trip time: " + totalTime + " miliseconds");

	System.exit(0);
    }
}
