import gov.fnal.cd.rms.provider.RmsConnection;
import gov.fnal.cd.rms.provider.RmsConnectionFactory;
import gov.fnal.cd.rms.base.RmsDestination;
import gov.fnal.cd.rms.base.RmsMessage;
import gov.fnal.cd.rms.base.RmsMessageBody;
import gov.fnal.cd.rms.RmsProducer;
import gov.fnal.cd.rms.RmsConsumer;

public class RmsSendAndReceiveTest {
    private RmsConnection _myConnection;
    private RmsProducer _myProducer;
    private RmsConsumer _myConsumer;

    /**
     * Initialize the RMS provider routines.  Startup the Localhost provider
     * and create a producer and consumer with the same destination so we
     * can send messages between them.
     *
     * @return NONE
     */
    private void initializeRms() {
	RmsDestination myDestination;

	try {
	    _myConnection = RmsConnectionFactory.getConnection("RmsSendAndReceive", -1,
                                                               RmsConnectionFactory.LOCALHOST_PROVIDER);
	}
	catch (Exception excp) {
	    System.out.println("Had some problems creating the connection");
	    System.out.println(excp);

	    return;
	}

	myDestination = new RmsDestination("configManager", "control", "request");

	try {
	    _myProducer = new RmsProducer(_myConnection, myDestination);
	}
	catch (Exception excp) {
	    System.out.println("Exception when trying to create a producer: " + excp);
	    return;
	}

	try {
	    _myConsumer = new RmsConsumer(_myConnection, myDestination);
	}
	catch (Exception excp) {
	    System.out.println("Exception when trying to create a consumer: " + excp);
	    return;
	}

	return;
    }

    /**
     * Send a message with "Hello World!" in the body through the consumer.
     *
     * @return NONE
     */
    private void sendMessage() {
	RmsMessage myMessage;
	RmsMessageBody myMessageBody;

	myMessageBody = new RmsMessageBody();
	myMessageBody.setContent("Hello World!");
	myMessage = new RmsMessage(myMessageBody);

	System.out.println("Sending the following:");
	System.out.println(myMessageBody.getContent());
	System.out.println("");

	try {
	    _myProducer.sendMessage(myMessage);
	}
	catch (Exception excp) {
	    System.out.println("Sending of the message failed: " + excp);
	}
	
	return;
    }

    /**
     * Receive a message through the consumer, and send a reply.
     */
    private void receiveMessage() {
	RmsMessage incomingMessage;
	RmsMessage reply;
	RmsMessageBody replyMessageBody;

	incomingMessage = _myConsumer.receiveMessage();

	System.out.println("Received the following:");
	System.out.println(incomingMessage.getBody().getContent());
	System.out.println("");

	System.out.println("\nGenerating a reply...\n");

	reply = incomingMessage.createReply();

	replyMessageBody = new RmsMessageBody();
	replyMessageBody.setContent("Hello World! REPLY!!!111!!1!");
	reply.setBody(replyMessageBody);

	try {
	    _myConsumer.sendReply(reply);
	}
	catch (Exception excp) {
	    System.out.println("Sending of the reply failed: " + excp);
	}
	
	return;
    }

    /**
     * Query the producer for a reply, and print out the reply.
     */
    private void receiveReply() {
	RmsMessage reply;

	reply = _myProducer.receiveReply();

	System.out.println("Received the following reply:");
	System.out.println(reply.getBody().getContent());
	System.out.println("");

	return;
    }

    public static void main(String[] args) {
	RmsSendAndReceiveTest testObject = new RmsSendAndReceiveTest();

	testObject.initializeRms();
	for(int i = 0; i < 10; i++) {
	    testObject.sendMessage();
	    testObject.receiveMessage();
	    testObject.receiveReply();
	}

	try {
	    testObject._myConnection.close();
	}
	catch (Exception excp) {
	    return;
	}

	return;
    }
}
