#ifndef _RMS_BUFFEREDPROVIDERLISTENER_H
#define _RMS_BUFFEREDPROVIDERLISTENER_H

#include <rms/provider/ProviderListener.h>
#include <rms/util/LinkedBlockingQueue.h>

#include <boost/shared_ptr.hpp>

#include <string>

namespace gov {

namespace fnal {

namespace cd {

namespace rms {

namespace provider {

/**
 * This class provides an implementation of the ProviderListener interface
 * that copies the messages that are received into a specified queue.
 *
 * @author Kurt Biery
 * @author Steve Foulkes
 * @version $Revision: 1.1.1.1 $ $Date: 2009/10/09 20:03:00 $
 */

class BufferedProviderListener : public ProviderListener {
  public:

    /**
     * Creates a new instance associated with the specified queue.
     *
     * @param inputQueue The LinkedBlockingQueue to use.
     */
    BufferedProviderListener(util::LinkedBlockingQueue 
                             <boost::shared_ptr<std::string> > *inputQueue) {
        _inputQueue = inputQueue;
    }

    /**
     * Method called by the provider when a message is received.  This
     * will add the message to the queue.
     *
     * @param messageText The message that has been received.
     */
    void textReceived(boost::shared_ptr<std::string> messageText) {
        _inputQueue->add(messageText);
    }

  private:
    /**
     * The queue that provides the buffering of the messages
     * that are received from the provider.
     */
    util::LinkedBlockingQueue <boost::shared_ptr<std::string> > *_inputQueue;

};

};  // end of namespace provider

};  // end of namespace rms

};  // end of namespace cd

};  // end of namespace fnal

};  // end of namespace gov

#endif
