#ifndef _RMS_PROVIDERLISTENER_H
#define _RMS_PROVIDERLISTENER_H

#include <boost/shared_ptr.hpp>

#include <string>

namespace gov {

namespace fnal {

namespace cd {

namespace rms {

namespace provider {

/**
 * The interface for classes that want to be notified when a
 * string message has been received from a provider.
 *
 * Classes that implement this interface and are declared
 * as listeners for provider-level messages are notified
 * when messages are received.
 *
 * @author Kurt Biery
 * @author Steve Foulkes
 * @version $Revision: 1.1.1.1 $ $Date: 2009/10/09 20:03:00 $
 */

class ProviderListener {
  public:

    /**
     * Destructor (declared virtual to ensure that derived class
     * destructors are always called).
     */
    virtual ~ProviderListener() {}
    
    /**
     * Invoked when a string message is received from a provider.
     *
     * @param messageText The string containing the message that was
     * received.
     */
    virtual void textReceived(boost::shared_ptr<std::string> messageText) = 0;
};

};  // end of namespace provider

};  // end of namespace rms

};  // end of namespace cd

};  // end of namespace fnal

};  // end of namespace gov

#endif
