#ifndef _RMS_CONNECTIONFACTORY_H
#define _RMS_CONNECTIONFACTORY_H

#include <rms/provider/RmsConnection.h>
#include <boost/shared_ptr.hpp>
#include <string>

namespace gov {

namespace fnal {

namespace cd {

namespace rms {

namespace provider {

// NOTE:  when adding a new provider type, be sure to check for all places
// that existing ones are used and add the new type as needed.  (There is
// more to adding a provider than just adding a new constant string.)

/**
 * This class provides methods for creating connection objects for
 * different providers.
 * (The connection objects implement the RmsConnection interface and
 * are used by consumers and producers to communicate with
 * the underlying messaging provider.)
 *
 * @author Kurt Biery
 * @author Steve Foulkes
 * @version $Revision: 1.4 $ $Date: 2010/02/18 15:32:12 $
 */
class RmsConnectionFactory {
 public:
    static const std::string EPICS_PROVIDER;
    static const std::string LOCALHOST_PROVIDER;

    boost::shared_ptr<RmsConnection> getConnection(const std::string applicationName,
                                                   const int applicationPartition);
    boost::shared_ptr<RmsConnection> getConnection(const std::string applicationName,
                                                   const int applicationPartition,
                                                   std::string providerType);
};

};  // end of namespace gov
};  // end of namespace fnal
};  // end of namespace cd
};  // end of namespace rms
};  // end of namespace provider


#endif
