# this function processes the ddsidl files
# it will fill the variable name DDSIDL_LIB_SOURCES with the 
# processed .cpp files
function(process_ddsidl DDSIDL_LIB_SOURCES)

  SET(DDSIDL_GEN_LANG  	     "cpp")				       
  SET(DDSIDL_COMPILE   	     "$ENV{OSDDS_DIR}/bin/idlpp")                  
  SET(DDSIDL_COMPILE_OPTIONS "-S")
  SET(DDSIDL_TAILOR          "${TOPLEVEL_NOVADAQ_SRC_DIR}/ups/ddsidl_tailor.sh")
  SET(DDSIDL_DEFAULT_INC     "${TOPLEVEL_NOVADAQ_SRC_DIR}/rms/config/")

  MESSAGE("DDSIDL_IDL_FILES set to ${DDSIDL_IDL_FILES}")

  # loop over all .idl files in the DDSIDL_IDL_FILES variable
  foreach(source ${DDSIDL_IDL_FILES})
    #define some local variables for the produced .h and .cpp files
    get_filename_component(IDL_BASENAME     ${source} NAME_WE)
    get_filename_component(IDL_BASENAME_EXT ${source} NAME)

    list(APPEND TMP_SOURCES ${IDL_BASENAME}.cpp 	      
			    ${IDL_BASENAME}Dcps.cpp     
			    ${IDL_BASENAME}Dcps_impl.cpp
			    ${IDL_BASENAME}SplDcps.cpp  )

    # call ddsidl and generate the source and header files

    add_custom_command(OUTPUT ${CMAKE_CURRENT_BINARY_DIR}/${DDSIDL_TOPLEVEL}ccpp_${IDL_BASENAME}.h
    			      ${CMAKE_CURRENT_BINARY_DIR}/${DDSIDL_TOPLEVEL}${IDL_BASENAME}.h 
			      ${CMAKE_CURRENT_BINARY_DIR}/${DDSIDL_TOPLEVEL}${IDL_BASENAME}Dcps.h
			      ${CMAKE_CURRENT_BINARY_DIR}/${DDSIDL_TOPLEVEL}${IDL_BASENAME}Dcps_impl.h 
			      ${CMAKE_CURRENT_BINARY_DIR}/${DDSIDL_TOPLEVEL}${IDL_BASENAME}SplDcps.h
			      ${CMAKE_CURRENT_BINARY_DIR}/${IDL_BASENAME}.cpp 
			      ${CMAKE_CURRENT_BINARY_DIR}/${IDL_BASENAME}Dcps.cpp
		       	      ${CMAKE_CURRENT_BINARY_DIR}/${IDL_BASENAME}Dcps_impl.cpp
			      ${CMAKE_CURRENT_BINARY_DIR}/${IDL_BASENAME}SplDcps.cpp

      COMMAND ${DDSIDL_COMPILE} ${DDSIDL_COMPILE_OPTIONS} -l ${DDSIDL_GEN_LANG} -I ${DDSIDL_DEFAULT_INC} -I ${DDSIDL_IDL_DIR} -d ${CMAKE_CURRENT_BINARY_DIR} ${source}
      COMMAND ${DDSIDL_TAILOR} ${IDL_BASENAME} $ENV{PACKAGE}
      COMMAND ${CMAKE_COMMAND} -E copy ${CMAKE_CURRENT_BINARY_DIR}/ccpp_${IDL_BASENAME}.h     ${CMAKE_CURRENT_BINARY_DIR}/${DDSIDL_TOPLEVEL}ccpp_${IDL_BASENAME}.h
      COMMAND ${CMAKE_COMMAND} -E copy ${CMAKE_CURRENT_BINARY_DIR}/${IDL_BASENAME}.h          ${CMAKE_CURRENT_BINARY_DIR}/${DDSIDL_TOPLEVEL}${IDL_BASENAME}.h
      COMMAND ${CMAKE_COMMAND} -E copy ${CMAKE_CURRENT_BINARY_DIR}/${IDL_BASENAME}Dcps.h      ${CMAKE_CURRENT_BINARY_DIR}/${DDSIDL_TOPLEVEL}${IDL_BASENAME}Dcps.h
      COMMAND ${CMAKE_COMMAND} -E copy ${CMAKE_CURRENT_BINARY_DIR}/${IDL_BASENAME}Dcps_impl.h ${CMAKE_CURRENT_BINARY_DIR}/${DDSIDL_TOPLEVEL}${IDL_BASENAME}Dcps_impl.h
      COMMAND ${CMAKE_COMMAND} -E copy ${CMAKE_CURRENT_BINARY_DIR}/${IDL_BASENAME}SplDcps.h   ${CMAKE_CURRENT_BINARY_DIR}/${DDSIDL_TOPLEVEL}${IDL_BASENAME}SplDcps.h
      COMMAND ${CMAKE_COMMAND} -E remove ${CMAKE_CURRENT_BINARY_DIR}/ccpp_${IDL_BASENAME}.h
      COMMAND ${CMAKE_COMMAND} -E remove ${CMAKE_CURRENT_BINARY_DIR}/${IDL_BASENAME}.h
      COMMAND ${CMAKE_COMMAND} -E remove ${CMAKE_CURRENT_BINARY_DIR}/${IDL_BASENAME}Dcps.h
      COMMAND ${CMAKE_COMMAND} -E remove ${CMAKE_CURRENT_BINARY_DIR}/${IDL_BASENAME}Dcps_impl.h
      COMMAND ${CMAKE_COMMAND} -E remove ${CMAKE_CURRENT_BINARY_DIR}/${IDL_BASENAME}SplDcps.h
    )

    #install the header and source files
    install(FILES ${CMAKE_CURRENT_BINARY_DIR}/${DDSIDL_TOPLEVEL}ccpp_${IDL_BASENAME}.h
		  ${CMAKE_CURRENT_BINARY_DIR}/${DDSIDL_TOPLEVEL}${IDL_BASENAME}.h
		  ${CMAKE_CURRENT_BINARY_DIR}/${DDSIDL_TOPLEVEL}${IDL_BASENAME}Dcps.h 
		  ${CMAKE_CURRENT_BINARY_DIR}/${DDSIDL_TOPLEVEL}${IDL_BASENAME}Dcps_impl.h 
		  ${CMAKE_CURRENT_BINARY_DIR}/${DDSIDL_TOPLEVEL}${IDL_BASENAME}SplDcps.h
	    DESTINATION ${${product}_inc_dir}/$ENV{PACKAGE}/
           )
    install_source(LIST  ${CMAKE_CURRENT_BINARY_DIR}/${IDL_BASENAME}.cpp 
			 ${CMAKE_CURRENT_BINARY_DIR}/${IDL_BASENAME}Dcps.cpp 
			 ${CMAKE_CURRENT_BINARY_DIR}/${IDL_BASENAME}Dcps_impl.cpp 
			 ${CMAKE_CURRENT_BINARY_DIR}/${IDL_BASENAME}SplDcps.cpp)
  endforeach()

  # set the output variable
  set(${DDSIDL_LIB_SOURCES} ${TMP_SOURCES} PARENT_SCOPE)
endfunction()
